@@ -32,6 +32,7 @@
 #include "advpub.h"
 #include "wine/unicode.h"
 #include "wine/debug.h"
+#include "advpack_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(advpack);
 
@@ -40,8 +41,6 @@ typedef HRESULT (WINAPI *DLLREGISTER) (void);
 #define MAX_FIELD_LENGTH    512
 #define PREFIX_LEN          5
 
-HRESULT launch_exe(LPCWSTR cmd, LPCWSTR dir, HANDLE *phEXE);
-
 /* registry path of the Installed Components key for per-user stubs */
 static const WCHAR setup_key[] = {
     'S','O','F','T','W','A','R','E','\\',
@@ -86,22 +85,24 @@ static void get_dest_dir(HINF hInf, PCWSTR pszSection, PWSTR pszBuffer, DWORD dw
     if (RegOpenKeyW(root, key, &subkey) ||
         RegQueryValueExW(subkey, value, NULL, NULL, (LPBYTE)pszBuffer, &size))
     {
-        SetupGetStringFieldW(&context, 6, pszBuffer, dwSize, NULL);
+        SetupGetStringFieldW(&context, 5, pszBuffer, dwSize, NULL);
     }
 
     RegCloseKey(subkey);
 }
 
 /* loads the LDIDs specified in the install section of an INF */
-static void set_ldids(HINF hInf, LPCWSTR pszInstallSection)
+void set_ldids(HINF hInf, LPCWSTR pszInstallSection, LPCWSTR pszWorkingDir)
 {
     WCHAR field[MAX_FIELD_LENGTH];
-    WCHAR key[MAX_FIELD_LENGTH];
+    WCHAR line[MAX_FIELD_LENGTH];
     WCHAR dest[MAX_PATH];
     INFCONTEXT context;
     DWORD size;
     int ldid;
 
+    static const WCHAR source_dir[] = {'S','o','u','r','c','e','D','i','r',0};
+
     static const WCHAR custDestW[] = {
         'C','u','s','t','o','m','D','e','s','t','i','n','a','t','i','o','n',0
     };
@@ -115,13 +116,50 @@ static void set_ldids(HINF hInf, LPCWSTR pszInstallSection)
 
     do
     {
-        SetupGetIntField(&context, 0, &ldid);
+        LPWSTR value, ptr, key, key_copy = NULL;
+
         SetupGetLineTextW(&context, NULL, NULL, NULL,
-                          key, MAX_FIELD_LENGTH, &size);
+                          line, MAX_FIELD_LENGTH, &size);
 
-        get_dest_dir(hInf, key, dest, MAX_PATH);
+        /* SetupGetLineTextW returns the value if there is only one key, but
+         * returns the whole line if there is more than one key
+         */
+        if (!(value = strchrW(line, '=')))
+        {
+            SetupGetStringFieldW(&context, 0, NULL, 0, &size);
+            key = HeapAlloc(GetProcessHeap(), 0, size * sizeof(WCHAR));
+            key_copy = key;
+            SetupGetStringFieldW(&context, 0, key, size, &size);
+            value = line;
+        }
+        else
+        {
+            key = line;
+            *(value++) = '\0';
+        }
 
-        SetupSetDirectoryIdW(hInf, ldid, dest);
+        /* remove leading whitespace from the value */
+        while (*value == ' ')
+            value++;
+
+        /* FIXME: need to check the query option */
+        ptr = strchrW(value, ',');
+        if (ptr)
+            *ptr = '\0';
+
+        /* set dest to pszWorkingDir if key is SourceDir */
+        if (pszWorkingDir && !lstrcmpiW(value, source_dir))
+            lstrcpynW(dest, pszWorkingDir, MAX_PATH);
+        else
+            get_dest_dir(hInf, value, dest, MAX_PATH);
+
+        /* set all ldids to dest */
+        while ((ptr = get_parameter(&key, ',')))
+        {
+            ldid = atolW(ptr);
+            SetupSetDirectoryIdW(hInf, ldid, dest);
+        }
+        HeapFree(GetProcessHeap(), 0, key_copy);
     } while (SetupFindNextLine(&context, &context));
 }
 
@@ -327,7 +365,7 @@ HRESULT WINAPI OpenINFEngineW(LPCWSTR pszInfFilename, LPCWSTR pszInstallSection,
     if (*phInf == INVALID_HANDLE_VALUE)
         return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
 
-    set_ldids(*phInf, pszInstallSection);
+    set_ldids(*phInf, pszInstallSection, NULL);
     
     return S_OK;
 }
@@ -392,48 +430,82 @@ HRESULT WINAPI RebootCheckOnInstallW(HWND hWnd, LPCWSTR pszINF,
     return E_FAIL;
 }
 
+/* registers the OCX if do_reg is TRUE, unregisters it otherwise */
+HRESULT do_ocx_reg(HMODULE hocx, BOOL do_reg)
+{
+    DLLREGISTER reg_func;
+
+    if (do_reg)
+        reg_func = (DLLREGISTER)GetProcAddress(hocx, "DllRegisterServer");
+    else
+        reg_func = (DLLREGISTER)GetProcAddress(hocx, "DllUnregisterServer");
+
+    if (!reg_func)
+        return E_FAIL;
+
+    reg_func();
+    return S_OK;
+}
+
 /***********************************************************************
  *             RegisterOCX    (ADVPACK.@)
+ *
+ * Registers an OCX.
+ *
+ * PARAMS
+ *   hWnd    [I] Handle to the window used for the display.
+ *   hInst   [I] Instance of the process.
+ *   cmdline [I] Contains parameters in the order OCX,flags,param.
+ *   show    [I] How the window should be shown.
+ *
+ * RETURNS
+ *   Success: S_OK.
+ *   Failure: E_FAIL.
+ *
+ * NOTES
+ *   OCX - Filename of the OCX to register.
+ *   flags - Controls the operation of RegisterOCX.
+ *    'I' Call DllRegisterServer and DllInstall.
+ *    'N' Only call DllInstall.
+ *   param - Command line passed to DllInstall.
  */
-void WINAPI RegisterOCX(HWND hWnd, HINSTANCE hInst, LPCSTR cmdline, INT show)
+HRESULT WINAPI RegisterOCX(HWND hWnd, HINSTANCE hInst, LPCSTR cmdline, INT show)
 {
-    WCHAR wszBuff[MAX_PATH];
-    WCHAR* pwcComma;
-    HMODULE hm;
-    DLLREGISTER pfnRegister;
-    HRESULT hr;
+    LPWSTR ocx_filename, str_flags, param;
+    LPWSTR cmdline_copy, cmdline_ptr;
+    UNICODE_STRING cmdlineW;
+    HRESULT hr = E_FAIL;
+    HMODULE hm = NULL;
+    DWORD size;
 
     TRACE("(%s)\n", debugstr_a(cmdline));
 
-    MultiByteToWideChar(CP_ACP, 0, cmdline, strlen(cmdline), wszBuff, MAX_PATH);
-    if ((pwcComma = strchrW( wszBuff, ',' ))) *pwcComma = 0;
+    RtlCreateUnicodeStringFromAsciiz(&cmdlineW, cmdline);
 
-    TRACE("Parsed DLL name (%s)\n", debugstr_w(wszBuff));
+    size = (lstrlenW(cmdlineW.Buffer) + 1) * sizeof(WCHAR);
+    cmdline_copy = HeapAlloc(GetProcessHeap(), 0, size);
+    cmdline_ptr = cmdline_copy;
+    lstrcpyW(cmdline_copy, cmdlineW.Buffer);
 
-    hm = LoadLibraryExW(wszBuff, 0, LOAD_WITH_ALTERED_SEARCH_PATH);
+    ocx_filename = get_parameter(&cmdline_ptr, ',');
+    if (!ocx_filename || !*ocx_filename)
+        goto done;
+
+    str_flags = get_parameter(&cmdline_ptr, ',');
+    param = get_parameter(&cmdline_ptr, ',');
+
+    hm = LoadLibraryExW(ocx_filename, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
     if (!hm)
-    {
-        ERR("Couldn't load DLL: %s\n", debugstr_w(wszBuff));
-        return;
-    }
+        goto done;
 
-    pfnRegister = (DLLREGISTER)GetProcAddress(hm, "DllRegisterServer");
-    if (pfnRegister == NULL)
-    {
-        ERR("DllRegisterServer entry point not found\n");
-    }
-    else
-    {
-        hr = pfnRegister();
-        if (hr != S_OK)
-        {
-            ERR("DllRegisterServer entry point returned %08lx\n", hr);
-        }
-    }
-
-    TRACE("Successfully registered OCX\n");
+    hr = do_ocx_reg(hm, TRUE);
 
+done:
     FreeLibrary(hm);
+    HeapFree(GetProcessHeap(), 0, cmdline_copy);
+    RtlFreeUnicodeString(&cmdlineW);
+
+    return hr;
 }
 
 /***********************************************************************
@@ -652,7 +724,7 @@ HRESULT WINAPI TranslateInfStringW(LPCWSTR pszInfFilename, LPCWSTR pszInstallSec
     if (hInf == INVALID_HANDLE_VALUE)
         return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
 
-    set_ldids(hInf, pszInstallSection);
+    set_ldids(hInf, pszInstallSection, NULL);
 
     if (!SetupGetLineTextW(NULL, hInf, pszTranslateSection, pszTranslateKey,
                            pszBuffer, dwBufferSize, pdwRequiredSize))
