@@ -26,7 +26,6 @@
 #include "winuser.h"
 #include "winnls.h"
 #include "winerror.h"
-#include "windowsx.h"
 #include "mmsystem.h"
 #include "vfw.h"
 #include "msacm.h"
@@ -116,7 +115,7 @@ HRESULT AVIFILE_CreateACMStream(REFIID riid, LPVOID *ppv)
 
   *ppv = NULL;
 
-  pstream = (IAVIStreamImpl*)LocalAlloc(LPTR, sizeof(IAVIStreamImpl));
+  pstream = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(IAVIStreamImpl));
   if (pstream == NULL)
     return AVIERR_MEMORY;
 
@@ -124,7 +123,7 @@ HRESULT AVIFILE_CreateACMStream(REFIID riid, LPVOID *ppv)
 
   hr = IAVIStream_QueryInterface((IAVIStream*)pstream, riid, ppv);
   if (FAILED(hr))
-    LocalFree((HLOCAL)pstream);
+    HeapFree(GetProcessHeap(), 0, pstream);
 
   return hr;
 }
@@ -177,20 +176,20 @@ static ULONG WINAPI ACMStream_fnRelease(IAVIStream* iface)
       This->has = NULL;
     }
     if (This->acmStreamHdr.pbSrc != NULL) {
-      GlobalFreePtr(This->acmStreamHdr.pbSrc);
+      HeapFree(GetProcessHeap(), 0, This->acmStreamHdr.pbSrc);
       This->acmStreamHdr.pbSrc = NULL;
     }
     if (This->acmStreamHdr.pbDst != NULL) {
-      GlobalFreePtr(This->acmStreamHdr.pbDst);
+      HeapFree(GetProcessHeap(), 0, This->acmStreamHdr.pbDst);
       This->acmStreamHdr.pbDst = NULL;
     }
     if (This->lpInFormat != NULL) {
-      GlobalFreePtr(This->lpInFormat);
+      HeapFree(GetProcessHeap(), 0, This->lpInFormat);
       This->lpInFormat = NULL;
       This->cbInFormat = 0;
     }
     if (This->lpOutFormat != NULL) {
-      GlobalFreePtr(This->lpOutFormat);
+      HeapFree(GetProcessHeap(), 0, This->lpOutFormat);
       This->lpOutFormat = NULL;
       This->cbOutFormat = 0;
     }
@@ -198,7 +197,7 @@ static ULONG WINAPI ACMStream_fnRelease(IAVIStream* iface)
       IAVIStream_Release(This->pStream);
       This->pStream = NULL;
     }
-    LocalFree((HLOCAL)This);
+    HeapFree(GetProcessHeap(), 0, This);
 
     return 0;
   }
@@ -250,7 +249,7 @@ static HRESULT WINAPI ACMStream_fnCreate(IAVIStream *iface, LPARAM lParam1,
     else
       This->cbOutFormat = sizeof(PCMWAVEFORMAT);
 
-    This->lpOutFormat = (LPWAVEFORMATEX)GlobalAllocPtr(GHND, This->cbOutFormat);
+    This->lpOutFormat = HeapAlloc(GetProcessHeap(), 0, This->cbOutFormat);
     if (This->lpOutFormat == NULL)
       return AVIERR_MEMORY;
 
@@ -383,7 +382,7 @@ static HRESULT WINAPI ACMStream_fnSetFormat(IAVIStream *iface, LONG pos,
   if ((This->sInfo.dwCaps & AVIFILECAPS_CANWRITE) == 0)
     return AVIERR_READONLY;
 
-  This->lpInFormat = (LPWAVEFORMATEX)GlobalAllocPtr(GMEM_MOVEABLE, formatsize);
+  This->lpInFormat = HeapAlloc(GetProcessHeap(), 0, formatsize);
   if (This->lpInFormat == NULL)
     return AVIERR_MEMORY;
   This->cbInFormat = formatsize;
@@ -463,7 +462,7 @@ static HRESULT WINAPI ACMStream_fnRead(IAVIStream *iface, LONG start,
 
   /* Need to free destination buffer used for writing? */
   if (This->acmStreamHdr.pbDst != NULL) {
-    GlobalFreePtr(This->acmStreamHdr.pbDst);
+    HeapFree(GetProcessHeap(), 0, This->acmStreamHdr.pbDst);
     This->acmStreamHdr.pbDst     = NULL;
     This->acmStreamHdr.dwDstUser = 0;
   }
@@ -472,10 +471,9 @@ static HRESULT WINAPI ACMStream_fnRead(IAVIStream *iface, LONG start,
   if (This->acmStreamHdr.pbSrc == NULL ||
       This->acmStreamHdr.dwSrcUser < size) {
     if (This->acmStreamHdr.pbSrc == NULL)
-      This->acmStreamHdr.pbSrc = GlobalAllocPtr(GMEM_MOVEABLE, size);
+      This->acmStreamHdr.pbSrc = HeapAlloc(GetProcessHeap(), 0, size);
     else
-      This->acmStreamHdr.pbSrc = GlobalReAllocPtr(This->acmStreamHdr.pbSrc,
-						  size, GMEM_MOVEABLE);
+      This->acmStreamHdr.pbDst = HeapReAlloc(GetProcessHeap(), 0, This->acmStreamHdr.pbSrc, size);
     if (This->acmStreamHdr.pbSrc == NULL)
       return AVIERR_MEMORY;
     This->acmStreamHdr.dwSrcUser = size;
@@ -567,7 +565,7 @@ static HRESULT WINAPI ACMStream_fnWrite(IAVIStream *iface, LONG start,
 
   /* Need to free source buffer used for reading? */
   if (This->acmStreamHdr.pbSrc != NULL) {
-    GlobalFreePtr(This->acmStreamHdr.pbSrc);
+    HeapFree(GetProcessHeap(), 0, This->acmStreamHdr.pbSrc);
     This->acmStreamHdr.pbSrc     = NULL;
     This->acmStreamHdr.dwSrcUser = 0;
   }
@@ -576,10 +574,9 @@ static HRESULT WINAPI ACMStream_fnWrite(IAVIStream *iface, LONG start,
   if (This->acmStreamHdr.pbDst == NULL ||
       This->acmStreamHdr.dwDstUser < size) {
     if (This->acmStreamHdr.pbDst == NULL)
-      This->acmStreamHdr.pbDst = GlobalAllocPtr(GMEM_MOVEABLE, size);
+      This->acmStreamHdr.pbDst = HeapAlloc(GetProcessHeap(), 0, size);
     else
-      This->acmStreamHdr.pbDst = GlobalReAllocPtr(This->acmStreamHdr.pbDst,
-						  size, GMEM_MOVEABLE);
+      This->acmStreamHdr.pbDst = HeapReAlloc(GetProcessHeap(), 0, This->acmStreamHdr.pbDst, size);
     if (This->acmStreamHdr.pbDst == NULL)
       return AVIERR_MEMORY;
     This->acmStreamHdr.dwDstUser = size;
@@ -703,7 +700,7 @@ static HRESULT AVIFILE_OpenCompressor(IAVIStreamImpl *This)
     hr = AVIStreamFormatSize(This->pStream, This->sInfo.dwStart, &This->cbInFormat);
     if (FAILED(hr))
       return hr;
-    This->lpInFormat = (LPWAVEFORMATEX)GlobalAllocPtr(GMEM_MOVEABLE, This->cbInFormat);
+    This->lpInFormat = HeapAlloc(GetProcessHeap(), 0, This->cbInFormat);
     if (This->lpInFormat == NULL)
       return AVIERR_MEMORY;
 
@@ -715,7 +712,7 @@ static HRESULT AVIFILE_OpenCompressor(IAVIStreamImpl *This)
     if (This->lpOutFormat == NULL) {
       /* we must decode to default format */
       This->cbOutFormat = sizeof(PCMWAVEFORMAT);
-      This->lpOutFormat = (LPWAVEFORMATEX)GlobalAllocPtr(GHND, This->cbOutFormat);
+      This->lpOutFormat = HeapAlloc(GetProcessHeap(), 0, This->cbOutFormat);
       if (This->lpOutFormat == NULL)
 	return AVIERR_MEMORY;
 
