@@ -127,7 +127,7 @@ HRESULT AVIFILE_CreateICMStream(REFIID riid, LPVOID *ppv)
 
   *ppv = NULL;
 
-  pstream = HeapAlloc(GetProcessHeap(), 0, sizeof(IAVIStreamImpl));
+  pstream = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(IAVIStreamImpl));
   if (pstream == NULL)
     return AVIERR_MEMORY;
 
