@@ -77,7 +77,6 @@ typedef struct
     BOOL      bCaptured;	/* Is the mouse captured? */
     BOOL      bPressed;		/* Is a header item pressed (down)? */
     BOOL      bTracking;	/* Is in tracking mode? */
-    BOOL      bUnicode;		/* Unicode flag */
     INT       iMoveItem;	/* index of tracked item. (Tracking mode) */
     INT       xTrackOffset;	/* distance between the right side of the tracked item and the cursor */
     INT       xOldTrack;	/* track offset (see above) after the last WM_MOUSEMOVE */
@@ -94,11 +93,22 @@ typedef struct
 
 #define VERT_BORDER     3
 #define DIVIDER_WIDTH  10
+#define HDN_UNICODE_OFFSET 20
+#define HDN_FIRST_UNICODE (HDN_FIRST-HDN_UNICODE_OFFSET)
 
 #define HEADER_GetInfoPtr(hwnd) ((HEADER_INFO *)GetWindowLongPtrW(hwnd,0))
 
 static const WCHAR themeClass[] = {'H','e','a','d','e','r',0};
+static WCHAR emptyString[] = {0};
 
+static void HEADER_DisposeItem(HEADER_ITEM *lpItem)
+{
+    if (lpItem->pszText && lpItem->pszText != emptyString &&
+        lpItem->pszText != LPSTR_TEXTCALLBACKW) /* covers LPSTR_TEXTCALLBACKA too */
+    {
+        Free(lpItem->pszText);
+    }
+}
 
 inline static LRESULT
 HEADER_IndexToOrder (HWND hwnd, INT iItem)
@@ -168,7 +178,7 @@ HEADER_DrawItem (HWND hwnd, HDC hdc, INT iItem, BOOL bHotTrack)
     HTHEME theme = GetWindowTheme (hwnd);
     NMCUSTOMDRAW nmcd;
 
-    TRACE("DrawItem(iItem %d bHotTrack %d unicode flag %d)\n", iItem, bHotTrack, infoPtr->bUnicode);
+    TRACE("DrawItem(iItem %d bHotTrack %d unicode flag %d)\n", iItem, bHotTrack, (infoPtr->nNotifyFormat == NFR_UNICODE));
 
     if (!infoPtr->bRectsValid)
     	HEADER_SetItemBounds(hwnd);
@@ -567,6 +577,52 @@ HEADER_DrawTrackLine (HWND hwnd, HDC hdc, INT x)
     SelectObject (hdc, hOldPen);
 }
 
+/***
+ * DESCRIPTION:
+ * Convert a HDITEM into the correct format (ANSI/Unicode) to send it in a notify
+ *
+ * PARAMETER(S):
+ * [I] infoPtr : the header that wants to send the notify
+ * [O] dest : The buffer to store the HDITEM for notify. It may be set to a HDITEMA of HDITEMW
+ * [I] src  : The source HDITEM. It may be a HDITEMA or HDITEMW
+ * [I] fSourceUnicode : is src a HDITEMW or HDITEMA
+ * [O] ppvScratch : a pointer to a scratch buffer that needs to be freed after
+ *                  the HDITEM is no longer in use or NULL if none was needed
+ * 
+ * NOTE: We depend on HDITEMA and HDITEMW having the same structure
+ */
+static void HEADER_CopyHDItemForNotify(HEADER_INFO *infoPtr, HDITEMW *dest,
+    HDITEMW *src, BOOL fSourceUnicode, LPVOID *ppvScratch)
+{
+    *ppvScratch = NULL;
+    *dest = *src;
+    
+    if (src->mask & HDI_TEXT && src->pszText != LPSTR_TEXTCALLBACKW) /* covers TEXTCALLBACKA as well */
+    {
+        if (fSourceUnicode && infoPtr->nNotifyFormat != NFR_UNICODE)
+        {
+            dest->pszText = NULL;
+            Str_SetPtrWtoA((LPSTR *)&dest->pszText, src->pszText);
+            *ppvScratch = dest->pszText;
+        }
+        
+        if (!fSourceUnicode && infoPtr->nNotifyFormat == NFR_UNICODE)
+        {
+            dest->pszText = NULL;
+            Str_SetPtrAtoW(&dest->pszText, (LPSTR)src->pszText);
+            *ppvScratch = dest->pszText;
+        }
+    }
+}
+
+static UINT HEADER_NotifyCodeWtoA(UINT code)
+{
+    /* we use the fact that all the unicode messages are in HDN_FIRST_UNICODE..HDN_LAST*/
+    if (code >= HDN_LAST && code <= HDN_FIRST_UNICODE)
+        return code + HDN_UNICODE_OFFSET;
+    else
+        return code;
+}
 
 static BOOL
 HEADER_SendSimpleNotify (HWND hwnd, UINT code)
@@ -583,29 +639,38 @@ HEADER_SendSimpleNotify (HWND hwnd, UINT code)
 }
 
 static BOOL
-HEADER_SendHeaderNotify (HWND hwnd, UINT code, INT iItem, INT mask)
+HEADER_SendHeaderNotifyT (HWND hwnd, UINT code, INT iItem, INT mask, HDITEMW *lpItem)
 {
     HEADER_INFO *infoPtr = HEADER_GetInfoPtr (hwnd);
-    NMHEADERA nmhdr;
-    HDITEMA nmitem;
+    NMHEADERW nmhdr;
+    HDITEMW nmitem;
+    
+    if (lpItem == NULL)
+    {
+        /* lpItem == NULL means that we should take the actual data from the item */
+        if (mask & HDI_TEXT)
+        {
+            FIXME("(): invalid parameters - lpItem == NULL and (mask & HDI_TEXT)\n");
+            mask &= ~HDI_TEXT;
+        }
+        nmitem.mask = mask;
+        nmitem.cxy = infoPtr->items[iItem].cxy;
+        nmitem.hbm = infoPtr->items[iItem].hbm;
+        nmitem.pszText = NULL;
+        nmitem.cchTextMax = 0;
+        nmitem.fmt = infoPtr->items[iItem].fmt;
+        nmitem.lParam = infoPtr->items[iItem].lParam;
+        nmitem.iOrder = infoPtr->items[iItem].iOrder;
+        nmitem.iImage = infoPtr->items[iItem].iImage;
+        lpItem = &nmitem;
+    }
 
     nmhdr.hdr.hwndFrom = hwnd;
     nmhdr.hdr.idFrom   = GetWindowLongPtrW (hwnd, GWLP_ID);
-    nmhdr.hdr.code = code;
+    nmhdr.hdr.code = (infoPtr->nNotifyFormat == NFR_UNICODE ? code : HEADER_NotifyCodeWtoA(code));
     nmhdr.iItem = iItem;
     nmhdr.iButton = 0;
-    nmhdr.pitem = &nmitem;
-    nmitem.mask = mask;
-    nmitem.cxy = infoPtr->items[iItem].cxy;
-    nmitem.hbm = infoPtr->items[iItem].hbm;
-    nmitem.pszText = NULL;
-    nmitem.cchTextMax = 0;
-/*    nmitem.pszText = infoPtr->items[iItem].pszText; */
-/*    nmitem.cchTextMax = infoPtr->items[iItem].cchTextMax; */
-    nmitem.fmt = infoPtr->items[iItem].fmt;
-    nmitem.lParam = infoPtr->items[iItem].lParam;
-    nmitem.iOrder = infoPtr->items[iItem].iOrder;
-    nmitem.iImage = infoPtr->items[iItem].iImage;
+    nmhdr.pitem = lpItem;
 
     return (BOOL)SendMessageW (infoPtr->hwndNotify, WM_NOTIFY,
                                (WPARAM)nmhdr.hdr.idFrom, (LPARAM)&nmhdr);
@@ -740,8 +805,7 @@ HEADER_DeleteItem (HWND hwnd, WPARAM wParam)
 
     if (infoPtr->uNumItem == 1) {
         TRACE("Simple delete!\n");
-        if (infoPtr->items[0].pszText)
-            Free (infoPtr->items[0].pszText);
+        HEADER_DisposeItem(&infoPtr->items[0]);
         Free (infoPtr->items);
         Free(infoPtr->order);
         infoPtr->items = 0;
@@ -756,8 +820,7 @@ HEADER_DeleteItem (HWND hwnd, WPARAM wParam)
 
         for (i = 0; i < infoPtr->uNumItem; i++)
            TRACE("%d: order=%d, iOrder=%d, ->iOrder=%d\n", i, infoPtr->order[i], infoPtr->items[i].iOrder, infoPtr->items[infoPtr->order[i]].iOrder);
-        if (infoPtr->items[iItem].pszText)
-            Free (infoPtr->items[iItem].pszText);
+        HEADER_DisposeItem(&infoPtr->items[iItem]);
         iOrder = infoPtr->items[iItem].iOrder;
 
         infoPtr->uNumItem--;
@@ -937,7 +1000,7 @@ inline static LRESULT
 HEADER_GetUnicodeFormat (HWND hwnd)
 {
     HEADER_INFO *infoPtr = HEADER_GetInfoPtr (hwnd);
-    return infoPtr->bUnicode;
+    return (infoPtr->nNotifyFormat == NFR_UNICODE);
 }
 
 
@@ -1055,7 +1118,7 @@ HEADER_InsertItemT (HWND hwnd, INT nItem, LPHDITEMW phdi, BOOL bUnicode)
 
     if (phdi->mask & HDI_TEXT)
     {
-	if (!phdi->pszText) phdi->pszText = '\0'; /* null pointer check */
+        if (!phdi->pszText) phdi->pszText = emptyString; /* null pointer check */
         if (phdi->pszText != LPSTR_TEXTCALLBACKW) /* covers != TEXTCALLBACKA too */
         {
             if (bUnicode)
@@ -1150,6 +1213,8 @@ HEADER_SetItemT (HWND hwnd, INT nItem, LPHDITEMW phdi, BOOL bUnicode)
 {
     HEADER_INFO *infoPtr = HEADER_GetInfoPtr (hwnd);
     HEADER_ITEM *lpItem;
+    HDITEMW hdNotify;
+    void *pvScratch;
 
     if (phdi == NULL)
 	return FALSE;
@@ -1158,9 +1223,12 @@ HEADER_SetItemT (HWND hwnd, INT nItem, LPHDITEMW phdi, BOOL bUnicode)
 
     TRACE("[nItem=%d]\n", nItem);
 
-    if (HEADER_SendHeaderNotify (hwnd, bUnicode ? HDN_ITEMCHANGINGW : HDN_ITEMCHANGINGA,
-                                 nItem, phdi->mask))
+    HEADER_CopyHDItemForNotify(infoPtr, &hdNotify, phdi, bUnicode, &pvScratch);
+    if (HEADER_SendHeaderNotifyT (hwnd, HDN_ITEMCHANGINGW, nItem, phdi->mask, &hdNotify))
+    {
+        if (pvScratch) Free(pvScratch);
 	return FALSE;
+    }
 
     lpItem = &infoPtr->items[nItem];
     if (phdi->mask & HDI_BITMAP)
@@ -1194,7 +1262,8 @@ HEADER_SetItemT (HWND hwnd, INT nItem, LPHDITEMW phdi, BOOL bUnicode)
         {
             if (lpItem->pszText)
             {
-                Free(lpItem->pszText);
+                if (lpItem->pszText != emptyString && lpItem->pszText != LPSTR_TEXTCALLBACKW)
+                    Free(lpItem->pszText);
                 lpItem->pszText = NULL;
             }
             if (phdi->pszText)
@@ -1237,13 +1306,14 @@ HEADER_SetItemT (HWND hwnd, INT nItem, LPHDITEMW phdi, BOOL bUnicode)
         }
       }
 
-    HEADER_SendHeaderNotify (hwnd, bUnicode ? HDN_ITEMCHANGEDW : HDN_ITEMCHANGEDA,
-                             nItem, phdi->mask);
+    HEADER_SendHeaderNotifyT (hwnd, HDN_ITEMCHANGEDW, nItem, phdi->mask, &hdNotify);
 
     HEADER_SetItemBounds (hwnd);
 
     InvalidateRect(hwnd, NULL, FALSE);
 
+    if (pvScratch != NULL)
+        Free(pvScratch);
     return TRUE;
 }
 
@@ -1251,9 +1321,9 @@ inline static LRESULT
 HEADER_SetUnicodeFormat (HWND hwnd, WPARAM wParam)
 {
     HEADER_INFO *infoPtr = HEADER_GetInfoPtr (hwnd);
-    BOOL bTemp = infoPtr->bUnicode;
+    BOOL bTemp = (infoPtr->nNotifyFormat == NFR_UNICODE);
 
-    infoPtr->bUnicode = (BOOL)wParam;
+    infoPtr->nNotifyFormat = ((BOOL)wParam ? NFR_UNICODE : NFR_ANSI);
 
     return bTemp;
 }
@@ -1284,7 +1354,6 @@ HEADER_Create (HWND hwnd, WPARAM wParam, LPARAM lParam)
     infoPtr->iMoveItem = 0;
     infoPtr->himl = 0;
     infoPtr->iHotItem = -1;
-    infoPtr->bUnicode = IsWindowUnicode (hwnd);
     infoPtr->iMargin = 3*GetSystemMetrics(SM_CXEDGE);
     infoPtr->nNotifyFormat =
 	SendMessageW (infoPtr->hwndNotify, WM_NOTIFYFORMAT, (WPARAM)hwnd, NF_QUERY);
@@ -1313,8 +1382,7 @@ HEADER_Destroy (HWND hwnd, WPARAM wParam, LPARAM lParam)
     if (infoPtr->items) {
         lpItem = infoPtr->items;
         for (nItem = 0; nItem < infoPtr->uNumItem; nItem++, lpItem++) {
-	    if ((lpItem->pszText) && (lpItem->pszText != LPSTR_TEXTCALLBACKW))
-		Free (lpItem->pszText);
+            HEADER_DisposeItem(lpItem);
         }
         Free (infoPtr->items);
     }
@@ -1355,9 +1423,9 @@ HEADER_LButtonDblClk (HWND hwnd, WPARAM wParam, LPARAM lParam)
     HEADER_InternalHitTest (hwnd, &pt, &flags, &nItem);
 
     if ((GetWindowLongW (hwnd, GWL_STYLE) & HDS_BUTTONS) && (flags == HHT_ONHEADER))
-	HEADER_SendHeaderNotify (hwnd, HDN_ITEMDBLCLICKA, nItem,0);
+        HEADER_SendHeaderNotifyT (hwnd, HDN_ITEMDBLCLICKW, nItem, 0, NULL);
     else if ((flags == HHT_ONDIVIDER) || (flags == HHT_ONDIVOPEN))
-	HEADER_SendHeaderNotify (hwnd, HDN_DIVIDERDBLCLICKA, nItem,0);
+        HEADER_SendHeaderNotifyT (hwnd, HDN_DIVIDERDBLCLICKW, nItem, 0, NULL);
 
     return 0;
 }
@@ -1393,7 +1461,7 @@ HEADER_LButtonDown (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	TRACE("Pressed item %d!\n", nItem);
     }
     else if ((flags == HHT_ONDIVIDER) || (flags == HHT_ONDIVOPEN)) {
-	if (!(HEADER_SendHeaderNotify (hwnd, HDN_BEGINTRACKA, nItem,0))) {
+        if (!(HEADER_SendHeaderNotifyT (hwnd, HDN_BEGINTRACKW, nItem, 0, NULL))) {
 	    SetCapture (hwnd);
 	    infoPtr->bCaptured = TRUE;
 	    infoPtr->bTracking = TRUE;
@@ -1468,7 +1536,7 @@ HEADER_LButtonUp (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	TRACE("End tracking item %d!\n", infoPtr->iMoveItem);
 	infoPtr->bTracking = FALSE;
 
-	HEADER_SendHeaderNotify (hwnd, HDN_ENDTRACKA, infoPtr->iMoveItem,HDI_WIDTH);
+        HEADER_SendHeaderNotifyT (hwnd, HDN_ENDTRACKW, infoPtr->iMoveItem, HDI_WIDTH, NULL);
 
         if (!(dwStyle & HDS_FULLDRAG)) {
 	    hdc = GetDC (hwnd);
@@ -1476,7 +1544,7 @@ HEADER_LButtonUp (HWND hwnd, WPARAM wParam, LPARAM lParam)
             ReleaseDC (hwnd, hdc);
         }
           
-	if (HEADER_SendHeaderNotify(hwnd, HDN_ITEMCHANGINGA, infoPtr->iMoveItem, HDI_WIDTH))
+        if (HEADER_SendHeaderNotifyT(hwnd, HDN_ITEMCHANGINGW, infoPtr->iMoveItem, HDI_WIDTH, NULL))
 	{
 	    infoPtr->items[infoPtr->iMoveItem].cxy = infoPtr->nOldWidth;
 	}
@@ -1489,7 +1557,7 @@ HEADER_LButtonUp (HWND hwnd, WPARAM wParam, LPARAM lParam)
 
 	HEADER_SetItemBounds (hwnd);
 	InvalidateRect(hwnd, NULL, TRUE);
-	HEADER_SendHeaderNotify(hwnd, HDN_ITEMCHANGEDA, infoPtr->iMoveItem, HDI_WIDTH);
+        HEADER_SendHeaderNotifyT(hwnd, HDN_ITEMCHANGEDW, infoPtr->iMoveItem, HDI_WIDTH, NULL);
     }
 
     if (infoPtr->bCaptured) {
@@ -1582,13 +1650,13 @@ HEADER_MouseMove (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	}
 	else if (infoPtr->bTracking) {
 	    if (dwStyle & HDS_FULLDRAG) {
-		if (!HEADER_SendHeaderNotify (hwnd, HDN_ITEMCHANGINGA, infoPtr->iMoveItem, HDI_WIDTH))
+                if (!HEADER_SendHeaderNotifyT (hwnd, HDN_ITEMCHANGINGW, infoPtr->iMoveItem, HDI_WIDTH, NULL))
 		{
 		nWidth = pt.x - infoPtr->items[infoPtr->iMoveItem].rect.left + infoPtr->xTrackOffset;
 		if (nWidth < 0)
 		  nWidth = 0;
 		infoPtr->items[infoPtr->iMoveItem].cxy = nWidth;
-			HEADER_SendHeaderNotify(hwnd, HDN_ITEMCHANGEDA, infoPtr->iMoveItem, HDI_WIDTH);
+                        HEADER_SendHeaderNotifyT(hwnd, HDN_ITEMCHANGEDW, infoPtr->iMoveItem, HDI_WIDTH, NULL);
 		}
 		HEADER_SetItemBounds (hwnd);
 		InvalidateRect(hwnd, NULL, FALSE);
@@ -1603,7 +1671,7 @@ HEADER_MouseMove (HWND hwnd, WPARAM wParam, LPARAM lParam)
 		    infoPtr->xOldTrack - infoPtr->items[infoPtr->iMoveItem].rect.left;
 		HEADER_DrawTrackLine (hwnd, hdc, infoPtr->xOldTrack);
 		ReleaseDC (hwnd, hdc);
-	    HEADER_SendHeaderNotify (hwnd, HDN_TRACKA, infoPtr->iMoveItem, HDI_WIDTH);
+                HEADER_SendHeaderNotifyT (hwnd, HDN_TRACKW, infoPtr->iMoveItem, HDI_WIDTH, NULL);
 	    }
 
 	    TRACE("Tracking item %d!\n", infoPtr->iMoveItem);
