@@ -1507,7 +1507,7 @@ TOOLTIPS_GetTextA (HWND hwnd, WPARAM wParam, LPARAM lParam)
 
     /* NB this API is broken, there is no way for the app to determine
        what size buffer it requires nor a way to specify how long the
-       one it supplies is.  We'll assume it's upto INFOTIPSIZE */
+       one it supplies is.  We'll assume it's up to INFOTIPSIZE */
 
     WideCharToMultiByte(CP_ACP, 0, infoPtr->tools[nTool].lpszText, -1,
 			lpToolInfo->lpszText, INFOTIPSIZE, NULL, NULL);
@@ -2025,7 +2025,8 @@ TOOLTIPS_SetToolInfoA (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	else {
 	    if ( (toolPtr->lpszText) &&
 		 !IS_INTRESOURCE(toolPtr->lpszText) ) {
-		Free (toolPtr->lpszText);
+		if( toolPtr->lpszText != LPSTR_TEXTCALLBACKW)
+                    Free (toolPtr->lpszText);
 		toolPtr->lpszText = NULL;
 	    }
 	    if (lpToolInfo->lpszText) {
@@ -2082,7 +2083,8 @@ TOOLTIPS_SetToolInfoW (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	else {
 	    if ( (toolPtr->lpszText) &&
 		 !IS_INTRESOURCE(toolPtr->lpszText) ) {
-		Free (toolPtr->lpszText);
+		if( toolPtr->lpszText != LPSTR_TEXTCALLBACKW)
+                    Free (toolPtr->lpszText);
 		toolPtr->lpszText = NULL;
 	    }
 	    if (lpToolInfo->lpszText) {
@@ -2208,7 +2210,8 @@ TOOLTIPS_UpdateTipTextA (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	else {
 	    if ( (toolPtr->lpszText) &&
 		 !IS_INTRESOURCE(toolPtr->lpszText) ) {
-		Free (toolPtr->lpszText);
+		if( toolPtr->lpszText != LPSTR_TEXTCALLBACKW)
+                    Free (toolPtr->lpszText);
 		toolPtr->lpszText = NULL;
 	    }
 	    if (lpToolInfo->lpszText) {
@@ -2265,7 +2268,8 @@ TOOLTIPS_UpdateTipTextW (HWND hwnd, WPARAM wParam, LPARAM lParam)
 	else {
 	    if ( (toolPtr->lpszText)  &&
 		 !IS_INTRESOURCE(toolPtr->lpszText) ) {
-		Free (toolPtr->lpszText);
+		if( toolPtr->lpszText != LPSTR_TEXTCALLBACKW)
+                    Free (toolPtr->lpszText);
 		toolPtr->lpszText = NULL;
 	    }
 	    if (lpToolInfo->lpszText) {
@@ -2400,7 +2404,7 @@ TOOLTIPS_NCCreate (HWND hwnd, WPARAM wParam, LPARAM lParam)
     DWORD dwStyle = GetWindowLongW (hwnd, GWL_STYLE);
     DWORD dwExStyle = GetWindowLongW (hwnd, GWL_EXSTYLE);
 
-    dwStyle &= 0x0000FFFF;
+    dwStyle &= ~(WS_CHILD | /*WS_MAXIMIZE |*/ WS_BORDER | WS_DLGFRAME);
     dwStyle |= (WS_POPUP | WS_BORDER | WS_CLIPSIBLINGS);
 
     /* WS_BORDER only draws a border round the window rect, not the
