@@ -26,6 +26,107 @@
 
 #include "wine/test.h"
 
+struct OIDToAlgID
+{
+    LPCSTR oid;
+    DWORD algID;
+};
+
+static const struct OIDToAlgID oidToAlgID[] = {
+ { szOID_RSA_RSA, CALG_RSA_KEYX },
+ { szOID_RSA_MD2RSA, CALG_MD2 },
+ { szOID_RSA_MD4RSA, CALG_MD4 },
+ { szOID_RSA_MD5RSA, CALG_MD5 },
+ { szOID_RSA_SHA1RSA, CALG_SHA },
+ { szOID_RSA_DH, CALG_DH_SF },
+ { szOID_RSA_SMIMEalgESDH, CALG_DH_EPHEM },
+ { szOID_RSA_SMIMEalgCMS3DESwrap, CALG_3DES },
+ { szOID_RSA_SMIMEalgCMSRC2wrap, CALG_RC2 },
+ { szOID_RSA_MD2, CALG_MD2 },
+ { szOID_RSA_MD4, CALG_MD4 },
+ { szOID_RSA_MD5, CALG_MD5 },
+ { szOID_RSA_RC2CBC, CALG_RC2 },
+ { szOID_RSA_RC4, CALG_RC4 },
+ { szOID_RSA_DES_EDE3_CBC, CALG_3DES },
+ { szOID_ANSI_X942_DH, CALG_DH_SF },
+ { szOID_X957_DSA, CALG_DSS_SIGN },
+ { szOID_X957_SHA1DSA, CALG_SHA },
+ { szOID_OIWSEC_md4RSA, CALG_MD4 },
+ { szOID_OIWSEC_md5RSA, CALG_MD5 },
+ { szOID_OIWSEC_md4RSA2, CALG_MD4 },
+ { szOID_OIWSEC_desCBC, CALG_DES },
+ { szOID_OIWSEC_dsa, CALG_DSS_SIGN },
+ { szOID_OIWSEC_shaDSA, CALG_SHA },
+ { szOID_OIWSEC_shaRSA, CALG_SHA },
+ { szOID_OIWSEC_sha, CALG_SHA },
+ { szOID_OIWSEC_rsaXchg, CALG_RSA_KEYX },
+ { szOID_OIWSEC_sha1, CALG_SHA },
+ { szOID_OIWSEC_dsaSHA1, CALG_SHA },
+ { szOID_OIWSEC_sha1RSASign, CALG_SHA },
+ { szOID_OIWDIR_md2RSA, CALG_MD2 },
+ { szOID_INFOSEC_mosaicUpdatedSig, CALG_SHA },
+ { szOID_INFOSEC_mosaicKMandUpdSig, CALG_DSS_SIGN },
+};
+
+static const struct OIDToAlgID algIDToOID[] = {
+ { szOID_RSA_RSA, CALG_RSA_KEYX },
+ { szOID_RSA_SMIMEalgESDH, CALG_DH_EPHEM },
+ { szOID_RSA_MD2, CALG_MD2 },
+ { szOID_RSA_MD4, CALG_MD4 },
+ { szOID_RSA_MD5, CALG_MD5 },
+ { szOID_RSA_RC2CBC, CALG_RC2 },
+ { szOID_RSA_RC4, CALG_RC4 },
+ { szOID_RSA_DES_EDE3_CBC, CALG_3DES },
+ { szOID_ANSI_X942_DH, CALG_DH_SF },
+ { szOID_X957_DSA, CALG_DSS_SIGN },
+ { szOID_OIWSEC_desCBC, CALG_DES },
+ { szOID_OIWSEC_sha1, CALG_SHA },
+};
+
+static void testOIDToAlgID(void)
+{
+    int i;
+    DWORD alg;
+
+    /* Test with a bogus one */
+    SetLastError(0xdeadbeef);
+    alg = CertOIDToAlgId("1.2.3");
+    ok(!alg && (GetLastError() == 0xdeadbeef ||
+     GetLastError() == ERROR_RESOURCE_NAME_NOT_FOUND),
+     "Expected ERROR_RESOURCE_NAME_NOT_FOUND or no error set, got %08lx\n",
+     GetLastError());
+
+    for (i = 0; i < sizeof(oidToAlgID) / sizeof(oidToAlgID[0]); i++)
+    {
+        alg = CertOIDToAlgId(oidToAlgID[i].oid);
+        /* Not all Windows installations support all these, so make sure it's
+         * at least not the wrong one.
+         */
+        ok(alg == 0 || alg == oidToAlgID[i].algID,
+         "Expected %ld, got %ld\n", oidToAlgID[i].algID, alg);
+    }
+}
+
+static void testAlgIDToOID(void)
+{
+    int i;
+    LPCSTR oid;
+
+    /* Test with a bogus one */
+    SetLastError(0xdeadbeef);
+    oid = CertAlgIdToOID(ALG_CLASS_SIGNATURE | ALG_TYPE_ANY | 80);
+    ok(!oid && GetLastError() == 0xdeadbeef,
+     "Didn't expect last error (%08lx) to be set\n", GetLastError());
+    for (i = 0; i < sizeof(algIDToOID) / sizeof(algIDToOID[0]); i++)
+    {
+        oid = CertAlgIdToOID(algIDToOID[i].algID);
+        /* Allow failure, not every version of Windows supports every algo */
+        if (oid)
+            ok(!strcmp(oid, algIDToOID[i].oid), 
+             "Expected %s, got %s\n", algIDToOID[i].oid, oid);
+    }
+}
+
 static void test_oidFunctionSet(void)
 {
     HCRYPTOIDFUNCSET set1, set2;
@@ -213,8 +314,86 @@ static void test_registerOIDFunction(void)
     ok(ret, "CryptUnregisterOIDFunction failed: %ld\n", GetLastError());
 }
 
+static BOOL WINAPI countOidInfo(PCCRYPT_OID_INFO pInfo, void *pvArg)
+{
+    (*(DWORD *)pvArg)++;
+    return TRUE;
+}
+
+static BOOL WINAPI noOidInfo(PCCRYPT_OID_INFO pInfo, void *pvArg)
+{
+    return FALSE;
+}
+
+static void test_enumOIDInfo(void)
+{
+    BOOL ret;
+    DWORD count = 0;
+
+    /* This crashes
+    ret = CryptEnumOIDInfo(7, 0, NULL, NULL);
+     */
+
+    /* Silly tests, check that more than one thing is enumerated */
+    ret = CryptEnumOIDInfo(0, 0, &count, countOidInfo);
+    ok(ret && count > 0, "Expected more than item enumerated\n");
+    ret = CryptEnumOIDInfo(0, 0, NULL, noOidInfo);
+    ok(!ret, "Expected FALSE\n");
+}
+
+static void test_findOIDInfo(void)
+{
+    static const WCHAR sha1[] = { 's','h','a','1',0 };
+    ALG_ID alg = CALG_SHA1;
+    ALG_ID algs[2] = { CALG_MD5, CALG_RSA_SIGN };
+    PCCRYPT_OID_INFO info;
+
+    info = CryptFindOIDInfo(0, NULL, 0);
+    ok(info == NULL, "Expected NULL\n");
+    info = CryptFindOIDInfo(CRYPT_OID_INFO_OID_KEY, szOID_RSA_MD5, 0);
+    ok(info != NULL, "Expected to find szOID_RSA_MD5\n");
+    if (info)
+    {
+        ok(!strcmp(info->pszOID, szOID_RSA_MD5), "Expected %s, got %s\n",
+         szOID_RSA_MD5, info->pszOID);
+        ok(U(*info).Algid == CALG_MD5, "Expected CALG_MD5, got %d\n",
+           U(*info).Algid);
+    }
+    info = CryptFindOIDInfo(CRYPT_OID_INFO_NAME_KEY, (void *)sha1, 0);
+    ok(info != NULL, "Expected to find sha1\n");
+    if (info)
+    {
+        ok(!strcmp(info->pszOID, szOID_OIWSEC_sha1), "Expected %s, got %s\n",
+         szOID_OIWSEC_sha1, info->pszOID);
+        ok(U(*info).Algid == CALG_SHA1, "Expected CALG_SHA1, got %d\n",
+           U(*info).Algid);
+    }
+    info = CryptFindOIDInfo(CRYPT_OID_INFO_ALGID_KEY, &alg, 0);
+    ok(info != NULL, "Expected to find sha1\n");
+    if (info)
+    {
+        ok(!strcmp(info->pszOID, szOID_OIWSEC_sha1), "Expected %s, got %s\n",
+         szOID_OIWSEC_sha1, info->pszOID);
+        ok(U(*info).Algid == CALG_SHA1, "Expected CALG_SHA1, got %d\n",
+           U(*info).Algid);
+    }
+    info = CryptFindOIDInfo(CRYPT_OID_INFO_SIGN_KEY, algs, 0);
+    ok(info != NULL, "Expected to find md5RSA\n");
+    if (info)
+    {
+        ok(!strcmp(info->pszOID, szOID_RSA_MD5RSA), "Expected %s, got %s\n",
+         szOID_RSA_MD5RSA, info->pszOID);
+        ok(U(*info).Algid == CALG_MD5, "Expected CALG_MD5, got %d\n",
+           U(*info).Algid);
+    }
+}
+
 START_TEST(oid)
 {
+    testOIDToAlgID();
+    testAlgIDToOID();
+    test_enumOIDInfo();
+    test_findOIDInfo();
     test_oidFunctionSet();
     test_installOIDFunctionAddress();
     test_registerOIDFunction();
