@@ -49,7 +49,7 @@
 
 #include "gdi.h"
 #include "d3d9.h"
-#include "d3d9_private.h"
+#include "ddraw.h"
 #include "wine/wined3d_interface.h"
 
 /* ===========================================================================
