@@ -25,6 +25,7 @@
 #include "psapi.h"
 #include "wine/debug.h"
 #include "wdbgexts.h"
+#include "winnls.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 
@@ -115,10 +116,28 @@ BOOL validate_addr64(DWORD64 addr)
 }
 
 /******************************************************************
- *		SymSetSearchPath (DBGHELP.@)
+ *		fetch_buffer
+ *
+ * Ensures process' internal buffer is large enough.
+ */
+void* fetch_buffer(struct process* pcs, unsigned size)
+{
+    if (size > pcs->buffer_size)
+    {
+        if (pcs->buffer)
+            pcs->buffer = HeapReAlloc(GetProcessHeap(), 0, pcs->buffer, size);
+        else
+            pcs->buffer = HeapAlloc(GetProcessHeap(), 0, size);
+        pcs->buffer_size = (pcs->buffer) ? size : 0;
+    }
+    return pcs->buffer;
+}
+
+/******************************************************************
+ *		SymSetSearchPathW (DBGHELP.@)
  *
  */
-BOOL WINAPI SymSetSearchPath(HANDLE hProcess, PCSTR searchPath)
+BOOL WINAPI SymSetSearchPathW(HANDLE hProcess, PCWSTR searchPath)
 {
     struct process* pcs = process_find_by_handle(hProcess);
 
@@ -126,8 +145,43 @@ BOOL WINAPI SymSetSearchPath(HANDLE hProcess, PCSTR searchPath)
     if (!searchPath) return FALSE;
 
     HeapFree(GetProcessHeap(), 0, pcs->search_path);
-    pcs->search_path = strcpy(HeapAlloc(GetProcessHeap(), 0, strlen(searchPath) + 1),
-                              searchPath);
+    pcs->search_path = lstrcpyW(HeapAlloc(GetProcessHeap(), 0, 
+                                          (lstrlenW(searchPath) + 1) * sizeof(WCHAR)),
+                                searchPath);
+    return TRUE;
+}
+
+/******************************************************************
+ *		SymSetSearchPath (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymSetSearchPath(HANDLE hProcess, PCSTR searchPath)
+{
+    BOOL        ret = FALSE;
+    unsigned    len;
+    WCHAR*      sp;
+
+    len = MultiByteToWideChar(CP_ACP, 0, searchPath, -1, NULL, 0);
+    if ((sp = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR))))
+    {
+        MultiByteToWideChar(CP_ACP, 0, searchPath, -1, sp, len);
+
+        ret = SymSetSearchPathW(hProcess, sp);
+        HeapFree(GetProcessHeap(), 0, sp);
+    }
+    return ret;
+}
+
+/***********************************************************************
+ *		SymGetSearchPathW (DBGHELP.@)
+ */
+BOOL WINAPI SymGetSearchPathW(HANDLE hProcess, LPWSTR szSearchPath, 
+                              DWORD SearchPathLength)
+{
+    struct process* pcs = process_find_by_handle(hProcess);
+    if (!pcs) return FALSE;
+
+    lstrcpynW(szSearchPath, pcs->search_path, SearchPathLength);
     return TRUE;
 }
 
@@ -137,11 +191,19 @@ BOOL WINAPI SymSetSearchPath(HANDLE hProcess, PCSTR searchPath)
 BOOL WINAPI SymGetSearchPath(HANDLE hProcess, LPSTR szSearchPath, 
                              DWORD SearchPathLength)
 {
-    struct process* pcs = process_find_by_handle(hProcess);
-    if (!pcs) return FALSE;
+    WCHAR*      buffer = HeapAlloc(GetProcessHeap(), 0, SearchPathLength);
+    BOOL        ret = FALSE;
 
-    lstrcpynA(szSearchPath, pcs->search_path, SearchPathLength);
-    return TRUE;
+    if (buffer)
+    {
+        ret = SymGetSearchPathW(hProcess, buffer,
+                                SearchPathLength * sizeof(WCHAR));
+        if (ret)
+            WideCharToMultiByte(CP_ACP, 0, buffer, SearchPathLength,
+                                szSearchPath, SearchPathLength, NULL, NULL);
+        HeapFree(GetProcessHeap(), 0, buffer);
+    }
+    return ret;
 }
 
 /******************************************************************
@@ -171,11 +233,12 @@ static BOOL check_live_target(struct process* pcs)
 {
     if (!GetProcessId(pcs->handle)) return FALSE;
     if (GetEnvironmentVariableA("DBGHELP_NOLIVE", NULL, 0)) return FALSE;
-    return elf_read_wine_loader_dbg_info(pcs);
+    elf_read_wine_loader_dbg_info(pcs);
+    return TRUE;
 }
 
 /******************************************************************
- *		SymInitialize (DBGHELP.@)
+ *		SymInitializeW (DBGHELP.@)
  *
  * The initialisation of a dbghelp's context.
  * Note that hProcess doesn't need to be a valid process handle (except
@@ -200,11 +263,11 @@ static BOOL check_live_target(struct process* pcs)
  * Note also that this scheme can be intertwined with the deferred loading 
  * mechanism (ie only load the debug information when we actually need it).
  */
-BOOL WINAPI SymInitialize(HANDLE hProcess, PCSTR UserSearchPath, BOOL fInvadeProcess)
+BOOL WINAPI SymInitializeW(HANDLE hProcess, PCWSTR UserSearchPath, BOOL fInvadeProcess)
 {
     struct process*     pcs;
 
-    TRACE("(%p %s %u)\n", hProcess, debugstr_a(UserSearchPath), fInvadeProcess);
+    TRACE("(%p %s %u)\n", hProcess, debugstr_w(UserSearchPath), fInvadeProcess);
 
     if (process_find_by_handle(hProcess))
         FIXME("what to do ??\n");
@@ -216,33 +279,36 @@ BOOL WINAPI SymInitialize(HANDLE hProcess, PCSTR UserSearchPath, BOOL fInvadePro
 
     if (UserSearchPath)
     {
-        pcs->search_path = strcpy(HeapAlloc(GetProcessHeap(), 0, strlen(UserSearchPath) + 1), 
-                                  UserSearchPath);
+        pcs->search_path = lstrcpyW(HeapAlloc(GetProcessHeap(), 0,      
+                                              (lstrlenW(UserSearchPath) + 1) * sizeof(WCHAR)),
+                                    UserSearchPath);
     }
     else
     {
         unsigned        size;
         unsigned        len;
+        static const WCHAR      sym_path[] = {'_','N','T','_','S','Y','M','B','O','L','_','P','A','T','H',0};
+        static const WCHAR      alt_sym_path[] = {'_','N','T','_','A','L','T','E','R','N','A','T','E','_','S','Y','M','B','O','L','_','P','A','T','H',0};
 
-        pcs->search_path = HeapAlloc(GetProcessHeap(), 0, len = MAX_PATH);
-        while ((size = GetCurrentDirectoryA(len, pcs->search_path)) >= len)
-            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, len *= 2);
-        pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, size + 1);
+        pcs->search_path = HeapAlloc(GetProcessHeap(), 0, (len = MAX_PATH) * sizeof(WCHAR));
+        while ((size = GetCurrentDirectoryW(len, pcs->search_path)) >= len)
+            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, (len *= 2) * sizeof(WCHAR));
+        pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, (size + 1) * sizeof(WCHAR));
 
-        len = GetEnvironmentVariableA("_NT_SYMBOL_PATH", NULL, 0);
+        len = GetEnvironmentVariableW(sym_path, NULL, 0);
         if (len)
         {
-            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, size + 1 + len + 1);
+            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, (size + 1 + len + 1) * sizeof(WCHAR));
             pcs->search_path[size] = ';';
-            GetEnvironmentVariableA("_NT_SYMBOL_PATH", pcs->search_path + size + 1, len);
+            GetEnvironmentVariableW(sym_path, pcs->search_path + size + 1, len);
             size += 1 + len;
         }
-        len = GetEnvironmentVariableA("_NT_ALTERNATE_SYMBOL_PATH", NULL, 0);
+        len = GetEnvironmentVariableW(alt_sym_path, NULL, 0);
         if (len)
         {
-            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, size + 1 + len + 1);
+            pcs->search_path = HeapReAlloc(GetProcessHeap(), 0, pcs->search_path, (size + 1 + len + 1) * sizeof(WCHAR));
             pcs->search_path[size] = ';';
-            GetEnvironmentVariableA("_NT_ALTERNATE_SYMBOL_PATH", pcs->search_path + size + 1, len);
+            GetEnvironmentVariableW(alt_sym_path, pcs->search_path + size + 1, len);
             size += 1 + len;
         }
     }
@@ -268,6 +334,30 @@ BOOL WINAPI SymInitialize(HANDLE hProcess, PCSTR UserSearchPath, BOOL fInvadePro
     return TRUE;
 }
 
+/******************************************************************
+ *		SymInitialize (DBGHELP.@)
+ *
+ *
+ */
+BOOL WINAPI SymInitialize(HANDLE hProcess, PCSTR UserSearchPath, BOOL fInvadeProcess)
+{
+    WCHAR*              sp = NULL;
+    BOOL                ret;
+
+    if (UserSearchPath)
+    {
+        unsigned len;
+
+        len = MultiByteToWideChar(CP_ACP, 0, UserSearchPath, -1, NULL, 0);
+        sp = HeapAlloc(GetProcessHeap(), 0, len * sizeof(WCHAR));
+        MultiByteToWideChar(CP_ACP, 0, UserSearchPath, -1, sp, len);
+    }
+
+    ret = SymInitializeW(hProcess, sp, fInvadeProcess);
+    HeapFree(GetProcessHeap(), 0, sp);
+    return ret;
+}
+
 /******************************************************************
  *		SymCleanup (DBGHELP.@)
  *
@@ -407,10 +497,64 @@ static BOOL CALLBACK reg_cb64to32(HANDLE hProcess, ULONG action, ULONG64 data, U
 BOOL pcs_callback(const struct process* pcs, ULONG action, void* data)
 {
     TRACE("%p %lu %p\n", pcs, action, data);
+
     if (!pcs->reg_cb) return FALSE;
+    if (pcs->reg_is_unicode)
+    {
+        IMAGEHLP_DEFERRED_SYMBOL_LOAD64*    idsl;
+        IMAGEHLP_DEFERRED_SYMBOL_LOADW64    idslW;
+
+        switch (action)
+        {
+        case CBA_DEBUG_INFO:
+        case CBA_DEFERRED_SYMBOL_LOAD_CANCEL:
+        case CBA_SET_OPTIONS:
+        case CBA_SYMBOLS_UNLOADED:
+            break;
+        case CBA_DEFERRED_SYMBOL_LOAD_COMPLETE:
+        case CBA_DEFERRED_SYMBOL_LOAD_FAILURE:
+        case CBA_DEFERRED_SYMBOL_LOAD_PARTIAL:
+        case CBA_DEFERRED_SYMBOL_LOAD_START:
+            idsl = (IMAGEHLP_DEFERRED_SYMBOL_LOAD64*)(DWORD)data;
+            idslW.SizeOfStruct = sizeof(idslW);
+            idslW.BaseOfImage = idsl->BaseOfImage;
+            idslW.CheckSum = idsl->CheckSum;
+            idslW.TimeDateStamp = idsl->TimeDateStamp;
+            MultiByteToWideChar(CP_ACP, 0, idsl->FileName, -1, 
+                                idslW.FileName, sizeof(idslW.FileName) / sizeof(WCHAR));
+            idslW.Reparse = idsl->Reparse;
+            data = &idslW;
+            break;
+        case CBA_DUPLICATE_SYMBOL:
+        case CBA_EVENT:
+        case CBA_READ_MEMORY:
+        default:
+            FIXME("No mapping for action %lu\n", action);
+            return FALSE;
+        }
+    }
     return pcs->reg_cb(pcs->handle, action, (ULONG64)(DWORD_PTR)data, pcs->reg_user);
 }
 
+/******************************************************************
+ *		sym_register_cb
+ *
+ * Helper for registering a callback.
+ */
+static BOOL sym_register_cb(HANDLE hProcess, 
+                            PSYMBOL_REGISTERED_CALLBACK64 cb,
+                            DWORD64 user, BOOL unicode)
+{
+    struct process* pcs = process_find_by_handle(hProcess);
+
+    if (!pcs) return FALSE;
+    pcs->reg_cb = cb;
+    pcs->reg_is_unicode = unicode;
+    pcs->reg_user = user;
+
+    return TRUE;
+}
+
 /***********************************************************************
  *		SymRegisterCallback (DBGHELP.@)
  */
@@ -419,7 +563,9 @@ BOOL WINAPI SymRegisterCallback(HANDLE hProcess,
                                 PVOID UserContext)
 {
     DWORD64 tmp = ((ULONGLONG)(DWORD)CallbackFunction << 32) | (DWORD)UserContext;
-    return SymRegisterCallback64(hProcess, reg_cb64to32, tmp);
+    TRACE("(%p, %p, %p)\n", 
+          hProcess, CallbackFunction, UserContext);
+    return sym_register_cb(hProcess, reg_cb64to32, tmp, FALSE);
 }
 
 /***********************************************************************
@@ -429,15 +575,21 @@ BOOL WINAPI SymRegisterCallback64(HANDLE hProcess,
                                   PSYMBOL_REGISTERED_CALLBACK64 CallbackFunction,
                                   ULONG64 UserContext)
 {
-    struct process* pcs = process_find_by_handle(hProcess);
-
     TRACE("(%p, %p, %s)\n", 
           hProcess, CallbackFunction, wine_dbgstr_longlong(UserContext));
-    if (!pcs) return FALSE;
-    pcs->reg_cb = CallbackFunction;
-    pcs->reg_user = UserContext;
+    return sym_register_cb(hProcess, CallbackFunction, UserContext, FALSE);
+}
 
-    return TRUE;
+/***********************************************************************
+ *		SymRegisterCallbackW64 (DBGHELP.@)
+ */
+BOOL WINAPI SymRegisterCallbackW64(HANDLE hProcess, 
+                                   PSYMBOL_REGISTERED_CALLBACK64 CallbackFunction,
+                                   ULONG64 UserContext)
+{
+    TRACE("(%p, %p, %s)\n", 
+          hProcess, CallbackFunction, wine_dbgstr_longlong(UserContext));
+    return sym_register_cb(hProcess, CallbackFunction, UserContext, TRUE);
 }
 
 /* This is imagehlp version not dbghelp !! */
