@@ -291,15 +291,19 @@ struct process
 {
     struct process*             next;
     HANDLE                      handle;
-    char*                       search_path;
+    WCHAR*                      search_path;
     
     PSYMBOL_REGISTERED_CALLBACK64       reg_cb;
+    BOOL                        reg_is_unicode;
     DWORD64                     reg_user;
 
     struct module*              lmodules;
     unsigned long               dbg_hdr_addr;
 
     IMAGEHLP_STACK_FRAME        ctx_frame;
+
+    unsigned                    buffer_size;
+    void*                       buffer;
 };
 
 struct line_info
@@ -315,11 +319,40 @@ struct line_info
     } u;
 };
 
+struct module_pair
+{
+    struct module*              requested; /* in:  to module_get_debug() */
+    struct module*              effective; /* out: module with debug info */
+};
+
+enum pdb_kind {PDB_JG, PDB_DS};
+
+struct pdb_lookup
+{
+    const char*                 filename;
+    DWORD                       age;
+    enum pdb_kind               kind;
+    union
+    {
+        struct
+        {
+            DWORD               timestamp;
+            struct PDB_JG_TOC*  toc;
+        } jg;
+        struct
+        {
+            GUID                guid;
+            struct PDB_DS_TOC*  toc;
+        } ds;
+    } u;
+};
+
 /* dbghelp.c */
 extern struct process* process_find_by_handle(HANDLE hProcess);
 extern HANDLE hMsvcrt;
 extern BOOL         validate_addr64(DWORD64 addr);
 extern BOOL         pcs_callback(const struct process* pcs, ULONG action, void* data);
+extern void*        fetch_buffer(struct process* pcs, unsigned size);
 
 /* elf_module.c */
 typedef BOOL (*elf_enum_modules_cb)(const char*, unsigned long addr, void* user);
@@ -342,8 +375,7 @@ extern struct module*
 extern struct module*
                     module_find_by_name(const struct process* pcs, 
                                         const char* name, enum module_type type);
-extern struct module*
-                    module_get_debug(const struct process* pcs, struct module*);
+extern BOOL         module_get_debug(const struct process* pcs, struct module_pair*);
 extern struct module*
                     module_new(struct process* pcs, const char* name, 
                                enum module_type type, BOOL virtual,
@@ -366,6 +398,8 @@ extern BOOL         pe_load_debug_directory(const struct process* pcs,
                                             const BYTE* mapping,
                                             const IMAGE_SECTION_HEADER* sectp, DWORD nsect,
                                             const IMAGE_DEBUG_DIRECTORY* dbg, int nDbg);
+extern BOOL         pdb_fetch_file_info(struct pdb_lookup* pdb_lookup);
+
 /* pe_module.c */
 extern BOOL         pe_load_nt_header(HANDLE hProc, DWORD base, IMAGE_NT_HEADERS* nth);
 extern struct module*
