@@ -69,6 +69,7 @@
 # include <sys/link.h>
 #endif
 
+#include "wine/library.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
@@ -117,7 +118,7 @@ struct symtab_elt
 {
     struct hash_table_elt       ht_elt;
     const Elf32_Sym*            symp;
-    const char*                 filename;
+    struct symt_compiland*      compiland;
     unsigned                    used;
 };
 
@@ -250,14 +251,14 @@ static void elf_unmap_file(struct elf_file_map* fmap)
  *
  * creating an internal hash table to ease use ELF symtab information lookup
  */
-static void elf_hash_symtab(const struct module* module, struct pool* pool, 
+static void elf_hash_symtab(struct module* module, struct pool* pool, 
                             struct hash_table* ht_symtab, struct elf_file_map* fmap,
                             int symtab_idx, struct thunk_area* thunks)
 {
     int		                i, j, nsym;
     const char*                 strp;
     const char*                 symname;
-    const char*                 filename = NULL;
+    struct symt_compiland*      compiland = NULL;
     const char*                 ptr;
     const Elf32_Sym*            symp;
     struct symtab_elt*          ste;
@@ -288,7 +289,7 @@ static void elf_hash_symtab(const struct module* module, struct pool* pool,
 
         if (ELF32_ST_TYPE(symp->st_info) == STT_FILE)
         {
-            filename = symname;
+            compiland = symname ? symt_new_compiland(module, symname) : NULL;
             continue;
         }
         for (j = 0; thunks[j].symname; j++)
@@ -330,7 +331,7 @@ static void elf_hash_symtab(const struct module* module, struct pool* pool,
             }
         }
         ste->symp        = symp;
-        ste->filename    = filename;
+        ste->compiland   = compiland;
         ste->used        = 0;
         hash_table_add(ht_symtab, &ste->ht_elt);
     }
@@ -376,22 +377,24 @@ static const Elf32_Sym* elf_lookup_symtab(const struct module* module,
         if (ste->used || strcmp(ste->ht_elt.name, name)) continue;
 
         weak_result = ste;
-        if ((ste->filename && !compiland_name) || (!ste->filename && compiland_name))
+        if ((ste->compiland && !compiland_name) || (!ste->compiland && compiland_name))
             continue;
-        if (ste->filename && compiland_name)
+        if (ste->compiland && compiland_name)
         {
-            if (strcmp(ste->filename, compiland_name))
+            const char* filename = source_get(module, ste->compiland->source);
+            if (strcmp(filename, compiland_name))
             {
-                base = strrchr(ste->filename, '/');
-                if (!base++) base = ste->filename;
+                base = strrchr(filename, '/');
+                if (!base++) base = filename;
                 if (strcmp(base, compiland_basename)) continue;
             }
         }
         if (result)
         {
             FIXME("Already found symbol %s (%s) in symtab %s @%08x and %s @%08x\n", 
-                  name, compiland_name, result->filename, result->symp->st_value,
-                  ste->filename, ste->symp->st_value);
+                  name, compiland_name,
+                  source_get(module, result->compiland->source), result->symp->st_value,
+                  source_get(module, ste->compiland->source), ste->symp->st_value);
         }
         else
         {
@@ -495,8 +498,6 @@ static int elf_new_wine_thunks(struct module* module, struct hash_table* ht_symt
                                unsigned num_areas, struct thunk_area* thunks)
 {
     int		                j;
-    struct symt_compiland*      compiland = NULL;
-    const char*                 compiland_name = NULL;
     struct hash_table_iter      hti;
     struct symtab_elt*          ste;
     DWORD                       addr;
@@ -507,16 +508,6 @@ static int elf_new_wine_thunks(struct module* module, struct hash_table* ht_symt
     {
         if (ste->used) continue;
 
-        /* FIXME: this is not a good idea anyway... we are creating several
-         * compiland objects for a same compilation unit
-         * We try to cache the last compiland used, but it's not enough
-         * (we should here only create compilands if they are not yet 
-         *  defined)
-         */
-        if (!compiland_name || compiland_name != ste->filename)
-            compiland = symt_new_compiland(module, 
-                                           compiland_name = ste->filename);
-
         addr = module->elf_info->elf_addr + ste->symp->st_value;
 
         for (j = 0; j < num_areas; j++)
@@ -527,7 +518,7 @@ static int elf_new_wine_thunks(struct module* module, struct hash_table* ht_symt
         }
         if (j < num_areas) /* thunk found */
         {
-            symt_new_thunk(module, compiland, ste->ht_elt.name, thunks[j].ordinal,
+            symt_new_thunk(module, ste->compiland, ste->ht_elt.name, thunks[j].ordinal,
                            addr, ste->symp->st_size);
         }
         else
@@ -547,11 +538,11 @@ static int elf_new_wine_thunks(struct module* module, struct hash_table* ht_symt
                 switch (ELF32_ST_TYPE(ste->symp->st_info))
                 {
                 case STT_FUNC:
-                    symt_new_function(module, compiland, ste->ht_elt.name,
+                    symt_new_function(module, ste->compiland, ste->ht_elt.name,
                                       addr, ste->symp->st_size, NULL);
                     break;
                 case STT_OBJECT:
-                    symt_new_global_variable(module, compiland, ste->ht_elt.name,
+                    symt_new_global_variable(module, ste->compiland, ste->ht_elt.name,
                                              ELF32_ST_BIND(ste->symp->st_info) == STB_LOCAL,
                                              addr, ste->symp->st_size, NULL);
                     break;
@@ -604,8 +595,6 @@ static int elf_new_wine_thunks(struct module* module, struct hash_table* ht_symt
  */
 static int elf_new_public_symbols(struct module* module, struct hash_table* symtab)
 {
-    struct symt_compiland*      compiland = NULL;
-    const char*                 compiland_name = NULL;
     struct hash_table_iter      hti;
     struct symtab_elt*          ste;
 
@@ -616,17 +605,7 @@ static int elf_new_public_symbols(struct module* module, struct hash_table* symt
     hash_table_iter_init(symtab, &hti, NULL);
     while ((ste = hash_table_iter_up(&hti)))
     {
-        /* FIXME: this is not a good idea anyway... we are creating several
-         * compiland objects for a same compilation unit
-         * We try to cache the last compiland used, but it's not enough
-         * (we should here only create compilands if they are not yet 
-         *  defined)
-         */
-        if (!compiland_name || compiland_name != ste->filename)
-            compiland = symt_new_compiland(module, 
-                                           compiland_name = ste->filename);
-
-        symt_new_public(module, compiland, ste->ht_elt.name,
+        symt_new_public(module, ste->compiland, ste->ht_elt.name,
                         module->elf_info->elf_addr + ste->symp->st_value,
                         ste->symp->st_size, TRUE /* FIXME */, 
                         ELF32_ST_TYPE(ste->symp->st_info) == STT_FUNC);
@@ -1090,7 +1069,7 @@ static BOOL elf_load_file(struct process* pcs, const char* filename,
         struct elf_module_info *elf_module_info = 
             HeapAlloc(GetProcessHeap(), 0, sizeof(struct elf_module_info));
         if (!elf_module_info) goto leave;
-        elf_info->module = module_new(pcs, filename, DMT_ELF, 
+        elf_info->module = module_new(pcs, filename, DMT_ELF, FALSE,
                                       (load_offset) ? load_offset : fmap.elf_start, 
                                       fmap.elf_size, 0, calc_crc32(&fmap));
         if (!elf_info->module)
@@ -1159,6 +1138,33 @@ static BOOL elf_load_file_from_path(HANDLE hProcess,
     return ret;
 }
 
+/******************************************************************
+ *		elf_load_file_from_dll_path
+ *
+ * Tries to load an ELF file from the dll path
+ */
+static BOOL elf_load_file_from_dll_path(HANDLE hProcess,
+                                        const char* filename,
+                                        unsigned long load_offset,
+                                        struct elf_info* elf_info)
+{
+    BOOL ret = FALSE;
+    unsigned int index = 0;
+    const char *path;
+
+    while (!ret && (path = wine_dll_enum_load_path( index++ )))
+    {
+        char *name = HeapAlloc( GetProcessHeap(), 0, strlen(path) + strlen(filename) + 2 );
+        if (!name) break;
+        strcpy( name, path );
+        strcat( name, "/" );
+        strcat( name, filename );
+        ret = elf_load_file(hProcess, name, load_offset, elf_info);
+        HeapFree( GetProcessHeap(), 0, name );
+    }
+    return ret;
+}
+
 /******************************************************************
  *		elf_search_and_load_file
  *
@@ -1187,9 +1193,8 @@ static BOOL elf_search_and_load_file(struct process* pcs, const char* filename,
         ret = elf_load_file_from_path(pcs, filename, load_offset, 
                                       getenv("PATH"), elf_info) ||
             elf_load_file_from_path(pcs, filename, load_offset,
-                                    getenv("LD_LIBRARY_PATH"), elf_info) ||
-            elf_load_file_from_path(pcs, filename, load_offset,
-                                    getenv("WINEDLLPATH"), elf_info);
+                                    getenv("LD_LIBRARY_PATH"), elf_info);
+        if (!ret) ret = elf_load_file_from_dll_path(pcs, filename, load_offset, elf_info);
     }
     
     return ret;
@@ -1265,7 +1270,8 @@ BOOL	elf_synchronize_module_list(struct process* pcs)
 
     for (module = pcs->lmodules; module; module = module->next)
     {
-        if (module->type == DMT_ELF) module->elf_info->elf_mark = 0;
+        if (module->type == DMT_ELF && !module->is_virtual)
+            module->elf_info->elf_mark = 0;
     }
 
     es.pcs = pcs;
@@ -1276,8 +1282,8 @@ BOOL	elf_synchronize_module_list(struct process* pcs)
     module = pcs->lmodules;
     while (module)
     {
-        if (module->type == DMT_ELF && !module->elf_info->elf_mark && 
-            !module->elf_info->elf_loader)
+        if (module->type == DMT_ELF && !module->is_virtual &&
+            !module->elf_info->elf_mark && !module->elf_info->elf_loader)
         {
             module_remove(pcs, module);
             /* restart all over */
@@ -1394,7 +1400,7 @@ static BOOL elf_load_cb(const char* name, unsigned long addr, void* user)
  * Also, find module real name and load address from
  * the real loaded modules list in pcs address space
  */
-struct module*  elf_load_module(struct process* pcs, const char* name, DWORD addr)
+struct module*  elf_load_module(struct process* pcs, const char* name, unsigned long addr)
 {
     struct elf_load     el;
 
