@@ -30,6 +30,7 @@
 #include "winreg.h"
 #include "winternl.h"
 #include "wine/debug.h"
+#include "winnls.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 
@@ -78,13 +79,12 @@ static void module_fill_module(const char* in, char* out, size_t size)
     while ((*out = tolower(*out))) out++;
 }
 
-static const char*      get_module_type(enum module_type type)
+static const char*      get_module_type(enum module_type type, BOOL virtual)
 {
     switch (type)
     {
-    case DMT_ELF: return "ELF";
-    case DMT_PE: return "PE";
-    case DMT_VIRTUAL: return "Virtual";
+    case DMT_ELF: return virtual ? "Virtual ELF" : "ELF";
+    case DMT_PE: return virtual ? "Virtual PE" : "PE";
     default: return "---";
     }
 }
@@ -93,13 +93,13 @@ static const char*      get_module_type(enum module_type type)
  * Creates and links a new module to a process 
  */
 struct module* module_new(struct process* pcs, const char* name, 
-                          enum module_type type, 
+                          enum module_type type, BOOL virtual,
                           unsigned long mod_addr, unsigned long size,
                           unsigned long stamp, unsigned long checksum) 
 {
     struct module*      module;
 
-    assert(type == DMT_ELF || type == DMT_PE || type == DMT_VIRTUAL);
+    assert(type == DMT_ELF || type == DMT_PE);
     if (!(module = HeapAlloc(GetProcessHeap(), 0, sizeof(*module))))
 	return NULL;
 
@@ -109,7 +109,7 @@ struct module* module_new(struct process* pcs, const char* name,
     pcs->lmodules = module;
 
     TRACE("=> %s %08lx-%08lx %s\n", 
-          get_module_type(type), mod_addr, mod_addr + size, name);
+          get_module_type(type, virtual), mod_addr, mod_addr + size, name);
 
     pool_init(&module->pool, 65536);
     
@@ -126,6 +126,7 @@ struct module* module_new(struct process* pcs, const char* name,
     module->module.CheckSum = checksum;
 
     module->type              = type;
+    module->is_virtual        = virtual ? TRUE : FALSE;
     module->sortlist_valid    = FALSE;
     module->addr_sorttab      = NULL;
     /* FIXME: this seems a bit too high (on a per module basis)
@@ -154,8 +155,7 @@ struct module* module_find_by_name(const struct process* pcs,
     if (type == DMT_UNKNOWN)
     {
         if ((module = module_find_by_name(pcs, name, DMT_PE)) ||
-            (module = module_find_by_name(pcs, name, DMT_ELF)) ||
-            (module = module_find_by_name(pcs, name, DMT_VIRTUAL)))
+            (module = module_find_by_name(pcs, name, DMT_ELF)))
             return module;
     }
     else
@@ -230,50 +230,49 @@ struct module* module_get_containee(const struct process* pcs,
  *   container (and also force the ELF container's debug info loading if deferred)
  * - otherwise return the module itself if it has some debug info
  */
-struct module* module_get_debug(const struct process* pcs, struct module* module)
+BOOL module_get_debug(const struct process* pcs, struct module_pair* pair)
 {
-    struct module*                      parent;
     IMAGEHLP_DEFERRED_SYMBOL_LOAD64     idsl64;
 
-    if (!module) return NULL;
-    /* for a PE builtin, always get info from parent */
-    if ((parent = module_get_container(pcs, module)))
-        module = parent;
+    if (!pair->requested) return FALSE;
+    /* for a PE builtin, always get info from container */
+    if (!(pair->effective = module_get_container(pcs, pair->requested)))
+        pair->effective = pair->requested;
     /* if deferred, force loading */
-    if (module->module.SymType == SymDeferred)
+    if (pair->effective->module.SymType == SymDeferred)
     {
         BOOL ret;
-
-        switch (module->type)
+        
+        if (pair->effective->is_virtual) ret = FALSE;
+        else switch (pair->effective->type)
         {
         case DMT_ELF:
-            ret = elf_load_debug_info(module, NULL);
+            ret = elf_load_debug_info(pair->effective, NULL);
             break;
         case DMT_PE:
             idsl64.SizeOfStruct = sizeof(idsl64);
-            idsl64.BaseOfImage = module->module.BaseOfImage;
-            idsl64.CheckSum = module->module.CheckSum;
-            idsl64.TimeDateStamp = module->module.TimeDateStamp;
-            strcpy(idsl64.FileName, module->module.ImageName);
+            idsl64.BaseOfImage = pair->effective->module.BaseOfImage;
+            idsl64.CheckSum = pair->effective->module.CheckSum;
+            idsl64.TimeDateStamp = pair->effective->module.TimeDateStamp;
+            strcpy(idsl64.FileName, pair->effective->module.ImageName);
             idsl64.Reparse = FALSE;
             idsl64.hFile = INVALID_HANDLE_VALUE;
 
             pcs_callback(pcs, CBA_DEFERRED_SYMBOL_LOAD_START, &idsl64);
-            ret = pe_load_debug_info(pcs, module);
+            ret = pe_load_debug_info(pcs, pair->effective);
             pcs_callback(pcs,
                          ret ? CBA_DEFERRED_SYMBOL_LOAD_COMPLETE : CBA_DEFERRED_SYMBOL_LOAD_FAILURE,
                          &idsl64);
             break;
-        case DMT_VIRTUAL: /* fall through */
         default:
             ret = FALSE;
             break;
         }
-        if (!ret) module->module.SymType = SymNone;
-        assert(module->module.SymType != SymDeferred);
-        module_compute_num_syms(module);
+        if (!ret) pair->effective->module.SymType = SymNone;
+        assert(pair->effective->module.SymType != SymDeferred);
+        module_compute_num_syms(pair->effective);
     }
-    return (module && module->module.SymType != SymNone) ? module : NULL;
+    return pair->effective->module.SymType != SymNone;
 }
 
 /***********************************************************************
@@ -290,8 +289,7 @@ struct module* module_find_by_addr(const struct process* pcs, unsigned long addr
     if (type == DMT_UNKNOWN)
     {
         if ((module = module_find_by_addr(pcs, addr, DMT_PE)) ||
-            (module = module_find_by_addr(pcs, addr, DMT_ELF)) ||
-            (module = module_find_by_addr(pcs, addr, DMT_VIRTUAL)))
+            (module = module_find_by_addr(pcs, addr, DMT_ELF)))
             return module;
     }
     else
@@ -436,6 +434,10 @@ DWORD64 WINAPI  SymLoadModuleEx(HANDLE hProcess, HANDLE hFile, PCSTR ImageName,
                                 PCSTR ModuleName, DWORD64 BaseOfDll, DWORD DllSize,
                                 PMODLOAD_DATA Data, DWORD Flags)
 {
+    TRACE("(%p %p %s %s %s %08lx %p %08lx)\n", 
+          hProcess, hFile, debugstr_a(ImageName), debugstr_a(ModuleName), 
+          wine_dbgstr_longlong(BaseOfDll), DllSize, Data, Flags);
+
     if (Data)
         FIXME("Unsupported load data parameter %p for %s\n", Data, ImageName);
     if (!validate_addr64(BaseOfDll)) return FALSE;
@@ -445,7 +447,8 @@ DWORD64 WINAPI  SymLoadModuleEx(HANDLE hProcess, HANDLE hFile, PCSTR ImageName,
         struct module* module;
         if (!pcs) return FALSE;
 
-        module = module_new(pcs, ImageName, DMT_VIRTUAL, (DWORD)BaseOfDll, DllSize, 0, 0);
+        module = module_new(pcs, ImageName, module_get_type_by_name(ImageName), TRUE, 
+                            (DWORD)BaseOfDll, DllSize, 0, 0);
         if (!module) return FALSE;
         if (ModuleName)
             lstrcpynA(module->module.ModuleName, ModuleName, sizeof(module->module.ModuleName));
@@ -460,6 +463,39 @@ DWORD64 WINAPI  SymLoadModuleEx(HANDLE hProcess, HANDLE hFile, PCSTR ImageName,
                          (DWORD)BaseOfDll, DllSize);
 }
 
+/***********************************************************************
+ *			SymLoadModuleExW (DBGHELP.@)
+ */
+DWORD64 WINAPI  SymLoadModuleExW(HANDLE hProcess, HANDLE hFile, PCWSTR wImageName,
+                                 PCWSTR wModuleName, DWORD64 BaseOfDll, DWORD DllSize,
+                                 PMODLOAD_DATA Data, DWORD Flags)
+{
+    LPSTR       ImageName, ModuleName;
+    unsigned    len;
+    BOOL        ret;
+
+    if (wImageName)
+    {
+        len = WideCharToMultiByte(CP_ACP,0, wImageName, -1, NULL, 0, NULL, NULL);
+        ImageName = HeapAlloc(GetProcessHeap(), 0, len);
+        WideCharToMultiByte(CP_ACP,0, wImageName, -1, ImageName, len, NULL, NULL);
+    }
+    else ImageName = NULL;
+    if (wModuleName)
+    {
+        len = WideCharToMultiByte(CP_ACP,0, wModuleName, -1, NULL, 0, NULL, NULL);
+        ModuleName = HeapAlloc(GetProcessHeap(), 0, len);
+        WideCharToMultiByte(CP_ACP,0, wModuleName, -1, ModuleName, len, NULL, NULL);
+    }
+    else ModuleName = NULL;
+
+    ret = SymLoadModuleEx(hProcess, hFile, ImageName, ModuleName,
+                          BaseOfDll, DllSize, Data, Flags);
+    HeapFree(GetProcessHeap(), 0, ImageName);
+    HeapFree(GetProcessHeap(), 0, ModuleName);
+    return ret;
+}
+
 /***********************************************************************
  *                     SymLoadModule64 (DBGHELP.@)
  */
@@ -484,9 +520,9 @@ BOOL module_remove(struct process* pcs, struct module* module)
     HeapFree(GetProcessHeap(), 0, (char*)module->sources);
     HeapFree(GetProcessHeap(), 0, module->addr_sorttab);
     pool_destroy(&module->pool);
-    if (module->module.SymType != SymNone)
-        pcs_callback(pcs, CBA_SYMBOLS_UNLOADED, NULL);
-
+    /* native dbghelp doesn't invoke registered callback(,CBA_SYMBOLS_UNLOADED,) here
+     * so do we
+     */
     for (p = &pcs->lmodules; *p; p = &(*p)->next)
     {
         if (*p == module)
@@ -624,6 +660,39 @@ BOOL  WINAPI SymGetModuleInfo(HANDLE hProcess, DWORD dwAddr,
     return TRUE;
 }
 
+/******************************************************************
+ *		SymGetModuleInfoW (DBGHELP.@)
+ *
+ */
+BOOL  WINAPI SymGetModuleInfoW(HANDLE hProcess, DWORD dwAddr, 
+                               PIMAGEHLP_MODULEW ModuleInfo)
+{
+    IMAGEHLP_MODULE     mi;
+    IMAGEHLP_MODULEW    miw;
+
+    if (sizeof(miw) < ModuleInfo->SizeOfStruct) FIXME("Wrong size\n");
+
+    mi.SizeOfStruct = sizeof(mi);
+    if (!SymGetModuleInfo(hProcess, dwAddr, &mi)) return FALSE;
+
+    miw.SizeOfStruct  = mi.SizeOfStruct;
+    miw.BaseOfImage   = mi.BaseOfImage;
+    miw.ImageSize     = mi.ImageSize;
+    miw.TimeDateStamp = mi.TimeDateStamp;
+    miw.CheckSum      = mi.CheckSum;
+    miw.NumSyms       = mi.NumSyms;
+    miw.SymType       = mi.SymType;
+    MultiByteToWideChar(CP_ACP, 0, mi.ModuleName, -1,   
+                        miw.ModuleName, sizeof(miw.ModuleName) / sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, mi.ImageName, -1,   
+                        miw.ImageName, sizeof(miw.ImageName) / sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, mi.LoadedImageName, -1,   
+                        miw.LoadedImageName, sizeof(miw.LoadedImageName) / sizeof(WCHAR));
+    memcpy(ModuleInfo, &miw, ModuleInfo->SizeOfStruct);
+
+    return TRUE;
+}
+
 /******************************************************************
  *		SymGetModuleInfo64 (DBGHELP.@)
  *
@@ -684,6 +753,54 @@ BOOL  WINAPI SymGetModuleInfo64(HANDLE hProcess, DWORD64 dwAddr,
     return TRUE;
 }
 
+/******************************************************************
+ *		SymGetModuleInfoW64 (DBGHELP.@)
+ *
+ */
+BOOL  WINAPI SymGetModuleInfoW64(HANDLE hProcess, DWORD64 dwAddr, 
+                                 PIMAGEHLP_MODULEW64 ModuleInfo)
+{
+    IMAGEHLP_MODULE64   mi;
+    IMAGEHLP_MODULEW64  miw;
+
+    if (sizeof(miw) < ModuleInfo->SizeOfStruct) FIXME("Wrong size\n");
+
+    mi.SizeOfStruct = sizeof(mi);
+    if (!SymGetModuleInfo64(hProcess, dwAddr, &mi)) return FALSE;
+
+    miw.SizeOfStruct  = mi.SizeOfStruct;
+    miw.BaseOfImage   = mi.BaseOfImage;
+    miw.ImageSize     = mi.ImageSize;
+    miw.TimeDateStamp = mi.TimeDateStamp;
+    miw.CheckSum      = mi.CheckSum;
+    miw.NumSyms       = mi.NumSyms;
+    miw.SymType       = mi.SymType;
+    MultiByteToWideChar(CP_ACP, 0, mi.ModuleName, -1,   
+                        miw.ModuleName, sizeof(miw.ModuleName) / sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, mi.ImageName, -1,   
+                        miw.ImageName, sizeof(miw.ImageName) / sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, mi.LoadedImageName, -1,   
+                        miw.LoadedImageName, sizeof(miw.LoadedImageName) / sizeof(WCHAR));
+
+    miw.CVSig         = mi.CVSig;
+    MultiByteToWideChar(CP_ACP, 0, mi.CVData, -1,   
+                        miw.CVData, sizeof(miw.CVData) / sizeof(WCHAR));
+    miw.PdbSig        = mi.PdbSig;
+    miw.PdbSig70      = mi.PdbSig70;
+    miw.PdbAge        = mi.PdbAge;
+    miw.PdbUnmatched  = mi.PdbUnmatched;
+    miw.DbgUnmatched  = mi.DbgUnmatched;
+    miw.LineNumbers   = mi.LineNumbers;
+    miw.GlobalSymbols = mi.GlobalSymbols;
+    miw.TypeInfo      = mi.TypeInfo;
+    miw.SourceIndexed = mi.SourceIndexed;
+    miw.Publics       = mi.Publics;
+
+    memcpy(ModuleInfo, &miw, ModuleInfo->SizeOfStruct);
+
+    return TRUE;
+}
+
 /***********************************************************************
  *		SymGetModuleBase (DBGHELP.@)
  */
