@@ -229,7 +229,6 @@ static BOOL CALLBACK sffip_cb(LPCSTR buffer, void* user)
 {
     struct sffip*       s = (struct sffip*)user;
     DWORD               size, checksum;
-    DWORD_PTR           timestamp;
 
     /* FIXME: should check that id/two/three match the file pointed
      * by buffer
@@ -240,6 +239,7 @@ static BOOL CALLBACK sffip_cb(LPCSTR buffer, void* user)
         {
             HANDLE  hFile, hMap;
             void*   mapping;
+            DWORD   timestamp;
 
             timestamp = ~(DWORD_PTR)s->id;
             size = ~s->two;
@@ -282,7 +282,49 @@ static BOOL CALLBACK sffip_cb(LPCSTR buffer, void* user)
         }
         break;
     case DMT_PDB:
-        FIXME("NIY on '%s'\n", buffer);
+        {
+            struct pdb_lookup   pdb_lookup;
+
+            pdb_lookup.filename = buffer;
+
+            if (!pdb_fetch_file_info(&pdb_lookup)) return FALSE;
+            switch (pdb_lookup.kind)
+            {
+            case PDB_JG:
+                if (s->flags & SSRVOPT_GUIDPTR)
+                {
+                    WARN("Found %s, but wrong PDB version\n", buffer);
+                    return FALSE;
+                }
+                if (pdb_lookup.u.jg.timestamp != (DWORD_PTR)s->id)
+                {
+                    WARN("Found %s, but wrong signature: %08lx %08lx\n",
+                         buffer, pdb_lookup.u.jg.timestamp, (DWORD_PTR)s->id);
+                    return FALSE;
+                }
+                break;
+            case PDB_DS:
+                if (!(s->flags & SSRVOPT_GUIDPTR))
+                {
+                    WARN("Found %s, but wrong PDB version\n", buffer);
+                    return FALSE;
+                }
+                if (!(memcmp(&pdb_lookup.u.ds.guid, (GUID*)s->id, sizeof(GUID))))
+                {
+                    WARN("Found %s, but wrong GUID: %s %s\n",
+                         buffer, debugstr_guid(&pdb_lookup.u.ds.guid),
+                         debugstr_guid((GUID*)s->id));
+                    return FALSE;
+                }
+                break;
+            }
+            if (pdb_lookup.age != s->two)
+            {
+                WARN("Found %s, but wrong age: %08lx %08lx\n",
+                     buffer, pdb_lookup.age, s->two);
+                return FALSE;
+            }
+        }
         break;
     default:
         FIXME("What the heck??\n");
@@ -298,7 +340,7 @@ static BOOL CALLBACK sffip_cb(LPCSTR buffer, void* user)
  *		SymFindFileInPath (DBGHELP.@)
  *
  */
-BOOL WINAPI SymFindFileInPath(HANDLE hProcess, PCSTR searchPath, PCSTR full_path,
+BOOL WINAPI SymFindFileInPath(HANDLE hProcess, PCSTR inSearchPath, PCSTR full_path,
                               PVOID id, DWORD two, DWORD three, DWORD flags,
                               LPSTR buffer, PFINDFILEINPATHCALLBACK cb,
                               PVOID user)
@@ -308,13 +350,22 @@ BOOL WINAPI SymFindFileInPath(HANDLE hProcess, PCSTR searchPath, PCSTR full_path
     char                tmp[MAX_PATH];
     char*               ptr;
     const char*         filename;
+    const char*         searchPath = inSearchPath;
 
     TRACE("(%p %s %s %p %08lx %08lx %08lx %p %p %p)\n",
           hProcess, searchPath, full_path, id, two, three, flags, 
           buffer, cb, user);
 
     if (!pcs) return FALSE;
-    if (!searchPath) searchPath = pcs->search_path;
+    if (!searchPath)
+    {
+        unsigned len = WideCharToMultiByte(CP_ACP, 0, pcs->search_path, -1, NULL, 0, NULL, NULL);
+        char* buf;
+
+        searchPath = buf = HeapAlloc(GetProcessHeap(), 0, len);
+        if (!searchPath) return FALSE;
+        WideCharToMultiByte(CP_ACP, 0, pcs->search_path, -1, buf, len, NULL, NULL);
+    }
 
     s.id = id;
     s.two = two;
@@ -330,6 +381,8 @@ BOOL WINAPI SymFindFileInPath(HANDLE hProcess, PCSTR searchPath, PCSTR full_path
     if (sffip_cb(full_path, &s))
     {
         strcpy(buffer, full_path);
+        if (searchPath != inSearchPath)
+            HeapFree(GetProcessHeap(), 0, (char*)searchPath);
         return TRUE;
     }
 
@@ -350,8 +403,12 @@ BOOL WINAPI SymFindFileInPath(HANDLE hProcess, PCSTR searchPath, PCSTR full_path
         if (do_search(filename, tmp, FALSE, sffip_cb, &s))
         {
             strcpy(buffer, tmp);
+            if (searchPath != inSearchPath)
+                HeapFree(GetProcessHeap(), 0, (char*)searchPath);
             return TRUE;
         }
     }
+    if (searchPath != inSearchPath)
+        HeapFree(GetProcessHeap(), 0, (char*)searchPath);
     return FALSE;
 }
