@@ -31,6 +31,7 @@
 #include "winreg.h"
 #include "winternl.h"
 #include "wine/debug.h"
+#include "winnls.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 
@@ -214,13 +215,13 @@ static BOOL pe_load_export_debug_info(const struct process* pcs,
 #if 0
     /* Add start of DLL (better use the (yet unimplemented) Exe SymTag for this) */
     /* FIXME: module.ModuleName isn't correctly set yet if it's passed in SymLoadModule */
-    symt_new_public(module, NULL, module->module.ModuleName, base, 0,
+    symt_new_public(module, NULL, module->module.ModuleName, base, 1,
                     TRUE /* FIXME */, TRUE /* FIXME */);
 #endif
     
     /* Add entry point */
     symt_new_public(module, NULL, "EntryPoint", 
-                    base + nth->OptionalHeader.AddressOfEntryPoint, 0,
+                    base + nth->OptionalHeader.AddressOfEntryPoint, 1,
                     TRUE, TRUE);
 #if 0
     /* FIXME: we'd better store addresses linked to sections rather than 
@@ -233,7 +234,7 @@ static BOOL pe_load_export_debug_info(const struct process* pcs,
     {
 	symt_new_public(module, NULL, section->Name, 
                         RtlImageRvaToVa(nth, (void*)mapping, section->VirtualAddress, NULL), 
-                        0, TRUE /* FIXME */, TRUE /* FIXME */);
+                        1, TRUE /* FIXME */, TRUE /* FIXME */);
     }
 #endif
 
@@ -257,7 +258,7 @@ static BOOL pe_load_export_debug_info(const struct process* pcs,
             symt_new_public(module, NULL, 
                             RtlImageRvaToVa(nth, (void*)mapping, names[i], NULL),
                             base + functions[ordinals[i]], 
-                            0, TRUE /* FIXME */, TRUE /* FIXME */);
+                            1, TRUE /* FIXME */, TRUE /* FIXME */);
         }
 	    
         for (i = 0; i < exports->NumberOfFunctions; i++) 
@@ -268,7 +269,7 @@ static BOOL pe_load_export_debug_info(const struct process* pcs,
                 if ((ordinals[j] == i) && names[j]) break;
             if (j < exports->NumberOfNames) continue;
             snprintf(buffer, sizeof(buffer), "%ld", i + exports->Base);
-            symt_new_public(module, NULL, buffer, base + (DWORD)functions[i], 0,
+            symt_new_public(module, NULL, buffer, base + (DWORD)functions[i], 1,
                             TRUE /* FIXME */, TRUE /* FIXME */);
         }
     }
@@ -336,12 +337,19 @@ struct module* pe_load_module(struct process* pcs, const char* name,
     loaded_name[0] = '\0';
     if (!hFile)
     {
+        unsigned len = WideCharToMultiByte(CP_ACP,0, pcs->search_path, -1, NULL, 0, NULL, NULL);
+        char* sp;
+
         if (!name)
         {
             /* FIXME SetLastError */
             return NULL;
         }
-        if ((hFile = FindExecutableImage(name, pcs->search_path, loaded_name)) == NULL)
+        sp = HeapAlloc(GetProcessHeap(), 0, len);
+        if (!sp) return FALSE;
+        WideCharToMultiByte(CP_ACP,0, pcs->search_path, -1, sp, len, NULL, NULL);
+
+        if ((hFile = FindExecutableImage(name, sp, loaded_name)) == NULL)
             return NULL;
         opened = TRUE;
     }
@@ -360,7 +368,7 @@ struct module* pe_load_module(struct process* pcs, const char* name,
                 if (!base) base = nth->OptionalHeader.ImageBase;
                 if (!size) size = nth->OptionalHeader.SizeOfImage;
             
-                module = module_new(pcs, loaded_name, DMT_PE, base, size,
+                module = module_new(pcs, loaded_name, DMT_PE, FALSE, base, size,
                                     nth->FileHeader.TimeDateStamp, 
                                     nth->OptionalHeader.CheckSum);
                 if (module)
@@ -428,11 +436,11 @@ struct module* pe_load_module_from_pcs(struct process* pcs, const char* name,
             if (pe_load_nt_header(pcs->handle, base, &nth))
             {
                 if (!size) size = nth.OptionalHeader.SizeOfImage;
-                module = module_new(pcs, name, DMT_PE, base, size,
+                module = module_new(pcs, name, DMT_PE, FALSE, base, size,
                                     nth.FileHeader.TimeDateStamp, nth.OptionalHeader.CheckSum);
             }
         } else if (size)
-            module = module_new(pcs, name, DMT_PE, base, size, 0 /* FIXME */, 0 /* FIXME */);
+            module = module_new(pcs, name, DMT_PE, FALSE, base, size, 0 /* FIXME */, 0 /* FIXME */);
     }
     return module;
 }
