@@ -36,6 +36,7 @@
 
 #include "wine/debug.h"
 #include "dbghelp_private.h"
+#include "winnls.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 WINE_DECLARE_DEBUG_CHANNEL(dbghelp_symt);
@@ -452,7 +453,7 @@ struct symt_thunk* symt_new_thunk(struct module* module,
 }
 
 /* expect sym_info->MaxNameLen to be set before being called */
-static void symt_fill_sym_info(const struct module* module, 
+static void symt_fill_sym_info(const struct module_pair* pair, 
                                const struct symt* sym, SYMBOL_INFO* sym_info)
 {
     const char* name;
@@ -467,7 +468,7 @@ static void symt_fill_sym_info(const struct module* module,
          !symt_get_info((struct symt*)sym_info->TypeIndex, TI_GET_LENGTH, &size)))
         size = 0;
     sym_info->Size = (DWORD)size;
-    sym_info->ModBase = module->module.BaseOfImage;
+    sym_info->ModBase = pair->requested->module.BaseOfImage;
     sym_info->Flags = 0;
     sym_info->Value = 0;
 
@@ -556,26 +557,44 @@ static void symt_fill_sym_info(const struct module* module,
                          wine_dbgstr_longlong(sym_info->Address));
 }
 
-static BOOL symt_enum_module(struct module* module, regex_t* regex,
-                             PSYM_ENUMERATESYMBOLS_CALLBACK cb, PVOID user)
+struct sym_enum
+{
+    PSYM_ENUMERATESYMBOLS_CALLBACK      cb;
+    PVOID                               user;
+    SYMBOL_INFO*                        sym_info;
+    DWORD                               index;
+    DWORD                               tag;
+    DWORD64                             addr;
+    char                                buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME];
+};
+
+static BOOL send_symbol(const struct sym_enum* se, struct module_pair* pair,
+                        const struct symt* sym)
+{
+    symt_fill_sym_info(pair, sym, se->sym_info);
+    if (se->index && se->sym_info->info != se->index) return FALSE;
+    if (se->tag && se->sym_info->Tag != se->tag) return FALSE;
+    if (se->addr && !(se->addr >= se->sym_info->Address && se->addr < se->sym_info->Address + se->sym_info->Size)) return FALSE;
+    return !se->cb(se->sym_info, se->sym_info->Size, se->user);
+}
+
+static BOOL symt_enum_module(struct module_pair* pair, regex_t* regex,
+                             const struct sym_enum* se)
 {
-    char                        buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME];
-    SYMBOL_INFO*                sym_info = (SYMBOL_INFO*)buffer;
     void*                       ptr;
     struct symt_ht*             sym = NULL;
     struct hash_table_iter      hti;
 
-    hash_table_iter_init(&module->ht_symbols, &hti, NULL);
+    hash_table_iter_init(&pair->effective->ht_symbols, &hti, NULL);
     while ((ptr = hash_table_iter_up(&hti)))
     {
         sym = GET_ENTRY(ptr, struct symt_ht, hash_elt);
         if (sym->hash_elt.name &&
             regexec(regex, sym->hash_elt.name, 0, NULL, 0) == 0)
         {
-            sym_info->SizeOfStruct = sizeof(SYMBOL_INFO);
-            sym_info->MaxNameLen = sizeof(buffer) - sizeof(SYMBOL_INFO);
-            symt_fill_sym_info(module, &sym->symt, sym_info);
-            if (!cb(sym_info, sym_info->Size, user)) return TRUE;
+            se->sym_info->SizeOfStruct = sizeof(SYMBOL_INFO);
+            se->sym_info->MaxNameLen = sizeof(se->buffer) - sizeof(SYMBOL_INFO);
+            if (send_symbol(se, pair, &sym->symt)) return TRUE;
         }
     }   
     return FALSE;
@@ -681,9 +700,8 @@ int symt_find_nearest(struct module* module, DWORD addr)
     return low;
 }
 
-static BOOL symt_enum_locals_helper(struct process* pcs, struct module* module,
-                                    regex_t* preg, PSYM_ENUMERATESYMBOLS_CALLBACK cb,
-                                    PVOID user, SYMBOL_INFO* sym_info,
+static BOOL symt_enum_locals_helper(struct process* pcs, struct module_pair* pair,
+                                    regex_t* preg, const struct sym_enum* se,
                                     struct vector* v)
 {
     struct symt**       plsym = NULL;
@@ -700,17 +718,14 @@ static BOOL symt_enum_locals_helper(struct process* pcs, struct module* module,
                 struct symt_block*  block = (struct symt_block*)lsym;
                 if (pc < block->address || block->address + block->size <= pc)
                     continue;
-                if (!symt_enum_locals_helper(pcs, module, preg, cb, user, 
-                                             sym_info, &block->vchildren))
+                if (!symt_enum_locals_helper(pcs, pair, preg, se, &block->vchildren))
                     return FALSE;
             }
             break;
         case SymTagData:
             if (regexec(preg, symt_get_name(lsym), 0, NULL, 0) == 0)
             {
-                symt_fill_sym_info(module, lsym, sym_info);
-                if (!cb(sym_info, sym_info->Size, user))
-                    return FALSE;
+                if (send_symbol(se, pair, lsym)) return FALSE;
             }
             break;
         case SymTagLabel:
@@ -725,25 +740,22 @@ static BOOL symt_enum_locals_helper(struct process* pcs, struct module* module,
     return TRUE;
 }
 
-static BOOL symt_enum_locals(struct process* pcs, const char* mask,
-                             PSYM_ENUMERATESYMBOLS_CALLBACK EnumSymbolsCallback,
-                             PVOID UserContext)
+static BOOL symt_enum_locals(struct process* pcs, const char* mask, 
+                             const struct sym_enum* se)
 {
-    struct module*      module;
+    struct module_pair  pair;
     struct symt_ht*     sym;
-    char                buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME];
-    SYMBOL_INFO*        sym_info = (SYMBOL_INFO*)buffer;
     DWORD               pc = pcs->ctx_frame.InstructionOffset;
     int                 idx;
 
-    sym_info->SizeOfStruct = sizeof(*sym_info);
-    sym_info->MaxNameLen = sizeof(buffer) - sizeof(SYMBOL_INFO);
+    se->sym_info->SizeOfStruct = sizeof(*se->sym_info);
+    se->sym_info->MaxNameLen = sizeof(se->buffer) - sizeof(SYMBOL_INFO);
 
-    module = module_find_by_addr(pcs, pc, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module))) return FALSE;
-    if ((idx = symt_find_nearest(module, pc)) == -1) return FALSE;
+    pair.requested = module_find_by_addr(pcs, pc, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair)) return FALSE;
+    if ((idx = symt_find_nearest(pair.effective, pc)) == -1) return FALSE;
 
-    sym = module->addr_sorttab[idx];
+    sym = pair.effective->addr_sorttab[idx];
     if (sym->symt.tag == SymTagFunction)
     {
         BOOL            ret;
@@ -751,15 +763,112 @@ static BOOL symt_enum_locals(struct process* pcs, const char* mask,
 
         compile_regex(mask ? mask : "*", -1, &preg,
                       dbghelp_options & SYMOPT_CASE_INSENSITIVE);
-        ret = symt_enum_locals_helper(pcs, module, &preg, EnumSymbolsCallback, 
-                                      UserContext, sym_info, 
+        ret = symt_enum_locals_helper(pcs, &pair, &preg, se, 
                                       &((struct symt_function*)sym)->vchildren);
         regfree(&preg);
         return ret;
         
     }
-    symt_fill_sym_info(module, &sym->symt, sym_info);
-    return EnumSymbolsCallback(sym_info, sym_info->Size, UserContext);
+    return send_symbol(se, &pair, &sym->symt);
+}
+
+/******************************************************************
+ *		copy_symbolW
+ *
+ * Helper for transforming an ANSI symbol info into an UNICODE one.
+ * Assume that MaxNameLen is the same for both version (A & W).
+ */
+static void copy_symbolW(SYMBOL_INFOW* siw, const SYMBOL_INFO* si)
+{
+    siw->SizeOfStruct = si->SizeOfStruct;
+    siw->TypeIndex = si->TypeIndex; 
+    siw->Reserved[0] = si->Reserved[0];
+    siw->Reserved[1] = si->Reserved[1];
+    siw->Index = si->info; /* FIXME: see dbghelp.h */
+    siw->Size = si->Size;
+    siw->ModBase = si->ModBase;
+    siw->Flags = si->Flags;
+    siw->Value = si->Value;
+    siw->Address = si->Address;
+    siw->Register = si->Register;
+    siw->Scope = si->Scope;
+    siw->Tag = si->Tag;
+    siw->NameLen = si->NameLen;
+    siw->MaxNameLen = si->MaxNameLen;
+    MultiByteToWideChar(CP_ACP, 0, si->Name, -1, siw->Name, siw->MaxNameLen);
+}
+
+/******************************************************************
+ *		sym_enum
+ *
+ * Core routine for most of the enumeration of symbols
+ */
+static BOOL sym_enum(HANDLE hProcess, ULONG64 BaseOfDll, PCSTR Mask,
+                     const struct sym_enum* se)
+{
+    struct process*     pcs = process_find_by_handle(hProcess);
+    struct module_pair  pair;
+    const char*         bang;
+    regex_t             mod_regex, sym_regex;
+
+    if (BaseOfDll == 0)
+    {
+        /* do local variables ? */
+        if (!Mask || !(bang = strchr(Mask, '!')))
+            return symt_enum_locals(pcs, Mask, se);
+
+        if (bang == Mask) return FALSE;
+
+        compile_regex(Mask, bang - Mask, &mod_regex, TRUE);
+        compile_regex(bang + 1, -1, &sym_regex, 
+                      dbghelp_options & SYMOPT_CASE_INSENSITIVE);
+        
+        for (pair.requested = pcs->lmodules; pair.requested; pair.requested = pair.requested->next)
+        {
+            if (pair.requested->type == DMT_PE && module_get_debug(pcs, &pair))
+            {
+                if (regexec(&mod_regex, pair.requested->module.ModuleName, 0, NULL, 0) == 0 &&
+                    symt_enum_module(&pair, &sym_regex, se))
+                    break;
+            }
+        }
+        /* not found in PE modules, retry on the ELF ones
+         */
+        if (!pair.requested && (dbghelp_options & SYMOPT_WINE_WITH_ELF_MODULES))
+        {
+            for (pair.requested = pcs->lmodules; pair.requested; pair.requested = pair.requested->next)
+            {
+                if (pair.requested->type == DMT_ELF &&
+                    !module_get_containee(pcs, pair.requested) &&
+                    module_get_debug(pcs, &pair))
+                {
+                    if (regexec(&mod_regex, pair.requested->module.ModuleName, 0, NULL, 0) == 0 &&
+                        symt_enum_module(&pair, &sym_regex, se))
+                    break;
+                }
+            }
+        }
+        regfree(&mod_regex);
+        regfree(&sym_regex);
+        return TRUE;
+    }
+    pair.requested = module_find_by_addr(pcs, BaseOfDll, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair))
+        return FALSE;
+
+    /* we always ignore module name from Mask when BaseOfDll is defined */
+    if (Mask && (bang = strchr(Mask, '!')))
+    {
+        if (bang == Mask) return FALSE;
+        Mask = bang + 1;
+    }
+
+    compile_regex(Mask ? Mask : "*", -1, &sym_regex, 
+                  dbghelp_options & SYMOPT_CASE_INSENSITIVE);
+    symt_enum_module(&pair, &sym_regex, se);
+    regfree(&sym_regex);
+
+    return TRUE;
 }
 
 /******************************************************************
@@ -777,77 +886,67 @@ BOOL WINAPI SymEnumSymbols(HANDLE hProcess, ULONG64 BaseOfDll, PCSTR Mask,
                            PSYM_ENUMERATESYMBOLS_CALLBACK EnumSymbolsCallback,
                            PVOID UserContext)
 {
-    struct process*     pcs = process_find_by_handle(hProcess);
-    struct module*      module;
-    struct module*      dbg_module;
-    const char*         bang;
-    regex_t             mod_regex, sym_regex;
+    struct sym_enum     se;
 
     TRACE("(%p %s %s %p %p)\n", 
           hProcess, wine_dbgstr_longlong(BaseOfDll), debugstr_a(Mask),
           EnumSymbolsCallback, UserContext);
 
-    if (!pcs) return FALSE;
+    se.cb = EnumSymbolsCallback;
+    se.user = UserContext;
+    se.index = 0;
+    se.tag = 0;
+    se.addr = 0;
+    se.sym_info = (PSYMBOL_INFO)se.buffer;
 
-    if (BaseOfDll == 0)
+    return sym_enum(hProcess, BaseOfDll, Mask, &se);
+}
+
+struct sym_enumW
+{
+    PSYM_ENUMERATESYMBOLS_CALLBACKW     cb;
+    void*                               ctx;
+    PSYMBOL_INFOW                       sym_info;
+    char                                buffer[sizeof(SYMBOL_INFOW) + MAX_SYM_NAME];
+
+};
+    
+static BOOL CALLBACK sym_enumW(PSYMBOL_INFO si, ULONG size, PVOID ctx)
+{
+    struct sym_enumW*   sew = ctx;
+
+    copy_symbolW(sew->sym_info, si);
+
+    return (sew->cb)(sew->sym_info, size, sew->ctx);
+}
+
+/******************************************************************
+ *		SymEnumSymbolsW (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymEnumSymbolsW(HANDLE hProcess, ULONG64 BaseOfDll, PCWSTR Mask,
+                            PSYM_ENUMERATESYMBOLS_CALLBACKW EnumSymbolsCallback,
+                            PVOID UserContext)
+{
+    struct sym_enumW    sew;
+    BOOL                ret = FALSE;
+    char*               maskA = NULL;
+
+    sew.ctx = UserContext;
+    sew.cb = EnumSymbolsCallback;
+    sew.sym_info = (PSYMBOL_INFOW)sew.buffer;
+
+    if (Mask)
     {
-        /* do local variables ? */
-        if (!Mask || !(bang = strchr(Mask, '!')))
-            return symt_enum_locals(pcs, Mask, EnumSymbolsCallback, UserContext);
-
-        if (bang == Mask) return FALSE;
-
-        compile_regex(Mask, bang - Mask, &mod_regex, TRUE);
-        compile_regex(bang + 1, -1, &sym_regex, 
-                      dbghelp_options & SYMOPT_CASE_INSENSITIVE);
-        
-        for (module = pcs->lmodules; module; module = module->next)
-        {
-            if (module->type == DMT_PE && (dbg_module = module_get_debug(pcs, module)))
-            {
-                if (regexec(&mod_regex, module->module.ModuleName, 0, NULL, 0) == 0 &&
-                    symt_enum_module(dbg_module, &sym_regex, 
-                                     EnumSymbolsCallback, UserContext))
-                    break;
-            }
-        }
-        /* not found in PE modules, retry on the ELF ones
-         */
-        if (!module && (dbghelp_options & SYMOPT_WINE_WITH_ELF_MODULES))
-        {
-            for (module = pcs->lmodules; module; module = module->next)
-            {
-                if (module->type == DMT_ELF &&
-                    !module_get_containee(pcs, module) &&
-                    (dbg_module = module_get_debug(pcs, module)))
-                {
-                    if (regexec(&mod_regex, module->module.ModuleName, 0, NULL, 0) == 0 &&
-                        symt_enum_module(dbg_module, &sym_regex, EnumSymbolsCallback, UserContext))
-                    break;
-                }
-            }
-        }
-        regfree(&mod_regex);
-        regfree(&sym_regex);
-        return TRUE;
+        unsigned len = WideCharToMultiByte(CP_ACP, 0, Mask, -1, NULL, 0, NULL, NULL);
+        maskA = HeapAlloc(GetProcessHeap(), 0, len);
+        if (!maskA) return FALSE;
+        WideCharToMultiByte(CP_ACP, 0, Mask, -1, maskA, len, NULL, NULL);
     }
-    module = module_find_by_addr(pcs, BaseOfDll, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module)))
-        return FALSE;
+    ret = SymEnumSymbols(hProcess, BaseOfDll, maskA, sym_enumW, &sew);
+    HeapFree(GetProcessHeap(), 0, maskA);
 
-    /* we always ignore module name from Mask when BaseOfDll is defined */
-    if (Mask && (bang = strchr(Mask, '!')))
-    {
-        if (bang == Mask) return FALSE;
-        Mask = bang + 1;
-    }
-
-    compile_regex(Mask ? Mask : "*", -1, &sym_regex, 
-                      dbghelp_options & SYMOPT_CASE_INSENSITIVE);
-    symt_enum_module(module, &sym_regex, EnumSymbolsCallback, UserContext);
-    regfree(&sym_regex);
-
-    return TRUE;
+    return ret;
 }
 
 struct sym_enumerate
@@ -885,22 +984,47 @@ BOOL WINAPI SymFromAddr(HANDLE hProcess, DWORD64 Address,
                         DWORD64* Displacement, PSYMBOL_INFO Symbol)
 {
     struct process*     pcs = process_find_by_handle(hProcess);
-    struct module*      module;
+    struct module_pair  pair;
     struct symt_ht*     sym;
     int                 idx;
 
     if (!pcs) return FALSE;
-    module = module_find_by_addr(pcs, Address, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module))) return FALSE;
-    if ((idx = symt_find_nearest(module, Address)) == -1) return FALSE;
+    pair.requested = module_find_by_addr(pcs, Address, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair)) return FALSE;
+    if ((idx = symt_find_nearest(pair.effective, Address)) == -1) return FALSE;
 
-    sym = module->addr_sorttab[idx];
+    sym = pair.effective->addr_sorttab[idx];
 
-    symt_fill_sym_info(module, &sym->symt, Symbol);
+    symt_fill_sym_info(&pair, &sym->symt, Symbol);
     *Displacement = Address - Symbol->Address;
     return TRUE;
 }
 
+/******************************************************************
+ *		SymFromAddrW (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymFromAddrW(HANDLE hProcess, DWORD64 Address, 
+                         DWORD64* Displacement, PSYMBOL_INFOW Symbol)
+{
+    PSYMBOL_INFO        si;
+    unsigned            len;
+    BOOL                ret;
+
+    len = sizeof(*si) + Symbol->MaxNameLen * sizeof(WCHAR);
+    si = HeapAlloc(GetProcessHeap(), 0, len);
+    if (!si) return FALSE;
+
+    si->SizeOfStruct = sizeof(*si);
+    si->MaxNameLen = Symbol->MaxNameLen;
+    if ((ret = SymFromAddr(hProcess, Address, Displacement, si)))
+    {
+        copy_symbolW(Symbol, si);
+    }
+    HeapFree(GetProcessHeap(), 0, si);
+    return ret;
+}
+
 /******************************************************************
  *		SymGetSymFromAddr (DBGHELP.@)
  *
@@ -929,6 +1053,31 @@ BOOL WINAPI SymGetSymFromAddr(HANDLE hProcess, DWORD Address,
     return TRUE;
 }
 
+static BOOL find_name(struct process* pcs, struct module* module, const char* name,
+                      SYMBOL_INFO* symbol)
+{
+    struct hash_table_iter      hti;
+    void*                       ptr;
+    struct symt_ht*             sym = NULL;
+    struct module_pair          pair;
+
+    if (!(pair.requested = module)) return FALSE;
+    if (!module_get_debug(pcs, &pair)) return FALSE;
+
+    hash_table_iter_init(&pair.effective->ht_symbols, &hti, name);
+    while ((ptr = hash_table_iter_up(&hti)))
+    {
+        sym = GET_ENTRY(ptr, struct symt_ht, hash_elt);
+
+        if (!strcmp(sym->hash_elt.name, name))
+        {
+            symt_fill_sym_info(&pair, &sym->symt, symbol);
+            return TRUE;
+        }
+    }
+    return FALSE;
+
+}
 /******************************************************************
  *		SymFromName (DBGHELP.@)
  *
@@ -937,9 +1086,6 @@ BOOL WINAPI SymFromName(HANDLE hProcess, PCSTR Name, PSYMBOL_INFO Symbol)
 {
     struct process*             pcs = process_find_by_handle(hProcess);
     struct module*              module;
-    struct hash_table_iter      hti;
-    void*                       ptr;
-    struct symt_ht*             sym = NULL;
     const char*                 name;
 
     TRACE("(%p, %s, %p)\n", hProcess, Name, Symbol);
@@ -953,30 +1099,22 @@ BOOL WINAPI SymFromName(HANDLE hProcess, PCSTR Name, PSYMBOL_INFO Symbol)
         memcpy(tmp, Name, name - Name);
         tmp[name - Name] = '\0';
         module = module_find_by_name(pcs, tmp, DMT_UNKNOWN);
-        if (!module) return FALSE;
-        Name = (char*)(name + 1);
+        return find_name(pcs, module, (char*)(name + 1), Symbol);
     }
-    else module = pcs->lmodules;
-
-    /* FIXME: Name could be made out of a regular expression */
-    for (; module; module = (name) ? NULL : module->next)
+    for (module = pcs->lmodules; module; module = module->next)
     {
-        if (module->module.SymType == SymNone) continue;
-        if (module->module.SymType == SymDeferred)
+        if (module->type == DMT_PE && find_name(pcs, module, Name, Symbol))
+            return TRUE;
+    }
+    /* not found in PE modules, retry on the ELF ones
+     */
+    if (dbghelp_options & SYMOPT_WINE_WITH_ELF_MODULES)
+    {
+        for (module = pcs->lmodules; module; module = module->next)
         {
-            struct module*      xmodule = module_get_debug(pcs, module);
-            if (!xmodule || xmodule != module) continue;
-        }
-        hash_table_iter_init(&module->ht_symbols, &hti, Name);
-        while ((ptr = hash_table_iter_up(&hti)))
-        {
-            sym = GET_ENTRY(ptr, struct symt_ht, hash_elt);
-
-            if (!strcmp(sym->hash_elt.name, Name))
-            {
-                symt_fill_sym_info(module, &sym->symt, Symbol);
+            if (module->type == DMT_ELF && !module_get_containee(pcs, module) &&
+                find_name(pcs, module, Name, Symbol))
                 return TRUE;
-            }
         }
     }
     return FALSE;
@@ -1072,7 +1210,7 @@ BOOL WINAPI SymGetLineFromAddr(HANDLE hProcess, DWORD dwAddr,
                                PDWORD pdwDisplacement, PIMAGEHLP_LINE Line)
 {
     struct process*     pcs = process_find_by_handle(hProcess);
-    struct module*      module;
+    struct module_pair  pair;
     int                 idx;
 
     TRACE("%p %08lx %p %p\n", hProcess, dwAddr, pdwDisplacement, Line);
@@ -1080,13 +1218,13 @@ BOOL WINAPI SymGetLineFromAddr(HANDLE hProcess, DWORD dwAddr,
     if (Line->SizeOfStruct < sizeof(*Line)) return FALSE;
 
     if (!pcs) return FALSE;
-    module = module_find_by_addr(pcs, dwAddr, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module))) return FALSE;
-    if ((idx = symt_find_nearest(module, dwAddr)) == -1) return FALSE;
+    pair.requested = module_find_by_addr(pcs, dwAddr, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair)) return FALSE;
+    if ((idx = symt_find_nearest(pair.effective, dwAddr)) == -1) return FALSE;
 
-    if (module->addr_sorttab[idx]->symt.tag != SymTagFunction) return FALSE;
-    if (!symt_fill_func_line_info(module, 
-                                  (struct symt_function*)module->addr_sorttab[idx],
+    if (pair.effective->addr_sorttab[idx]->symt.tag != SymTagFunction) return FALSE;
+    if (!symt_fill_func_line_info(pair.effective, 
+                                  (struct symt_function*)pair.effective->addr_sorttab[idx],
                                   dwAddr, Line)) return FALSE;
     *pdwDisplacement = dwAddr - Line->Address;
     return TRUE;
@@ -1105,6 +1243,22 @@ static void copy_line_64_from_32(IMAGEHLP_LINE64* l64, const IMAGEHLP_LINE* l32)
     l64->Address = l32->Address;
 }
 
+/******************************************************************
+ *		copy_line_W64_from_32 (internal)
+ *
+ */
+static void copy_line_W64_from_32(struct process* pcs, IMAGEHLP_LINEW64* l64, const IMAGEHLP_LINE* l32)
+{
+    unsigned len;
+
+    l64->Key = l32->Key;
+    l64->LineNumber = l32->LineNumber;
+    len = MultiByteToWideChar(CP_ACP, 0, l32->FileName, -1, NULL, 0);
+    if ((l64->FileName = fetch_buffer(pcs, len * sizeof(WCHAR))))
+        MultiByteToWideChar(CP_ACP, 0, l32->FileName, -1, l64->FileName, len);
+    l64->Address = l32->Address;
+}
+
 /******************************************************************
  *		copy_line_32_from_64 (internal)
  *
@@ -1136,6 +1290,26 @@ BOOL WINAPI SymGetLineFromAddr64(HANDLE hProcess, DWORD64 dwAddr,
     return TRUE;
 }
 
+/******************************************************************
+ *		SymGetLineFromAddrW64 (DBGHELP.@)
+ *
+ */
+BOOL WINAPI SymGetLineFromAddrW64(HANDLE hProcess, DWORD64 dwAddr, 
+                                  PDWORD pdwDisplacement, PIMAGEHLP_LINEW64 Line)
+{
+    struct process*     pcs = process_find_by_handle(hProcess);
+    IMAGEHLP_LINE       line32;
+
+    if (!pcs) return FALSE;
+    if (Line->SizeOfStruct < sizeof(*Line)) return FALSE;
+    if (!validate_addr64(dwAddr)) return FALSE;
+    line32.SizeOfStruct = sizeof(line32);
+    if (!SymGetLineFromAddr(hProcess, (DWORD)dwAddr, pdwDisplacement, &line32))
+        return FALSE;
+    copy_line_W64_from_32(pcs, Line, &line32);
+    return TRUE;
+}
+
 /******************************************************************
  *		SymGetLinePrev (DBGHELP.@)
  *
@@ -1143,7 +1317,7 @@ BOOL WINAPI SymGetLineFromAddr64(HANDLE hProcess, DWORD64 dwAddr,
 BOOL WINAPI SymGetLinePrev(HANDLE hProcess, PIMAGEHLP_LINE Line)
 {
     struct process*     pcs = process_find_by_handle(hProcess);
-    struct module*      module;
+    struct module_pair  pair;
     struct line_info*   li;
     BOOL                in_search = FALSE;
 
@@ -1152,8 +1326,8 @@ BOOL WINAPI SymGetLinePrev(HANDLE hProcess, PIMAGEHLP_LINE Line)
     if (Line->SizeOfStruct < sizeof(*Line)) return FALSE;
 
     if (!pcs) return FALSE;
-    module = module_find_by_addr(pcs, Line->Address, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module))) return FALSE;
+    pair.requested = module_find_by_addr(pcs, Line->Address, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair)) return FALSE;
 
     if (Line->Key == 0) return FALSE;
     li = (struct line_info*)Line->Key;
@@ -1176,7 +1350,7 @@ BOOL WINAPI SymGetLinePrev(HANDLE hProcess, PIMAGEHLP_LINE Line)
         {
             if (in_search)
             {
-                Line->FileName = (char*)source_get(module, li->u.source_file);
+                Line->FileName = (char*)source_get(pair.effective, li->u.source_file);
                 return TRUE;
             }
             in_search = TRUE;
@@ -1229,16 +1403,16 @@ BOOL symt_get_func_line_next(struct module* module, PIMAGEHLP_LINE line)
 BOOL WINAPI SymGetLineNext(HANDLE hProcess, PIMAGEHLP_LINE Line)
 {
     struct process*     pcs = process_find_by_handle(hProcess);
-    struct module*      module;
+    struct module_pair  pair;
 
     TRACE("(%p %p)\n", hProcess, Line);
 
     if (Line->SizeOfStruct < sizeof(*Line)) return FALSE;
     if (!pcs) return FALSE;
-    module = module_find_by_addr(pcs, Line->Address, DMT_UNKNOWN);
-    if (!(module = module_get_debug(pcs, module))) return FALSE;
+    pair.requested = module_find_by_addr(pcs, Line->Address, DMT_UNKNOWN);
+    if (!module_get_debug(pcs, &pair)) return FALSE;
 
-    if (symt_get_func_line_next(module, Line)) return TRUE;
+    if (symt_get_func_line_next(pair.effective, Line)) return TRUE;
     SetLastError(ERROR_NO_MORE_ITEMS); /* FIXME */
     return FALSE;
 }
@@ -1341,34 +1515,61 @@ BOOL WINAPI SymSearch(HANDLE hProcess, ULONG64 BaseOfDll, DWORD Index,
                       PSYM_ENUMERATESYMBOLS_CALLBACK EnumSymbolsCallback,
                       PVOID UserContext, DWORD Options)
 {
+    struct sym_enum     se;
+
     TRACE("(%p %s %lu %lu %s %s %p %p %lx)\n",
           hProcess, wine_dbgstr_longlong(BaseOfDll), Index, SymTag, Mask, 
           wine_dbgstr_longlong(Address), EnumSymbolsCallback,
           UserContext, Options);
 
-    if (Index != 0)
-    {
-        FIXME("Unsupported searching for a given Index (%lu)\n", Index);
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return FALSE;
-    }
-    if (SymTag != 0)
-    {
-        FIXME("Unsupported searching for a given SymTag (%lu)\n", SymTag);
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return FALSE;
-    }
-    if (Address != 0)
-    {
-        FIXME("Unsupported searching for a given Address (%s)\n", wine_dbgstr_longlong(Address));
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return FALSE;
-    }
     if (Options != SYMSEARCH_GLOBALSONLY)
     {
         FIXME("Unsupported searching with options (%lx)\n", Options);
         SetLastError(ERROR_INVALID_PARAMETER);
         return FALSE;
     }
-    return SymEnumSymbols(hProcess, BaseOfDll, Mask, EnumSymbolsCallback, UserContext);
+
+    se.cb = EnumSymbolsCallback;
+    se.user = UserContext;
+    se.index = Index;
+    se.tag = SymTag;
+    se.addr = Address;
+    se.sym_info = (PSYMBOL_INFO)se.buffer;
+
+    return sym_enum(hProcess, BaseOfDll, Mask, &se);
+}
+
+/******************************************************************
+ *		SymSearchW (DBGHELP.@)
+ */
+BOOL WINAPI SymSearchW(HANDLE hProcess, ULONG64 BaseOfDll, DWORD Index,
+                       DWORD SymTag, PCWSTR Mask, DWORD64 Address,
+                       PSYM_ENUMERATESYMBOLS_CALLBACKW EnumSymbolsCallback,
+                       PVOID UserContext, DWORD Options)
+{
+    struct sym_enumW    sew;
+    BOOL                ret = FALSE;
+    char*               maskA = NULL;
+
+    TRACE("(%p %s %lu %lu %s %s %p %p %lx)\n",
+          hProcess, wine_dbgstr_longlong(BaseOfDll), Index, SymTag, debugstr_w(Mask), 
+          wine_dbgstr_longlong(Address), EnumSymbolsCallback,
+          UserContext, Options);
+
+    sew.ctx = UserContext;
+    sew.cb = EnumSymbolsCallback;
+    sew.sym_info = (PSYMBOL_INFOW)sew.buffer;
+
+    if (Mask)
+    {
+        unsigned len = WideCharToMultiByte(CP_ACP, 0, Mask, -1, NULL, 0, NULL, NULL);
+        maskA = HeapAlloc(GetProcessHeap(), 0, len);
+        if (!maskA) return FALSE;
+        WideCharToMultiByte(CP_ACP, 0, Mask, -1, maskA, len, NULL, NULL);
+    }
+    ret = SymSearch(hProcess, BaseOfDll, Index, SymTag, maskA, Address,
+                    sym_enumW, &sew, Options);
+    HeapFree(GetProcessHeap(), 0, maskA);
+
+    return ret;
 }
