@@ -458,11 +458,6 @@ HRESULT WINAPI IDirectSoundCaptureImpl_CreateCaptureBuffer(
 
     TRACE( "(%p,%p,%p,%p)\n",iface,lpcDSCBufferDesc,lplpDSCaptureBuffer,pUnk);
 
-    if (This == NULL) {
-	WARN("invalid parameter: This == NULL\n");
-	return DSERR_INVALIDPARAM;
-    }
-
     if (lpcDSCBufferDesc == NULL) {
 	WARN("invalid parameter: lpcDSCBufferDesc == NULL)\n");
 	return DSERR_INVALIDPARAM;
@@ -793,10 +788,7 @@ IDirectSoundCaptureBufferImpl_Release( LPDIRECTSOUNDCAPTUREBUFFER8 iface )
 	    IDsCaptureDriverBuffer_Release(This->device->hwbuf);
 
         /* remove from DirectSoundCaptureDevice */
-        if (This->device)
-            This->device->capture_buffer = NULL;
-        else
-            ERR("does not reference dsound\n");
+        This->device->capture_buffer = NULL;
 
         if (This->notify)
 	    IDirectSoundNotify_Release((LPDIRECTSOUNDNOTIFY)This->notify);
@@ -816,11 +808,6 @@ IDirectSoundCaptureBufferImpl_GetCaps(
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     TRACE( "(%p,%p)\n", This, lpDSCBCaps );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (lpDSCBCaps == NULL) {
         WARN("invalid parameter: lpDSCBCaps == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -856,11 +843,6 @@ IDirectSoundCaptureBufferImpl_GetCurrentPosition(
     HRESULT hres = DS_OK;
     TRACE( "(%p,%p,%p)\n", This, lpdwCapturePosition, lpdwReadPosition );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -916,11 +898,6 @@ IDirectSoundCaptureBufferImpl_GetFormat(
     TRACE( "(%p,%p,0x%08lx,%p)\n", This, lpwfxFormat, dwSizeAllocated,
         lpdwSizeWritten );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -954,11 +931,6 @@ IDirectSoundCaptureBufferImpl_GetStatus(
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     TRACE( "(%p, %p), thread is %04lx\n", This, lpdwStatus, GetCurrentThreadId() );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -1019,11 +991,6 @@ IDirectSoundCaptureBufferImpl_Lock(
         dwReadBytes, lplpvAudioPtr1, lpdwAudioBytes1, lplpvAudioPtr2,
         lpdwAudioBytes2, dwFlags, GetTickCount() );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -1083,11 +1050,6 @@ IDirectSoundCaptureBufferImpl_Start(
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     TRACE( "(%p,0x%08lx)\n", This, dwFlags );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -1240,11 +1202,6 @@ IDirectSoundCaptureBufferImpl_Stop( LPDIRECTSOUNDCAPTUREBUFFER8 iface )
     IDirectSoundCaptureBufferImpl *This = (IDirectSoundCaptureBufferImpl *)iface;
     TRACE( "(%p)\n", This );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (This->device == NULL) {
         WARN("invalid parameter: This->device == NULL\n");
         return DSERR_INVALIDPARAM;
@@ -1291,11 +1248,6 @@ IDirectSoundCaptureBufferImpl_Unlock(
     TRACE( "(%p,%p,%08lu,%p,%08lu)\n", This, lpvAudioPtr1, dwAudioBytes1,
         lpvAudioPtr2, dwAudioBytes2 );
 
-    if (This == NULL) {
-        WARN("invalid parameter: This == NULL\n");
-        return DSERR_INVALIDPARAM;
-    }
-
     if (lpvAudioPtr1 == NULL) {
         WARN("invalid parameter: lpvAudioPtr1 == NULL\n");
         return DSERR_INVALIDPARAM;
