@@ -1,7 +1,7 @@
 /*
  * iphlpapi dll implementation
  *
- * Copyright (C) 2003 Juan Lang
+ * Copyright (C) 2003,2006 Juan Lang
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -55,11 +55,9 @@ BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
   switch (fdwReason) {
     case DLL_PROCESS_ATTACH:
       DisableThreadLibraryCalls( hinstDLL );
-      interfaceMapInit();
       break;
 
     case DLL_PROCESS_DETACH:
-      interfaceMapFree();
       break;
   }
   return TRUE;
@@ -87,7 +85,6 @@ BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
 DWORD WINAPI AddIPAddress(IPAddr Address, IPMask IpMask, DWORD IfIndex, PULONG NTEContext, PULONG NTEInstance)
 {
   FIXME(":stub\n");
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -101,7 +98,7 @@ DWORD WINAPI AddIPAddress(IPAddr Address, IPMask IpMask, DWORD IfIndex, PULONG N
  * PARAMS
  *  ppIfTable [Out] pointer into which the MIB_IFTABLE is
  *                  allocated and returned.
- *  bOrder    [In]  passed to GetIfTable() to order the table
+ *  bOrder    [In]  whether to sort the table
  *  heap      [In]  heap from which the table is allocated
  *  flags     [In]  flags to HeapAlloc
  *
@@ -132,6 +129,18 @@ DWORD WINAPI AllocateAndGetIfTableFromStack(PMIB_IFTABLE *ppIfTable,
 }
 
 
+static int IpAddrTableSorter(const void *a, const void *b)
+{
+  int ret;
+
+  if (a && b)
+    ret = ((const MIB_IPADDRROW*)a)->dwAddr - ((const MIB_IPADDRROW*)b)->dwAddr;
+  else
+    ret = 0;
+  return ret;
+}
+
+
 /******************************************************************
  *    AllocateAndGetIpAddrTableFromStack (IPHLPAPI.@)
  *
@@ -141,13 +150,13 @@ DWORD WINAPI AllocateAndGetIfTableFromStack(PMIB_IFTABLE *ppIfTable,
  * PARAMS
  *  ppIpAddrTable [Out] pointer into which the MIB_IPADDRTABLE is
  *                      allocated and returned.
- *  bOrder        [In]  passed to GetIpAddrTable to order the table
+ *  bOrder        [In]  whether to sort the table
  *  heap          [In]  heap from which the table is allocated
  *  flags         [In]  flags to HeapAlloc
  *
  * RETURNS
- *  ERROR_INVALID_PARAMETER if ppIpAddrTable is NULL, whatever GetIpAddrTable()
- *  returns otherwise.
+ *  ERROR_INVALID_PARAMETER if ppIpAddrTable is NULL, other error codes on
+ *  failure, NO_ERROR on success.
  */
 DWORD WINAPI AllocateAndGetIpAddrTableFromStack(PMIB_IPADDRTABLE *ppIpAddrTable,
  BOOL bOrder, HANDLE heap, DWORD flags)
@@ -156,18 +165,35 @@ DWORD WINAPI AllocateAndGetIpAddrTableFromStack(PMIB_IPADDRTABLE *ppIpAddrTable,
 
   TRACE("ppIpAddrTable %p, bOrder %d, heap %p, flags 0x%08lx\n",
    ppIpAddrTable, bOrder, heap, flags);
-  if (!ppIpAddrTable)
-    ret = ERROR_INVALID_PARAMETER;
-  else {
-    DWORD dwSize = 0;
+  ret = getIPAddrTable(ppIpAddrTable, heap, flags);
+  if (!ret && bOrder)
+    qsort((*ppIpAddrTable)->table, (*ppIpAddrTable)->dwNumEntries,
+     sizeof(MIB_IPADDRROW), IpAddrTableSorter);
+  TRACE("returning %ld\n", ret);
+  return ret;
+}
 
-    ret = GetIpAddrTable(*ppIpAddrTable, &dwSize, bOrder);
-    if (ret == ERROR_INSUFFICIENT_BUFFER) {
-      *ppIpAddrTable = HeapAlloc(heap, flags, dwSize);
-      ret = GetIpAddrTable(*ppIpAddrTable, &dwSize, bOrder);
+
+static int IpForwardTableSorter(const void *a, const void *b)
+{
+  int ret;
+
+  if (a && b) {
+   const MIB_IPFORWARDROW* rowA = (const MIB_IPFORWARDROW*)a;
+   const MIB_IPFORWARDROW* rowB = (const MIB_IPFORWARDROW*)b;
+
+    ret = rowA->dwForwardDest - rowB->dwForwardDest;
+    if (ret == 0) {
+      ret = rowA->dwForwardProto - rowB->dwForwardProto;
+      if (ret == 0) {
+        ret = rowA->dwForwardPolicy - rowB->dwForwardPolicy;
+        if (ret == 0)
+          ret = rowA->dwForwardNextHop - rowB->dwForwardNextHop;
+      }
     }
   }
-  TRACE("returning %ld\n", ret);
+  else
+    ret = 0;
   return ret;
 }
 
@@ -181,13 +207,13 @@ DWORD WINAPI AllocateAndGetIpAddrTableFromStack(PMIB_IPADDRTABLE *ppIpAddrTable,
  * PARAMS
  *  ppIpForwardTable [Out] pointer into which the MIB_IPFORWARDTABLE is
  *                         allocated and returned.
- *  bOrder           [In]  passed to GetIfTable to order the table
+ *  bOrder           [In]  whether to sort the table
  *  heap             [In]  heap from which the table is allocated
  *  flags            [In]  flags to HeapAlloc
  *
  * RETURNS
- *  ERROR_INVALID_PARAMETER if ppIfTable is NULL, whatever
- *  GetIpForwardTable() returns otherwise.
+ *  ERROR_INVALID_PARAMETER if ppIfTable is NULL, other error codes
+ *  on failure, NO_ERROR on success.
  */
 DWORD WINAPI AllocateAndGetIpForwardTableFromStack(PMIB_IPFORWARDTABLE *
  ppIpForwardTable, BOOL bOrder, HANDLE heap, DWORD flags)
@@ -196,22 +222,27 @@ DWORD WINAPI AllocateAndGetIpForwardTableFromStack(PMIB_IPFORWARDTABLE *
 
   TRACE("ppIpForwardTable %p, bOrder %d, heap %p, flags 0x%08lx\n",
    ppIpForwardTable, bOrder, heap, flags);
-  if (!ppIpForwardTable)
-    ret = ERROR_INVALID_PARAMETER;
-  else {
-    DWORD dwSize = 0;
-
-    ret = GetIpForwardTable(*ppIpForwardTable, &dwSize, bOrder);
-    if (ret == ERROR_INSUFFICIENT_BUFFER) {
-      *ppIpForwardTable = HeapAlloc(heap, flags, dwSize);
-      ret = GetIpForwardTable(*ppIpForwardTable, &dwSize, bOrder);
-    }
-  }
+  ret = getRouteTable(ppIpForwardTable, heap, flags);
+  if (!ret && bOrder)
+    qsort((*ppIpForwardTable)->table, (*ppIpForwardTable)->dwNumEntries,
+     sizeof(MIB_IPFORWARDROW), IpForwardTableSorter);
   TRACE("returning %ld\n", ret);
   return ret;
 }
 
 
+static int IpNetTableSorter(const void *a, const void *b)
+{
+  int ret;
+
+  if (a && b)
+    ret = ((const MIB_IPNETROW*)a)->dwAddr - ((const MIB_IPNETROW*)b)->dwAddr;
+  else
+    ret = 0;
+  return ret;
+}
+
+
 /******************************************************************
  *    AllocateAndGetIpNetTableFromStack (IPHLPAPI.@)
  *
@@ -221,13 +252,13 @@ DWORD WINAPI AllocateAndGetIpForwardTableFromStack(PMIB_IPFORWARDTABLE *
  * PARAMS
  *  ppIpNetTable [Out] pointer into which the MIB_IPNETTABLE is
  *                     allocated and returned.
- *  bOrder       [In]  passed to GetIpNetTable to order the table
+ *  bOrder       [In]  whether to sort the table
  *  heap         [In]  heap from which the table is allocated
  *  flags        [In]  flags to HeapAlloc
  *
  * RETURNS
- *  ERROR_INVALID_PARAMETER if ppIpNetTable is NULL, whatever GetIpNetTable()
- *  returns otherwise.
+ *  ERROR_INVALID_PARAMETER if ppIpNetTable is NULL, other error codes
+ *  on failure, NO_ERROR on success.
  */
 DWORD WINAPI AllocateAndGetIpNetTableFromStack(PMIB_IPNETTABLE *ppIpNetTable,
  BOOL bOrder, HANDLE heap, DWORD flags)
@@ -236,18 +267,35 @@ DWORD WINAPI AllocateAndGetIpNetTableFromStack(PMIB_IPNETTABLE *ppIpNetTable,
 
   TRACE("ppIpNetTable %p, bOrder %d, heap %p, flags 0x%08lx\n",
    ppIpNetTable, bOrder, heap, flags);
-  if (!ppIpNetTable)
-    ret = ERROR_INVALID_PARAMETER;
-  else {
-    DWORD dwSize = 0;
+  ret = getArpTable(ppIpNetTable, heap, flags);
+  if (!ret && bOrder)
+    qsort((*ppIpNetTable)->table, (*ppIpNetTable)->dwNumEntries,
+     sizeof(MIB_IPADDRROW), IpNetTableSorter);
+  TRACE("returning %ld\n", ret);
+  return ret;
+}
 
-    ret = GetIpNetTable(*ppIpNetTable, &dwSize, bOrder);
-    if (ret == ERROR_INSUFFICIENT_BUFFER) {
-      *ppIpNetTable = HeapAlloc(heap, flags, dwSize);
-      ret = GetIpNetTable(*ppIpNetTable, &dwSize, bOrder);
+
+static int TcpTableSorter(const void *a, const void *b)
+{
+  int ret;
+
+  if (a && b) {
+    const MIB_TCPROW* rowA = a;
+    const MIB_TCPROW* rowB = b;
+
+    ret = rowA->dwLocalAddr - rowB->dwLocalAddr;
+    if (ret == 0) {
+      ret = rowA->dwLocalPort - rowB->dwLocalPort;
+      if (ret == 0) {
+        ret = rowA->dwRemoteAddr - rowB->dwRemoteAddr;
+        if (ret == 0)
+          ret = rowA->dwRemotePort - rowB->dwRemotePort;
+      }
     }
   }
-  TRACE("returning %ld\n", ret);
+  else
+    ret = 0;
   return ret;
 }
 
@@ -261,7 +309,7 @@ DWORD WINAPI AllocateAndGetIpNetTableFromStack(PMIB_IPNETTABLE *ppIpNetTable,
  * PARAMS
  *  ppTcpTable [Out] pointer into which the MIB_TCPTABLE is
  *                   allocated and returned.
- *  bOrder     [In]  passed to GetTcpTable to order the table
+ *  bOrder     [In]  whether to sort the table
  *  heap       [In]  heap from which the table is allocated
  *  flags      [In]  flags to HeapAlloc
  *
@@ -276,22 +324,33 @@ DWORD WINAPI AllocateAndGetTcpTableFromStack(PMIB_TCPTABLE *ppTcpTable,
 
   TRACE("ppTcpTable %p, bOrder %d, heap %p, flags 0x%08lx\n",
    ppTcpTable, bOrder, heap, flags);
-  if (!ppTcpTable)
-    ret = ERROR_INVALID_PARAMETER;
-  else {
-    DWORD dwSize = 0;
-
-    ret = GetTcpTable(*ppTcpTable, &dwSize, bOrder);
-    if (ret == ERROR_INSUFFICIENT_BUFFER) {
-      *ppTcpTable = HeapAlloc(heap, flags, dwSize);
-      ret = GetTcpTable(*ppTcpTable, &dwSize, bOrder);
-    }
-  }
+  ret = getTcpTable(ppTcpTable, heap, flags);
+  if (!ret && bOrder)
+    qsort((*ppTcpTable)->table, (*ppTcpTable)->dwNumEntries,
+     sizeof(MIB_TCPROW), TcpTableSorter);
   TRACE("returning %ld\n", ret);
   return ret;
 }
 
 
+static int UdpTableSorter(const void *a, const void *b)
+{
+  int ret;
+
+  if (a && b) {
+    const MIB_UDPROW* rowA = (const MIB_UDPROW*)a;
+    const MIB_UDPROW* rowB = (const MIB_UDPROW*)b;
+
+    ret = rowA->dwLocalAddr - rowB->dwLocalAddr;
+    if (ret == 0)
+      ret = rowA->dwLocalPort - rowB->dwLocalPort;
+  }
+  else
+    ret = 0;
+  return ret;
+}
+
+
 /******************************************************************
  *    AllocateAndGetUdpTableFromStack (IPHLPAPI.@)
  *
@@ -301,7 +360,7 @@ DWORD WINAPI AllocateAndGetTcpTableFromStack(PMIB_TCPTABLE *ppTcpTable,
  * PARAMS
  *  ppUdpTable [Out] pointer into which the MIB_UDPTABLE is
  *                   allocated and returned.
- *  bOrder     [In]  passed to GetUdpTable to order the table
+ *  bOrder     [In]  whether to sort the table
  *  heap       [In]  heap from which the table is allocated
  *  flags      [In]  flags to HeapAlloc
  *
@@ -316,17 +375,10 @@ DWORD WINAPI AllocateAndGetUdpTableFromStack(PMIB_UDPTABLE *ppUdpTable,
 
   TRACE("ppUdpTable %p, bOrder %d, heap %p, flags 0x%08lx\n",
    ppUdpTable, bOrder, heap, flags);
-  if (!ppUdpTable)
-    ret = ERROR_INVALID_PARAMETER;
-  else {
-    DWORD dwSize = 0;
-
-    ret = GetUdpTable(*ppUdpTable, &dwSize, bOrder);
-    if (ret == ERROR_INSUFFICIENT_BUFFER) {
-      *ppUdpTable = HeapAlloc(heap, flags, dwSize);
-      ret = GetUdpTable(*ppUdpTable, &dwSize, bOrder);
-    }
-  }
+  ret = getUdpTable(ppUdpTable, heap, flags);
+  if (!ret && bOrder)
+    qsort((*ppUdpTable)->table, (*ppUdpTable)->dwNumEntries,
+     sizeof(MIB_UDPROW), UdpTableSorter);
   TRACE("returning %ld\n", ret);
   return ret;
 }
@@ -399,7 +451,6 @@ DWORD WINAPI CreateProxyArpEntry(DWORD dwAddress, DWORD dwMask, DWORD dwIfIndex)
 {
   FIXME("(dwAddress 0x%08lx, dwMask 0x%08lx, dwIfIndex 0x%08lx): stub\n",
    dwAddress, dwMask, dwIfIndex);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -422,7 +473,6 @@ DWORD WINAPI CreateProxyArpEntry(DWORD dwAddress, DWORD dwMask, DWORD dwIfIndex)
 DWORD WINAPI DeleteIPAddress(ULONG NTEContext)
 {
   FIXME("(NTEContext %ld): stub\n", NTEContext);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -494,7 +544,6 @@ DWORD WINAPI DeleteProxyArpEntry(DWORD dwAddress, DWORD dwMask, DWORD dwIfIndex)
 {
   FIXME("(dwAddress 0x%08lx, dwMask 0x%08lx, dwIfIndex 0x%08lx): stub\n",
    dwAddress, dwMask, dwIfIndex);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -520,7 +569,7 @@ DWORD WINAPI EnableRouter(HANDLE * pHandle, OVERLAPPED * pOverlapped)
   FIXME("(pHandle %p, pOverlapped %p): stub\n", pHandle, pOverlapped);
   /* could echo "1" > /proc/net/sys/net/ipv4/ip_forward, not sure I want to
      could map EACCESS to ERROR_ACCESS_DENIED, I suppose
-     marking Win2K+ functions not supported */
+   */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -543,8 +592,7 @@ DWORD WINAPI EnableRouter(HANDLE * pHandle, OVERLAPPED * pOverlapped)
 DWORD WINAPI FlushIpNetTable(DWORD dwIfIndex)
 {
   FIXME("(dwIfIndex 0x%08lx): stub\n", dwIfIndex);
-  /* this flushes the arp cache of the given index
-     marking Win2K+ functions not supported */
+  /* this flushes the arp cache of the given index */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -568,7 +616,7 @@ DWORD WINAPI FlushIpNetTable(DWORD dwIfIndex)
 DWORD WINAPI GetAdapterIndex(LPWSTR AdapterName, PULONG IfIndex)
 {
   FIXME("(AdapterName %p, IfIndex %p): stub\n", AdapterName, IfIndex);
-  /* marking Win2K+ functions not supported */
+  /* FIXME: implement using getInterfaceIndexByName */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -597,22 +645,30 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
     DWORD numNonLoopbackInterfaces = getNumNonLoopbackInterfaces();
 
     if (numNonLoopbackInterfaces > 0) {
-      /* this calculation assumes only one address in the IP_ADDR_STRING lists.
-         that's okay, because:
-         - we don't get multiple addresses per adapter anyway
-         - we don't know about per-adapter gateways
-         - DHCP and WINS servers can have max one entry per list */
-      ULONG size = sizeof(IP_ADAPTER_INFO) * numNonLoopbackInterfaces;
+      DWORD numIPAddresses = getNumIPAddresses();
+      ULONG size;
 
+      /* This may slightly overestimate the amount of space needed, because
+       * the IP addresses include the loopback address, but it's easier
+       * to make sure there's more than enough space than to make sure there's
+       * precisely enough space.
+       */
+      size = sizeof(IP_ADAPTER_INFO) * numNonLoopbackInterfaces;
+      size += numIPAddresses  * sizeof(IP_ADDR_STRING); 
       if (!pAdapterInfo || *pOutBufLen < size) {
         *pOutBufLen = size;
         ret = ERROR_BUFFER_OVERFLOW;
       }
       else {
-        InterfaceIndexTable *table = getNonLoopbackInterfaceIndexTable();
+        InterfaceIndexTable *table = NULL;
+        PMIB_IPADDRTABLE ipAddrTable = NULL;
 
+        ret = getIPAddrTable(&ipAddrTable, GetProcessHeap(), 0);
+        if (!ret)
+          table = getNonLoopbackInterfaceIndexTable();
         if (table) {
           size = sizeof(IP_ADAPTER_INFO) * table->numIndexes;
+          size += ipAddrTable->dwNumEntries * sizeof(IP_ADDR_STRING); 
           if (*pOutBufLen < size) {
             *pOutBufLen = size;
             ret = ERROR_INSUFFICIENT_BUFFER;
@@ -622,10 +678,13 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
             HKEY hKey;
             BOOL winsEnabled = FALSE;
             IP_ADDRESS_STRING primaryWINS, secondaryWINS;
+            PIP_ADDR_STRING nextIPAddr = (PIP_ADDR_STRING)((LPBYTE)pAdapterInfo
+             + numNonLoopbackInterfaces * sizeof(IP_ADAPTER_INFO));
 
             memset(pAdapterInfo, 0, size);
             /* @@ Wine registry key: HKCU\Software\Wine\Network */
-            if (RegOpenKeyA(HKEY_CURRENT_USER, "Software\\Wine\\Network", &hKey) == ERROR_SUCCESS) {
+            if (RegOpenKeyA(HKEY_CURRENT_USER, "Software\\Wine\\Network",
+             &hKey) == ERROR_SUCCESS) {
               DWORD size = sizeof(primaryWINS.String);
               unsigned long addr;
 
@@ -644,12 +703,12 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
             }
             for (ndx = 0; ndx < table->numIndexes; ndx++) {
               PIP_ADAPTER_INFO ptr = &pAdapterInfo[ndx];
-              DWORD addrLen = sizeof(ptr->Address), type;
+              DWORD addrLen = sizeof(ptr->Address), type, i;
+              PIP_ADDR_STRING currentIPAddr = &ptr->IpAddressList;
+              BOOL firstIPAddr = TRUE;
 
               /* on Win98 this is left empty, but whatever */
-              lstrcpynA(ptr->AdapterName,
-               getInterfaceNameByIndex(table->indexes[ndx]),
-               MAX_ADAPTER_NAME_LENGTH+1);
+              getInterfaceNameByIndex(table->indexes[ndx], ptr->AdapterName);
               getInterfacePhysicalByIndex(table->indexes[ndx], &addrLen,
                ptr->Address, &type);
               /* MS defines address length and type as UINT in some places and
@@ -658,10 +717,26 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
               ptr->AddressLength = addrLen;
               ptr->Type = type;
               ptr->Index = table->indexes[ndx];
-              toIPAddressString(getInterfaceIPAddrByIndex(table->indexes[ndx]),
-               ptr->IpAddressList.IpAddress.String);
-              toIPAddressString(getInterfaceMaskByIndex(table->indexes[ndx]),
-               ptr->IpAddressList.IpMask.String);
+              for (i = 0; i < ipAddrTable->dwNumEntries; i++) {
+                if (ipAddrTable->table[i].dwIndex == ptr->Index) {
+                  if (firstIPAddr) {
+                    toIPAddressString(ipAddrTable->table[i].dwAddr,
+                     ptr->IpAddressList.IpAddress.String);
+                    toIPAddressString(ipAddrTable->table[i].dwBCastAddr,
+                     ptr->IpAddressList.IpMask.String);
+                    firstIPAddr = FALSE;
+                  }
+                  else {
+                    currentIPAddr->Next = nextIPAddr;
+                    currentIPAddr = nextIPAddr;
+                    toIPAddressString(ipAddrTable->table[i].dwAddr,
+                     currentIPAddr->IpAddress.String);
+                    toIPAddressString(ipAddrTable->table[i].dwBCastAddr,
+                     currentIPAddr->IpMask.String);
+                    nextIPAddr++;
+                  }
+                }
+              }
               if (winsEnabled) {
                 ptr->HaveWins = TRUE;
                 memcpy(ptr->PrimaryWinsServer.IpAddress.String,
@@ -680,6 +755,7 @@ DWORD WINAPI GetAdaptersInfo(PIP_ADAPTER_INFO pAdapterInfo, PULONG pOutBufLen)
         }
         else
           ret = ERROR_OUTOFMEMORY;
+        HeapFree(GetProcessHeap(), 0, ipAddrTable);
       }
     }
     else
@@ -751,7 +827,8 @@ DWORD WINAPI GetBestRoute(DWORD dwDestAddr, DWORD dwSourceAddr, PMIB_IPFORWARDRO
     DWORD ndx, matchedBits, matchedNdx = 0;
 
     for (ndx = 0, matchedBits = 0; ndx < table->dwNumEntries; ndx++) {
-      if ((dwDestAddr & table->table[ndx].dwForwardMask) ==
+      if (table->table[ndx].dwForwardType != MIB_IPROUTE_TYPE_INVALID &&
+       (dwDestAddr & table->table[ndx].dwForwardMask) ==
        (table->table[ndx].dwForwardDest & table->table[ndx].dwForwardMask)) {
         DWORD numShifts, mask;
 
@@ -764,9 +841,15 @@ DWORD WINAPI GetBestRoute(DWORD dwDestAddr, DWORD dwSourceAddr, PMIB_IPFORWARDRO
         }
       }
     }
-    memcpy(pBestRoute, &table->table[matchedNdx], sizeof(MIB_IPFORWARDROW));
+    if (matchedNdx < table->dwNumEntries) {
+      memcpy(pBestRoute, &table->table[matchedNdx], sizeof(MIB_IPFORWARDROW));
+      ret = ERROR_SUCCESS;
+    }
+    else {
+      /* No route matches, which can happen if there's no default route. */
+      ret = ERROR_HOST_UNREACHABLE;
+    }
     HeapFree(GetProcessHeap(), 0, table);
-    ret = ERROR_SUCCESS;
   }
   else
     ret = ERROR_OUTOFMEMORY;
@@ -837,13 +920,14 @@ DWORD WINAPI GetIcmpStatistics(PMIB_ICMP pStats)
 DWORD WINAPI GetIfEntry(PMIB_IFROW pIfRow)
 {
   DWORD ret;
-  const char *name;
+  char nameBuf[MAX_ADAPTER_NAME];
+  char *name;
 
   TRACE("pIfRow %p\n", pIfRow);
   if (!pIfRow)
     return ERROR_INVALID_PARAMETER;
 
-  name = getInterfaceNameByIndex(pIfRow->dwIndex);
+  name = getInterfaceNameByIndex(pIfRow->dwIndex, nameBuf);
   if (name) {
     ret = getInterfaceEntryByName(name, pIfRow);
     if (ret == NO_ERROR)
@@ -917,6 +1001,7 @@ DWORD WINAPI GetIfTable(PMIB_IFTABLE pIfTable, PULONG pdwSize, BOOL bOrder)
         else {
           DWORD ndx;
 
+          *pdwSize = size;
           pIfTable->dwNumEntries = 0;
           for (ndx = 0; ndx < table->numIndexes; ndx++) {
             pIfTable->table[ndx].dwIndex = table->indexes[ndx];
@@ -945,12 +1030,15 @@ DWORD WINAPI GetIfTable(PMIB_IFTABLE pIfTable, PULONG pdwSize, BOOL bOrder)
  * Get a list of network interface adapters.
  *
  * PARAMS
- *  pIfTable    [Out] buffer for intertace adapters
+ *  pIfTable    [Out] buffer for interface adapters
  *  dwOutBufLen [Out] if buffer is too small, returns required size
  *
  * RETURNS
  *  Success: NO_ERROR
  *  Failure: error code from winerror.h
+ *
+ * BUGS
+ *  MSDN states this should return non-loopback interfaces only.
  */
 DWORD WINAPI GetInterfaceInfo(PIP_INTERFACE_INFO pIfTable, PULONG dwOutBufLen)
 {
@@ -980,14 +1068,16 @@ DWORD WINAPI GetInterfaceInfo(PIP_INTERFACE_INFO pIfTable, PULONG dwOutBufLen)
         }
         else {
           DWORD ndx;
+          char nameBuf[MAX_ADAPTER_NAME];
 
+          *dwOutBufLen = size;
           pIfTable->NumAdapters = 0;
           for (ndx = 0; ndx < table->numIndexes; ndx++) {
             const char *walker, *name;
             WCHAR *assigner;
 
             pIfTable->Adapter[ndx].Index = table->indexes[ndx];
-            name = getInterfaceNameByIndex(table->indexes[ndx]);
+            name = getInterfaceNameByIndex(table->indexes[ndx], nameBuf);
             for (walker = name, assigner = pIfTable->Adapter[ndx].Name;
              walker && *walker &&
              assigner - pIfTable->Adapter[ndx].Name < MAX_ADAPTER_NAME - 1;
@@ -1009,18 +1099,6 @@ DWORD WINAPI GetInterfaceInfo(PIP_INTERFACE_INFO pIfTable, PULONG dwOutBufLen)
 }
 
 
-static int IpAddrTableSorter(const void *a, const void *b)
-{
-  int ret;
-
-  if (a && b)
-    ret = ((const MIB_IPADDRROW*)a)->dwAddr - ((const MIB_IPADDRROW*)b)->dwAddr;
-  else
-    ret = 0;
-  return ret;
-}
-
-
 /******************************************************************
  *    GetIpAddrTable (IPHLPAPI.@)
  *
@@ -1051,56 +1129,28 @@ DWORD WINAPI GetIpAddrTable(PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL
   if (!pdwSize)
     ret = ERROR_INVALID_PARAMETER;
   else {
-    DWORD numInterfaces = getNumInterfaces();
-    ULONG size = sizeof(MIB_IPADDRTABLE) + (numInterfaces - 1) *
-     sizeof(MIB_IPADDRROW);
+    PMIB_IPADDRTABLE table;
 
-    if (!pIpAddrTable || *pdwSize < size) {
-      *pdwSize = size;
-      ret = ERROR_INSUFFICIENT_BUFFER;
-    }
-    else {
-      InterfaceIndexTable *table = getInterfaceIndexTable();
+    ret = getIPAddrTable(&table, GetProcessHeap(), 0);
+    if (ret == NO_ERROR)
+    {
+      ULONG size = sizeof(MIB_IPADDRTABLE) + (table->dwNumEntries - 1) *
+       sizeof(MIB_IPADDRROW);
 
-      if (table) {
-        size = sizeof(MIB_IPADDRTABLE) + (table->numIndexes - 1) *
-         sizeof(MIB_IPADDRROW);
-        if (*pdwSize < size) {
-          *pdwSize = size;
-          ret = ERROR_INSUFFICIENT_BUFFER;
-        }
-        else {
-          DWORD ndx, bcast;
-
-          pIpAddrTable->dwNumEntries = 0;
-          for (ndx = 0; ndx < table->numIndexes; ndx++) {
-            pIpAddrTable->table[ndx].dwIndex = table->indexes[ndx];
-            pIpAddrTable->table[ndx].dwAddr =
-             getInterfaceIPAddrByIndex(table->indexes[ndx]);
-            pIpAddrTable->table[ndx].dwMask =
-             getInterfaceMaskByIndex(table->indexes[ndx]);
-            /* the dwBCastAddr member isn't the broadcast address, it indicates
-             * whether the interface uses the 1's broadcast address (1) or the
-             * 0's broadcast address (0).
-             */
-            bcast = getInterfaceBCastAddrByIndex(table->indexes[ndx]);
-            pIpAddrTable->table[ndx].dwBCastAddr =
-             (bcast & pIpAddrTable->table[ndx].dwMask) ? 1 : 0;
-            /* FIXME: hardcoded reasm size, not sure where to get it */
-            pIpAddrTable->table[ndx].dwReasmSize = 65535;
-            pIpAddrTable->table[ndx].unused1 = 0;
-            pIpAddrTable->table[ndx].wType = 0; /* aka unused2 */
-            pIpAddrTable->dwNumEntries++;
-          }
-          if (bOrder)
-            qsort(pIpAddrTable->table, pIpAddrTable->dwNumEntries,
-             sizeof(MIB_IPADDRROW), IpAddrTableSorter);
-          ret = NO_ERROR;
-        }
-        HeapFree(GetProcessHeap(), 0, table);
+      if (!pIpAddrTable || *pdwSize < size) {
+        *pdwSize = size;
+        ret = ERROR_INSUFFICIENT_BUFFER;
       }
-      else
-        ret = ERROR_OUTOFMEMORY;
+      else {
+        *pdwSize = size;
+        memcpy(pIpAddrTable, table, sizeof(MIB_IPADDRTABLE) +
+         (table->dwNumEntries - 1) * sizeof(MIB_IPADDRROW));
+        if (bOrder)
+          qsort(pIpAddrTable->table, pIpAddrTable->dwNumEntries,
+           sizeof(MIB_IPADDRROW), IpAddrTableSorter);
+        ret = NO_ERROR;
+      }
+      HeapFree(GetProcessHeap(), 0, table);
     }
   }
   TRACE("returning %ld\n", ret);
@@ -1108,30 +1158,6 @@ DWORD WINAPI GetIpAddrTable(PMIB_IPADDRTABLE pIpAddrTable, PULONG pdwSize, BOOL
 }
 
 
-static int IpForwardTableSorter(const void *a, const void *b)
-{
-  int ret;
-
-  if (a && b) {
-   const MIB_IPFORWARDROW* rowA = (const MIB_IPFORWARDROW*)a;
-   const MIB_IPFORWARDROW* rowB = (const MIB_IPFORWARDROW*)b;
-
-    ret = rowA->dwForwardDest - rowB->dwForwardDest;
-    if (ret == 0) {
-      ret = rowA->dwForwardProto - rowB->dwForwardProto;
-      if (ret == 0) {
-        ret = rowA->dwForwardPolicy - rowB->dwForwardPolicy;
-        if (ret == 0)
-          ret = rowA->dwForwardNextHop - rowB->dwForwardNextHop;
-      }
-    }
-  }
-  else
-    ret = 0;
-  return ret;
-}
-
-
 /******************************************************************
  *    GetIpForwardTable (IPHLPAPI.@)
  *
@@ -1171,45 +1197,19 @@ DWORD WINAPI GetIpForwardTable(PMIB_IPFORWARDTABLE pIpForwardTable, PULONG pdwSi
       ret = ERROR_INSUFFICIENT_BUFFER;
     }
     else {
-      RouteTable *table = getRouteTable();
-      if (table) {
-        sizeNeeded = sizeof(MIB_IPFORWARDTABLE) + (table->numRoutes - 1) *
+      PMIB_IPFORWARDTABLE table;
+
+      ret = getRouteTable(&table, GetProcessHeap(), 0);
+      if (!ret) {
+        sizeNeeded = sizeof(MIB_IPFORWARDTABLE) + (table->dwNumEntries - 1) *
          sizeof(MIB_IPFORWARDROW);
         if (*pdwSize < sizeNeeded) {
           *pdwSize = sizeNeeded;
           ret = ERROR_INSUFFICIENT_BUFFER;
         }
         else {
-          DWORD ndx;
-
-          pIpForwardTable->dwNumEntries = table->numRoutes;
-          for (ndx = 0; ndx < numRoutes; ndx++) {
-            pIpForwardTable->table[ndx].dwForwardIfIndex =
-             table->routes[ndx].ifIndex;
-            pIpForwardTable->table[ndx].dwForwardDest =
-             table->routes[ndx].dest;
-            pIpForwardTable->table[ndx].dwForwardMask =
-             table->routes[ndx].mask;
-            pIpForwardTable->table[ndx].dwForwardPolicy = 0;
-            pIpForwardTable->table[ndx].dwForwardNextHop =
-             table->routes[ndx].gateway;
-            /* FIXME: this type is appropriate for local interfaces; may not
-               always be appropriate */
-            pIpForwardTable->table[ndx].dwForwardType = MIB_IPROUTE_TYPE_DIRECT;
-            /* FIXME: other protos might be appropriate, e.g. the default route
-               is typically set with MIB_IPPROTO_NETMGMT instead */
-            pIpForwardTable->table[ndx].dwForwardProto = MIB_IPPROTO_LOCAL;
-            /* punt on age and AS */
-            pIpForwardTable->table[ndx].dwForwardAge = 0;
-            pIpForwardTable->table[ndx].dwForwardNextHopAS = 0;
-            pIpForwardTable->table[ndx].dwForwardMetric1 =
-             table->routes[ndx].metric;
-            /* rest of the metrics are 0.. */
-            pIpForwardTable->table[ndx].dwForwardMetric2 = 0;
-            pIpForwardTable->table[ndx].dwForwardMetric3 = 0;
-            pIpForwardTable->table[ndx].dwForwardMetric4 = 0;
-            pIpForwardTable->table[ndx].dwForwardMetric5 = 0;
-          }
+          *pdwSize = sizeNeeded;
+          memcpy(pIpForwardTable, table, sizeNeeded);
           if (bOrder)
             qsort(pIpForwardTable->table, pIpForwardTable->dwNumEntries,
              sizeof(MIB_IPFORWARDROW), IpForwardTableSorter);
@@ -1217,8 +1217,6 @@ DWORD WINAPI GetIpForwardTable(PMIB_IPFORWARDTABLE pIpForwardTable, PULONG pdwSi
         }
         HeapFree(GetProcessHeap(), 0, table);
       }
-      else
-        ret = ERROR_OUTOFMEMORY;
     }
   }
   TRACE("returning %ld\n", ret);
@@ -1226,18 +1224,6 @@ DWORD WINAPI GetIpForwardTable(PMIB_IPFORWARDTABLE pIpForwardTable, PULONG pdwSi
 }
 
 
-static int IpNetTableSorter(const void *a, const void *b)
-{
-  int ret;
-
-  if (a && b)
-    ret = ((const MIB_IPNETROW*)a)->dwAddr - ((const MIB_IPNETROW*)b)->dwAddr;
-  else
-    ret = 0;
-  return ret;
-}
-
-
 /******************************************************************
  *    GetIpNetTable (IPHLPAPI.@)
  *
@@ -1276,9 +1262,10 @@ DWORD WINAPI GetIpNetTable(PMIB_IPNETTABLE pIpNetTable, PULONG pdwSize, BOOL bOr
       ret = ERROR_INSUFFICIENT_BUFFER;
     }
     else {
-      PMIB_IPNETTABLE table = getArpTable();
+      PMIB_IPNETTABLE table;
 
-      if (table) {
+      ret = getArpTable(&table, GetProcessHeap(), 0);
+      if (!ret) {
         size = sizeof(MIB_IPNETTABLE) + (table->dwNumEntries - 1) *
          sizeof(MIB_IPNETROW);
         if (*pdwSize < size) {
@@ -1286,6 +1273,7 @@ DWORD WINAPI GetIpNetTable(PMIB_IPNETTABLE pIpNetTable, PULONG pdwSize, BOOL bOr
           ret = ERROR_INSUFFICIENT_BUFFER;
         }
         else {
+          *pdwSize = size;
           memcpy(pIpNetTable, table, size);
           if (bOrder)
             qsort(pIpNetTable->table, pIpNetTable->dwNumEntries,
@@ -1294,8 +1282,6 @@ DWORD WINAPI GetIpNetTable(PMIB_IPNETTABLE pIpNetTable, PULONG pdwSize, BOOL bOr
         }
         HeapFree(GetProcessHeap(), 0, table);
       }
-      else
-        ret = ERROR_OUTOFMEMORY;
     }
   }
   TRACE("returning %ld\n", ret);
@@ -1454,7 +1440,6 @@ DWORD WINAPI GetPerAdapterInfo(ULONG IfIndex, PIP_PER_ADAPTER_INFO pPerAdapterIn
 {
   TRACE("(IfIndex %ld, pPerAdapterInfo %p, pOutBufLen %p)\n", IfIndex,
    pPerAdapterInfo, pOutBufLen);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -1509,30 +1494,6 @@ DWORD WINAPI GetTcpStatistics(PMIB_TCPSTATS pStats)
 }
 
 
-static int TcpTableSorter(const void *a, const void *b)
-{
-  int ret;
-
-  if (a && b) {
-    const MIB_TCPROW* rowA = a;
-    const MIB_TCPROW* rowB = b;
-
-    ret = rowA->dwLocalAddr - rowB->dwLocalAddr;
-    if (ret == 0) {
-      ret = rowA->dwLocalPort - rowB->dwLocalPort;
-      if (ret == 0) {
-        ret = rowA->dwRemoteAddr - rowB->dwRemoteAddr;
-        if (ret == 0)
-          ret = rowA->dwRemotePort - rowB->dwRemotePort;
-      }
-    }
-  }
-  else
-    ret = 0;
-  return ret;
-}
-
-
 /******************************************************************
  *    GetTcpTable (IPHLPAPI.@)
  *
@@ -1565,16 +1526,17 @@ DWORD WINAPI GetTcpTable(PMIB_TCPTABLE pTcpTable, PDWORD pdwSize, BOOL bOrder)
     ret = ERROR_INVALID_PARAMETER;
   else {
     DWORD numEntries = getNumTcpEntries();
-    ULONG size = sizeof(MIB_TCPTABLE) + (numEntries - 1) * sizeof(MIB_TCPROW);
+    DWORD size = sizeof(MIB_TCPTABLE) + (numEntries - 1) * sizeof(MIB_TCPROW);
 
     if (!pTcpTable || *pdwSize < size) {
       *pdwSize = size;
       ret = ERROR_INSUFFICIENT_BUFFER;
     }
     else {
-      PMIB_TCPTABLE table = getTcpTable();
+      PMIB_TCPTABLE table;
 
-      if (table) {
+      ret = getTcpTable(&table, GetProcessHeap(), 0);
+      if (!ret) {
         size = sizeof(MIB_TCPTABLE) + (table->dwNumEntries - 1) *
          sizeof(MIB_TCPROW);
         if (*pdwSize < size) {
@@ -1582,6 +1544,7 @@ DWORD WINAPI GetTcpTable(PMIB_TCPTABLE pTcpTable, PDWORD pdwSize, BOOL bOrder)
           ret = ERROR_INSUFFICIENT_BUFFER;
         }
         else {
+          *pdwSize = size;
           memcpy(pTcpTable, table, size);
           if (bOrder)
             qsort(pTcpTable->table, pTcpTable->dwNumEntries,
@@ -1622,24 +1585,6 @@ DWORD WINAPI GetUdpStatistics(PMIB_UDPSTATS pStats)
 }
 
 
-static int UdpTableSorter(const void *a, const void *b)
-{
-  int ret;
-
-  if (a && b) {
-    const MIB_UDPROW* rowA = (const MIB_UDPROW*)a;
-    const MIB_UDPROW* rowB = (const MIB_UDPROW*)b;
-
-    ret = rowA->dwLocalAddr - rowB->dwLocalAddr;
-    if (ret == 0)
-      ret = rowA->dwLocalPort - rowB->dwLocalPort;
-  }
-  else
-    ret = 0;
-  return ret;
-}
-
-
 /******************************************************************
  *    GetUdpTable (IPHLPAPI.@)
  *
@@ -1671,16 +1616,17 @@ DWORD WINAPI GetUdpTable(PMIB_UDPTABLE pUdpTable, PDWORD pdwSize, BOOL bOrder)
     ret = ERROR_INVALID_PARAMETER;
   else {
     DWORD numEntries = getNumUdpEntries();
-    ULONG size = sizeof(MIB_UDPTABLE) + (numEntries - 1) * sizeof(MIB_UDPROW);
+    DWORD size = sizeof(MIB_UDPTABLE) + (numEntries - 1) * sizeof(MIB_UDPROW);
 
     if (!pUdpTable || *pdwSize < size) {
       *pdwSize = size;
       ret = ERROR_INSUFFICIENT_BUFFER;
     }
     else {
-      PMIB_UDPTABLE table = getUdpTable();
+      PMIB_UDPTABLE table;
 
-      if (table) {
+      ret = getUdpTable(&table, GetProcessHeap(), 0);
+      if (!ret) {
         size = sizeof(MIB_UDPTABLE) + (table->dwNumEntries - 1) *
          sizeof(MIB_UDPROW);
         if (*pdwSize < size) {
@@ -1688,6 +1634,7 @@ DWORD WINAPI GetUdpTable(PMIB_UDPTABLE pUdpTable, PDWORD pdwSize, BOOL bOrder)
           ret = ERROR_INSUFFICIENT_BUFFER;
         }
         else {
+          *pdwSize = size;
           memcpy(pUdpTable, table, size);
           if (bOrder)
             qsort(pUdpTable->table, pUdpTable->dwNumEntries,
@@ -1806,7 +1753,6 @@ DWORD WINAPI IpRenewAddress(PIP_ADAPTER_INDEX_MAP AdapterInfo)
 DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 {
   FIXME("(Handle %p, overlapped %p): stub\n", Handle, overlapped);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -1830,7 +1776,6 @@ DWORD WINAPI NotifyAddrChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 DWORD WINAPI NotifyRouteChange(PHANDLE Handle, LPOVERLAPPED overlapped)
 {
   FIXME("(Handle %p, overlapped %p): stub\n", Handle, overlapped);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -1841,7 +1786,7 @@ DWORD WINAPI NotifyRouteChange(PHANDLE Handle, LPOVERLAPPED overlapped)
  * Send an ARP request.
  *
  * PARAMS
- *  DestIP     [In]     attemp to obtain this IP
+ *  DestIP     [In]     attempt to obtain this IP
  *  SrcIP      [In]     optional sender IP address
  *  pMacAddr   [Out]    buffer for the mac address
  *  PhyAddrLen [In/Out] length of the output buffer
@@ -1857,7 +1802,6 @@ DWORD WINAPI SendARP(IPAddr DestIP, IPAddr SrcIP, PULONG pMacAddr, PULONG PhyAdd
 {
   FIXME("(DestIP 0x%08lx, SrcIP 0x%08lx, pMacAddr %p, PhyAddrLen %p): stub\n",
    DestIP, SrcIP, pMacAddr, PhyAddrLen);
-  /* marking Win2K+ functions not supported */
   return ERROR_NOT_SUPPORTED;
 }
 
@@ -1880,7 +1824,7 @@ DWORD WINAPI SendARP(IPAddr DestIP, IPAddr SrcIP, PULONG pMacAddr, PULONG PhyAdd
 DWORD WINAPI SetIfEntry(PMIB_IFROW pIfRow)
 {
   FIXME("(pIfRow %p): stub\n", pIfRow);
-  /* this is supposed to set an administratively interface up or down.
+  /* this is supposed to set an interface administratively up or down.
      Could do SIOCSIFFLAGS and set/clear IFF_UP, but, not sure I want to, and
      this sort of down is indistinguishable from other sorts of down (e.g. no
      link). */
@@ -2027,6 +1971,6 @@ DWORD WINAPI UnenableRouter(OVERLAPPED * pOverlapped, LPDWORD lpdwEnableCount)
    lpdwEnableCount);
   /* could echo "0" > /proc/net/sys/net/ipv4/ip_forward, not sure I want to
      could map EACCESS to ERROR_ACCESS_DENIED, I suppose
-     marking Win2K+ functions not supported */
+   */
   return ERROR_NOT_SUPPORTED;
 }
