@@ -431,7 +431,6 @@ DWORD WINAPI GetShortPathNameW( LPCWSTR longpath, LPWSTR shortpath, DWORD shortl
     LPCWSTR             p;
     DWORD               sp = 0, lp = 0;
     DWORD               tmplen;
-    BOOL                unixabsolute = (longpath[0] == '/');
     WIN32_FIND_DATAW    wfd;
     HANDLE              goit;
     UNICODE_STRING      ustr;
@@ -451,7 +450,7 @@ DWORD WINAPI GetShortPathNameW( LPCWSTR longpath, LPWSTR shortpath, DWORD shortl
     }
 
     /* check for drive letter */
-    if (!unixabsolute && longpath[1] == ':' )
+    if (longpath[0] != '/' && longpath[1] == ':' )
     {
         tmpshortpath[0] = longpath[0];
         tmpshortpath[1] = ':';
@@ -722,7 +721,7 @@ inline static BOOL contains_pathW (LPCWSTR name)
  * Searches for a specified file in the search path.
  *
  * PARAMS
- *    path	[I] Path to search
+ *    path	[I] Path to search (NULL means default)
  *    name	[I] Filename to search for.
  *    ext	[I] File extension to append to file name. The first
  *		    character must be a period. This parameter is
@@ -818,12 +817,13 @@ DWORD WINAPI SearchPathW( LPCWSTR path, LPCWSTR name, LPCWSTR ext, DWORD buflen,
 DWORD WINAPI SearchPathA( LPCSTR path, LPCSTR name, LPCSTR ext,
                           DWORD buflen, LPSTR buffer, LPSTR *lastpart )
 {
-    WCHAR *pathW, *nameW = NULL, *extW = NULL;
+    WCHAR *pathW = NULL, *nameW = NULL, *extW = NULL;
     WCHAR bufferW[MAX_PATH];
     DWORD ret;
 
-    if (name && !(nameW = FILE_name_AtoW( name, FALSE ))) return 0;
-    if (!(pathW = FILE_name_AtoW( path, TRUE ))) return 0;
+    if (!name || !(nameW = FILE_name_AtoW( name, FALSE ))) return 0;
+    if (path && !(pathW = FILE_name_AtoW( path, TRUE ))) return 0;
+    
     if (ext && !(extW = FILE_name_AtoW( ext, TRUE )))
     {
         HeapFree( GetProcessHeap(), 0, pathW );
