@@ -31,6 +31,9 @@
 #ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
+#ifdef HAVE_SYS_PRCTL_H
+# include <sys/prctl.h>
+#endif
 #include <sys/types.h>
 
 #include "ntstatus.h"
@@ -592,22 +595,6 @@ static BOOL build_command_line( WCHAR **argv )
 }
 
 
-static void version(void)
-{
-    MESSAGE( "%s\n", PACKAGE_STRING );
-    ExitProcess(0);
-}
-
-static void usage(void)
-{
-    MESSAGE( "%s\n", PACKAGE_STRING );
-    MESSAGE( "Usage: wine PROGRAM [ARGUMENTS...]   Run the specified program\n" );
-    MESSAGE( "       wine --help                   Display this help and exit\n");
-    MESSAGE( "       wine --version                Output version information and exit\n");
-    ExitProcess(0);
-}
-
-
 /***********************************************************************
  *           init_current_directory
  *
@@ -852,6 +839,68 @@ static void start_process( void *arg )
 }
 
 
+/***********************************************************************
+ *           set_process_name
+ *
+ * Change the process name in the ps output.
+ */
+static void set_process_name( int *argc, char *argv[], char *name )
+{
+#ifdef HAVE_PRCTL
+    int i, offset;
+    char *prctl_name = NULL;
+    char *end = argv[*argc-1] + strlen(argv[*argc-1]) + 1;
+
+#ifndef PR_SET_NAME
+# define PR_SET_NAME 15
+#endif
+
+    if (!name)
+    {
+        char *p;
+        prctl_name = argv[1];
+        if ((p = strrchr( prctl_name, '\\' ))) prctl_name = p + 1;
+        if ((p = strrchr( prctl_name, '/' ))) prctl_name = p + 1;
+    }
+    else
+    {
+        if (strlen(name) <= strlen(argv[0])) prctl_name = name;
+    }
+
+    if (prctl_name && prctl( PR_SET_NAME, prctl_name ) != -1)
+    {
+        if (name)
+        {
+            strcpy( argv[0], name );
+            offset = argv[1] - (argv[0] + strlen(name) + 1);
+            memmove( argv[1] - offset, argv[1], end - argv[1] );
+            memset( end - offset, 0, offset );
+            for (i = 1; i < *argc; i++) argv[i] -= offset;
+        }
+        else
+        {
+            offset = argv[1] - argv[0];
+            memmove( argv[1] - offset, argv[1], end - argv[1] );
+            memset( end - offset, 0, offset );
+            for (i = 1; i < *argc; i++) argv[i-1] = argv[i] - offset;
+            argv[i-1] = NULL;
+            (*argc)--;
+        }
+    }
+    else
+#endif  /* HAVE_PRCTL */
+    {
+        if (name) argv[0] = name;
+        else
+        {
+            /* remove argv[0] */
+            memmove( argv, argv + 1, *argc * sizeof(argv[0]) );
+            (*argc)--;
+        }
+    }
+}
+
+
 /***********************************************************************
  *           __wine_kernel_init
  *
@@ -861,18 +910,17 @@ void __wine_kernel_init(void)
 {
     static const WCHAR dotW[] = {'.',0};
     static const WCHAR exeW[] = {'.','e','x','e',0};
+    static char winevdm[] = "winevdm.exe";
 
     WCHAR *p, main_exe_name[MAX_PATH];
     HMODULE module;
     DWORD type, error = 0;
     PEB *peb = NtCurrentTeb()->Peb;
+    char *new_argv0 = NULL;
 
     /* Initialize everything */
     if (!process_init()) exit(1);
 
-    __wine_main_argv++;  /* remove argv[0] (wine itself) */
-    __wine_main_argc--;
-
     if (peb->ProcessParameters->ImagePathName.Buffer)
     {
         strcpyW( main_exe_name, peb->ProcessParameters->ImagePathName.Buffer );
@@ -881,18 +929,11 @@ void __wine_kernel_init(void)
     {
         WCHAR exe_nameW[MAX_PATH];
 
-        if (!__wine_main_argv[0]) usage();
-        if (__wine_main_argc == 1)
-        {
-            if (strcmp(__wine_main_argv[0], "--help") == 0) usage();
-            if (strcmp(__wine_main_argv[0], "--version") == 0) version();
-        }
-
-        MultiByteToWideChar( CP_UNIXCP, 0, __wine_main_argv[0], -1, exe_nameW, MAX_PATH );
+        MultiByteToWideChar( CP_UNIXCP, 0, __wine_main_argv[1], -1, exe_nameW, MAX_PATH );
         if (!SearchPathW( NULL, exe_nameW, exeW, MAX_PATH, main_exe_name, NULL ) &&
             !get_builtin_path( exe_nameW, exeW, main_exe_name, MAX_PATH ))
         {
-            MESSAGE( "wine: cannot find '%s'\n", __wine_main_argv[0] );
+            MESSAGE( "wine: cannot find '%s'\n", __wine_main_argv[1] );
             ExitProcess( GetLastError() );
         }
     }
@@ -902,7 +943,7 @@ void __wine_kernel_init(void)
     if (!p || strchrW( p, '/' ) || strchrW( p, '\\' )) strcatW( main_exe_name, dotW );
 
     TRACE( "starting process name=%s argv[0]=%s\n",
-           debugstr_w(main_exe_name), debugstr_a(__wine_main_argv[0]) );
+           debugstr_w(main_exe_name), debugstr_a(__wine_main_argv[1]) );
 
     RtlInitUnicodeString( &NtCurrentTeb()->Peb->ProcessParameters->DllPath,
                           MODULE_get_dll_load_path(main_exe_name) );
@@ -916,9 +957,7 @@ void __wine_kernel_init(void)
             if (type == SCS_WOW_BINARY || type == SCS_DOS_BINARY ||
                 type == SCS_OS216_BINARY || type == SCS_PIF_BINARY)
             {
-                __wine_main_argv--;
-                __wine_main_argc++;
-                __wine_main_argv[0] = "winevdm.exe";
+                new_argv0 = winevdm;
                 module = LoadLibraryExW( winevdmW, 0, DONT_RESOLVE_DLL_REFERENCES );
             }
         }
@@ -932,6 +971,8 @@ void __wine_kernel_init(void)
         ExitProcess( error );
     }
 
+    set_process_name( &__wine_main_argc, __wine_main_argv, new_argv0 );
+
     peb->ImageBaseAddress = module;
 
     /* build command line */
@@ -1887,6 +1928,16 @@ HINSTANCE WINAPI LoadModule( LPCSTR name, LPVOID paramBlock )
 
 /******************************************************************************
  *           TerminateProcess   (KERNEL32.@)
+ *
+ * Terminates a process.
+ *
+ * PARAMS
+ *  handle    [I] Process to terminate.
+ *  exit_code [I] Exit code.
+ *
+ * RETURNS
+ *  Success: TRUE.
+ *  Failure: FALSE, check GetLastError().
  */
 BOOL WINAPI TerminateProcess( HANDLE handle, DWORD exit_code )
 {
@@ -1898,6 +1949,14 @@ BOOL WINAPI TerminateProcess( HANDLE handle, DWORD exit_code )
 
 /***********************************************************************
  *           ExitProcess   (KERNEL32.@)
+ *
+ * Exits the current process.
+ *
+ * PARAMS
+ *  status [I] Status code to exit with.
+ *
+ * RETURNS
+ *  Nothing.
  */
 void WINAPI ExitProcess( DWORD status )
 {
@@ -2247,6 +2306,17 @@ void WINAPI ExitProcess16( WORD status )
 
 /*********************************************************************
  *           OpenProcess   (KERNEL32.@)
+ *
+ * Opens a handle to a process.
+ *
+ * PARAMS
+ *  access  [I] Desired access rights assigned to the returned handle.
+ *  inherit [I] Determines whether or not child processes will inherit the handle.
+ *  id      [I] Process identifier of the process to get a handle to.
+ *
+ * RETURNS
+ *  Success: Valid handle to the specified process.
+ *  Failure: NULL, check GetLastError().
  */
 HANDLE WINAPI OpenProcess( DWORD access, BOOL inherit, DWORD id )
 {
@@ -2265,6 +2335,8 @@ HANDLE WINAPI OpenProcess( DWORD access, BOOL inherit, DWORD id )
     attr.SecurityQualityOfService = NULL;
     attr.ObjectName = NULL;
 
+    if (GetVersion() & 0x80000000) access = PROCESS_ALL_ACCESS;
+
     status = NtOpenProcess(&handle, access, &attr, &cid);
     if (status != STATUS_SUCCESS)
     {
@@ -2278,6 +2350,20 @@ HANDLE WINAPI OpenProcess( DWORD access, BOOL inherit, DWORD id )
 /*********************************************************************
  *           MapProcessHandle   (KERNEL.483)
  *           GetProcessId       (KERNEL32.@)
+ *
+ * Gets the a unique identifier of a process.
+ *
+ * PARAMS
+ *  hProcess [I] Handle to the process.
+ *
+ * RETURNS
+ *  Success: TRUE.
+ *  Failure: FALSE, check GetLastError().
+ *
+ * NOTES
+ *
+ * The identifier is unique only on the machine and only until the process
+ * exits (including system shutdown).
  */
 DWORD WINAPI GetProcessId( HANDLE hProcess )
 {
@@ -2295,6 +2381,15 @@ DWORD WINAPI GetProcessId( HANDLE hProcess )
 /*********************************************************************
  *           CloseW32Handle (KERNEL.474)
  *           CloseHandle    (KERNEL32.@)
+ *
+ * Closes a handle.
+ *
+ * PARAMS
+ *  handle [I] Handle to close.
+ *
+ * RETURNS
+ *  Success: TRUE.
+ *  Failure: FALSE, check GetLastError().
  */
 BOOL WINAPI CloseHandle( HANDLE handle )
 {
@@ -2687,7 +2782,6 @@ BOOL WINAPI WriteProcessMemory( HANDLE process, LPVOID addr, LPCVOID buffer, SIZ
 BOOL WINAPI FlushInstructionCache(HANDLE hProcess, LPCVOID lpBaseAddress, SIZE_T dwSize)
 {
     NTSTATUS status;
-    if (GetVersion() & 0x80000000) return TRUE; /* not NT, always TRUE */
     status = NtFlushInstructionCache( hProcess, lpBaseAddress, dwSize );
     if (status) SetLastError( RtlNtStatusToDosError(status) );
     return !status;
@@ -2755,9 +2849,14 @@ HANDLE WINAPI GetCurrentProcess(void)
 /***********************************************************************
  *           CmdBatNotification   (KERNEL32.@)
  *
- *  Called by cmd.exe with
- *    (1) when a batch file is started 
- *    (0) when a batch file finishes executing
+ * Notifies the system that a batch file has started or finished.
+ *
+ * PARAMS
+ *  bBatchRunning [I]  TRUE if a batch file has started or 
+ *                     FALSE if a batch file has finished executing.
+ *
+ * RETURNS
+ *  Unknown.
  */
 BOOL WINAPI CmdBatNotification( BOOL bBatchRunning )
 {
