@@ -45,6 +45,7 @@
 
 typedef struct BindStatusCallback BindStatusCallback;
 typedef struct HTMLDOMNode HTMLDOMNode;
+typedef struct ConnectionPoint ConnectionPoint;
 
 typedef struct {
     const IHTMLDocument2Vtbl              *lpHTMLDocument2Vtbl;
@@ -62,6 +63,7 @@ typedef struct {
     const IOleCommandTargetVtbl           *lpOleCommandTargetVtbl;
     const IOleControlVtbl                 *lpOleControlVtbl;
     const IHlinkTargetVtbl                *lpHlinkTargetVtbl;
+    const IConnectionPointContainerVtbl   *lpConnectionPointContainerVtbl;
 
     LONG ref;
 
@@ -73,6 +75,7 @@ typedef struct {
     IOleInPlaceFrame *frame;
 
     HWND hwnd;
+    HWND tooltips_hwnd;
 
     BOOL in_place_active;
     BOOL ui_active;
@@ -82,15 +85,21 @@ typedef struct {
 
     BindStatusCallback *status_callback;
 
+    ConnectionPoint *cp_htmldocevents;
+    ConnectionPoint *cp_htmldocevents2;
+
     HTMLDOMNode *nodes;
 } HTMLDocument;
 
 struct NSContainer {
-    const nsIWebBrowserChromeVtbl     *lpWebBrowserChromeVtbl;
-    const nsIContextMenuListenerVtbl  *lpContextMenuListenerVtbl;
-    const nsIURIContentListenerVtbl   *lpURIContentListenerVtbl;
-    const nsIEmbeddingSiteWindowVtbl  *lpEmbeddingSiteWindowVtbl;
-    const nsIInterfaceRequestorVtbl   *lpInterfaceRequestorVtbl;
+    const nsIWebBrowserChromeVtbl       *lpWebBrowserChromeVtbl;
+    const nsIContextMenuListenerVtbl    *lpContextMenuListenerVtbl;
+    const nsIURIContentListenerVtbl     *lpURIContentListenerVtbl;
+    const nsIEmbeddingSiteWindowVtbl    *lpEmbeddingSiteWindowVtbl;
+    const nsITooltipListenerVtbl        *lpTooltipListenerVtbl;
+    const nsIInterfaceRequestorVtbl     *lpInterfaceRequestorVtbl;
+    const nsIWeakReferenceVtbl          *lpWeakReferenceVtbl;
+    const nsISupportsWeakReferenceVtbl  *lpSupportsWeakReferenceVtbl;
 
     nsIWebBrowser *webbrowser;
     nsIWebNavigation *navigation;
@@ -131,6 +140,7 @@ struct HTMLDOMNode {
 
 typedef struct {
     const IHTMLElementVtbl *lpHTMLElementVtbl;
+    const IHTMLElement2Vtbl *lpHTMLElement2Vtbl;
 
     void (*destructor)(IUnknown*);
 
@@ -140,6 +150,12 @@ typedef struct {
     IUnknown *impl;
 } HTMLElement;
 
+typedef struct {
+    const IHTMLTextContainerVtbl *lpHTMLTextContainerVtbl;
+
+    IUnknown *impl;
+} HTMLTextContainer;
+
 #define HTMLDOC(x)       ((IHTMLDocument2*)               &(x)->lpHTMLDocument2Vtbl)
 #define HTMLDOC3(x)      ((IHTMLDocument3*)               &(x)->lpHTMLDocument3Vtbl)
 #define PERSIST(x)       ((IPersist*)                     &(x)->lpPersistFileVtbl)
@@ -160,16 +176,23 @@ typedef struct {
 #define CONTROL(x)       ((IOleControl*)                  &(x)->lpOleControlVtbl)
 #define STATUSCLB(x)     ((IBindStatusCallback*)          &(x)->lpBindStatusCallbackVtbl)
 #define HLNKTARGET(x)    ((IHlinkTarget*)                 &(x)->lpHlinkTargetVtbl)
+#define CONPTCONT(x)     ((IConnectionPointContainer*)    &(x)->lpConnectionPointContainerVtbl)
 
 #define NSWBCHROME(x)    ((nsIWebBrowserChrome*)          &(x)->lpWebBrowserChromeVtbl)
 #define NSCML(x)         ((nsIContextMenuListener*)       &(x)->lpContextMenuListenerVtbl)
 #define NSURICL(x)       ((nsIURIContentListener*)        &(x)->lpURIContentListenerVtbl)
 #define NSEMBWNDS(x)     ((nsIEmbeddingSiteWindow*)       &(x)->lpEmbeddingSiteWindowVtbl)
 #define NSIFACEREQ(x)    ((nsIInterfaceRequestor*)        &(x)->lpInterfaceRequestorVtbl)
+#define NSTOOLTIP(x)     ((nsITooltipListener*)           &(x)->lpTooltipListenerVtbl)
+#define NSWEAKREF(x)     ((nsIWeakReference*)             &(x)->lpWeakReferenceVtbl)
+#define NSSUPWEAKREF(x)  ((nsISupportsWeakReference*)     &(x)->lpSupportsWeakReferenceVtbl)
 
 #define HTMLELEM(x)      ((IHTMLElement*)                 &(x)->lpHTMLElementVtbl)
+#define HTMLELEM2(x)     ((IHTMLElement2*)                &(x)->lpHTMLElement2Vtbl)
 #define HTMLDOMNODE(x)   ((IHTMLDOMNode*)                 &(x)->lpHTMLDOMNodeVtbl)
 
+#define HTMLTEXTCONT(x)  ((IHTMLTextContainer*)           &(x)->lpHTMLTextContainerVtbl)
+
 #define DEFINE_THIS(cls,ifc,iface) ((cls*)((BYTE*)(iface)-offsetof(cls,lp ## ifc ## Vtbl)))
 
 HRESULT HTMLDocument_Create(IUnknown*,REFIID,void**);
@@ -181,6 +204,7 @@ void HTMLDocument_View_Init(HTMLDocument*);
 void HTMLDocument_Window_Init(HTMLDocument*);
 void HTMLDocument_Service_Init(HTMLDocument*);
 void HTMLDocument_Hlink_Init(HTMLDocument*);
+void HTMLDocument_ConnectionPoints_Init(HTMLDocument*);
 
 NSContainer *NSContainer_Create(HTMLDocument*,NSContainer*);
 void NSContainer_Release(NSContainer*);
@@ -188,6 +212,9 @@ void NSContainer_Release(NSContainer*);
 void HTMLDocument_LockContainer(HTMLDocument*,BOOL);
 void HTMLDocument_ShowContextMenu(HTMLDocument*,DWORD,POINT*);
 
+void show_tooltip(HTMLDocument*,DWORD,DWORD,LPCWSTR);
+void hide_tooltip(HTMLDocument*);
+
 HRESULT ProtocolFactory_Create(REFCLSID,REFIID,void**);
 
 void close_gecko(void);
@@ -198,10 +225,12 @@ void hlink_frame_navigate(HTMLDocument*,IHlinkFrame*,LPCWSTR,nsIInputStream*,DWO
 
 nsIURI *get_nsIURI(LPCWSTR);
 
-nsACString *nsACString_Create(void);
+void *nsalloc(size_t);
+void nsfree(void*);
+
+void nsACString_Init(nsACString*,const char*);
 PRUint32 nsACString_GetData(const nsACString*,const char**,PRBool*);
-void nsACString_SetData(nsACString*,const char*);
-void nsACString_Destroy(nsACString*);
+void nsACString_Finish(nsACString*);
 
 void nsAString_Init(nsAString*,const PRUnichar*);
 PRUint32 nsAString_GetData(const nsAString*,const PRUnichar**,PRBool*);
@@ -217,6 +246,13 @@ void HTMLInputElement_Create(HTMLElement*);
 void HTMLSelectElement_Create(HTMLElement*);
 void HTMLTextAreaElement_Create(HTMLElement*);
 
+void HTMLElement2_Init(HTMLElement*);
+
+void HTMLTextContainer_Init(HTMLTextContainer*,IUnknown*);
+
+HRESULT HTMLDOMNode_QI(HTMLDOMNode*,REFIID,void**);
+HRESULT HTMLElement_QI(HTMLElement*,REFIID,void**);
+
 HTMLDOMNode *get_node(HTMLDocument*,nsIDOMNode*);
 void release_nodes(HTMLDocument*);
 
