@@ -32,6 +32,7 @@
 #include "shlguid.h"
 
 #include "wine/debug.h"
+#include "wine/unicode.h"
 
 #include "mshtml_private.h"
 
@@ -114,18 +115,18 @@ static HRESULT WINAPI BindStatusCallback_OnStartBinding(IBindStatusCallback *ifa
     IBinding_AddRef(pbind);
 
     if(This->doc->nscontainer && This->doc->nscontainer->stream) {
-        nsACString *strTextHtml;
+        nsACString strTextHtml;
         nsresult nsres;
         nsIURI *uri = get_nsIURI(This->url);
 
-        strTextHtml = nsACString_Create();
         /* FIXME: Set it correctly */
-        nsACString_SetData(strTextHtml, "text/html");
+        nsACString_Init(&strTextHtml, "text/html");
 
-        nsres = nsIWebBrowserStream_OpenStream(This->doc->nscontainer->stream, uri, strTextHtml);
+        nsres = nsIWebBrowserStream_OpenStream(This->doc->nscontainer->stream, uri, &strTextHtml);
         if(NS_FAILED(nsres))
             ERR("OpenStream failed: %08lx\n", nsres);
 
+        nsACString_Finish(&strTextHtml);
         nsIURI_Release(uri);
     }
 
@@ -221,6 +222,8 @@ static HRESULT WINAPI BindStatusCallback_OnObjectAvailable(IBindStatusCallback *
     return E_NOTIMPL;
 }
 
+#undef STATUSCLB_THIS
+
 static const IBindStatusCallbackVtbl BindStatusCallbackVtbl = {
     BindStatusCallback_QueryInterface,
     BindStatusCallback_AddRef,
@@ -238,12 +241,14 @@ static const IBindStatusCallbackVtbl BindStatusCallbackVtbl = {
 static BindStatusCallback *BindStatusCallback_Create(HTMLDocument *doc, LPOLESTR url)
 {
     BindStatusCallback *ret = HeapAlloc(GetProcessHeap(), 0, sizeof(BindStatusCallback));
+
     ret->lpBindStatusCallbackVtbl = &BindStatusCallbackVtbl;
     ret->ref = 0;
     ret->url = url;
     ret->doc = doc;
     ret->stream = NULL;
     IHTMLDocument2_AddRef(HTMLDOC(doc));
+
     return ret;
 }
 
@@ -285,6 +290,80 @@ static HRESULT WINAPI PersistMoniker_IsDirty(IPersistMoniker *iface)
     return E_NOTIMPL;
 }
 
+static nsIInputStream *get_post_data_stream(IBindCtx *bctx)
+{
+    nsIInputStream *ret = NULL;
+    IBindStatusCallback *callback;
+    IHttpNegotiate *http_negotiate;
+    BINDINFO bindinfo;
+    DWORD bindf = 0;
+    DWORD post_len = 0, headers_len = 0;
+    LPWSTR headers = NULL;
+    WCHAR emptystr[] = {0};
+    char *data;
+    HRESULT hres;
+
+    static WCHAR _BSCB_Holder_[] =
+        {'_','B','S','C','B','_','H','o','l','d','e','r','_',0};
+
+
+    /* FIXME: This should be done in URLMoniker */
+    if(!bctx)
+        return NULL;
+
+    hres = IBindCtx_GetObjectParam(bctx, _BSCB_Holder_, (IUnknown**)&callback);
+    if(FAILED(hres))
+        return NULL;
+
+    hres = IBindStatusCallback_QueryInterface(callback, &IID_IHttpNegotiate,
+                                              (void**)&http_negotiate);
+    if(SUCCEEDED(hres)) {
+        hres = IHttpNegotiate_BeginningTransaction(http_negotiate, emptystr,
+                                                   emptystr, 0, &headers);
+        IHttpNegotiate_Release(http_negotiate);
+
+        if(SUCCEEDED(hres) && headers)
+            headers_len = WideCharToMultiByte(CP_ACP, 0, headers, -1, NULL, 0, NULL, NULL);
+    }
+
+    memset(&bindinfo, 0, sizeof(bindinfo));
+    bindinfo.cbSize = sizeof(bindinfo);
+
+    hres = IBindStatusCallback_GetBindInfo(callback, &bindf, &bindinfo);
+
+    if(SUCCEEDED(hres) && bindinfo.dwBindVerb == BINDVERB_POST)
+        post_len = bindinfo.cbStgmedData;
+
+    if(headers_len || post_len) {
+        int len = headers_len ? headers_len-1 : 0;
+
+        static const char content_length[] = "Content-Length: %lu\r\n\r\n";
+
+        data = HeapAlloc(GetProcessHeap(), 0, headers_len+post_len+sizeof(content_length)+8);
+
+        if(headers_len) {
+            WideCharToMultiByte(CP_ACP, 0, headers, -1, data, -1, NULL, NULL);
+            CoTaskMemFree(headers);
+        }
+
+        if(post_len) {
+            sprintf(data+len, content_length, post_len);
+            len = strlen(data);
+
+            memcpy(data+len, bindinfo.stgmedData.u.hGlobal, post_len);
+        }
+
+        TRACE("data = %s\n", debugstr_an(data, len+post_len));
+
+        ret = create_nsstream(data, len+post_len);
+    }
+
+    ReleaseBindInfo(&bindinfo);
+    IBindStatusCallback_Release(callback);
+
+    return ret;
+}
+
 static HRESULT WINAPI PersistMoniker_Load(IPersistMoniker *iface, BOOL fFullyAvailable,
         IMoniker *pimkName, LPBC pibc, DWORD grfMode)
 {
@@ -301,10 +380,6 @@ static HRESULT WINAPI PersistMoniker_Load(IPersistMoniker *iface, BOOL fFullyAva
     if(pibc) {
         IUnknown *unk = NULL;
 
-        static WCHAR wszClientSiteParam[] = {'{','d','4','d','b','6','8','5','0','-',
-            '5','3','8','5','-','1','1','d','0','-','8','9','e','9','-','0','0','a',
-            '0','c','9','0','a','9','0','a','c','}',0};
-
         /* FIXME:
          * Use params:
          * "__PrecreatedObject"
@@ -317,7 +392,7 @@ static HRESULT WINAPI PersistMoniker_Load(IPersistMoniker *iface, BOOL fFullyAva
          * "_EnumFORMATETC_"
          */
 
-        IBindCtx_GetObjectParam(pibc, wszClientSiteParam, &unk);
+        IBindCtx_GetObjectParam(pibc, (LPOLESTR)SZ_HTML_CLIENTSITE_OBJECTPARAM, &unk);
         if(unk) {
             IOleClientSite *client = NULL;
 
@@ -362,17 +437,22 @@ static HRESULT WINAPI PersistMoniker_Load(IPersistMoniker *iface, BOOL fFullyAva
          * It uses Gecko's LoadURI instead of IMoniker's BindToStorage. Should we improve
          * it (to do so we'd have to use not frozen interfaces)?
          */
-        LPOLESTR old_url = This->nscontainer->url;
 
-        This->nscontainer->url = url;
+        nsIInputStream *post_data_stream = get_post_data_stream(pibc);
+
+        This->nscontainer->load_call = TRUE;
         nsres = nsIWebNavigation_LoadURI(This->nscontainer->navigation, url,
-                LOAD_FLAGS_NONE, NULL, NULL, NULL);
+                LOAD_FLAGS_NONE, NULL, post_data_stream, NULL);
+        This->nscontainer->load_call = FALSE;
+
+        if(post_data_stream)
+            nsIInputStream_Release(post_data_stream);
+
         if(NS_SUCCEEDED(nsres)) {
-            CoTaskMemFree(old_url);
+            CoTaskMemFree(url);
             return S_OK;
         }else {
             WARN("LoadURI failed: %08lx\n", nsres);
-            This->nscontainer->url = old_url;
         }
     }    
 
@@ -563,34 +643,6 @@ static const IPersistFileVtbl PersistFileVtbl = {
     PersistFile_GetCurFile
 };
 
-BOOL HTMLDocument_OnLoad(HTMLDocument *This, LPCWSTR url)
-{
-    IOleCommandTarget *cmdtrg = NULL;
-    HRESULT hres;
-
-    TRACE("(%p)->(%s)\n", This, debugstr_w(url));
-
-    hres = IOleClientSite_QueryInterface(This->client, &IID_IOleCommandTarget, (void**)&cmdtrg);
-    if(SUCCEEDED(hres)) {
-        VARIANT varUrl, varRes;
-
-        V_VT(&varUrl) = VT_BSTR;
-        V_BSTR(&varUrl) = SysAllocString(url);
-        V_VT(&varRes) = VT_BOOL;
-
-        hres = IOleCommandTarget_Exec(cmdtrg, &CGID_ShellDocView, 67, 0, &varUrl, &varRes);
-        SysFreeString(V_BSTR(&varUrl));
-
-        if(SUCCEEDED(hres) && !V_BOOL(&varRes)) {
-            TRACE("got VARIANT_FALSE, do not load\n");
-            IOleCommandTarget_Release(cmdtrg);
-            return FALSE;
-        }
-    }
-
-    return TRUE;
-}
-
 void HTMLDocument_Persist_Init(HTMLDocument *This)
 {
     This->lpPersistMonikerVtbl = &PersistMonikerVtbl;
