@@ -118,15 +118,15 @@ static UINT ACTION_AppSearchGetSignature(MSIPACKAGE *package, MSISIGNATURE *sig,
         }
 
         /* get properties */
-        sig->File = load_dynamic_stringW(row,2);
-        minVersion = load_dynamic_stringW(row,3);
+        sig->File = msi_dup_record_field(row,2);
+        minVersion = msi_dup_record_field(row,3);
         if (minVersion)
         {
             ACTION_VerStrToInteger(minVersion, &sig->MinVersionMS,
              &sig->MinVersionLS);
             msi_free( minVersion);
         }
-        maxVersion = load_dynamic_stringW(row,4);
+        maxVersion = msi_dup_record_field(row,4);
         if (maxVersion)
         {
             ACTION_VerStrToInteger(maxVersion, &sig->MaxVersionMS,
@@ -139,7 +139,7 @@ static UINT ACTION_AppSearchGetSignature(MSIPACKAGE *package, MSISIGNATURE *sig,
         sig->MaxSize = MSI_RecordGetInteger(row,6);
         if (sig->MaxSize == MSI_NULL_INTEGER)
             sig->MaxSize = 0;
-        sig->Languages = load_dynamic_stringW(row,9);
+        sig->Languages = msi_dup_record_field(row,9);
         time = MSI_RecordGetInteger(row,7);
         if (time != MSI_NULL_INTEGER)
             DosDateTimeToFileTime(HIWORD(time), LOWORD(time), &sig->MinTime);
@@ -158,7 +158,8 @@ static UINT ACTION_AppSearchGetSignature(MSIPACKAGE *package, MSISIGNATURE *sig,
         TRACE("Languages is %s\n", debugstr_w(sig->Languages));
 
 end:
-        msiobj_release(&row->hdr);
+        if (row)
+            msiobj_release(&row->hdr);
         MSI_ViewClose(view);
         msiobj_release(&view->hdr);
     }
@@ -220,7 +221,8 @@ static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, BOOL *appFound,
          debugstr_w(guid));
 
 end:
-        msiobj_release(&row->hdr);
+        if (row)
+            msiobj_release(&row->hdr);
         MSI_ViewClose(view);
         msiobj_release(&view->hdr);
     }
@@ -276,9 +278,9 @@ static UINT ACTION_AppSearchReg(MSIPACKAGE *package, BOOL *appFound,
         }
 
         root = MSI_RecordGetInteger(row,2);
-        keyPath = load_dynamic_stringW(row,3);
+        keyPath = msi_dup_record_field(row,3);
         /* FIXME: keyPath needs to be expanded for properties */
-        valueName = load_dynamic_stringW(row,4);
+        valueName = msi_dup_record_field(row,4);
         /* FIXME: valueName probably does too */
         type = MSI_RecordGetInteger(row,5);
 
@@ -393,7 +395,8 @@ end:
         msi_free( keyPath);
         msi_free( valueName);
 
-        msiobj_release(&row->hdr);
+        if (row)
+            msiobj_release(&row->hdr);
         MSI_ViewClose(view);
         msiobj_release(&view->hdr);
     }
@@ -442,13 +445,14 @@ static UINT ACTION_AppSearchIni(MSIPACKAGE *package, BOOL *appFound,
         }
 
         /* get file name */
-        fileName = load_dynamic_stringW(row,2);
+        fileName = msi_dup_record_field(row,2);
         FIXME("AppSearch unimplemented for IniLocator (ini file name %s)\n",
          debugstr_w(fileName));
         msi_free( fileName);
 
 end:
-        msiobj_release(&row->hdr);
+        if (row)
+            msiobj_release(&row->hdr);
         MSI_ViewClose(view);
         msiobj_release(&view->hdr);
     }
@@ -549,7 +553,7 @@ static UINT ACTION_FileVersionMatches(MSISIGNATURE *sig, LPCWSTR filePath,
 
             if (buf)
             {
-                static const WCHAR rootW[] = { '\\',0 };
+                static WCHAR rootW[] = { '\\',0 };
                 UINT versionLen;
                 LPVOID subBlock = NULL;
 
@@ -872,7 +876,8 @@ static UINT ACTION_AppSearchDr(MSIPACKAGE *package, MSISIGNATURE *sig)
         rc = ACTION_SearchDirectory(package, sig, expanded, depth);
 
 end:
-        msiobj_release(&row->hdr);
+        if (row)
+            msiobj_release(&row->hdr);
         MSI_ViewClose(view);
         msiobj_release(&view->hdr);
     }
