@@ -33,7 +33,7 @@
 
 #include "query.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(msi);
+WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 
 
 /*
@@ -182,6 +182,14 @@ static UINT DELETE_delete( struct tagMSIVIEW *view )
     return ERROR_SUCCESS;
 }
 
+static UINT DELETE_find_matching_rows( struct tagMSIVIEW *view, UINT col,
+    UINT val, UINT *row, MSIITERHANDLE *handle )
+{
+    TRACE("%p, %d, %u, %p\n", view, col, val, *handle);
+
+    return ERROR_FUNCTION_FAILED;
+}
+
 
 MSIVIEWOPS delete_ops =
 {
@@ -194,7 +202,8 @@ MSIVIEWOPS delete_ops =
     DELETE_get_dimensions,
     DELETE_get_column_info,
     DELETE_modify,
-    DELETE_delete
+    DELETE_delete,
+    DELETE_find_matching_rows
 };
 
 UINT DELETE_CreateView( MSIDATABASE *db, MSIVIEW **view, MSIVIEW *table )
