@@ -214,6 +214,14 @@ static UINT INSERT_delete( struct tagMSIVIEW *view )
     return ERROR_SUCCESS;
 }
 
+static UINT INSERT_find_matching_rows( struct tagMSIVIEW *view, UINT col,
+    UINT val, UINT *row, MSIITERHANDLE *handle )
+{
+    TRACE("%p, %d, %u, %p\n", view, col, val, *handle);
+
+    return ERROR_FUNCTION_FAILED;
+}
+
 
 MSIVIEWOPS insert_ops =
 {
@@ -226,7 +234,8 @@ MSIVIEWOPS insert_ops =
     INSERT_get_dimensions,
     INSERT_get_column_info,
     INSERT_modify,
-    INSERT_delete
+    INSERT_delete,
+    INSERT_find_matching_rows
 };
 
 UINT INSERT_CreateView( MSIDATABASE *db, MSIVIEW **view, LPWSTR table,
