@@ -161,7 +161,7 @@ static void und_free_all(struct parsed_symbol* sym)
     while (sym->alloc_list)
     {
         next = *(void**)sym->alloc_list;
-        sym->mem_free_ptr(sym->alloc_list);
+        if(sym->mem_free_ptr) sym->mem_free_ptr(sym->alloc_list);
         sym->alloc_list = next;
     }
     sym->avail_in_first = 0;
@@ -1233,8 +1233,7 @@ char* __unDNameEx(char* buffer, const char* mangled, int buflen,
     result = symbol_demangle(&sym) ? sym.result : mangled;
     if (buffer && buflen)
     {
-        memcpy(buffer, result, buflen - 1);
-        buffer[buflen - 1] = '\0';
+        lstrcpynA( buffer, result, buflen);
     }
     else
     {
