@@ -58,6 +58,7 @@ static const WCHAR szComplete4[] = {
                     'p','n','=','\'','w','i','n','e',' ','2','0','0','5','0','8','0','4','\'','>','\n',
             'f','n','2','.','t','x','t','\n',
         '<','/','p','r','>','\n',
+        '<','e','m','p','t','y','>','<','/','e','m','p','t','y','>','\n',
     '<','/','l','c','>','\n',0
 };
 static const WCHAR szNonExistentFile[] = {
@@ -69,10 +70,13 @@ static const WCHAR szDocument[] = {
 
 static const WCHAR szOpen[] = { 'o','p','e','n',0 };
 static const WCHAR szdl[] = { 'd','l',0 };
+static const WCHAR szvr[] = { 'v','r',0 };
 static const WCHAR szlc[] = { 'l','c',0 };
 static const WCHAR szbs[] = { 'b','s',0 };
 static const WCHAR szstr1[] = { 's','t','r','1',0 };
 static const WCHAR szstr2[] = { 's','t','r','2',0 };
+static const WCHAR szstar[] = { '*',0 };
+static const WCHAR szfn1_txt[] = {'f','n','1','.','t','x','t',0};
 
 void test_domdoc( void )
 {
@@ -347,6 +351,17 @@ void test_domnode( void )
 
     if (map)
     {
+        ISupportErrorInfo *support_error;
+        r = IXMLDOMNamedNodeMap_QueryInterface( map, &IID_ISupportErrorInfo, (LPVOID*)&support_error );
+        ok( r == S_OK, "ret %08lx\n", r );
+
+        r = ISupportErrorInfo_InterfaceSupportsErrorInfo( support_error, &IID_IXMLDOMNamedNodeMap );
+todo_wine
+{
+        ok( r == S_OK, "ret %08lx\n", r );
+}
+        ISupportErrorInfo_Release( support_error );
+
         str = SysAllocString( szdl );
         r = IXMLDOMNamedNodeMap_getNamedItem( map, str, &node );
         ok( r == S_OK, "getNamedItem returned wrong code\n");
@@ -480,7 +495,7 @@ void test_domnode( void )
     {
         r = IXMLDOMNodeList_get_length( list, &count );
         ok( r == S_OK, "get_length returns %08lx\n", r );
-        ok( count == 2, "get_length got %ld\n", count );
+        ok( count == 3, "get_length got %ld\n", count );
 
         r = IXMLDOMNodeList_nextNode( list, &node );
         ok( r == S_OK, "nextNode returned wrong code\n");
@@ -764,7 +779,184 @@ static void test_create(void)
     IXMLDOMDocument_Release( doc );
 }
 
+static void test_getElementsByTagName(void)
+{
+    HRESULT r;
+    BSTR str;
+    VARIANT_BOOL b;
+    IXMLDOMDocument *doc;
+    IXMLDOMNodeList *node_list;
+    LONG len;
 
+    r = CoCreateInstance( &CLSID_DOMDocument, NULL, 
+        CLSCTX_INPROC_SERVER, &IID_IXMLDOMDocument, (LPVOID*)&doc );
+    if( r != S_OK )
+        return;
+
+    str = SysAllocString( szComplete4 );
+    r = IXMLDOMDocument_loadXML( doc, str, &b );
+    ok( r == S_OK, "loadXML failed\n");
+    ok( b == VARIANT_TRUE, "failed to load XML string\n");
+    SysFreeString( str );
+
+    str = SysAllocString( szstar );
+    r = IXMLDOMDocument_getElementsByTagName(doc, str, &node_list);
+    ok( r == S_OK, "ret %08lx\n", r );
+    r = IXMLDOMNodeList_get_length( node_list, &len );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( len == 4, "len %ld\n", len );
+    IXMLDOMNodeList_Release( node_list );
+    SysFreeString( str );
+
+    str = SysAllocString( szbs );
+    r = IXMLDOMDocument_getElementsByTagName(doc, str, &node_list);
+    ok( r == S_OK, "ret %08lx\n", r );
+    r = IXMLDOMNodeList_get_length( node_list, &len );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( len == 1, "len %ld\n", len );
+    IXMLDOMNodeList_Release( node_list );
+    SysFreeString( str );
+
+    str = SysAllocString( szdl );
+    r = IXMLDOMDocument_getElementsByTagName(doc, str, &node_list);
+    ok( r == S_OK, "ret %08lx\n", r );
+    r = IXMLDOMNodeList_get_length( node_list, &len );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( len == 0, "len %ld\n", len );
+    IXMLDOMNodeList_Release( node_list );
+    SysFreeString( str );
+
+    str = SysAllocString( szstr1 );
+    r = IXMLDOMDocument_getElementsByTagName(doc, str, &node_list);
+    ok( r == S_OK, "ret %08lx\n", r );
+    r = IXMLDOMNodeList_get_length( node_list, &len );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( len == 0, "len %ld\n", len );
+    IXMLDOMNodeList_Release( node_list );
+    SysFreeString( str );
+
+    IXMLDOMDocument_Release( doc );
+}
+
+static void test_get_text(void)
+{
+    HRESULT r;
+    BSTR str;
+    VARIANT_BOOL b;
+    IXMLDOMDocument *doc;
+    IXMLDOMNode *node, *node2, *node3;
+    IXMLDOMNodeList *node_list;
+    IXMLDOMNamedNodeMap *node_map;
+
+    r = CoCreateInstance( &CLSID_DOMDocument, NULL, 
+        CLSCTX_INPROC_SERVER, &IID_IXMLDOMDocument, (LPVOID*)&doc );
+    if( r != S_OK )
+        return;
+
+    str = SysAllocString( szComplete4 );
+    r = IXMLDOMDocument_loadXML( doc, str, &b );
+    ok( r == S_OK, "loadXML failed\n");
+    ok( b == VARIANT_TRUE, "failed to load XML string\n");
+    SysFreeString( str );
+
+    str = SysAllocString( szbs );
+    r = IXMLDOMDocument_getElementsByTagName( doc, str, &node_list );
+    ok( r == S_OK, "ret %08lx\n", r );
+    SysFreeString(str);
+    
+    r = IXMLDOMNodeList_get_item( node_list, 0, &node );
+    ok( r == S_OK, "ret %08lx\n", r ); 
+    IXMLDOMNodeList_Release( node_list );
+
+    r = IXMLDOMNode_get_text( node, &str );
+    ok( r == S_OK, "ret %08lx\n", r );
+todo_wine {
+    ok( !memcmp(str, szfn1_txt, sizeof(szfn1_txt)), "wrong string\n" );
+ }
+    ok( !memcmp(str, szfn1_txt, sizeof(szfn1_txt)-4), "wrong string\n" );
+    SysFreeString(str);
+
+    r = IXMLDOMNode_get_attributes( node, &node_map );
+    ok( r == S_OK, "ret %08lx\n", r );
+    
+    str = SysAllocString( szvr );
+    r = IXMLDOMNamedNodeMap_getNamedItem( node_map, str, &node2 );
+    ok( r == S_OK, "ret %08lx\n", r );
+    SysFreeString(str);
+
+    r = IXMLDOMNode_get_text( node2, &str );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( !memcmp(str, szstr2, sizeof(szstr2)), "wrong string\n" );
+    SysFreeString(str);
+
+    r = IXMLDOMNode_get_firstChild( node2, &node3 );
+    ok( r == S_OK, "ret %08lx\n", r );
+
+    r = IXMLDOMNode_get_text( node3, &str );
+    ok( r == S_OK, "ret %08lx\n", r );
+    ok( !memcmp(str, szstr2, sizeof(szstr2)), "wrong string\n" );
+    SysFreeString(str);
+
+
+    IXMLDOMNode_Release( node3 );
+    IXMLDOMNode_Release( node2 );
+    IXMLDOMNamedNodeMap_Release( node_map );
+    IXMLDOMNode_Release( node );
+    IXMLDOMDocument_Release( doc );
+}
+
+static void test_get_childNodes(void)
+{
+    HRESULT r;
+    BSTR str;
+    VARIANT_BOOL b;
+    IXMLDOMDocument *doc;
+    IXMLDOMElement *element;
+    IXMLDOMNode *node, *node2;
+    IXMLDOMNodeList *node_list, *node_list2;
+    long len;
+
+    r = CoCreateInstance( &CLSID_DOMDocument, NULL, 
+        CLSCTX_INPROC_SERVER, &IID_IXMLDOMDocument, (LPVOID*)&doc );
+    if( r != S_OK )
+        return;
+
+    str = SysAllocString( szComplete4 );
+    r = IXMLDOMDocument_loadXML( doc, str, &b );
+    ok( r == S_OK, "loadXML failed\n");
+    ok( b == VARIANT_TRUE, "failed to load XML string\n");
+    SysFreeString( str );
+
+    r = IXMLDOMDocument_get_documentElement( doc, &element );
+    ok( r == S_OK, "ret %08lx\n", r);
+
+    r = IXMLDOMElement_get_childNodes( element, &node_list );
+    ok( r == S_OK, "ret %08lx\n", r);
+
+    r = IXMLDOMNodeList_get_length( node_list, &len );
+    ok( r == S_OK, "ret %08lx\n", r);
+    ok( len == 3, "len %ld\n", len);
+
+    r = IXMLDOMNodeList_get_item( node_list, 2, &node );
+    ok( r == S_OK, "ret %08lx\n", r);
+
+    r = IXMLDOMNode_get_childNodes( node, &node_list2 );
+    ok( r == S_OK, "ret %08lx\n", r);
+
+    r = IXMLDOMNodeList_get_length( node_list2, &len );
+    ok( r == S_OK, "ret %08lx\n", r);
+    ok( len == 0, "len %ld\n", len);
+
+    r = IXMLDOMNodeList_get_item( node_list2, 0, &node2);
+    ok( r == S_FALSE, "ret %08lx\n", r);
+
+    IXMLDOMNodeList_Release( node_list2 );
+    IXMLDOMNode_Release( node );
+    IXMLDOMNodeList_Release( node_list );
+    IXMLDOMElement_Release( element );
+    IXMLDOMDocument_Release( doc );
+}
+    
 START_TEST(domdoc)
 {
     HRESULT r;
@@ -776,6 +968,9 @@ START_TEST(domdoc)
     test_domnode();
     test_refs();
     test_create();
+    test_getElementsByTagName();
+    test_get_text();
+    test_get_childNodes();
 
     CoUninitialize();
 }
