@@ -27,6 +27,8 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <time.h>
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winternl.h"
 #include "wine/debug.h"
