@@ -27,6 +27,7 @@
 #include <stdarg.h>
 
 #include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "winternl.h"
 #include "wine/exception.h"
@@ -76,7 +77,6 @@ static RTL_CRITICAL_SECTION vectored_handlers_section = { &critsect_debug, -1, 0
 # error You must define GET_IP for this CPU
 #endif
 
-
 /*******************************************************************
  *         EXC_RaiseHandler
  *
@@ -128,10 +128,7 @@ static DWORD EXC_CallHandler( EXCEPTION_RECORD *record, EXCEPTION_REGISTRATION_R
     newframe.frame.Handler = nested_handler;
     newframe.prevFrame     = frame;
     __wine_push_frame( &newframe.frame );
-    TRACE( "calling handler at %p code=%lx flags=%lx\n",
-           handler, record->ExceptionCode, record->ExceptionFlags );
     ret = handler( record, frame, context, dispatcher );
-    TRACE( "handler returned %lx\n", ret );
     __wine_pop_frame( &newframe.frame );
     return ret;
 }
@@ -184,7 +181,7 @@ void wait_suspend( CONTEXT *context )
  *
  * Send an EXCEPTION_DEBUG_EVENT event to the debugger.
  */
-static int send_debug_event( EXCEPTION_RECORD *rec, int first_chance, CONTEXT *context )
+static NTSTATUS send_debug_event( EXCEPTION_RECORD *rec, int first_chance, CONTEXT *context )
 {
     int ret;
     HANDLE handle = 0;
@@ -240,63 +237,15 @@ static LONG call_vectored_handlers( EXCEPTION_RECORD *rec, CONTEXT *context )
 }
 
 
-/*******************************************************************
- *         EXC_DefaultHandling
+/**********************************************************************
+ *           call_stack_handlers
  *
- * Default handling for exceptions. Called when we didn't find a suitable handler.
+ * Call the stack handlers chain.
  */
-static void EXC_DefaultHandling( EXCEPTION_RECORD *rec, CONTEXT *context )
-{
-    if (send_debug_event( rec, FALSE, context ) == DBG_CONTINUE) return;  /* continue execution */
-
-    if (rec->ExceptionFlags & EH_STACK_INVALID)
-        ERR("Exception frame is not in stack limits => unable to dispatch exception.\n");
-    else if (rec->ExceptionCode == STATUS_NONCONTINUABLE_EXCEPTION)
-        ERR("Process attempted to continue execution after noncontinuable exception.\n");
-    else
-        ERR("Unhandled exception code %lx flags %lx addr %p\n",
-            rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
-    NtTerminateProcess( NtCurrentProcess(), 1 );
-}
-
-
-/***********************************************************************
- *		RtlRaiseException (NTDLL.@)
- */
-void WINAPI __regs_RtlRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context )
+static NTSTATUS call_stack_handlers( EXCEPTION_RECORD *rec, CONTEXT *context )
 {
     EXCEPTION_REGISTRATION_RECORD *frame, *dispatch, *nested_frame;
-    EXCEPTION_RECORD newrec;
-    DWORD res, c;
-
-    TRACE( "code=%lx flags=%lx addr=%p\n", rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
-    for (c=0; c<rec->NumberParameters; c++) TRACE(" info[%ld]=%08lx\n", c, rec->ExceptionInformation[c]);
-    if (rec->ExceptionCode == EXCEPTION_WINE_STUB)
-    {
-        if (HIWORD(rec->ExceptionInformation[1]))
-            MESSAGE( "wine: Call from %p to unimplemented function %s.%s, aborting\n",
-                   rec->ExceptionAddress,
-                   (char*)rec->ExceptionInformation[0], (char*)rec->ExceptionInformation[1] );
-        else
-            MESSAGE( "wine: Call from %p to unimplemented function %s.%ld, aborting\n",
-                   rec->ExceptionAddress,
-                   (char*)rec->ExceptionInformation[0], rec->ExceptionInformation[1] );
-    }
-#ifdef __i386__
-    else
-    {
-        TRACE(" eax=%08lx ebx=%08lx ecx=%08lx edx=%08lx esi=%08lx edi=%08lx\n",
-              context->Eax, context->Ebx, context->Ecx,
-              context->Edx, context->Esi, context->Edi );
-        TRACE(" ebp=%08lx esp=%08lx cs=%04lx ds=%04lx es=%04lx fs=%04lx gs=%04lx flags=%08lx\n",
-              context->Ebp, context->Esp, context->SegCs, context->SegDs,
-              context->SegEs, context->SegFs, context->SegGs, context->EFlags );
-    }
-#endif
-
-    if (send_debug_event( rec, TRUE, context ) == DBG_CONTINUE) return;  /* continue execution */
-
-    if (call_vectored_handlers( rec, context ) == EXCEPTION_CONTINUE_EXECUTION) return;
+    DWORD res;
 
     frame = NtCurrentTeb()->Tib.ExceptionList;
     nested_frame = NULL;
@@ -312,7 +261,11 @@ void WINAPI __regs_RtlRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context )
         }
 
         /* Call handler */
+        TRACE( "calling handler at %p code=%lx flags=%lx\n",
+               frame->Handler, rec->ExceptionCode, rec->ExceptionFlags );
         res = EXC_CallHandler( rec, frame, context, &dispatch, frame->Handler, EXC_RaiseHandler );
+        TRACE( "handler at %p returned %lx\n", frame->Handler, res );
+
         if (frame == nested_frame)
         {
             /* no longer nested */
@@ -323,13 +276,8 @@ void WINAPI __regs_RtlRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context )
         switch(res)
         {
         case ExceptionContinueExecution:
-            if (!(rec->ExceptionFlags & EH_NONCONTINUABLE)) return;
-            newrec.ExceptionCode    = STATUS_NONCONTINUABLE_EXCEPTION;
-            newrec.ExceptionFlags   = EH_NONCONTINUABLE;
-            newrec.ExceptionRecord  = rec;
-            newrec.NumberParameters = 0;
-            RtlRaiseException( &newrec );  /* never returns */
-            break;
+            if (!(rec->ExceptionFlags & EH_NONCONTINUABLE)) return STATUS_SUCCESS;
+            return STATUS_NONCONTINUABLE_EXCEPTION;
         case ExceptionContinueSearch:
             break;
         case ExceptionNestedException:
@@ -337,16 +285,107 @@ void WINAPI __regs_RtlRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context )
             rec->ExceptionFlags |= EH_NESTED_CALL;
             break;
         default:
-            newrec.ExceptionCode    = STATUS_INVALID_DISPOSITION;
-            newrec.ExceptionFlags   = EH_NONCONTINUABLE;
-            newrec.ExceptionRecord  = rec;
-            newrec.NumberParameters = 0;
-            RtlRaiseException( &newrec );  /* never returns */
-            break;
+            return STATUS_INVALID_DISPOSITION;
         }
         frame = frame->Prev;
     }
-    EXC_DefaultHandling( rec, context );
+    return STATUS_UNHANDLED_EXCEPTION;
+}
+
+
+/*******************************************************************
+ *		raise_exception
+ *
+ * Implementation of NtRaiseException.
+ */
+static NTSTATUS raise_exception( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance )
+{
+    NTSTATUS status;
+
+    if (first_chance)
+    {
+        DWORD c;
+
+        TRACE( "code=%lx flags=%lx addr=%p\n",
+               rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
+        for (c = 0; c < rec->NumberParameters; c++)
+            TRACE( " info[%ld]=%08lx\n", c, rec->ExceptionInformation[c] );
+        if (rec->ExceptionCode == EXCEPTION_WINE_STUB)
+        {
+            if (rec->ExceptionInformation[1] >> 16)
+                MESSAGE( "wine: Call from %p to unimplemented function %s.%s, aborting\n",
+                         rec->ExceptionAddress,
+                         (char*)rec->ExceptionInformation[0], (char*)rec->ExceptionInformation[1] );
+            else
+                MESSAGE( "wine: Call from %p to unimplemented function %s.%ld, aborting\n",
+                         rec->ExceptionAddress,
+                         (char*)rec->ExceptionInformation[0], rec->ExceptionInformation[1] );
+        }
+#ifdef __i386__
+        else
+        {
+            TRACE(" eax=%08lx ebx=%08lx ecx=%08lx edx=%08lx esi=%08lx edi=%08lx\n",
+                  context->Eax, context->Ebx, context->Ecx,
+                  context->Edx, context->Esi, context->Edi );
+            TRACE(" ebp=%08lx esp=%08lx cs=%04lx ds=%04lx es=%04lx fs=%04lx gs=%04lx flags=%08lx\n",
+                  context->Ebp, context->Esp, context->SegCs, context->SegDs,
+                  context->SegEs, context->SegFs, context->SegGs, context->EFlags );
+        }
+#endif
+        status = send_debug_event( rec, TRUE, context );
+        if (status == DBG_CONTINUE || status == DBG_EXCEPTION_HANDLED)
+            return STATUS_SUCCESS;
+
+        if (call_vectored_handlers( rec, context ) == EXCEPTION_CONTINUE_EXECUTION)
+            return STATUS_SUCCESS;
+
+        if ((status = call_stack_handlers( rec, context )) != STATUS_UNHANDLED_EXCEPTION)
+            return status;
+    }
+
+    /* last chance exception */
+
+    status = send_debug_event( rec, FALSE, context );
+    if (status != DBG_CONTINUE)
+    {
+        if (rec->ExceptionFlags & EH_STACK_INVALID)
+            ERR("Exception frame is not in stack limits => unable to dispatch exception.\n");
+        else if (rec->ExceptionCode == STATUS_NONCONTINUABLE_EXCEPTION)
+            ERR("Process attempted to continue execution after noncontinuable exception.\n");
+        else
+            ERR("Unhandled exception code %lx flags %lx addr %p\n",
+                rec->ExceptionCode, rec->ExceptionFlags, rec->ExceptionAddress );
+        NtTerminateProcess( NtCurrentProcess(), 1 );
+    }
+    return STATUS_SUCCESS;
+}
+
+
+/*******************************************************************
+ *		NtRaiseException (NTDLL.@)
+ */
+NTSTATUS WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance )
+{
+    NTSTATUS status = raise_exception( rec, context, first_chance );
+    if (status == STATUS_SUCCESS) NtSetContextThread( GetCurrentThread(), context );
+    return status;
+}
+
+/***********************************************************************
+ *		RtlRaiseException (NTDLL.@)
+ */
+void WINAPI __regs_RtlRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context )
+{
+    NTSTATUS status = raise_exception( rec, context, TRUE );
+    if (status != STATUS_SUCCESS)
+    {
+        EXCEPTION_RECORD newrec;
+        newrec.ExceptionCode    = status;
+        newrec.ExceptionFlags   = EH_NONCONTINUABLE;
+        newrec.ExceptionRecord  = rec;
+        newrec.NumberParameters = 0;
+        RtlRaiseException( &newrec );  /* never returns */
+    }
 }
 
 /**********************************************************************/
@@ -371,6 +410,7 @@ void WINAPI __regs_RtlUnwind( EXCEPTION_REGISTRATION_RECORD* pEndFrame, PVOID un
 {
     EXCEPTION_RECORD record, newrec;
     EXCEPTION_REGISTRATION_RECORD *frame, *dispatch;
+    DWORD res;
 
 #ifdef __i386__
     context->Eax = (DWORD)returnEax;
@@ -416,8 +456,12 @@ void WINAPI __regs_RtlUnwind( EXCEPTION_REGISTRATION_RECORD* pEndFrame, PVOID un
         }
 
         /* Call handler */
-        switch(EXC_CallHandler( pRecord, frame, context, &dispatch,
-                                frame->Handler, EXC_UnwindHandler ))
+        TRACE( "calling handler at %p code=%lx flags=%lx\n",
+               frame->Handler, pRecord->ExceptionCode, pRecord->ExceptionFlags );
+        res = EXC_CallHandler( pRecord, frame, context, &dispatch, frame->Handler, EXC_UnwindHandler );
+        TRACE( "handler at %p returned %lx\n", frame->Handler, res );
+
+        switch(res)
         {
         case ExceptionContinueSearch:
             break;
@@ -451,28 +495,6 @@ void WINAPI RtlUnwind( PVOID pEndFrame, PVOID unusedEip,
 #endif
 
 
-/*******************************************************************
- *		NtRaiseException (NTDLL.@)
- */
-void WINAPI __regs_NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *ctx,
-                                  BOOL first, CONTEXT *context )
-{
-    __regs_RtlRaiseException( rec, ctx );
-    *context = *ctx;
-}
-
-#ifdef DEFINE_REGS_ENTRYPOINT
-DEFINE_REGS_ENTRYPOINT( NtRaiseException, 12, 12 );
-#else
-void WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *ctx, BOOL first )
-{
-    CONTEXT context;
-    memset( &context, 0, sizeof(context) );
-    __regs_NtRaiseException( rec, ctx, first, &context );
-}
-#endif
-
-
 /***********************************************************************
  *            RtlRaiseStatus  (NTDLL.@)
  *
@@ -545,7 +567,13 @@ DWORD __wine_exception_handler( EXCEPTION_RECORD *record, EXCEPTION_REGISTRATION
 
     if (record->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND | EH_NESTED_CALL))
         return ExceptionContinueSearch;
-    if (wine_frame->u.filter)
+
+    if (wine_frame->u.filter == (void *)1)  /* special hack for page faults */
+    {
+        if (record->ExceptionCode != STATUS_ACCESS_VIOLATION)
+            return ExceptionContinueSearch;
+    }
+    else if (wine_frame->u.filter)
     {
         EXCEPTION_POINTERS ptrs;
         ptrs.ExceptionRecord = record;
