@@ -202,9 +202,6 @@ NTSTATUS WINAPI NtQueryInformationToken(
 
     switch (tokeninfoclass)
     {
-    case TokenGroups:
-        len = sizeof(TOKEN_GROUPS);
-        break;
     case TokenOwner:
         len = sizeof(TOKEN_OWNER) + sizeof(SID);
         break;
@@ -409,7 +406,7 @@ NTSTATUS WINAPI NtCreatePort(PHANDLE PortHandle,POBJECT_ATTRIBUTES ObjectAttribu
 {
   FIXME("(%p,%p,%lu,%lu,%p),stub!\n",PortHandle,ObjectAttributes,
         MaxConnectInfoLength,MaxDataLength,reserved);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -432,7 +429,7 @@ NTSTATUS WINAPI NtConnectPort(
           pConnectInfoLength);
     if (ConnectInfo && pConnectInfoLength)
         TRACE("\tMessage = %s\n",debugstr_an(ConnectInfo,*pConnectInfoLength));
-    return 0;
+    return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -442,7 +439,7 @@ NTSTATUS WINAPI NtConnectPort(
 NTSTATUS WINAPI NtListenPort(HANDLE PortHandle,PLPC_MESSAGE pLpcMessage)
 {
   FIXME("(%p,%p),stub!\n",PortHandle,pLpcMessage);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -459,7 +456,7 @@ NTSTATUS WINAPI NtAcceptConnectPort(
 {
   FIXME("(%p,%lu,%p,%d,%p,%p),stub!\n",
         PortHandle,PortIdentifier,pLpcMessage,Accept,WriteSection,ReadSection);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -469,7 +466,7 @@ NTSTATUS WINAPI NtAcceptConnectPort(
 NTSTATUS WINAPI NtCompleteConnectPort(HANDLE PortHandle)
 {
   FIXME("(%p),stub!\n",PortHandle);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -479,7 +476,7 @@ NTSTATUS WINAPI NtCompleteConnectPort(HANDLE PortHandle)
 NTSTATUS WINAPI NtRegisterThreadTerminatePort(HANDLE PortHandle)
 {
   FIXME("(%p),stub!\n",PortHandle);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -506,7 +503,7 @@ NTSTATUS WINAPI NtRequestWaitReplyPort(
     TRACE("\tData                = %s\n",
       debugstr_an((const char*)pLpcMessageIn->Data,pLpcMessageIn->DataSize));
   }
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
@@ -520,7 +517,7 @@ NTSTATUS WINAPI NtReplyWaitReceivePort(
         PLPC_MESSAGE Message)
 {
   FIXME("(%p,%p,%p,%p),stub!\n",PortHandle,PortIdentifier,ReplyMessage,Message);
-  return 0;
+  return STATUS_NOT_IMPLEMENTED;
 }
 
 /*
@@ -904,6 +901,15 @@ NTSTATUS WINAPI NtQuerySystemInformation(
     return ret;
 }
 
+/******************************************************************************
+ * NtSetSystemInformation [NTDLL.@]
+ * ZwSetSystemInformation [NTDLL.@]
+ */
+NTSTATUS WINAPI NtSetSystemInformation(SYSTEM_INFORMATION_CLASS SystemInformationClass, PVOID SystemInformation, ULONG Length)
+{
+    FIXME("(0x%08x,%p,0x%08lx) stub\n",SystemInformationClass,SystemInformation,Length);
+    return STATUS_SUCCESS;
+}
 
 /******************************************************************************
  *  NtCreatePagingFile		[NTDLL.@]
