@@ -148,6 +148,17 @@ static void restore_fpu( CONTEXT *context, ucontext_t *ucontext )
 }
 
 
+/***********************************************************************
+ *              get_cpu_context
+ *
+ * Get the context of the current thread.
+ */
+void get_cpu_context( CONTEXT *context )
+{
+    FIXME("not implemented\n");
+}
+
+
 /***********************************************************************
  *           set_cpu_context
  *
