@@ -50,6 +50,8 @@
 #define NONAMELESSUNION
 #define NONAMELESSSTRUCT
 
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "windef.h"
 #include "thread.h"
 #include "wine/server.h"
@@ -83,6 +85,7 @@ NTSTATUS WINAPI NtCreateSemaphore( OUT PHANDLE SemaphoreHandle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         req->initial = InitialCount;
         req->max     = MaximumCount;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
@@ -109,6 +112,7 @@ NTSTATUS WINAPI NtOpenSemaphore( OUT PHANDLE SemaphoreHandle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         ret = wine_server_call( req );
         *SemaphoreHandle = reply->handle;
@@ -175,6 +179,7 @@ NTSTATUS WINAPI NtCreateEvent(
     {
         req->access = DesiredAccess;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         req->manual_reset = ManualReset;
         req->initial_state = InitialState;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
@@ -203,6 +208,7 @@ NTSTATUS WINAPI NtOpenEvent(
     {
         req->access  = DesiredAccess;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         ret = wine_server_call( req );
         *EventHandle = reply->handle;
@@ -322,6 +328,7 @@ NTSTATUS WINAPI NtCreateMutant(OUT HANDLE* MutantHandle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         req->owned   = InitialOwner;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         status = wine_server_call( req );
@@ -348,6 +355,7 @@ NTSTATUS WINAPI NtOpenMutant(OUT HANDLE* MutantHandle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         status = wine_server_call( req );
         *MutantHandle = reply->handle;
@@ -414,6 +422,7 @@ NTSTATUS WINAPI NtCreateTimer(OUT HANDLE *handle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         req->manual  = (timer_type == NotificationTimer) ? TRUE : FALSE;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         status = wine_server_call( req );
@@ -441,6 +450,7 @@ NTSTATUS WINAPI NtOpenTimer(OUT PHANDLE handle,
     {
         req->access  = access;
         req->attributes = (attr) ? attr->Attributes : 0;
+        req->rootdir = attr ? attr->RootDirectory : 0;
         if (len) wine_server_add_data( req, attr->ObjectName->Buffer, len );
         status = wine_server_call( req );
         *handle = reply->handle;
@@ -832,3 +842,29 @@ NTSTATUS WINAPI NtDelayExecution( BOOLEAN alertable, const LARGE_INTEGER *timeou
     }
     return STATUS_SUCCESS;
 }
+
+NTSTATUS WINAPI NtCreateIoCompletion( PHANDLE CompletionPort, ACCESS_MASK DesiredAccess,
+                                      POBJECT_ATTRIBUTES ObjectAttributes, ULONG NumberOfConcurrentThreads )
+{
+    FIXME("(%p, %lx, %p, %ld)\n", CompletionPort, DesiredAccess,
+          ObjectAttributes, NumberOfConcurrentThreads);
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+NTSTATUS WINAPI NtSetIoCompletion( HANDLE CompletionPort, ULONG_PTR CompletionKey,
+                                   PIO_STATUS_BLOCK iosb, ULONG NumberOfBytesTransferred,
+                                   ULONG NumberOfBytesToTransfer )
+{
+    FIXME("(%p, %lx, %p, %ld, %ld)\n", CompletionPort, CompletionKey,
+          iosb, NumberOfBytesTransferred, NumberOfBytesToTransfer);
+    return STATUS_NOT_IMPLEMENTED;
+}
+
+NTSTATUS WINAPI NtRemoveIoCompletion( HANDLE CompletionPort, PULONG_PTR CompletionKey,
+                                      PIO_STATUS_BLOCK iosb, PULONG OperationStatus,
+                                      PLARGE_INTEGER WaitTime )
+{
+    FIXME("(%p, %p, %p, %p, %p)\n", CompletionPort, CompletionKey,
+          iosb, OperationStatus, WaitTime);
+    return STATUS_NOT_IMPLEMENTED;
+}
