@@ -269,9 +269,12 @@ static const struct tagTZ_INFO TZ_INFO[] =
    {"SAMT",
     {'S','a','m','a','r','a',' ','S','t','a','n','d','a','r','d',' ','T','i',
      'm','e','\0'}, -270, 1},
+   {"YEKT",
+    {'U','r','a','l','s',' ','S','t','a','n','d','a','r','d',
+     ' ','T','i','m','e',' ','(','W','i','n','t','e','r',')','\0'}, -300, 0},
    {"YEKST",
-    {'E','k','a','t','e','r','i','n','b','u','r','g',' ','S','t','a','n','d',
-     'a','r','d',' ','T','i','m','e','\0'}, -360, 1},
+    {'U','r','a','l','s',' ','D','a','y','l','i','g','h','t',
+     ' ','T','i','m','e',' ','(','S','u','m','m','e','r',')','\0'}, -360, 1},
    {"PKT",
     {'W','e','s','t',' ','A','s','i','a',' ','S','t','a','n','d','a','r','d',
      ' ','T','i','m','e','\0'}, -300, 0},
@@ -874,6 +877,20 @@ NTSTATUS WINAPI NtQueryPerformanceCounter( PLARGE_INTEGER Counter, PLARGE_INTEGE
     return STATUS_SUCCESS;
 }
 
+
+/******************************************************************************
+ * NtGetTickCount   (NTDLL.@)
+ * ZwGetTickCount   (NTDLL.@)
+ */
+ULONG WINAPI NtGetTickCount(void)
+{
+    struct timeval current_time;
+
+    gettimeofday(&current_time, NULL);
+    return (current_time.tv_sec - server_start_time)*1000 + current_time.tv_usec/1000;
+}
+
+
 /***********************************************************************
  *        TIME_GetTZAsStr [internal]
  *
