@@ -1708,7 +1708,7 @@ HRESULT WINAPI CoGetClassObject(
     }
 
     if (FAILED(hres))
-        ERR("no class object %s could be created for for context 0x%lx\n",
+        ERR("no class object %s could be created for context 0x%lx\n",
             debugstr_guid(rclsid), dwClsContext);
     return hres;
 }
@@ -1878,11 +1878,8 @@ HRESULT WINAPI CoCreateInstance(
 			  &IID_IClassFactory,
 			  (LPVOID)&lpclf);
 
-  if (FAILED(hres)) {
-    FIXME("no classfactory created for CLSID %s, hres is 0x%08lx\n",
-	  debugstr_guid(rclsid),hres);
+  if (FAILED(hres))
     return hres;
-  }
 
   /*
    * Create the object and don't forget to release the factory
@@ -2643,6 +2640,14 @@ HRESULT WINAPI CoCopyProxy(IUnknown *pProxy, IUnknown **ppCopy)
 }
 
 
+static BOOL COM_PeekMessage(struct apartment *apt, MSG *msg)
+{
+    /* first try to retrieve messages for incoming COM calls to the apartment window */
+    return PeekMessageW(msg, apt->win, WM_USER, WM_APP - 1, PM_REMOVE|PM_NOYIELD) ||
+           /* next retrieve other messages necessary for the app to remain responsive */
+           PeekMessageW(msg, NULL, 0, WM_USER - 1, PM_REMOVE|PM_NOYIELD);
+}
+
 /***********************************************************************
  *           CoWaitForMultipleHandles [OLE32.@]
  *
@@ -2673,7 +2678,8 @@ HRESULT WINAPI CoWaitForMultipleHandles(DWORD dwFlags, DWORD dwTimeout,
 {
     HRESULT hr = S_OK;
     DWORD start_time = GetTickCount();
-    BOOL message_loop = TRUE;
+    APARTMENT *apt = COM_CurrentApt();
+    BOOL message_loop = apt && !apt->multi_threaded;
 
     TRACE("(0x%08lx, 0x%08lx, %ld, %p, %p)\n", dwFlags, dwTimeout, cHandles,
         pHandles, lpdwindex);
@@ -2703,7 +2709,7 @@ HRESULT WINAPI CoWaitForMultipleHandles(DWORD dwFlags, DWORD dwTimeout,
             if (res == WAIT_OBJECT_0 + cHandles)  /* messages available */
             {
                 MSG msg;
-                while (PeekMessageW(&msg, NULL, 0, 0, PM_REMOVE))
+                while (COM_PeekMessage(apt, &msg))
                 {
                     /* FIXME: filter the messages here */
                     TRACE("received message whilst waiting for RPC: 0x%04x\n", msg.message);
