@@ -122,6 +122,7 @@ static HRESULT WINAPI StgStreamImpl_QueryInterface(
    * Compare the riid with the interface IDs implemented by this object.
    */
   if (IsEqualGUID(&IID_IUnknown, riid)||
+      IsEqualGUID(&IID_IPersistStream, riid)||
       IsEqualGUID(&IID_IStream, riid))
   {
     *ppvObject = (IStream*)This;
@@ -267,6 +268,12 @@ static HRESULT WINAPI StgStreamImpl_Read(
   TRACE("(%p, %p, %ld, %p)\n",
 	iface, pv, cb, pcbRead);
 
+  if (!This->parentStorage)
+  {
+    WARN("storage reverted\n");
+    return STG_E_REVERTED;
+  }
+ 
   /*
    * If the caller is not interested in the number of bytes read,
    * we use another buffer to avoid "if" statements in the code.
@@ -369,12 +376,19 @@ static HRESULT WINAPI StgStreamImpl_Write(
   case STGM_READWRITE:
       break;
   default:
+      WARN("access denied by flags: 0x%lx\n", STGM_ACCESS_MODE(This->grfMode));
       return STG_E_ACCESSDENIED;
   }
 
   if (!pv)
     return STG_E_INVALIDPOINTER;
 
+  if (!This->parentStorage)
+  {
+    WARN("storage reverted\n");
+    return STG_E_REVERTED;
+  }
+ 
   /*
    * If the caller is not interested in the number of bytes written,
    * we use another buffer to avoid "if" statements in the code.
@@ -389,6 +403,7 @@ static HRESULT WINAPI StgStreamImpl_Write(
 
   if (cb == 0)
   {
+    TRACE("<-- S_OK, written 0\n");
     return S_OK;
   }
   else
@@ -435,6 +450,7 @@ static HRESULT WINAPI StgStreamImpl_Write(
    */
   This->currentPosition.u.LowPart += *pcbWritten;
 
+  TRACE("<-- S_OK, written %lu\n", *pcbWritten);
   return S_OK;
 }
 
@@ -463,9 +479,8 @@ static HRESULT WINAPI StgStreamImpl_Seek(
    * fail if the stream has no parent (as does windows)
    */
 
-  if(!(This->parentStorage)) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   /*
    * The caller is allowed to pass in NULL as the new position return value.
@@ -527,9 +542,8 @@ static HRESULT WINAPI StgStreamImpl_SetSize(
 
   TRACE("(%p, %ld)\n", iface, libNewSize.u.LowPart);
 
-  if(!This->parentStorage) {
+  if(!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   /*
    * As documented.
@@ -648,9 +662,8 @@ static HRESULT WINAPI StgStreamImpl_CopyTo(
    * Sanity check
    */
 
-  if(!This->parentStorage) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   if ( pstm == 0 )
     return STG_E_INVALIDPOINTER;
@@ -724,9 +737,9 @@ static HRESULT WINAPI StgStreamImpl_Commit(
 {
   StgStreamImpl* const This=(StgStreamImpl*)iface;
 
-  if(!This->parentStorage) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
+
   return S_OK;
 }
 
@@ -752,9 +765,8 @@ static HRESULT WINAPI StgStreamImpl_LockRegion(
 {
   StgStreamImpl* const This=(StgStreamImpl*)iface;
 
-  if(!This->parentStorage) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   FIXME("not implemented!\n");
   return E_NOTIMPL;
@@ -768,9 +780,8 @@ static HRESULT WINAPI StgStreamImpl_UnlockRegion(
 {
   StgStreamImpl* const This=(StgStreamImpl*)iface;
 
-  if(!This->parentStorage) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   FIXME("not implemented!\n");
   return E_NOTIMPL;
@@ -798,9 +809,8 @@ static HRESULT WINAPI StgStreamImpl_Stat(
    * if stream has no parent, return STG_E_REVERTED
    */
 
-  if(!This->parentStorage) {
-	return STG_E_REVERTED;
-  }
+  if (!This->parentStorage)
+    return STG_E_REVERTED;
 
   /*
    * Read the information from the property.
@@ -848,9 +858,8 @@ static HRESULT WINAPI StgStreamImpl_Clone(
    * Sanity check
    */
 
-  if(!This->parentStorage) {
+  if (!This->parentStorage)
     return STG_E_REVERTED;
-  }
 
   if ( ppstm == 0 )
     return STG_E_INVALIDPOINTER;
