@@ -472,6 +472,66 @@ static void test_proxy_marshal_and_unmarshal(void)
     end_host_object(tid, thread);
 }
 
+/* tests success case of an interthread marshal and then marshaling the proxy
+ * using an iid that hasn't previously been unmarshaled */
+static void test_proxy_marshal_and_unmarshal2(void)
+{
+    HRESULT hr;
+    IStream *pStream = NULL;
+    IUnknown *pProxy = NULL;
+    IUnknown *pProxy2 = NULL;
+    DWORD tid;
+    HANDLE thread;
+
+    cLocks = 0;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &pStream);
+    ok_ole_success(hr, CreateStreamOnHGlobal);
+    tid = start_host_object(pStream, &IID_IUnknown, (IUnknown*)&Test_ClassFactory, MSHLFLAGS_NORMAL, &thread);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IUnknown, (void **)&pProxy);
+    ok_ole_success(hr, CoUnmarshalInterface);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    /* marshal the proxy */
+    hr = CoMarshalInterface(pStream, &IID_IClassFactory, pProxy, MSHCTX_INPROC, NULL, MSHLFLAGS_NORMAL);
+    ok_ole_success(hr, CoMarshalInterface);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    /* unmarshal the second proxy to the object */
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy2);
+    ok_ole_success(hr, CoUnmarshalInterface);
+    IStream_Release(pStream);
+
+    /* now the proxies should be as follows:
+     *  pProxy -> &Test_ClassFactory
+     *  pProxy2 -> &Test_ClassFactory
+     * they should NOT be as follows:
+     *  pProxy -> &Test_ClassFactory
+     *  pProxy2 -> pProxy
+     * the above can only really be tested by looking in +ole traces
+     */
+
+    ok_more_than_one_lock();
+
+    IUnknown_Release(pProxy);
+
+    ok_more_than_one_lock();
+
+    IUnknown_Release(pProxy2);
+
+    ok_no_locks();
+
+    end_host_object(tid, thread);
+}
+
 /* tests that stubs are released when the containing apartment is destroyed */
 static void test_marshal_stub_apartment_shutdown(void)
 {
@@ -1538,18 +1598,15 @@ static void test_WM_QUIT_handling(void)
     }
 }
 
-/* doesn't pass with Win9x COM DLLs (even though Essential COM says it should) */
-#if 0
-
 static HANDLE heventShutdown;
 
-static void LockModuleOOP()
+static void LockModuleOOP(void)
 {
     InterlockedIncrement(&cLocks); /* for test purposes only */
     CoAddRefServerProcess();
 }
 
-static void UnlockModuleOOP()
+static void UnlockModuleOOP(void)
 {
     InterlockedDecrement(&cLocks); /* for test purposes only */
     if (!CoReleaseServerProcess())
@@ -1691,7 +1748,6 @@ static void test_out_of_process_com(void)
 
     CloseHandle(heventShutdown);
 }
-#endif
 
 static void test_ROT(void)
 {
@@ -1930,6 +1986,7 @@ START_TEST(marshal)
     test_marshal_and_unmarshal_invalid();
     test_interthread_marshal_and_unmarshal();
     test_proxy_marshal_and_unmarshal();
+    test_proxy_marshal_and_unmarshal2();
     test_marshal_stub_apartment_shutdown();
     test_marshal_proxy_apartment_shutdown();
     test_marshal_proxy_mta_apartment_shutdown();
@@ -1952,7 +2009,8 @@ START_TEST(marshal)
     test_message_reentrancy();
     test_WM_QUIT_handling();
 
-/*    test_out_of_process_com(); */
+    /* doesn't pass with Win9x COM DLLs (even though Essential COM says it should) */
+    if (0) test_out_of_process_com();
 
     test_ROT();
     /* FIXME: test GIT */
