@@ -55,9 +55,7 @@ static void test_hglobal_storage_stat(void)
 
     ok( stat.pwcsName == NULL, "storage name not null\n");
     ok( stat.type == 1, "type is wrong\n");
-    todo_wine {
     ok( stat.grfMode == 0x12, "grf mode is incorrect\n");
-    }
     ok( !memcmp(&stat.clsid, &test_stg_cls, sizeof test_stg_cls), "CLSID is wrong\n");
 
     refcount = IStorage_Release( stg );
@@ -118,15 +116,15 @@ static void test_create_storage_modes(void)
    ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile succeeded\n");
    ok(stg == NULL, "stg was set\n");
 
-   /* check what happens if the file already exists */
+   /* check what happens if the file already exists (which is how it's meant to be used) */
    r = StgCreateDocfile( filename, STGM_SHARE_EXCLUSIVE | STGM_READWRITE, 0, &stg);
    ok(r==S_OK, "StgCreateDocfile failed\n");
    r = IStorage_Release(stg);
    ok(r == 0, "storage not released\n");
    r = StgCreateDocfile( filename, STGM_SHARE_EXCLUSIVE | STGM_READWRITE |STGM_TRANSACTED, 0, &stg);
-   ok(r==STG_E_FILEALREADYEXISTS, "StgCreateDocfile wrong error\n");
+   ok(r==STG_E_FILEALREADYEXISTS, "StgCreateDocfile wrong error\n"); /* FAILIFTHERE is default */
    r = StgCreateDocfile( filename, STGM_READ, 0, &stg);
-   ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile succeeded\n");
+   ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile succeeded\n"); /* need at least readmode and sharemode */
    r = StgCreateDocfile( filename, STGM_SHARE_EXCLUSIVE, 0, &stg);
    ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile succeeded\n");
    r = StgCreateDocfile( filename, STGM_SHARE_DENY_WRITE, 0, &stg);
@@ -143,6 +141,8 @@ static void test_create_storage_modes(void)
    ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile failed\n");
    r = StgCreateDocfile( filename, STGM_SHARE_DENY_WRITE | STGM_READ, 0, &stg);
    ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile wrong error\n");
+   r = StgCreateDocfile( filename, STGM_TRANSACTED | STGM_SHARE_DENY_WRITE | STGM_READ, 0, &stg);
+   ok(r==STG_E_INVALIDFLAG, "StgCreateDocfile wrong error\n");
    ok(DeleteFileW(filename), "failed to delete file\n");
 
    r = StgCreateDocfile( filename, STGM_SHARE_EXCLUSIVE | STGM_READWRITE |STGM_TRANSACTED, 0, &stg);
@@ -160,7 +160,6 @@ static void test_create_storage_modes(void)
    ok(r==S_OK, "StgCreateDocfile failed\n");
    r = IStorage_Release(stg);
    ok(r == 0, "storage not released\n");
-
    ok(DeleteFileW(filename), "failed to delete file\n");
 
    /* test the way excel uses StgCreateDocFile */
@@ -173,6 +172,16 @@ static void test_create_storage_modes(void)
       ok(DeleteFileW(filename), "failed to delete file\n");
    }
 
+   /* and the way windows media uses it ... */
+   r = StgCreateDocfile( filename, STGM_CREATE | STGM_SHARE_DENY_NONE | STGM_READWRITE | STGM_TRANSACTED, 0, &stg);
+   ok(r==S_OK, "StgCreateDocfile the windows media way failed\n");
+   if (r == S_OK)
+   {
+      r = IStorage_Release(stg);
+      ok(r == 0, "storage not released\n");
+      ok(DeleteFileW(filename), "failed to delete file\n");
+   }
+
    /* looks like we need STGM_TRANSACTED or STGM_CREATE */
    r = StgCreateDocfile( filename, STGM_TRANSACTED|STGM_SHARE_DENY_WRITE|STGM_READWRITE, 0, &stg);
    ok(r==S_OK, "StgCreateDocfile the excel way failed\n");
@@ -474,6 +483,36 @@ static void test_open_storage(void)
     if (stg)
         IStorage_Release(stg);
 
+    /* open like Project 2003 */
+    stg = NULL;
+    r = StgOpenStorage( filename, NULL, STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == S_OK, "should succeed\n");
+    r = StgOpenStorage( filename, NULL, STGM_PRIORITY, NULL, 0, &stg2);
+    ok(r == S_OK, "should succeed\n");
+    if (stg2)
+        IStorage_Release(stg2);
+    if (stg)
+        IStorage_Release(stg);
+
+    stg = NULL;
+    r = StgOpenStorage( filename, NULL, STGM_PRIORITY | STGM_READWRITE, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFLAG, "should fail\n");
+
+    r = StgOpenStorage( filename, NULL, STGM_TRANSACTED | STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFLAG, "should fail\n");
+
+    r = StgOpenStorage( filename, NULL, STGM_SIMPLE | STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFLAG, "should fail\n");
+
+    r = StgOpenStorage( filename, NULL, STGM_DELETEONRELEASE | STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFUNCTION, "should fail\n");
+
+    r = StgOpenStorage( filename, NULL, STGM_NOSCRATCH | STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFLAG, "should fail\n");
+
+    r = StgOpenStorage( filename, NULL, STGM_NOSNAPSHOT | STGM_PRIORITY, NULL, 0, &stg);
+    ok(r == STG_E_INVALIDFLAG, "should fail\n");
+
     r = DeleteFileW(filename);
     ok(r, "file didn't exist\n");
 }
@@ -598,12 +637,14 @@ static void test_storage_refcount(void)
     static const WCHAR szDot[] = { '.',0 };
     WCHAR filename[MAX_PATH];
     IStorage *stg = NULL;
+    IStorage *stgprio = NULL;
     HRESULT r;
     IStream *stm = NULL;
     static const WCHAR stmname[] = { 'C','O','N','T','E','N','T','S',0 };
     LARGE_INTEGER pos;
     ULARGE_INTEGER upos;
     STATSTG stat;
+    char buffer[10];
 
     if(!GetTempFileNameW(szDot, szPrefix, 0, filename))
         return;
@@ -615,6 +656,12 @@ static void test_storage_refcount(void)
                             STGM_READWRITE |STGM_TRANSACTED, 0, &stg);
     ok(r==S_OK, "StgCreateDocfile failed\n");
 
+    r = WriteClassStg( stg, &test_stg_cls );
+    ok( r == S_OK, "WriteClassStg failed\n");
+
+    r = IStorage_Commit( stg, STGC_DEFAULT );
+    ok( r == S_OK, "IStorage_Commit failed\n");
+
     /* now create a stream */
     r = IStorage_CreateStream(stg, stmname, STGM_SHARE_EXCLUSIVE | STGM_READWRITE, 0, 0, &stm );
     ok(r==S_OK, "IStorage->CreateStream failed\n");
@@ -629,21 +676,84 @@ static void test_storage_refcount(void)
     r = IStream_Stat( stm, &stat, STATFLAG_DEFAULT );
     ok (r == STG_E_REVERTED, "stat should fail\n");
 
+    r = IStream_Write( stm, "Test string", strlen("Test string"), NULL);
+    ok (r == STG_E_REVERTED, "IStream_Write should return STG_E_REVERTED instead of 0x%08lx\n", r);
+
+    r = IStream_Read( stm, buffer, sizeof(buffer), NULL);
+    ok (r == STG_E_REVERTED, "IStream_Read should return STG_E_REVERTED instead of 0x%08lx\n", r);
+
     r = IStream_Release(stm);
     ok (r == 0, "stream not released\n");
 
-    /* test for grfMode open issue */
+    /* tests that STGM_PRIORITY doesn't prevent readwrite access from other
+     * StgOpenStorage calls in transacted mode */
+    r = StgOpenStorage( filename, NULL, STGM_PRIORITY, NULL, 0, &stgprio);
+    ok(r==S_OK, "StgOpenStorage failed with error 0x%08lx\n", r);
 
-    r = StgOpenStorage( filename, NULL, 0x00010020, NULL, 0, &stg);
-    ok(r==S_OK, "StgOpenStorage failed\n");
+    todo_wine {
+    /* non-transacted mode read/write fails */
+    r = StgOpenStorage( filename, NULL, STGM_SHARE_EXCLUSIVE|STGM_READWRITE, NULL, 0, &stg);
+    ok(r==STG_E_LOCKVIOLATION, "StgOpenStorage should return STG_E_LOCKVIOLATION instead of 0x%08lx\n", r);
+    }
+
+    /* non-transacted mode read-only succeeds */
+    r = StgOpenStorage( filename, NULL, STGM_SHARE_DENY_WRITE|STGM_READ, NULL, 0, &stg);
+    ok(r==S_OK, "StgOpenStorage failed with error 0x%08lx\n", r);
+    IStorage_Release(stg);
+
+    r = StgOpenStorage( filename, NULL, STGM_TRANSACTED|STGM_SHARE_DENY_WRITE|STGM_READWRITE, NULL, 0, &stg);
+    ok(r==S_OK, "StgOpenStorage failed with error 0x%08lx\n", r);
     if(stg)
     {
-        r = IStorage_OpenStream( stg, stmname, 0, STGM_SHARE_EXCLUSIVE|STGM_READWRITE, 0, &stm );
-        ok(r == S_OK, "OpenStream should succeed\n");
+        static const WCHAR stgname[] = { ' ',' ',' ','2','9',0 };
+        static const WCHAR stgname2[] = { 'C','V','_','i','e','w',0 };
+        static const WCHAR stmname2[] = { 'V','a','r','2','D','a','t','a',0 };
+        IStorage *stg2;
+        IStorage *stg3;
+        STATSTG statstg;
+
+        r = IStorage_Stat( stg, &statstg, STATFLAG_NONAME );
+        ok(r == S_OK, "Stat should have succeded instead of returning 0x%08lx\n", r);
+        ok(statstg.type == STGTY_STORAGE, "Statstg type should have been STGTY_STORAGE instead of %ld\n", statstg.type);
+        ok(statstg.cbSize.LowPart == 0, "Statstg cbSize.LowPart should have been 0 instead of %ld\n", statstg.cbSize.LowPart);
+        ok(statstg.cbSize.HighPart == 0, "Statstg cbSize.HighPart should have been 0 instead of %ld\n", statstg.cbSize.HighPart);
+        ok(statstg.grfMode == (STGM_TRANSACTED|STGM_SHARE_DENY_WRITE|STGM_READWRITE),
+            "Statstg grfMode should have been 0x10022 instead of 0x%lx\n", statstg.grfMode);
+        ok(statstg.grfLocksSupported == 0, "Statstg grfLocksSupported should have been 0 instead of %ld\n", statstg.grfLocksSupported);
+        ok(IsEqualCLSID(&statstg.clsid, &test_stg_cls), "Statstg clsid is not test_stg_cls\n");
+        ok(statstg.grfStateBits == 0, "Statstg grfStateBits should have been 0 instead of %ld\n", statstg.grfStateBits);
+        ok(statstg.reserved == 0, "Statstg reserved should have been 0 instead of %ld\n", statstg.reserved);
+
+        r = IStorage_CreateStorage( stg, stgname, STGM_SHARE_EXCLUSIVE, 0, 0, &stg2 );
+        ok(r == S_OK, "CreateStorage should have succeeded instead of returning 0x%08lx\n", r);
+
+        r = IStorage_Stat( stg2, &statstg, STATFLAG_DEFAULT );
+        ok(r == S_OK, "Stat should have succeded instead of returning 0x%08lx\n", r);
+        ok(!lstrcmpW(statstg.pwcsName, stgname),
+            "Statstg pwcsName should have been the name the storage was created with\n");
+        ok(statstg.type == STGTY_STORAGE, "Statstg type should have been STGTY_STORAGE instead of %ld\n", statstg.type);
+        ok(statstg.cbSize.LowPart == 0, "Statstg cbSize.LowPart should have been 0 instead of %ld\n", statstg.cbSize.LowPart);
+        ok(statstg.cbSize.HighPart == 0, "Statstg cbSize.HighPart should have been 0 instead of %ld\n", statstg.cbSize.HighPart);
+        ok(statstg.grfMode == STGM_SHARE_EXCLUSIVE,
+            "Statstg grfMode should have been STGM_SHARE_EXCLUSIVE instead of 0x%lx\n", statstg.grfMode);
+        ok(statstg.grfLocksSupported == 0, "Statstg grfLocksSupported should have been 0 instead of %ld\n", statstg.grfLocksSupported);
+        ok(IsEqualCLSID(&statstg.clsid, &CLSID_NULL), "Statstg clsid is not CLSID_NULL\n");
+        ok(statstg.grfStateBits == 0, "Statstg grfStateBits should have been 0 instead of %ld\n", statstg.grfStateBits);
+        ok(statstg.reserved == 0, "Statstg reserved should have been 0 instead of %ld\n", statstg.reserved);
+        CoTaskMemFree(statstg.pwcsName);
+
+        r = IStorage_CreateStorage( stg2, stgname2, STGM_SHARE_EXCLUSIVE|STGM_READWRITE, 0, 0, &stg3 );
+        ok(r == STG_E_ACCESSDENIED, "CreateStorage should have returned STG_E_ACCESSDENIED instead of 0x%08lx\n", r);
+
+        r = IStorage_CreateStream( stg2, stmname2, STGM_CREATE|STGM_SHARE_EXCLUSIVE, 0, 0, &stm );
+        ok(r == STG_E_ACCESSDENIED, "CreateStream should have returned STG_E_ACCESSDENIED instead of 0x%08lx\n", r);
+
+        IStorage_Release(stg2);
 
         r = IStorage_Release(stg);
         ok(r == 0, "wrong ref count\n");
     }
+    IStorage_Release(stgprio);
 
     DeleteFileW(filename);
 }
