@@ -42,6 +42,7 @@
 #include "winuser.h"
 
 #include "ole2.h"
+#include "propidl.h" /* for LPSAFEARRAY_User* functions */
 #include "typelib.h"
 #include "variant.h"
 #include "wine/debug.h"
@@ -68,19 +69,35 @@ static char *relaystr(WCHAR *in) {
 }
 
 static HRESULT
-xbuf_add(marshal_state *buf, LPBYTE stuff, DWORD size) {
-    while (buf->size - buf->curoff < size) {
-	if (buf->base) {
-	    buf->size += 100;
-	    buf->base = HeapReAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,buf->base,buf->size);
-	    if (!buf->base)
-		return E_OUTOFMEMORY;
-	} else {
-	    buf->base = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,32);
-	    buf->size = 32;
-	    if (!buf->base)
-		return E_OUTOFMEMORY;
-	}
+xbuf_resize(marshal_state *buf, DWORD newsize)
+{
+    if(buf->size >= newsize)
+        return S_FALSE;
+
+    if(buf->base)
+    {
+        buf->base = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, buf->base, newsize);
+        if(!buf->base)
+            return E_OUTOFMEMORY;
+    }
+    else
+    {
+        buf->base = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, newsize);
+        if(!buf->base)
+            return E_OUTOFMEMORY;
+    }
+    return S_OK;
+}
+
+static HRESULT
+xbuf_add(marshal_state *buf, LPBYTE stuff, DWORD size)
+{
+    HRESULT hr;
+
+    if(buf->size - buf->curoff < size)
+    {
+        hr = xbuf_resize(buf, buf->size + size + 100);
+        if(FAILED(hr)) return hr;
     }
     memcpy(buf->base+buf->curoff,stuff,size);
     buf->curoff += size;
@@ -786,6 +803,17 @@ serialize_param(
 	if (debugout) TRACE_(olerelay)("]");
 	return S_OK;
     }
+    case VT_SAFEARRAY: {
+        if (writeit)
+        {
+            unsigned long flags = MAKELONG(MSHCTX_DIFFERENTMACHINE, NDR_LOCAL_DATA_REPRESENTATION);
+            unsigned long size = LPSAFEARRAY_UserSize(&flags, buf->curoff, (LPSAFEARRAY *)arg);
+            xbuf_resize(buf, size);
+            LPSAFEARRAY_UserMarshal(&flags, buf->base + buf->curoff, (LPSAFEARRAY *)arg);
+            buf->curoff = size;
+        }
+        return S_OK;
+    }
     default:
 	ERR("Unhandled marshal type %d.\n",tdesc->vt);
 	return S_OK;
@@ -1115,6 +1143,16 @@ deserialize_param(
 		);
 	    return S_OK;
 	}
+    case VT_SAFEARRAY: {
+	    if (readit)
+	    {
+		unsigned long flags = MAKELONG(MSHCTX_DIFFERENTMACHINE, NDR_LOCAL_DATA_REPRESENTATION);
+		unsigned char *buffer;
+		buffer = LPSAFEARRAY_UserUnmarshal(&flags, buf->base + buf->curoff, (LPSAFEARRAY *)arg);
+		buf->curoff = buffer - buf->base;
+	    }
+	    return S_OK;
+	}
 	default:
 	    ERR("No handler for VT type %d!\n",tdesc->vt);
 	    return S_OK;
