@@ -39,6 +39,11 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(ole);
 
+#define ALIGNED_LENGTH(_Len, _Align) (((_Len)+(_Align))&~(_Align))
+#define ALIGNED_POINTER(_Ptr, _Align) ((LPVOID)ALIGNED_LENGTH((ULONG_PTR)(_Ptr), _Align))
+#define ALIGN_LENGTH(_Len, _Align) _Len = ALIGNED_LENGTH(_Len, _Align)
+#define ALIGN_POINTER(_Ptr, _Align) _Ptr = ALIGNED_POINTER(_Ptr, _Align)
+
 /* FIXME: not supposed to be here */
 
 const CLSID CLSID_PSDispatch = {
@@ -79,126 +84,202 @@ static void dump_user_flags(unsigned long *pFlags)
 }
 
 /* CLEANLOCALSTORAGE */
-/* I'm not sure how this is supposed to work yet */
+
+#define CLS_FUNCDESC  'f'
+#define CLS_LIBATTR   'l'
+#define CLS_TYPEATTR  't'
+#define CLS_VARDESC   'v'
 
 unsigned long WINAPI CLEANLOCALSTORAGE_UserSize(unsigned long *pFlags, unsigned long Start, CLEANLOCALSTORAGE *pstg)
 {
-  return Start + sizeof(DWORD);
+    ALIGN_LENGTH(Start, 3);
+    return Start + sizeof(DWORD);
 }
 
 unsigned char * WINAPI CLEANLOCALSTORAGE_UserMarshal(unsigned long *pFlags, unsigned char *Buffer, CLEANLOCALSTORAGE *pstg)
 {
-  *(DWORD*)Buffer = 0;
-  return Buffer + sizeof(DWORD);
+    ALIGN_POINTER(Buffer, 3);
+    *(DWORD*)Buffer = pstg->flags;
+    switch(pstg->flags)
+    {
+    case CLS_LIBATTR:
+        ITypeLib_ReleaseTLibAttr((ITypeLib*)pstg->pInterface, *(TLIBATTR**)pstg->pStorage);
+        break;
+    case CLS_TYPEATTR:
+        ITypeInfo_ReleaseTypeAttr((ITypeInfo*)pstg->pInterface, *(TYPEATTR**)pstg->pStorage); 
+        break;
+    case CLS_FUNCDESC:
+        ITypeInfo_ReleaseFuncDesc((ITypeInfo*)pstg->pInterface, *(FUNCDESC**)pstg->pStorage); 
+        break;
+    case CLS_VARDESC:
+        ITypeInfo_ReleaseVarDesc((ITypeInfo*)pstg->pInterface, *(VARDESC**)pstg->pStorage);
+        break;
+
+    default:
+        ERR("Unknown type %lx\n", pstg->flags);
+    }
+
+    *(VOID**)pstg->pStorage = NULL;
+    IUnknown_Release(pstg->pInterface);
+    pstg->pInterface = NULL;
+
+    return Buffer + sizeof(DWORD);
 }
 
 unsigned char * WINAPI CLEANLOCALSTORAGE_UserUnmarshal(unsigned long *pFlags, unsigned char *Buffer, CLEANLOCALSTORAGE *pstr)
 {
-  return Buffer + sizeof(DWORD);
+    ALIGN_POINTER(Buffer, 3);
+    pstr->flags = *(DWORD*)Buffer;
+    return Buffer + sizeof(DWORD);
 }
 
 void WINAPI CLEANLOCALSTORAGE_UserFree(unsigned long *pFlags, CLEANLOCALSTORAGE *pstr)
 {
+    /* Nothing to do */
 }
 
 /* BSTR */
 
+typedef struct
+{
+    DWORD len;          /* No. of chars not including trailing '\0' */
+    DWORD byte_len;     /* len * 2 or 0xffffffff if len == 0 */
+    DWORD len2;         /* == len */
+} bstr_wire_t;
+
 unsigned long WINAPI BSTR_UserSize(unsigned long *pFlags, unsigned long Start, BSTR *pstr)
 {
-  TRACE("(%lx,%ld,%p) => %p\n", *pFlags, Start, pstr, *pstr);
-  if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
-  Start += sizeof(FLAGGED_WORD_BLOB) + sizeof(OLECHAR) * (SysStringLen(*pstr) - 1);
-  TRACE("returning %ld\n", Start);
-  return Start;
+    TRACE("(%lx,%ld,%p) => %p\n", *pFlags, Start, pstr, *pstr);
+    if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
+    ALIGN_LENGTH(Start, 3);
+    Start += sizeof(bstr_wire_t) + ((SysStringByteLen(*pstr) + 1) & ~1);
+    TRACE("returning %ld\n", Start);
+    return Start;
 }
 
 unsigned char * WINAPI BSTR_UserMarshal(unsigned long *pFlags, unsigned char *Buffer, BSTR *pstr)
 {
-  wireBSTR str = (wireBSTR)Buffer;
+    bstr_wire_t *header;
+    DWORD len = SysStringByteLen(*pstr);
 
-  TRACE("(%lx,%p,%p) => %p\n", *pFlags, Buffer, pstr, *pstr);
-  if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
-  str->fFlags = 0;
-  str->clSize = SysStringLen(*pstr);
-  if (str->clSize)
-    memcpy(&str->asData, *pstr, sizeof(OLECHAR) * str->clSize);
-  return Buffer + sizeof(FLAGGED_WORD_BLOB) + sizeof(OLECHAR) * (str->clSize - 1);
+    TRACE("(%lx,%p,%p) => %p\n", *pFlags, Buffer, pstr, *pstr);
+    if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
+
+    ALIGN_POINTER(Buffer, 3);
+    header = (bstr_wire_t*)Buffer;
+    header->len = header->len2 = (len + 1) / 2;
+    if (*pstr)
+    {
+        header->byte_len = len;
+        memcpy(header + 1, *pstr, header->len * 2);
+    }
+    else
+        header->byte_len = 0xffffffff; /* special case for a null bstr */
+
+    return Buffer + sizeof(*header) + sizeof(OLECHAR) * header->len;
 }
 
 unsigned char * WINAPI BSTR_UserUnmarshal(unsigned long *pFlags, unsigned char *Buffer, BSTR *pstr)
 {
-  wireBSTR str = (wireBSTR)Buffer;
-  TRACE("(%lx,%p,%p) => %p\n", *pFlags, Buffer, pstr, *pstr);
-  if (str->clSize) {
-    SysReAllocStringLen(pstr, (OLECHAR*)&str->asData, str->clSize);
-  }
-  else if (*pstr) {
-    SysFreeString(*pstr);
-    *pstr = NULL;
-  }
-  if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
-  return Buffer + sizeof(FLAGGED_WORD_BLOB) + sizeof(OLECHAR) * (str->clSize - 1);
+    bstr_wire_t *header;
+    TRACE("(%lx,%p,%p) => %p\n", *pFlags, Buffer, pstr, *pstr);
+
+    ALIGN_POINTER(Buffer, 3);
+    header = (bstr_wire_t*)Buffer;
+    if(header->len != header->len2)
+        FIXME("len %08lx != len2 %08lx\n", header->len, header->len2);
+    
+    if(*pstr)
+    {
+        SysFreeString(*pstr);
+        *pstr = NULL;
+    }
+
+    if(header->byte_len != 0xffffffff)
+        *pstr = SysAllocStringByteLen((char*)(header + 1), header->byte_len);
+
+    if (*pstr) TRACE("string=%s\n", debugstr_w(*pstr));
+    return Buffer + sizeof(*header) + sizeof(OLECHAR) * header->len;
 }
 
 void WINAPI BSTR_UserFree(unsigned long *pFlags, BSTR *pstr)
 {
-  TRACE("(%lx,%p) => %p\n", *pFlags, pstr, *pstr);
-  if (*pstr) {
-    SysFreeString(*pstr);
-    *pstr = NULL;
-  }
+    TRACE("(%lx,%p) => %p\n", *pFlags, pstr, *pstr);
+    if (*pstr)
+    {
+        SysFreeString(*pstr);
+        *pstr = NULL;
+    }
 }
 
 /* VARIANT */
-/* I'm not too sure how to do this yet */
 
-#define VARIANT_wiresize sizeof(struct _wireVARIANT)
-
-static unsigned wire_size(VARTYPE vt)
+typedef struct
 {
-  if (vt & VT_ARRAY) return 0;
+    DWORD clSize;
+    DWORD rpcReserverd;
+    USHORT vt;
+    USHORT wReserved1;
+    USHORT wReserved2;
+    USHORT wReserved3;
+    DWORD switch_is;
+} variant_wire_t;
 
-  switch (vt & ~VT_BYREF) {
-  case VT_EMPTY:
-  case VT_NULL:
-    return 0;
-  case VT_I1:
-  case VT_UI1:
-    return sizeof(CHAR);
-  case VT_I2:
-  case VT_UI2:
-    return sizeof(SHORT);
-  case VT_I4:
-  case VT_UI4:
-    return sizeof(LONG);
-  case VT_INT:
-  case VT_UINT:
-    return sizeof(INT);
-  case VT_R4:
-    return sizeof(FLOAT);
-  case VT_R8:
-    return sizeof(DOUBLE);
-  case VT_BOOL:
-    return sizeof(VARIANT_BOOL);
-  case VT_ERROR:
-    return sizeof(SCODE);
-  case VT_DATE:
-    return sizeof(DATE);
-  case VT_CY:
-    return sizeof(CY);
-  case VT_DECIMAL:
-    return sizeof(DECIMAL);
-  case VT_BSTR:
-  case VT_VARIANT:
-  case VT_UNKNOWN:
-  case VT_DISPATCH:
-  case VT_SAFEARRAY:
-  case VT_RECORD:
-    return 0;
-  default:
-    FIXME("unhandled VT %d\n", vt);
-    return 0;
-  }
+static unsigned int get_type_size(unsigned long *pFlags, VARIANT *pvar)
+{
+    if (V_VT(pvar) & VT_ARRAY) return 4;
+
+    switch (V_VT(pvar) & ~VT_BYREF) {
+    case VT_EMPTY:
+    case VT_NULL:
+        return 0;
+    case VT_I1:
+    case VT_UI1:
+        return sizeof(CHAR);
+    case VT_I2:
+    case VT_UI2:
+        return sizeof(SHORT);
+    case VT_I4:
+    case VT_UI4:
+        return sizeof(LONG);
+    case VT_INT:
+    case VT_UINT:
+        return sizeof(INT);
+    case VT_R4:
+        return sizeof(FLOAT);
+    case VT_R8:
+        return sizeof(DOUBLE);
+    case VT_BOOL:
+        return sizeof(VARIANT_BOOL);
+    case VT_ERROR:
+        return sizeof(SCODE);
+    case VT_DATE:
+        return sizeof(DATE);
+    case VT_CY:
+        return sizeof(CY);
+    case VT_DECIMAL:
+        return sizeof(DECIMAL);
+    case VT_BSTR:
+        return sizeof(BSTR);
+    case VT_VARIANT:
+        return sizeof(VARIANT);
+    case VT_UNKNOWN:
+    case VT_DISPATCH:
+    case VT_RECORD:
+        return 0;
+    default:
+        FIXME("unhandled VT %d\n", V_VT(pvar));
+        return 0;
+    }
+}
+
+static unsigned int get_type_alignment(unsigned long *pFlags, VARIANT *pvar)
+{
+    unsigned int size = get_type_size(pFlags, pvar);
+    if(V_VT(pvar) & VT_BYREF) return 3;
+    if(size == 0) return 0;
+    if(size <= 4) return size - 1;
+    return 7;
 }
 
 static unsigned interface_variant_size(unsigned long *pFlags, REFIID riid, VARIANT *pvar)
@@ -219,32 +300,36 @@ static unsigned interface_variant_size(unsigned long *pFlags, REFIID riid, VARIA
   return size;
 }
 
-static unsigned wire_extra(unsigned long *pFlags, VARIANT *pvar)
+static unsigned long wire_extra_user_size(unsigned long *pFlags, unsigned long Start, VARIANT *pvar)
 {
-  if (V_ISARRAY(pvar)) {
-    FIXME("wire-size safearray\n");
-    return 0;
+  if (V_ISARRAY(pvar))
+  {
+    if (V_ISBYREF(pvar))
+      return LPSAFEARRAY_UserSize(pFlags, Start, V_ARRAYREF(pvar));
+    else 
+      return LPSAFEARRAY_UserSize(pFlags, Start, &V_ARRAY(pvar));
   }
+
   switch (V_VT(pvar)) {
   case VT_BSTR:
-    return BSTR_UserSize(pFlags, 0, &V_BSTR(pvar));
+    return BSTR_UserSize(pFlags, Start, &V_BSTR(pvar));
   case VT_BSTR | VT_BYREF:
-    return BSTR_UserSize(pFlags, 0, V_BSTRREF(pvar));
-  case VT_SAFEARRAY:
-  case VT_SAFEARRAY | VT_BYREF:
-    FIXME("wire-size safearray\n");
-    return 0;
+    return BSTR_UserSize(pFlags, Start, V_BSTRREF(pvar));
   case VT_VARIANT | VT_BYREF:
-    return VARIANT_UserSize(pFlags, 0, V_VARIANTREF(pvar));
+    return VARIANT_UserSize(pFlags, Start, V_VARIANTREF(pvar));
   case VT_UNKNOWN:
-    return interface_variant_size(pFlags, &IID_IUnknown, pvar);
+    return Start + interface_variant_size(pFlags, &IID_IUnknown, pvar);
   case VT_DISPATCH:
-    return interface_variant_size(pFlags, &IID_IDispatch, pvar);
+    return Start + interface_variant_size(pFlags, &IID_IDispatch, pvar);
   case VT_RECORD:
     FIXME("wire-size record\n");
-    return 0;
+    return Start;
+  case VT_SAFEARRAY:
+  case VT_SAFEARRAY | VT_BYREF:
+    FIXME("wire-size safearray: shouldn't be marshaling this\n");
+    return Start;
   default:
-    return 0;
+    return Start;
   }
 }
 
@@ -268,7 +353,7 @@ static unsigned char* interface_variant_marshal(unsigned long *pFlags, unsigned
    * but that would be overkill here, hence this implementation. We save the size because the unmarshal
    * code has no way to know how long the marshalled buffer is. */
 
-  size = wire_extra(pFlags, pvar);
+  size = wire_extra_user_size(pFlags, 0, pvar);
   
   working_mem = GlobalAlloc(0, size);
   if (!working_mem) return oldpos;
@@ -287,14 +372,14 @@ static unsigned char* interface_variant_marshal(unsigned long *pFlags, unsigned
 
   working_memlocked = GlobalLock(working_mem);
   memcpy(Buffer, &size, sizeof(ULONG)); /* copy the buffersize */
-  Buffer += sizeof(ULONG);
-  memcpy(Buffer, working_memlocked, size);
+  memcpy(Buffer + sizeof(ULONG), working_memlocked, size - sizeof(ULONG));
   GlobalUnlock(working_mem);
 
   IStream_Release(working);
 
-  TRACE("done, size=%ld\n", sizeof(ULONG) + size);
-  return Buffer + sizeof(ULONG) + size;
+  /* size includes the ULONG for the size written above */
+  TRACE("done, size=%ld\n", size);
+  return Buffer + size;
 }
 
 /* helper: called for VT_DISPATCH / VT_UNKNOWN variants to unmarshal the buffer. returns Buffer on failure, new position otherwise */
@@ -314,8 +399,7 @@ static unsigned char *interface_variant_unmarshal(unsigned long *pFlags, unsigne
   /* get the buffersize */
   memcpy(&size, Buffer, sizeof(ULONG));
   TRACE("buffersize=%ld\n", size);
-  Buffer += sizeof(ULONG);
-  
+
   working_mem = GlobalAlloc(0, size);
   if (!working_mem) return oldpos;
 
@@ -328,7 +412,7 @@ static unsigned char *interface_variant_unmarshal(unsigned long *pFlags, unsigne
   working_memlocked = GlobalLock(working_mem);
   
   /* now we copy the contents of the marshalling buffer to working_memlocked, unlock it, and demarshal the stream */
-  memcpy(working_memlocked, Buffer, size);
+  memcpy(working_memlocked, Buffer + sizeof(ULONG), size);
   GlobalUnlock(working_mem);
 
   hr = CoUnmarshalInterface(working, riid, (void**)&V_UNKNOWN(pvar));
@@ -339,162 +423,217 @@ static unsigned char *interface_variant_unmarshal(unsigned long *pFlags, unsigne
 
   IStream_Release(working); /* this also frees the underlying hglobal */
 
-  TRACE("done, processed=%ld bytes\n", sizeof(ULONG) + size);
-  return Buffer + sizeof(ULONG) + size;
+  /* size includes the ULONG for the size written above */
+  TRACE("done, processed=%ld bytes\n", size);
+  return Buffer + size;
 }
 
 
 unsigned long WINAPI VARIANT_UserSize(unsigned long *pFlags, unsigned long Start, VARIANT *pvar)
 {
-  TRACE("(%lx,%ld,%p)\n", *pFlags, Start, pvar);
-  TRACE("vt=%04x\n", V_VT(pvar));
-  Start += VARIANT_wiresize + wire_extra(pFlags, pvar);
-  TRACE("returning %ld\n", Start);
-  return Start;
+    int align;
+    TRACE("(%lx,%ld,%p)\n", *pFlags, Start, pvar);
+    TRACE("vt=%04x\n", V_VT(pvar));
+
+    ALIGN_LENGTH(Start, 7);
+    Start += sizeof(variant_wire_t);
+    if(V_VT(pvar) & VT_BYREF)
+        Start += 4;
+
+    align = get_type_alignment(pFlags, pvar);
+    ALIGN_LENGTH(Start, align);
+    if(V_VT(pvar) == (VT_VARIANT | VT_BYREF))
+        Start += 4;
+    else
+        Start += get_type_size(pFlags, pvar);
+    Start = wire_extra_user_size(pFlags, Start, pvar);
+
+    TRACE("returning %ld\n", Start);
+    return Start;
 }
 
 unsigned char * WINAPI VARIANT_UserMarshal(unsigned long *pFlags, unsigned char *Buffer, VARIANT *pvar)
 {
-  wireVARIANT var = (wireVARIANT)Buffer;
-  unsigned size, extra;
-  unsigned char *Pos = Buffer + VARIANT_wiresize;
+    variant_wire_t *header;
+    unsigned long type_size;
+    int align;
+    unsigned char *Pos;
 
-  TRACE("(%lx,%p,%p)\n", *pFlags, Buffer, pvar);
-  TRACE("vt=%04x\n", V_VT(pvar));
+    TRACE("(%lx,%p,%p)\n", *pFlags, Buffer, pvar);
+    TRACE("vt=%04x\n", V_VT(pvar));
 
-  memset(var, 0, sizeof(*var));
-  var->clSize = sizeof(*var);
-  var->vt = pvar->n1.n2.vt;
+    ALIGN_POINTER(Buffer, 7);
 
-  var->rpcReserved = var->vt;
-  if ((var->vt & VT_ARRAY) ||
-      ((var->vt & VT_TYPEMASK) == VT_SAFEARRAY))
-    var->vt = VT_ARRAY | (var->vt & VT_BYREF);
+    header = (variant_wire_t *)Buffer; 
 
-  if (var->vt == VT_DECIMAL) {
-    /* special case because decVal is on a different level */
-    var->u.decVal = pvar->n1.decVal;
-    return Pos;
-  }
+    header->clSize = 0; /* fixed up at the end */
+    header->rpcReserverd = 0;
+    header->vt = pvar->n1.n2.vt;
+    header->wReserved1 = pvar->n1.n2.wReserved1;
+    header->wReserved2 = pvar->n1.n2.wReserved2;
+    header->wReserved3 = pvar->n1.n2.wReserved3;
+    header->switch_is = pvar->n1.n2.vt;
+    if(header->switch_is & VT_ARRAY)
+        header->switch_is &= ~VT_TYPEMASK;
 
-  size = wire_size(V_VT(pvar));
-  extra = wire_extra(pFlags, pvar);
-  var->wReserved1 = pvar->n1.n2.wReserved1;
-  var->wReserved2 = pvar->n1.n2.wReserved2;
-  var->wReserved3 = pvar->n1.n2.wReserved3;
-  if (size) {
-    if (var->vt & VT_BYREF)
-      memcpy(&var->u.cVal, pvar->n1.n2.n3.byref, size);
+    Pos = (unsigned char*)(header + 1);
+    type_size = get_type_size(pFlags, pvar);
+    align = get_type_alignment(pFlags, pvar);
+    ALIGN_POINTER(Pos, align);
+
+    if(header->vt & VT_BYREF)
+    {
+        *(DWORD *)Pos = max(type_size, 4);
+        Pos += 4;
+        if((header->vt & VT_TYPEMASK) != VT_VARIANT)
+        {
+            memcpy(Pos, pvar->n1.n2.n3.byref, type_size);
+            Pos += type_size;
+        }
+        else
+        {
+            *(DWORD*)Pos = 'U' | 's' << 8 | 'e' << 16 | 'r' << 24;
+            Pos += 4;
+        }
+    } 
     else
-      memcpy(&var->u.cVal, &pvar->n1.n2.n3, size);
-  }
-  if (!extra) return Pos;
+    {
+        if((header->vt & VT_TYPEMASK) == VT_DECIMAL)
+            memcpy(Pos, pvar, type_size);
+        else
+            memcpy(Pos, &pvar->n1.n2.n3, type_size);
+        Pos += type_size;
+    }
 
-  switch (var->vt) {
-  case VT_BSTR:
-    Pos = BSTR_UserMarshal(pFlags, Pos, &V_BSTR(pvar));
-    break;
-  case VT_BSTR | VT_BYREF:
-    Pos = BSTR_UserMarshal(pFlags, Pos, V_BSTRREF(pvar));
-    break;
-  case VT_VARIANT | VT_BYREF:
-    Pos = VARIANT_UserMarshal(pFlags, Pos, V_VARIANTREF(pvar));
-    break;
-  case VT_DISPATCH | VT_BYREF:
-    FIXME("handle DISPATCH by ref\n");
-    break;
-  case VT_UNKNOWN:
-    /* this should probably call WdtpInterfacePointer_UserMarshal in ole32.dll */
-    Pos = interface_variant_marshal(pFlags, Pos, &IID_IUnknown, pvar);
-    break;
-  case VT_DISPATCH:
-    /* this should probably call WdtpInterfacePointer_UserMarshal in ole32.dll */
-    Pos = interface_variant_marshal(pFlags, Pos, &IID_IDispatch, pvar);
-    break;
-  case VT_RECORD:
-    FIXME("handle BRECORD by val\n");
-    break;
-  case VT_RECORD | VT_BYREF:
-    FIXME("handle BRECORD by ref\n");
-    break;
-  default:
-    FIXME("handle unknown complex type\n");
-    break;
-  }
-  var->clSize = Pos - Buffer;
-  TRACE("marshalled size=%ld\n", var->clSize);
-  return Pos;
+    if(header->vt & VT_ARRAY)
+    {
+        if(header->vt & VT_BYREF)
+            Pos = LPSAFEARRAY_UserMarshal(pFlags, Pos, V_ARRAYREF(pvar));
+        else
+            Pos = LPSAFEARRAY_UserMarshal(pFlags, Pos, &V_ARRAY(pvar));
+    }
+    else
+    {
+        switch (header->vt)
+        {
+        case VT_BSTR:
+            Pos = BSTR_UserMarshal(pFlags, Pos, &V_BSTR(pvar));
+            break;
+        case VT_BSTR | VT_BYREF:
+            Pos = BSTR_UserMarshal(pFlags, Pos, V_BSTRREF(pvar));
+            break;
+        case VT_VARIANT | VT_BYREF:
+            Pos = VARIANT_UserMarshal(pFlags, Pos, V_VARIANTREF(pvar));
+            break;
+        case VT_DISPATCH | VT_BYREF:
+            FIXME("handle DISPATCH by ref\n");
+            break;
+        case VT_UNKNOWN:
+            /* this should probably call WdtpInterfacePointer_UserMarshal in ole32.dll */
+            Pos = interface_variant_marshal(pFlags, Pos, &IID_IUnknown, pvar);
+            break;
+        case VT_DISPATCH:
+            /* this should probably call WdtpInterfacePointer_UserMarshal in ole32.dll */
+            Pos = interface_variant_marshal(pFlags, Pos, &IID_IDispatch, pvar);
+            break;
+        case VT_RECORD:
+            FIXME("handle BRECORD by val\n");
+            break;
+        case VT_RECORD | VT_BYREF:
+            FIXME("handle BRECORD by ref\n");
+            break;
+        }
+    }
+    header->clSize = ((Pos - Buffer) + 7) >> 3;
+    TRACE("marshalled size=%ld\n", header->clSize);
+    return Pos;
 }
 
 unsigned char * WINAPI VARIANT_UserUnmarshal(unsigned long *pFlags, unsigned char *Buffer, VARIANT *pvar)
 {
-  wireVARIANT var = (wireVARIANT)Buffer;
-  unsigned size;
-  unsigned char *Pos = Buffer + VARIANT_wiresize;
+    variant_wire_t *header;
+    unsigned long type_size;
+    int align;
+    unsigned char *Pos;
 
-  TRACE("(%lx,%p,%p)\n", *pFlags, Buffer, pvar);
-  VariantInit(pvar);
-  pvar->n1.n2.vt = var->rpcReserved;
-  TRACE("marshalled: clSize=%ld, vt=%04x\n", var->clSize, var->vt);
-  TRACE("vt=%04x\n", V_VT(pvar));
-  TRACE("reserved: %d, %d, %d\n", var->wReserved1, var->wReserved2, var->wReserved3);
-  TRACE("val: %ld\n", var->u.lVal);
+    TRACE("(%lx,%p,%p)\n", *pFlags, Buffer, pvar);
 
-  if (var->vt == VT_DECIMAL) {
-    /* special case because decVal is on a different level */
-    pvar->n1.decVal = var->u.decVal;
-    return Pos;
-  }
+    ALIGN_POINTER(Buffer, 7);
+    VariantInit(pvar);
 
-  size = wire_size(V_VT(pvar));
-  pvar->n1.n2.wReserved1 = var->wReserved1;
-  pvar->n1.n2.wReserved2 = var->wReserved2;
-  pvar->n1.n2.wReserved3 = var->wReserved3;
-  if (size) {
-    if (var->vt & VT_BYREF) {
-      pvar->n1.n2.n3.byref = CoTaskMemAlloc(size);
-      memcpy(pvar->n1.n2.n3.byref, &var->u.cVal, size);
+    header = (variant_wire_t *)Buffer; 
+    
+    pvar->n1.n2.vt = header->vt;
+    pvar->n1.n2.wReserved1 = header->wReserved1;
+    pvar->n1.n2.wReserved2 = header->wReserved2;
+    pvar->n1.n2.wReserved3 = header->wReserved3;
+
+    Pos = (unsigned char*)(header + 1);
+    type_size = get_type_size(pFlags, pvar);
+    align = get_type_alignment(pFlags, pvar);
+    ALIGN_POINTER(Pos, align);
+
+    if(header->vt & VT_BYREF)
+    {
+        Pos += 4;
+        pvar->n1.n2.n3.byref = CoTaskMemAlloc(type_size);
+        memcpy(pvar->n1.n2.n3.byref, Pos, type_size); 
+        if((header->vt & VT_TYPEMASK) != VT_VARIANT)
+           Pos += type_size;
+        else
+            Pos += 4;
     }
     else
-      memcpy(&pvar->n1.n2.n3, &var->u.cVal, size);
-  }
-  if (var->clSize <= VARIANT_wiresize) return Pos;
+    {
+        if((header->vt & VT_TYPEMASK) == VT_DECIMAL)
+            memcpy(pvar, Pos, type_size);
+        else
+            memcpy(&pvar->n1.n2.n3, Pos, type_size);
+        Pos += type_size;
+    }
 
-  switch (var->vt) {
-  case VT_BSTR:
-    Pos = BSTR_UserUnmarshal(pFlags, Pos, &V_BSTR(pvar));
-    break;
-  case VT_BSTR | VT_BYREF:
-    pvar->n1.n2.n3.byref = CoTaskMemAlloc(sizeof(BSTR));
-    *(BSTR*)pvar->n1.n2.n3.byref = NULL;
-    Pos = BSTR_UserUnmarshal(pFlags, Pos, V_BSTRREF(pvar));
-    break;
-  case VT_VARIANT | VT_BYREF:
-    pvar->n1.n2.n3.byref = CoTaskMemAlloc(sizeof(VARIANT));
-    Pos = VARIANT_UserUnmarshal(pFlags, Pos, V_VARIANTREF(pvar));
-    break;
-  case VT_RECORD:
-    FIXME("handle BRECORD by val\n");
-    break;
-  case VT_RECORD | VT_BYREF:
-    FIXME("handle BRECORD by ref\n");
-    break;
-  case VT_UNKNOWN:
-    Pos = interface_variant_unmarshal(pFlags, Pos, &IID_IUnknown, pvar);
-    break;
-  case VT_DISPATCH:
-    Pos = interface_variant_unmarshal(pFlags, Pos, &IID_IDispatch, pvar);
-    break;
-  case VT_DISPATCH | VT_BYREF:
-    FIXME("handle DISPATCH by ref\n");
-  default:
-    FIXME("handle unknown complex type\n");
-    break;
-  }
-  if (Pos != Buffer + var->clSize) {
-    ERR("size difference during unmarshal\n");
-  }
-  return Buffer + var->clSize;
+    if(header->vt & VT_ARRAY)
+    {
+        if(header->vt & VT_BYREF)
+            Pos = LPSAFEARRAY_UserUnmarshal(pFlags, Pos, V_ARRAYREF(pvar));
+        else
+            Pos = LPSAFEARRAY_UserUnmarshal(pFlags, Pos, &V_ARRAY(pvar));
+    }
+    else
+    {
+        switch (header->vt)
+        {
+        case VT_BSTR:
+            V_BSTR(pvar) = NULL;
+            Pos = BSTR_UserUnmarshal(pFlags, Pos, &V_BSTR(pvar));
+            break;
+        case VT_BSTR | VT_BYREF:
+            *V_BSTRREF(pvar) = NULL;
+            Pos = BSTR_UserUnmarshal(pFlags, Pos, V_BSTRREF(pvar));
+            break;
+        case VT_VARIANT | VT_BYREF:
+            Pos = VARIANT_UserUnmarshal(pFlags, Pos, V_VARIANTREF(pvar));
+            break;
+        case VT_DISPATCH | VT_BYREF:
+            FIXME("handle DISPATCH by ref\n");
+            break;
+        case VT_UNKNOWN:
+            /* this should probably call WdtpInterfacePointer_UserUnmarshal in ole32.dll */
+            Pos = interface_variant_unmarshal(pFlags, Pos, &IID_IUnknown, pvar);
+            break;
+        case VT_DISPATCH:
+            /* this should probably call WdtpInterfacePointer_UserUnmarshal in ole32.dll */
+            Pos = interface_variant_unmarshal(pFlags, Pos, &IID_IDispatch, pvar);
+            break;
+        case VT_RECORD:
+            FIXME("handle BRECORD by val\n");
+            break;
+        case VT_RECORD | VT_BYREF:
+            FIXME("handle BRECORD by ref\n");
+            break;
+        }
+    }
+    return Pos;
 }
 
 void WINAPI VARIANT_UserFree(unsigned long *pFlags, VARIANT *pvar)
@@ -510,23 +649,26 @@ void WINAPI VARIANT_UserFree(unsigned long *pFlags, VARIANT *pvar)
   VariantClear(pvar);
   if (!ref) return;
 
-  switch (vt) {
-  case VT_BSTR | VT_BYREF:
-    BSTR_UserFree(pFlags, ref);
-    break;
-  case VT_VARIANT | VT_BYREF:
-    VARIANT_UserFree(pFlags, ref);
-    break;
-  case VT_RECORD | VT_BYREF:
-    FIXME("handle BRECORD by ref\n");
-    break;
-  case VT_UNKNOWN:
-  case VT_DISPATCH:
-    IUnknown_Release(V_UNKNOWN(pvar));
-    break;
-  default:
-    FIXME("handle unknown complex type\n");
-    break;
+  if(vt & VT_ARRAY)
+    LPSAFEARRAY_UserFree(pFlags, V_ARRAYREF(pvar));
+  else
+  {
+    switch (vt)
+    {
+    case VT_BSTR | VT_BYREF:
+      BSTR_UserFree(pFlags, V_BSTRREF(pvar));
+      break;
+    case VT_VARIANT | VT_BYREF:
+      VARIANT_UserFree(pFlags, V_VARIANTREF(pvar));
+      break;
+    case VT_RECORD | VT_BYREF:
+      FIXME("handle BRECORD by ref\n");
+      break;
+    case VT_UNKNOWN | VT_BYREF:
+    case VT_DISPATCH | VT_BYREF:
+      IUnknown_Release(*V_UNKNOWNREF(pvar));
+      break;
+    }
   }
 
   CoTaskMemFree(ref);
@@ -601,6 +743,7 @@ unsigned long WINAPI LPSAFEARRAY_UserSize(unsigned long *pFlags, unsigned long S
 
     TRACE("("); dump_user_flags(pFlags); TRACE(", %ld, %p\n", StartingSize, *ppsa);
 
+    ALIGN_LENGTH(size, 3);
     size += sizeof(ULONG_PTR);
     if (*ppsa)
     {
@@ -665,10 +808,12 @@ unsigned long WINAPI LPSAFEARRAY_UserSize(unsigned long *pFlags, unsigned long S
                 }
                 break;
             }
+            case SF_I8:
+                ALIGN_LENGTH(size, 7);
+                /* fallthrough */
             case SF_I1:
             case SF_I2:
             case SF_I4:
-            case SF_I8:
                 size += ulCellCount * psa->cbElements;
                 break;
             default:
@@ -686,6 +831,7 @@ unsigned char * WINAPI LPSAFEARRAY_UserMarshal(unsigned long *pFlags, unsigned c
 
     TRACE("("); dump_user_flags(pFlags); TRACE(", %p, &%p\n", Buffer, *ppsa);
 
+    ALIGN_POINTER(Buffer, 3);
     *(ULONG_PTR *)Buffer = *ppsa ? TRUE : FALSE;
     Buffer += sizeof(ULONG_PTR);
     if (*ppsa)
@@ -775,10 +921,13 @@ unsigned char * WINAPI LPSAFEARRAY_UserMarshal(unsigned long *pFlags, unsigned c
                     }
                     break;
                 }
+
+                case SF_I8:
+                    ALIGN_POINTER(Buffer, 7);
+                    /* fallthrough */
                 case SF_I1:
                 case SF_I2:
                 case SF_I4:
-                case SF_I8:
                     /* Just copy the data over */
                     memcpy(Buffer, psa->pvData, ulCellCount * psa->cbElements);
                     Buffer += ulCellCount * psa->cbElements;
@@ -810,6 +959,7 @@ unsigned char * WINAPI LPSAFEARRAY_UserUnmarshal(unsigned long *pFlags, unsigned
 
     TRACE("("); dump_user_flags(pFlags); TRACE(", %p, %p\n", Buffer, ppsa);
 
+    ALIGN_POINTER(Buffer, 3);
     ptr = *(ULONG_PTR *)Buffer;
     Buffer += sizeof(ULONG_PTR);
 
@@ -857,7 +1007,8 @@ unsigned char * WINAPI LPSAFEARRAY_UserUnmarshal(unsigned long *pFlags, unsigned
 
     /* be careful about which flags we set since they could be a security
      * risk */
-    (*ppsa)->fFeatures = wiresa->fFeatures & ~(FADF_AUTOSETFLAGS);
+    (*ppsa)->fFeatures &= FADF_AUTOSETFLAGS;
+    (*ppsa)->fFeatures |= (wiresa->fFeatures & ~(FADF_AUTOSETFLAGS));
     /* FIXME: there should be a limit on how large wiresa->cbElements can be */
     (*ppsa)->cbElements = wiresa->cbElements;
     (*ppsa)->cLocks = LOWORD(wiresa->cLocks);
@@ -903,10 +1054,13 @@ unsigned char * WINAPI LPSAFEARRAY_UserUnmarshal(unsigned long *pFlags, unsigned
 
                 break;
             }
+
+            case SF_I8:
+                ALIGN_POINTER(Buffer, 7);
+                /* fallthrough */
             case SF_I1:
             case SF_I2:
             case SF_I4:
-            case SF_I8:
                 /* Just copy the data over */
                 memcpy((*ppsa)->pvData, Buffer, cell_count * (*ppsa)->cbElements);
                 Buffer += cell_count * (*ppsa)->cbElements;
@@ -1128,6 +1282,48 @@ HRESULT __RPC_STUB IEnumVARIANT_Next_Stub(
   return hr;
 }
 
+/* TypeInfo related freers */
+
+static void free_embedded_typedesc(TYPEDESC *tdesc);
+static void free_embedded_arraydesc(ARRAYDESC *adesc)
+{
+    switch(adesc->tdescElem.vt)
+    {
+    case VT_PTR:
+    case VT_SAFEARRAY:
+        free_embedded_typedesc(adesc->tdescElem.u.lptdesc);
+        CoTaskMemFree(adesc->tdescElem.u.lptdesc);
+        break;
+    case VT_CARRAY:
+        free_embedded_arraydesc(adesc->tdescElem.u.lpadesc);
+        CoTaskMemFree(adesc->tdescElem.u.lpadesc);
+        break;
+    }
+}
+
+static void free_embedded_typedesc(TYPEDESC *tdesc)
+{
+    switch(tdesc->vt)
+    {
+    case VT_PTR:
+    case VT_SAFEARRAY:
+        free_embedded_typedesc(tdesc->u.lptdesc);
+        CoTaskMemFree(tdesc->u.lptdesc);
+        break;
+    case VT_CARRAY:
+        free_embedded_arraydesc(tdesc->u.lpadesc);
+        CoTaskMemFree(tdesc->u.lpadesc);
+        break;
+    }
+}
+
+static void free_embedded_elemdesc(ELEMDESC *edesc)
+{
+    free_embedded_typedesc(&edesc->tdesc);
+    if(edesc->u.paramdesc.wParamFlags & PARAMFLAG_FHASDEFAULT)
+        CoTaskMemFree(edesc->u.paramdesc.pparamdescex);
+}
+
 /* ITypeComp */
 
 HRESULT CALLBACK ITypeComp_Bind_Proxy(
@@ -1185,9 +1381,16 @@ HRESULT __RPC_STUB ITypeComp_BindType_Stub(
 HRESULT CALLBACK ITypeInfo_GetTypeAttr_Proxy(
     ITypeInfo* This,
     TYPEATTR** ppTypeAttr)
+
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    CLEANLOCALSTORAGE stg;
+    TRACE("(%p, %p)\n", This, ppTypeAttr);
+
+    stg.flags = 0;
+    stg.pStorage = NULL;
+    stg.pInterface = NULL;
+
+    return ITypeInfo_RemoteGetTypeAttr_Proxy(This, ppTypeAttr, &stg);
 }
 
 HRESULT __RPC_STUB ITypeInfo_GetTypeAttr_Stub(
@@ -1195,8 +1398,18 @@ HRESULT __RPC_STUB ITypeInfo_GetTypeAttr_Stub(
     LPTYPEATTR* ppTypeAttr,
     CLEANLOCALSTORAGE* pDummy)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    HRESULT hr;
+    TRACE("(%p, %p)\n", This, ppTypeAttr);
+
+    hr = ITypeInfo_GetTypeAttr(This, ppTypeAttr);
+    if(hr != S_OK)
+        return hr;
+
+    pDummy->flags = CLS_TYPEATTR;
+    ITypeInfo_AddRef(This);
+    pDummy->pInterface = (IUnknown*)This;
+    pDummy->pStorage = ppTypeAttr;
+    return hr;
 }
 
 HRESULT CALLBACK ITypeInfo_GetFuncDesc_Proxy(
@@ -1204,8 +1417,14 @@ HRESULT CALLBACK ITypeInfo_GetFuncDesc_Proxy(
     UINT index,
     FUNCDESC** ppFuncDesc)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    CLEANLOCALSTORAGE stg;
+    TRACE("(%p, %d, %p)\n", This, index, ppFuncDesc);
+
+    stg.flags = 0;
+    stg.pStorage = NULL;
+    stg.pInterface = NULL;
+
+    return ITypeInfo_RemoteGetFuncDesc_Proxy(This, index, ppFuncDesc, &stg);
 }
 
 HRESULT __RPC_STUB ITypeInfo_GetFuncDesc_Stub(
@@ -1214,8 +1433,18 @@ HRESULT __RPC_STUB ITypeInfo_GetFuncDesc_Stub(
     LPFUNCDESC* ppFuncDesc,
     CLEANLOCALSTORAGE* pDummy)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    HRESULT hr;
+    TRACE("(%p, %d, %p)\n", This, index, ppFuncDesc);
+
+    hr = ITypeInfo_GetFuncDesc(This, index, ppFuncDesc);
+    if(hr != S_OK)
+        return hr;
+
+    pDummy->flags = CLS_FUNCDESC;
+    ITypeInfo_AddRef(This);
+    pDummy->pInterface = (IUnknown*)This;
+    pDummy->pStorage = ppFuncDesc;
+    return hr;
 }
 
 HRESULT CALLBACK ITypeInfo_GetVarDesc_Proxy(
@@ -1223,8 +1452,14 @@ HRESULT CALLBACK ITypeInfo_GetVarDesc_Proxy(
     UINT index,
     VARDESC** ppVarDesc)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    CLEANLOCALSTORAGE stg;
+    TRACE("(%p, %d, %p)\n", This, index, ppVarDesc);
+
+    stg.flags = 0;
+    stg.pStorage = NULL;
+    stg.pInterface = NULL;
+
+    return ITypeInfo_RemoteGetVarDesc_Proxy(This, index, ppVarDesc, &stg);
 }
 
 HRESULT __RPC_STUB ITypeInfo_GetVarDesc_Stub(
@@ -1233,8 +1468,18 @@ HRESULT __RPC_STUB ITypeInfo_GetVarDesc_Stub(
     LPVARDESC* ppVarDesc,
     CLEANLOCALSTORAGE* pDummy)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    HRESULT hr;
+    TRACE("(%p, %d, %p)\n", This, index, ppVarDesc);
+
+    hr = ITypeInfo_GetVarDesc(This, index, ppVarDesc);
+    if(hr != S_OK)
+        return hr;
+
+    pDummy->flags = CLS_VARDESC;
+    ITypeInfo_AddRef(This);
+    pDummy->pInterface = (IUnknown*)This;
+    pDummy->pStorage = ppVarDesc;
+    return hr;
 }
 
 HRESULT CALLBACK ITypeInfo_GetNames_Proxy(
@@ -1305,8 +1550,27 @@ HRESULT CALLBACK ITypeInfo_GetDocumentation_Proxy(
     DWORD* pdwHelpContext,
     BSTR* pBstrHelpFile)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    DWORD help_context;
+    BSTR name, doc_string, help_file;
+    HRESULT hr;
+    TRACE("(%p, %08lx, %p, %p, %p, %p)\n", This, memid, pBstrName, pBstrDocString, pdwHelpContext, pBstrHelpFile);
+
+    /* FIXME: presumably refPtrFlags is supposed to be a bitmask of which ptrs we actually want? */
+    hr = ITypeInfo_RemoteGetDocumentation_Proxy(This, memid, 0, &name, &doc_string, &help_context, &help_file);
+    if(SUCCEEDED(hr))
+    {
+        if(pBstrName) *pBstrName = name;
+        else SysFreeString(name);
+
+        if(pBstrDocString) *pBstrDocString = doc_string;
+        else SysFreeString(doc_string);
+
+        if(pBstrHelpFile) *pBstrHelpFile = help_file;
+        else SysFreeString(help_file);
+
+        if(pdwHelpContext) *pdwHelpContext = help_context;
+    }
+    return hr;
 }
 
 HRESULT __RPC_STUB ITypeInfo_GetDocumentation_Stub(
@@ -1318,8 +1582,9 @@ HRESULT __RPC_STUB ITypeInfo_GetDocumentation_Stub(
     DWORD* pdwHelpContext,
     BSTR* pBstrHelpFile)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    TRACE("(%p, %08lx, %08lx, %p, %p, %p, %p)\n", This, memid, refPtrFlags, pBstrName, pBstrDocString,
+          pdwHelpContext, pBstrHelpFile);
+    return ITypeInfo_GetDocumentation(This, memid, pBstrName, pBstrDocString, pdwHelpContext, pBstrHelpFile);
 }
 
 HRESULT CALLBACK ITypeInfo_GetDllEntry_Proxy(
@@ -1421,42 +1686,66 @@ void CALLBACK ITypeInfo_ReleaseTypeAttr_Proxy(
     ITypeInfo* This,
     TYPEATTR* pTypeAttr)
 {
-  FIXME("not implemented\n");
+    TRACE("(%p, %p)\n", This, pTypeAttr);
+    free_embedded_typedesc(&pTypeAttr->tdescAlias);
+    CoTaskMemFree(pTypeAttr);
 }
 
 HRESULT __RPC_STUB ITypeInfo_ReleaseTypeAttr_Stub(
     ITypeInfo* This)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    TRACE("nothing to do\n");
+    return S_OK;
 }
 
 void CALLBACK ITypeInfo_ReleaseFuncDesc_Proxy(
     ITypeInfo* This,
     FUNCDESC* pFuncDesc)
 {
-  FIXME("not implemented\n");
+    SHORT param;
+    TRACE("(%p, %p)\n", This, pFuncDesc);
+
+    for(param = 0; param < pFuncDesc->cParams; param++)
+        free_embedded_elemdesc(pFuncDesc->lprgelemdescParam + param);
+    if(param)
+        CoTaskMemFree(pFuncDesc->lprgelemdescParam);
+
+    free_embedded_elemdesc(&pFuncDesc->elemdescFunc);
+
+    if(pFuncDesc->cScodes != 0 && pFuncDesc->cScodes != -1)
+        CoTaskMemFree(pFuncDesc->lprgscode);
+
+    CoTaskMemFree(pFuncDesc);
 }
 
 HRESULT __RPC_STUB ITypeInfo_ReleaseFuncDesc_Stub(
     ITypeInfo* This)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    TRACE("nothing to do\n");
+    return S_OK;
 }
 
 void CALLBACK ITypeInfo_ReleaseVarDesc_Proxy(
     ITypeInfo* This,
     VARDESC* pVarDesc)
 {
-  FIXME("not implemented\n");
+    TRACE("(%p, %p)\n", This, pVarDesc);
+
+    if(pVarDesc->lpstrSchema)
+        CoTaskMemFree(pVarDesc->lpstrSchema);
+
+    if(pVarDesc->varkind == VAR_CONST)
+        CoTaskMemFree(pVarDesc->u.lpvarValue);
+
+    free_embedded_elemdesc(&pVarDesc->elemdescVar);
+    CoTaskMemFree(pVarDesc);
 }
 
 HRESULT __RPC_STUB ITypeInfo_ReleaseVarDesc_Stub(
     ITypeInfo* This)
 {
-  FIXME("not implemented\n");
-  return E_FAIL;
+    TRACE("nothing to do\n");
+    return S_OK;
 }
 
 
@@ -1515,6 +1804,11 @@ HRESULT CALLBACK ITypeLib_GetLibAttr_Proxy(
 {
     CLEANLOCALSTORAGE stg;
     TRACE("(%p, %p)\n", This, ppTLibAttr);
+
+    stg.flags = 0;
+    stg.pStorage = NULL;
+    stg.pInterface = NULL;
+
     return ITypeLib_RemoteGetLibAttr_Proxy(This, ppTLibAttr, &stg);    
 }
 
@@ -1523,8 +1817,18 @@ HRESULT __RPC_STUB ITypeLib_GetLibAttr_Stub(
     LPTLIBATTR* ppTLibAttr,
     CLEANLOCALSTORAGE* pDummy)
 {
+    HRESULT hr;
     TRACE("(%p, %p)\n", This, ppTLibAttr);
-    return ITypeLib_GetLibAttr(This, ppTLibAttr);
+    
+    hr = ITypeLib_GetLibAttr(This, ppTLibAttr);
+    if(hr != S_OK)
+        return hr;
+
+    pDummy->flags = CLS_LIBATTR;
+    ITypeLib_AddRef(This);
+    pDummy->pInterface = (IUnknown*)This;
+    pDummy->pStorage = ppTLibAttr;
+    return hr;
 }
 
 HRESULT CALLBACK ITypeLib_GetDocumentation_Proxy(
