@@ -38,6 +38,7 @@
 #include "objbase.h"
 #include "rpc.h"
 #include "rpcproxy.h"
+#include "ndrtypes.h"
 
 #include "wine/debug.h"
 #include "wine/rpcfc.h"
@@ -47,9 +48,11 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(rpc);
 
+#define NDR_TABLE_MASK 127
+
 static inline void call_buffer_sizer(PMIDL_STUB_MESSAGE pStubMsg, unsigned char *pMemory, PFORMAT_STRING pFormat)
 {
-    NDR_BUFFERSIZE m = NdrBufferSizer[pFormat[0]];
+    NDR_BUFFERSIZE m = NdrBufferSizer[pFormat[0] & NDR_TABLE_MASK];
     if (m) m(pStubMsg, pMemory, pFormat);
     else
     {
@@ -60,7 +63,7 @@ static inline void call_buffer_sizer(PMIDL_STUB_MESSAGE pStubMsg, unsigned char
 
 static inline unsigned char *call_marshaller(PMIDL_STUB_MESSAGE pStubMsg, unsigned char *pMemory, PFORMAT_STRING pFormat)
 {
-    NDR_MARSHALL m = NdrMarshaller[pFormat[0]];
+    NDR_MARSHALL m = NdrMarshaller[pFormat[0] & NDR_TABLE_MASK];
     if (m) return m(pStubMsg, pMemory, pFormat);
     else
     {
@@ -72,7 +75,7 @@ static inline unsigned char *call_marshaller(PMIDL_STUB_MESSAGE pStubMsg, unsign
 
 static inline unsigned char *call_unmarshaller(PMIDL_STUB_MESSAGE pStubMsg, unsigned char **ppMemory, PFORMAT_STRING pFormat, unsigned char fMustAlloc)
 {
-    NDR_UNMARSHALL m = NdrUnmarshaller[pFormat[0]];
+    NDR_UNMARSHALL m = NdrUnmarshaller[pFormat[0] & NDR_TABLE_MASK];
     if (m) return m(pStubMsg, ppMemory, pFormat, fMustAlloc);
     else
     {
@@ -84,7 +87,7 @@ static inline unsigned char *call_unmarshaller(PMIDL_STUB_MESSAGE pStubMsg, unsi
 
 static inline void call_freer(PMIDL_STUB_MESSAGE pStubMsg, unsigned char *pMemory, PFORMAT_STRING pFormat)
 {
-    NDR_FREE m = NdrFreer[pFormat[0]];
+    NDR_FREE m = NdrFreer[pFormat[0] & NDR_TABLE_MASK];
     if (m) m(pStubMsg, pMemory, pFormat);
     else
     {
@@ -95,7 +98,7 @@ static inline void call_freer(PMIDL_STUB_MESSAGE pStubMsg, unsigned char *pMemor
 
 static inline unsigned long call_memory_sizer(PMIDL_STUB_MESSAGE pStubMsg, PFORMAT_STRING pFormat)
 {
-    NDR_MEMORYSIZE m = NdrMemorySizer[pFormat[0]];
+    NDR_MEMORYSIZE m = NdrMemorySizer[pFormat[0] & NDR_TABLE_MASK];
     if (m) return m(pStubMsg, pFormat);
     else
     {
@@ -195,63 +198,12 @@ typedef struct _NDR_PROC_PARTIAL_OIF_HEADER
      * sizing pass */
     unsigned short constant_server_buffer_size;
 
-    /* -Oif flags:
-     * RPC_FC_PROC_OI2F_SRVMUSTSIZE = 0x01 - the server must perform a
-     *   sizing pass.
-     * RPC_FC_PROC_OI2F_CLTMUSTSIZE = 0x02 - the client must perform a
-     *   sizing pass.
-     * RPC_FC_PROC_OI2F_HASRETURN = 0x04 - procedure has a return value.
-     * RPC_FC_PROC_OI2F_HASPIPES = 0x08 - the pipe package should be used.
-     * RPC_FC_PROC_OI2F_HASASYNCUUID = 0x20 - indicates an asynchronous DCOM
-     *   procedure.
-     * RPC_FC_PROC_OI2F_HASEXTS = 0x40 - indicates that Windows 2000
-     *   extensions are in use.
-     * RPC_FC_PROC_OI2F_HASASYNCHND = 0x80 - indicates an asynchronous RPC
-     *   procedure.
-     */
-    unsigned char Oif_flags;
+    INTERPRETER_OPT_FLAGS Oi2Flags;
 
     /* number of params */
     unsigned char number_of_params;
 } NDR_PROC_PARTIAL_OIF_HEADER;
 
-/* Windows 2000 extensions */
-typedef struct _NDR_PROC_EXTENSION
-{
-    /* size in bytes of all following extensions */
-    unsigned char extension_version;
-
-    /* extension flags:
-     * HasNewCorrDesc = 0x01 - indicates new correlation descriptors in use
-     * ClientCorrCheck = 0x02 - client needs correlation check
-     * ServerCorrCheck = 0x04 - server needs correlation check
-     * HasNotify = 0x08 - should call MIDL [notify] routine @ NotifyIndex
-     * HasNotify2 = 0x10 - should call MIDL [notify_flag] routine @ 
-     *   NotifyIndex
-     */
-    unsigned char ext_flags;
-
-    /* client cache size hint */
-    unsigned short ClientCorrHint;
-
-    /* server cache size hint */
-    unsigned short ServerCorrHint;
-
-    /* index of routine in MIDL_STUB_DESC::NotifyRoutineTable to call if
-     * HasNotify or HasNotify2 flag set */
-    unsigned short NotifyIndex;
-} NDR_PROC_EXTENSION;
-
-/* usually generated only on IA64 */
-typedef struct _NDR_PROC_EXTENSION_64
-{
-    NDR_PROC_EXTENSION ext;
-
-    /* needed only on IA64 to cope with float/register loading */
-    unsigned short FloatDoubleMask;
-} NDR_PROC_EXTENSION_64;
-
-
 typedef struct _NDR_PARAM_OI_BASETYPE
 {
     /* parameter direction. One of:
@@ -286,37 +238,7 @@ typedef struct _NDR_PARAM_OI_OTHER
 
 typedef struct _NDR_PARAM_OIF_BASETYPE
 {
-    /* attributes of the parameter:
-     * RPC_FC_PROC_PF_MUSTSIZE = 0x0001 - client interpreter MUST size this
-     *  parameter, other parameters may be skipped, using the value in
-     *  NDR_PROC_PARTIAL_OIF_HEADER::constant_client_buffer_size instead.
-     * RPC_FC_PROC_PF_MUSTFREE = 0x0002 - server interpreter MUST size this
-     *  parameter, other parameters may be skipped, using the value in
-     *  NDR_PROC_PARTIAL_OIF_HEADER::constant_server_buffer_size instead.
-     * RPC_FC_PROC_PF_PIPE = 0x0004 - The parameter is a pipe handle. See
-     *  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/rpc/rpc/pipes.asp
-     *  for more information on pipes.
-     * RPC_FC_PROC_PF_IN = 0x0008 - The parameter is an input
-     * RPC_FC_PROC_PF_OUT = 0x0010 - The parameter is an output
-     * RPC_FC_PROC_PF_RETURN = 0x0020 - The parameter is to be returned
-     * RPC_FC_PROC_PF_BASETYPE = 0x0040 - The parameter is simple and has the
-     *  format defined by NDR_PARAM_OIF_BASETYPE rather than by
-     *  NDR_PARAM_OIF_OTHER.
-     * RPC_FC_PROC_PF_BYVAL = 0x0080 - Set for compound types being sent by
-     *  value. Can be of type: structure, union, transmit_as, represent_as,
-     *  wire_marshal and SAFEARRAY.
-     * RPC_FC_PROC_PF_SIMPLEREF = 0x0100 - parameter that is a reference
-     *  pointer to anything other than another pointer, and which has no
-     *  allocate attributes.
-     * RPC_FC_PROC_PF_DONTFREEINST = 0x0200 - Used for some represent_as types
-     *  for when the free instance routine should not be called.
-     * RPC_FC_PROC_PF_SAVEASYNC = 0x0400 - unknown
-     * RPC_FC_PROC_PF_SRVALLOCSIZE = 0xe000 - bitmask, which if non-zero
-     *  specifies the size of the object in numbers of 8byte blocks needed.
-     *  It will be stored on the server's stack rather than using an allocate
-     *  call.
-     */
-    unsigned short param_attributes;
+    PARAM_ATTRIBUTES param_attributes;
 
     /* the offset on the calling stack where the parameter is located */
     unsigned short stack_offset;
@@ -330,8 +252,7 @@ typedef struct _NDR_PARAM_OIF_BASETYPE
 
 typedef struct _NDR_PARAM_OIF_OTHER
 {
-    /* see NDR_PARAM_OIF_BASETYPE::param_attributes */
-    unsigned short param_attributes;
+    PARAM_ATTRIBUTES param_attributes;
 
     /* see NDR_PARAM_OIF_BASETYPE::stack_offset */
     unsigned short stack_offset;
@@ -418,20 +339,33 @@ void WINAPI NdrRpcSmSetClientToOsf(PMIDL_STUB_MESSAGE pMessage)
 #endif
 }
 
-static void WINAPI dump_RPC_FC_PROC_PF(unsigned short param_attributes)
+static void WINAPI dump_RPC_FC_PROC_PF(PARAM_ATTRIBUTES param_attributes)
 {
-    if (param_attributes & RPC_FC_PROC_PF_MUSTSIZE) TRACE(" RPC_FC_PROC_PF_MUSTSIZE");
-    if (param_attributes & RPC_FC_PROC_PF_MUSTFREE) TRACE(" RPC_FC_PROC_PF_MUSTFREE");
-    if (param_attributes & RPC_FC_PROC_PF_PIPE) TRACE(" RPC_FC_PROC_PF_PIPE");
-    if (param_attributes & RPC_FC_PROC_PF_IN) TRACE(" RPC_FC_PROC_PF_IN");
-    if (param_attributes & RPC_FC_PROC_PF_OUT) TRACE(" RPC_FC_PROC_PF_OUT");
-    if (param_attributes & RPC_FC_PROC_PF_RETURN) TRACE(" RPC_FC_PROC_PF_RETURN");
-    if (param_attributes & RPC_FC_PROC_PF_BASETYPE) TRACE(" RPC_FC_PROC_PF_BASETYPE");
-    if (param_attributes & RPC_FC_PROC_PF_BYVAL) TRACE(" RPC_FC_PROC_PF_BYVAL");
-    if (param_attributes & RPC_FC_PROC_PF_SIMPLEREF) TRACE(" RPC_FC_PROC_PF_SIMPLEREF");
-    if (param_attributes & RPC_FC_PROC_PF_DONTFREEINST) TRACE(" RPC_FC_PROC_PF_DONTFREEINST");
-    if (param_attributes & RPC_FC_PROC_PF_SAVEASYNC) TRACE(" RPC_FC_PROC_PF_SAVEASYNC");
-    if (param_attributes & RPC_FC_PROC_PF_SRVALLOCSIZE) TRACE(" RPC_FC_PROC_PF_SRVALLOCSIZE");
+    if (param_attributes.MustSize) TRACE(" MustSize");
+    if (param_attributes.MustFree) TRACE(" MustFree");
+    if (param_attributes.IsPipe) TRACE(" IsPipe");
+    if (param_attributes.IsIn) TRACE(" IsIn");
+    if (param_attributes.IsOut) TRACE(" IsOut");
+    if (param_attributes.IsReturn) TRACE(" IsReturn");
+    if (param_attributes.IsBasetype) TRACE(" IsBasetype");
+    if (param_attributes.IsByValue) TRACE(" IsByValue");
+    if (param_attributes.IsSimpleRef) TRACE(" IsSimpleRef");
+    if (param_attributes.IsDontCallFreeInst) TRACE(" IsDontCallFreeInst");
+    if (param_attributes.SaveForAsyncFinish) TRACE(" SaveForAsyncFinish");
+    if (param_attributes.ServerAllocSize) TRACE(" ServerAllocSize = %d", param_attributes.ServerAllocSize * 8);
+}
+
+static void WINAPI dump_INTERPRETER_OPT_FLAGS(INTERPRETER_OPT_FLAGS Oi2Flags)
+{
+    if (Oi2Flags.ServerMustSize) TRACE(" ServerMustSize");
+    if (Oi2Flags.ClientMustSize) TRACE(" ClientMustSize");
+    if (Oi2Flags.HasReturn) TRACE(" HasReturn");
+    if (Oi2Flags.HasPipes) TRACE(" HasPipes");
+    if (Oi2Flags.Unused) TRACE(" Unused");
+    if (Oi2Flags.HasAsyncUuid) TRACE(" HasAsyncUuid");
+    if (Oi2Flags.HasExtensions) TRACE(" HasExtensions");
+    if (Oi2Flags.HasAsyncHandle) TRACE(" HasAsyncHandle");
+    TRACE("\n");
 }
 
 /* FIXME: this will be different on other plaftorms than i386 */
@@ -459,9 +393,9 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
     /* counter */
     unsigned short i;
     /* cache of Oif_flags from v2 procedure header */
-    unsigned char Oif_flags = 0;
-    /* cache of extension flags from NDR_PROC_EXTENSION */
-    unsigned char ext_flags = 0;
+    INTERPRETER_OPT_FLAGS Oif_flags = { 0 };
+    /* cache of extension flags from NDR_PROC_HEADER_EXTS */
+    INTERPRETER_OPT_FLAGS2 ext_flags = { 0 };
     /* the type of pass we are currently doing */
     int phase;
     /* header for procedure string */
@@ -579,20 +513,20 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
         NDR_PROC_PARTIAL_OIF_HEADER * pOIFHeader =
             (NDR_PROC_PARTIAL_OIF_HEADER*)&pFormat[current_offset];
 
-        Oif_flags = pOIFHeader->Oif_flags;
+        Oif_flags = pOIFHeader->Oi2Flags;
         number_of_params = pOIFHeader->number_of_params;
 
         current_offset += sizeof(NDR_PROC_PARTIAL_OIF_HEADER);
     }
 
-    TRACE("Oif_flags = 0x%02x\n", Oif_flags);
+    TRACE("Oif_flags = "); dump_INTERPRETER_OPT_FLAGS(Oif_flags);
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASEXTS)
+    if (Oif_flags.HasExtensions)
     {
-        NDR_PROC_EXTENSION * pExtensions =
-            (NDR_PROC_EXTENSION *)&pFormat[current_offset];
-        ext_flags = pExtensions->ext_flags;
-        current_offset += pExtensions->extension_version;
+        NDR_PROC_HEADER_EXTS * pExtensions =
+            (NDR_PROC_HEADER_EXTS *)&pFormat[current_offset];
+        ext_flags = pExtensions->Flags2;
+        current_offset += pExtensions->Size;
     }
 
     if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
@@ -613,6 +547,8 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
 #endif
 
     stubMsg.BufferLength = 0;
+    /* needed for conformance of top-level objects */
+    stubMsg.StackTop = *(unsigned char **)args;
 
     /* store the RPC flags away */
     if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_RPCFLAGS)
@@ -622,14 +558,14 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
     if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_RPCSSALLOC)
         NdrRpcSmSetClientToOsf(&stubMsg);
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+    if (Oif_flags.HasPipes)
     {
         FIXME("pipes not supported yet\n");
         RpcRaiseException(RPC_X_WRONG_STUB_VERSION); /* FIXME: remove when implemented */
         /* init pipes package */
         /* NdrPipesInitialize(...) */
     }
-    if (ext_flags & RPC_FC_PROC_EXT_NEWCORRDESC)
+    if (ext_flags.HasNewCorrDesc)
     {
         /* initialize extra correlation package */
         FIXME("new correlation description not implemented\n");
@@ -654,7 +590,7 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
             /* allocate the buffer */
             if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
                 NdrProxyGetBuffer(This, &stubMsg);
-            else if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+            else if (Oif_flags.HasPipes)
                 /* NdrGetPipeBuffer(...) */
                 FIXME("pipes not supported yet\n");
             else
@@ -674,7 +610,7 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
              * params */
             if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
                 NdrProxySendReceive(This, &stubMsg);
-            else if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+            else if (Oif_flags.HasPipes)
                 /* NdrPipesSendReceive(...) */
                 FIXME("pipes not supported yet\n");
             else
@@ -721,12 +657,12 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
                     TRACE("\tstack_offset: 0x%x\n", current_stack_offset);
                     TRACE("\tmemory addr (before): %p\n", pArg);
 
-                    if (pParam->param_attributes & RPC_FC_PROC_PF_BASETYPE)
+                    if (pParam->param_attributes.IsBasetype)
                     {
                         const unsigned char * pTypeFormat =
                             &pParam->type_format_char;
 
-                        if (pParam->param_attributes & RPC_FC_PROC_PF_SIMPLEREF)
+                        if (pParam->param_attributes.IsSimpleRef)
                             pArg = *(unsigned char **)pArg;
 
                         TRACE("\tbase type: 0x%02x\n", *pTypeFormat);
@@ -734,18 +670,18 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
                         switch (phase)
                         {
                         case PROXY_CALCSIZE:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                                 call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
                             break;
                         case PROXY_MARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                                 call_marshaller(&stubMsg, pArg, pTypeFormat);
                             break;
                         case PROXY_UNMARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut)
                             {
                                 unsigned char *pRetVal = (unsigned char *)&RetVal;
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
+                                if (pParam->param_attributes.IsReturn)
                                     call_unmarshaller(&stubMsg, &pRetVal, pTypeFormat, 0);
                                 else
                                     call_unmarshaller(&stubMsg, &pArg, pTypeFormat, 0);
@@ -768,7 +704,7 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
 
                         /* if a simple ref pointer then we have to do the
                          * check for the pointer being non-NULL. */
-                        if (pParam->param_attributes & RPC_FC_PROC_PF_SIMPLEREF)
+                        if (pParam->param_attributes.IsSimpleRef)
                         {
                             if (!*(unsigned char **)pArg)
                                 RpcRaiseException(RPC_X_NULL_REF_POINTER);
@@ -779,30 +715,30 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
                         switch (phase)
                         {
                         case PROXY_CALCSIZE:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                if (pParam->param_attributes.IsByValue)
                                     call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
                                 else
                                     call_buffer_sizer(&stubMsg, *(unsigned char **)pArg, pTypeFormat);
                             }
                             break;
                         case PROXY_MARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                if (pParam->param_attributes.IsByValue)
                                     call_marshaller(&stubMsg, pArg, pTypeFormat);
                                 else
                                     call_marshaller(&stubMsg, *(unsigned char **)pArg, pTypeFormat);
                             }
                             break;
                         case PROXY_UNMARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut)
                             {
                                 unsigned char *pRetVal = (unsigned char *)&RetVal;
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
+                                if (pParam->param_attributes.IsReturn)
                                     call_unmarshaller(&stubMsg, &pRetVal, pTypeFormat, 0);
-                                else if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                else if (pParam->param_attributes.IsByValue)
                                     call_unmarshaller(&stubMsg, &pArg, pTypeFormat, 0);
                                 else
                                     call_unmarshaller(&stubMsg, (unsigned char **)pArg, pTypeFormat, 0);
@@ -914,13 +850,13 @@ LONG_PTR WINAPIV NdrClientCall2(PMIDL_STUB_DESC pStubDesc, PFORMAT_STRING pForma
 
     /* FIXME: unbind the binding handle */
 
-    if (ext_flags & RPC_FC_PROC_EXT_NEWCORRDESC)
+    if (ext_flags.HasNewCorrDesc)
     {
         /* free extra correlation package */
         /* NdrCorrelationFree(&stubMsg); */
     }
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+    if (Oif_flags.HasPipes)
     {
         /* NdrPipesDone(...) */
     }
@@ -964,9 +900,9 @@ __declspec(naked) LONG_PTR __cdecl call_server_func(SERVER_ROUTINE func, unsigne
         rep movsd           ; Copy dword blocks
         call [ebp+8]        ; Call function
         lea esp, [ebp-8]    ; Restore stack
-        pop ebp             ; Restore registers
-        pop esi
+        pop esi             ; Restore registers
         pop edi
+        pop ebp
         ret
     }
 }
@@ -1023,9 +959,9 @@ long WINAPI NdrStubCall2(
     /* counter */
     unsigned short i;
     /* cache of Oif_flags from v2 procedure header */
-    unsigned char Oif_flags = 0;
-    /* cache of extension flags from NDR_PROC_EXTENSION */
-    unsigned char ext_flags = 0;
+    INTERPRETER_OPT_FLAGS Oif_flags = { 0 };
+    /* cache of extension flags from NDR_PROC_HEADER_EXTS */
+    INTERPRETER_OPT_FLAGS2 ext_flags = { 0 };
     /* the type of pass we are currently doing */
     int phase;
     /* header for procedure string */
@@ -1036,9 +972,8 @@ long WINAPI NdrStubCall2(
     int current_offset;
     /* -Oif or -Oicf generated format */
     BOOL bV2Format = FALSE;
-    /* the return value (not from this function, but to be put back onto
-     * the wire */
-    LONG_PTR RetVal = 0;
+    /* location to put retval into */
+    LONG_PTR *retval_ptr = NULL;
 
     TRACE("pThis %p, pChannel %p, pRpcMsg %p, pdwStubPhase %p\n", pThis, pChannel, pRpcMsg, pdwStubPhase);
 
@@ -1111,20 +1046,20 @@ long WINAPI NdrStubCall2(
         NDR_PROC_PARTIAL_OIF_HEADER * pOIFHeader =
             (NDR_PROC_PARTIAL_OIF_HEADER*)&pFormat[current_offset];
 
-        Oif_flags = pOIFHeader->Oif_flags;
+        Oif_flags = pOIFHeader->Oi2Flags;
         number_of_params = pOIFHeader->number_of_params;
 
         current_offset += sizeof(NDR_PROC_PARTIAL_OIF_HEADER);
     }
 
-    TRACE("Oif_flags = 0x%02x\n", Oif_flags);
+    TRACE("Oif_flags = "); dump_INTERPRETER_OPT_FLAGS(Oif_flags);
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASEXTS)
+    if (Oif_flags.HasExtensions)
     {
-        NDR_PROC_EXTENSION * pExtensions =
-            (NDR_PROC_EXTENSION *)&pFormat[current_offset];
-        ext_flags = pExtensions->ext_flags;
-        current_offset += pExtensions->extension_version;
+        NDR_PROC_HEADER_EXTS * pExtensions =
+            (NDR_PROC_HEADER_EXTS *)&pFormat[current_offset];
+        ext_flags = pExtensions->Flags2;
+        current_offset += pExtensions->Size;
     }
 
     if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
@@ -1152,21 +1087,20 @@ long WINAPI NdrStubCall2(
           FIXME("Set RPCSS memory allocation routines\n");
 #endif
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+    if (Oif_flags.HasPipes)
     {
         FIXME("pipes not supported yet\n");
         RpcRaiseException(RPC_X_WRONG_STUB_VERSION); /* FIXME: remove when implemented */
         /* init pipes package */
         /* NdrPipesInitialize(...) */
     }
-    if (ext_flags & RPC_FC_PROC_EXT_NEWCORRDESC)
+    if (ext_flags.HasNewCorrDesc)
     {
         /* initialize extra correlation package */
         FIXME("new correlation description not implemented\n");
         stubMsg.fHasNewCorrDesc = TRUE;
     }
 
-
     /* convert strings, floating point values and endianess into our
      * preferred format */
     if ((pRpcMsg->DataRepresentation & 0x0000FFFFUL) != NDR_LOCAL_DATA_REPRESENTATION)
@@ -1176,8 +1110,8 @@ long WINAPI NdrStubCall2(
 
     TRACE("allocating memory for stack of size %x\n", stack_size);
 
-    args = HeapAlloc(GetProcessHeap(), 0, stack_size);
-    ZeroMemory(args, stack_size);
+    args = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, stack_size);
+    stubMsg.StackTop = args; /* used by conformance of top-level objects */
 
     /* add the implicit This pointer as the first arg to the function if we
      * are calling an object method */
@@ -1198,21 +1132,32 @@ long WINAPI NdrStubCall2(
         {
         case STUBLESS_CALLSERVER:
             /* call the server function */
-            if (pServerInfo->ThunkTable)
-            {
-                stubMsg.StackTop = args;
+            if (pServerInfo->ThunkTable && pServerInfo->ThunkTable[pRpcMsg->ProcNum])
                 pServerInfo->ThunkTable[pRpcMsg->ProcNum](&stubMsg);
-                /* FIXME: RetVal is stored as the last argument - retrieve it */
-            }
-            else if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
-            {
-                SERVER_ROUTINE *vtbl = *(SERVER_ROUTINE **)((CStdStubBuffer *)pThis)->pvServerObject;
-                RetVal = call_server_func(vtbl[pRpcMsg->ProcNum], args, stack_size);
-            }
             else
-                RetVal = call_server_func(pServerInfo->DispatchTable[pRpcMsg->ProcNum], args, stack_size);
+            {
+                SERVER_ROUTINE func;
+                LONG_PTR retval;
 
-            TRACE("stub implementation returned %p\n", (void *)RetVal);
+                if (pProcHeader->Oi_flags & RPC_FC_PROC_OIF_OBJECT)
+                {
+                    SERVER_ROUTINE *vtbl = *(SERVER_ROUTINE **)((CStdStubBuffer *)pThis)->pvServerObject;
+                    func = vtbl[pRpcMsg->ProcNum];
+                }
+                else
+                    func = pServerInfo->DispatchTable[pRpcMsg->ProcNum];
+
+                /* FIXME: what happens with return values that don't fit into a single register on x86? */
+                retval = call_server_func(func, args, stack_size);
+
+                if (retval_ptr)
+                {
+                    TRACE("stub implementation returned 0x%lx\n", retval);
+                    *retval_ptr = retval;
+                }
+                else
+                    TRACE("void stub implementation\n");
+            }
 
             stubMsg.Buffer = NULL;
             stubMsg.BufferLength = 0;
@@ -1261,10 +1206,11 @@ long WINAPI NdrStubCall2(
                     TRACE("\tstack_offset: %x\n", current_stack_offset);
                     TRACE("\tmemory addr (before): %p -> %p\n", pArg, *(unsigned char **)pArg);
 
-                    if (pParam->param_attributes & RPC_FC_PROC_PF_SRVALLOCSIZE)
-                        FIXME("RPC_FC_PROC_PF_SRVALLOCSIZE ignored for parameter %d\n", i);
+                    if (pParam->param_attributes.ServerAllocSize)
+                        FIXME("ServerAllocSize of %d ignored for parameter %d\n",
+                            pParam->param_attributes.ServerAllocSize * 8, i);
 
-                    if (pParam->param_attributes & RPC_FC_PROC_PF_BASETYPE)
+                    if (pParam->param_attributes.IsBasetype)
                     {
                         const unsigned char *pTypeFormat =
                             &pParam->type_format_char;
@@ -1274,11 +1220,9 @@ long WINAPI NdrStubCall2(
                         switch (phase)
                         {
                         case STUBLESS_MARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
-                                call_marshaller(&stubMsg, (unsigned char *)&RetVal, pTypeFormat);
-                            else if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut || pParam->param_attributes.IsReturn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_SIMPLEREF)
+                                if (pParam->param_attributes.IsSimpleRef)
                                     call_marshaller(&stubMsg, *(unsigned char **)pArg, pTypeFormat);
                                 else
                                     call_marshaller(&stubMsg, pArg, pTypeFormat);
@@ -1286,20 +1230,23 @@ long WINAPI NdrStubCall2(
                             /* FIXME: call call_freer here */
                             break;
                         case STUBLESS_UNMARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_SIMPLEREF)
+                                if (pParam->param_attributes.IsSimpleRef)
                                     call_unmarshaller(&stubMsg, (unsigned char **)pArg, pTypeFormat, 0);
                                 else
                                     call_unmarshaller(&stubMsg, &pArg, pTypeFormat, 0);
                             }
+
+                            /* make a note of the address of the return value parameter for later */
+                            if (pParam->param_attributes.IsReturn)
+                                retval_ptr = (LONG_PTR *)pArg;
+
                             break;
                         case STUBLESS_CALCSIZE:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
-                                call_buffer_sizer(&stubMsg, (unsigned char *)&RetVal, pTypeFormat);
-                            else if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut || pParam->param_attributes.IsReturn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_SIMPLEREF)
+                                if (pParam->param_attributes.IsSimpleRef)
                                     call_buffer_sizer(&stubMsg, *(unsigned char **)pArg, pTypeFormat);
                                 else
                                     call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
@@ -1324,11 +1271,9 @@ long WINAPI NdrStubCall2(
                         switch (phase)
                         {
                         case STUBLESS_MARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
-                                call_marshaller(&stubMsg, (unsigned char *)&RetVal, pTypeFormat);
-                            else if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut || pParam->param_attributes.IsReturn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                if (pParam->param_attributes.IsByValue)
                                     call_marshaller(&stubMsg, pArg, pTypeFormat);
                                 else
                                 {
@@ -1339,26 +1284,24 @@ long WINAPI NdrStubCall2(
                             /* FIXME: call call_freer here for IN types */
                             break;
                         case STUBLESS_UNMARSHAL:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_IN)
+                            if (pParam->param_attributes.IsIn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                if (pParam->param_attributes.IsByValue)
                                     call_unmarshaller(&stubMsg, &pArg, pTypeFormat, 0);
                                 else
                                     call_unmarshaller(&stubMsg, (unsigned char **)pArg, pTypeFormat, 0);
                             }
-                            else if ((pParam->param_attributes & RPC_FC_PROC_PF_OUT) && 
-                                      !(pParam->param_attributes & RPC_FC_PROC_PF_BYVAL))
+                            else if (pParam->param_attributes.IsOut &&
+                                     !pParam->param_attributes.IsByValue)
                             {
                                 *(void **)pArg = NdrAllocate(&stubMsg, sizeof(void *));
                                 **(void ***)pArg = 0;
                             }
                             break;
                         case STUBLESS_CALCSIZE:
-                            if (pParam->param_attributes & RPC_FC_PROC_PF_RETURN)
-                                call_buffer_sizer(&stubMsg, (unsigned char *)&RetVal, pTypeFormat);
-                            else if (pParam->param_attributes & RPC_FC_PROC_PF_OUT)
+                            if (pParam->param_attributes.IsOut || pParam->param_attributes.IsReturn)
                             {
-                                if (pParam->param_attributes & RPC_FC_PROC_PF_BYVAL)
+                                if (pParam->param_attributes.IsByValue)
                                     call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
                                 else
                                     call_buffer_sizer(&stubMsg, *(unsigned char **)pArg, pTypeFormat);
@@ -1396,10 +1339,7 @@ long WINAPI NdrStubCall2(
                         {
                         case STUBLESS_MARSHAL:
                             if (pParam->param_direction == RPC_FC_RETURN_PARAM_BASETYPE)
-                            {
-                                unsigned char *pRetVal = (unsigned char *)&RetVal;
-                                call_marshaller(&stubMsg, (unsigned char *)&pRetVal, pTypeFormat);
-                            }
+                                call_marshaller(&stubMsg, pArg, pTypeFormat);
                             break;
                         case STUBLESS_UNMARSHAL:
                             if (pParam->param_direction == RPC_FC_IN_PARAM_BASETYPE)
@@ -1407,10 +1347,7 @@ long WINAPI NdrStubCall2(
                             break;
                         case STUBLESS_CALCSIZE:
                             if (pParam->param_direction == RPC_FC_RETURN_PARAM_BASETYPE)
-                            {
-                                unsigned char * pRetVal = (unsigned char *)&RetVal;
-                                call_buffer_sizer(&stubMsg, (unsigned char *)&pRetVal, pTypeFormat);
-                            }
+                                call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
                             break;
                         default:
                             RpcRaiseException(RPC_S_INTERNAL_ERROR);
@@ -1432,13 +1369,9 @@ long WINAPI NdrStubCall2(
                         switch (phase)
                         {
                         case STUBLESS_MARSHAL:
-                            if (pParam->param_direction == RPC_FC_RETURN_PARAM)
-                            {
-                                unsigned char *pRetVal = (unsigned char *)&RetVal;
-                                call_marshaller(&stubMsg, (unsigned char *)&pRetVal, pTypeFormat);
-                            }
-                            else if (pParam->param_direction == RPC_FC_OUT_PARAM ||
-                                pParam->param_direction == RPC_FC_IN_OUT_PARAM)
+                            if (pParam->param_direction == RPC_FC_OUT_PARAM ||
+                                pParam->param_direction == RPC_FC_IN_OUT_PARAM ||
+                                pParam->param_direction == RPC_FC_RETURN_PARAM)
                                 call_marshaller(&stubMsg, pArg, pTypeFormat);
                             break;
                         case STUBLESS_UNMARSHAL:
@@ -1447,13 +1380,9 @@ long WINAPI NdrStubCall2(
                                 call_unmarshaller(&stubMsg, &pArg, pTypeFormat, 0);
                             break;
                         case STUBLESS_CALCSIZE:
-                            if (pParam->param_direction == RPC_FC_RETURN_PARAM)
-                            {
-                                unsigned char * pRetVal = (unsigned char *)&RetVal;
-                                call_buffer_sizer(&stubMsg, (unsigned char *)&pRetVal, pTypeFormat);
-                            }
-                            else if (pParam->param_direction == RPC_FC_OUT_PARAM ||
-                                pParam->param_direction == RPC_FC_IN_OUT_PARAM)
+                            if (pParam->param_direction == RPC_FC_OUT_PARAM ||
+                                pParam->param_direction == RPC_FC_IN_OUT_PARAM ||
+                                pParam->param_direction == RPC_FC_RETURN_PARAM)
                                 call_buffer_sizer(&stubMsg, pArg, pTypeFormat);
                             break;
                         default:
@@ -1475,13 +1404,13 @@ long WINAPI NdrStubCall2(
 
     pRpcMsg->BufferLength = (unsigned int)(stubMsg.Buffer - (unsigned char *)pRpcMsg->Buffer);
 
-    if (ext_flags & RPC_FC_PROC_EXT_NEWCORRDESC)
+    if (ext_flags.HasNewCorrDesc)
     {
         /* free extra correlation package */
         /* NdrCorrelationFree(&stubMsg); */
     }
 
-    if (Oif_flags & RPC_FC_PROC_OI2F_HASPIPES)
+    if (Oif_flags.HasPipes)
     {
         /* NdrPipesDone(...) */
     }
