@@ -99,210 +99,6 @@ void RPCRT4_strfree(LPSTR src)
   HeapFree(GetProcessHeap(), 0, src);
 }
 
-RPC_STATUS RPCRT4_CreateConnection(RpcConnection** Connection, BOOL server, LPSTR Protseq, LPSTR NetworkAddr, LPSTR Endpoint, LPSTR NetworkOptions, RpcBinding* Binding)
-{
-  RpcConnection* NewConnection;
-
-  NewConnection = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RpcConnection));
-  NewConnection->server = server;
-  NewConnection->Protseq = RPCRT4_strdupA(Protseq);
-  NewConnection->NetworkAddr = RPCRT4_strdupA(NetworkAddr);
-  NewConnection->Endpoint = RPCRT4_strdupA(Endpoint);
-  NewConnection->Used = Binding;
-  NewConnection->MaxTransmissionSize = RPC_MAX_PACKET_SIZE;
-
-  TRACE("connection: %p\n", NewConnection);
-  *Connection = NewConnection;
-
-  return RPC_S_OK;
-}
-
-RPC_STATUS RPCRT4_DestroyConnection(RpcConnection* Connection)
-{
-  TRACE("connection: %p\n", Connection);
-
-  RPCRT4_CloseConnection(Connection);
-  RPCRT4_strfree(Connection->Endpoint);
-  RPCRT4_strfree(Connection->NetworkAddr);
-  RPCRT4_strfree(Connection->Protseq);
-  HeapFree(GetProcessHeap(), 0, Connection);
-  return RPC_S_OK;
-}
-
-RPC_STATUS RPCRT4_OpenConnection(RpcConnection* Connection)
-{
-  TRACE("(Connection == ^%p)\n", Connection);
-  if (!Connection->conn) {
-    if (Connection->server) { /* server */
-      /* protseq=ncalrpc: supposed to use NT LPC ports,
-       * but we'll implement it with named pipes for now */
-      if (strcmp(Connection->Protseq, "ncalrpc") == 0) {
-        static LPCSTR prefix = "\\\\.\\pipe\\lrpc\\";
-        LPSTR pname;
-        pname = HeapAlloc(GetProcessHeap(), 0, strlen(prefix) + strlen(Connection->Endpoint) + 1);
-        strcat(strcpy(pname, prefix), Connection->Endpoint);
-        TRACE("listening on %s\n", pname);
-        Connection->conn = CreateNamedPipeA(pname, PIPE_ACCESS_DUPLEX,
-                                         PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE, PIPE_UNLIMITED_INSTANCES,
-                                         RPC_MAX_PACKET_SIZE, RPC_MAX_PACKET_SIZE, 5000, NULL);
-        HeapFree(GetProcessHeap(), 0, pname);
-        memset(&Connection->ovl, 0, sizeof(Connection->ovl));
-        Connection->ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
-        if (!ConnectNamedPipe(Connection->conn, &Connection->ovl)) {
-          WARN("Couldn't ConnectNamedPipe (error was %ld)\n", GetLastError());
-          if (GetLastError() == ERROR_PIPE_CONNECTED) {
-            SetEvent(Connection->ovl.hEvent);
-            return RPC_S_OK;
-          } else if (GetLastError() == ERROR_IO_PENDING) {
-            return RPC_S_OK;
-          }
-          return RPC_S_SERVER_UNAVAILABLE;
-        }
-      }
-      /* protseq=ncacn_np: named pipes */
-      else if (strcmp(Connection->Protseq, "ncacn_np") == 0) {
-        static LPCSTR prefix = "\\\\.";
-        LPSTR pname;
-        pname = HeapAlloc(GetProcessHeap(), 0, strlen(prefix) + strlen(Connection->Endpoint) + 1);
-        strcat(strcpy(pname, prefix), Connection->Endpoint);
-        TRACE("listening on %s\n", pname);
-        Connection->conn = CreateNamedPipeA(pname, PIPE_ACCESS_DUPLEX,
-                                         PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT, PIPE_UNLIMITED_INSTANCES,
-                                         RPC_MAX_PACKET_SIZE, RPC_MAX_PACKET_SIZE, 5000, NULL);
-        HeapFree(GetProcessHeap(), 0, pname);
-        memset(&Connection->ovl, 0, sizeof(Connection->ovl));
-        Connection->ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
-        if (!ConnectNamedPipe(Connection->conn, &Connection->ovl)) {
-          if (GetLastError() == ERROR_PIPE_CONNECTED) {
-            SetEvent(Connection->ovl.hEvent);
-            return RPC_S_OK;
-          } else if (GetLastError() == ERROR_IO_PENDING) {
-            return RPC_S_OK;
-          }
-          WARN("Couldn't ConnectNamedPipe (error was %ld)\n", GetLastError());
-          return RPC_S_SERVER_UNAVAILABLE;
-        }
-      }
-      else {
-        ERR("protseq %s not supported\n", Connection->Protseq);
-        return RPC_S_PROTSEQ_NOT_SUPPORTED;
-      }
-    }
-    else { /* client */
-      /* protseq=ncalrpc: supposed to use NT LPC ports,
-       * but we'll implement it with named pipes for now */
-      if (strcmp(Connection->Protseq, "ncalrpc") == 0) {
-        static LPCSTR prefix = "\\\\.\\pipe\\lrpc\\";
-        LPSTR pname;
-        HANDLE conn;
-        DWORD err;
-        DWORD dwMode;
-
-        pname = HeapAlloc(GetProcessHeap(), 0, strlen(prefix) + strlen(Connection->Endpoint) + 1);
-        strcat(strcpy(pname, prefix), Connection->Endpoint);
-        TRACE("connecting to %s\n", pname);
-        while (TRUE) {
-          if (WaitNamedPipeA(pname, NMPWAIT_WAIT_FOREVER)) {
-            conn = CreateFileA(pname, GENERIC_READ|GENERIC_WRITE, 0, NULL,
-                               OPEN_EXISTING, 0, 0);
-            if (conn != INVALID_HANDLE_VALUE) break;
-            err = GetLastError();
-            if (err == ERROR_PIPE_BUSY) continue;
-            TRACE("connection failed, error=%lx\n", err);
-            HeapFree(GetProcessHeap(), 0, pname);
-            return RPC_S_SERVER_TOO_BUSY;
-          } else {
-            err = GetLastError();
-            WARN("connection failed, error=%lx\n", err);
-            HeapFree(GetProcessHeap(), 0, pname);
-            return RPC_S_SERVER_UNAVAILABLE;
-          }
-        }
-
-        /* success */
-        HeapFree(GetProcessHeap(), 0, pname);
-        memset(&Connection->ovl, 0, sizeof(Connection->ovl));
-        /* pipe is connected; change to message-read mode. */
-        dwMode = PIPE_READMODE_MESSAGE; 
-        SetNamedPipeHandleState(conn, &dwMode, NULL, NULL);
-        Connection->ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
-        Connection->conn = conn;
-      }
-      /* protseq=ncacn_np: named pipes */
-      else if (strcmp(Connection->Protseq, "ncacn_np") == 0) {
-        static LPCSTR prefix = "\\\\.";
-        LPSTR pname;
-        HANDLE conn;
-        DWORD err;
-        DWORD dwMode;
-
-        pname = HeapAlloc(GetProcessHeap(), 0, strlen(prefix) + strlen(Connection->Endpoint) + 1);
-        strcat(strcpy(pname, prefix), Connection->Endpoint);
-        TRACE("connecting to %s\n", pname);
-        conn = CreateFileA(pname, GENERIC_READ|GENERIC_WRITE, 0, NULL,
-                           OPEN_EXISTING, 0, 0);
-        if (conn == INVALID_HANDLE_VALUE) {
-          err = GetLastError();
-          /* we don't need to handle ERROR_PIPE_BUSY here,
-           * the doc says that it is returned to the app */
-          WARN("connection failed, error=%lx\n", err);
-          HeapFree(GetProcessHeap(), 0, pname);
-          if (err == ERROR_PIPE_BUSY)
-            return RPC_S_SERVER_TOO_BUSY;
-          else
-            return RPC_S_SERVER_UNAVAILABLE;
-        }
-
-        /* success */
-        HeapFree(GetProcessHeap(), 0, pname);
-        memset(&Connection->ovl, 0, sizeof(Connection->ovl));
-        /* pipe is connected; change to message-read mode. */
-        dwMode = PIPE_READMODE_MESSAGE;
-        SetNamedPipeHandleState(conn, &dwMode, NULL, NULL);
-        Connection->ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
-        Connection->conn = conn;
-      } else {
-        ERR("protseq %s not supported\n", Connection->Protseq);
-        return RPC_S_PROTSEQ_NOT_SUPPORTED;
-      }
-    }
-  }
-  return RPC_S_OK;
-}
-
-RPC_STATUS RPCRT4_CloseConnection(RpcConnection* Connection)
-{
-  TRACE("(Connection == ^%p)\n", Connection);
-  if (Connection->conn) {
-    FlushFileBuffers(Connection->conn);
-    CloseHandle(Connection->conn);
-    Connection->conn = 0;
-  }
-  if (Connection->ovl.hEvent) {
-    CloseHandle(Connection->ovl.hEvent);
-    Connection->ovl.hEvent = 0;
-  }
-  return RPC_S_OK;
-}
-
-RPC_STATUS RPCRT4_SpawnConnection(RpcConnection** Connection, RpcConnection* OldConnection)
-{
-  RpcConnection* NewConnection;
-  RPC_STATUS err = RPCRT4_CreateConnection(&NewConnection, OldConnection->server, OldConnection->Protseq,
-                                           OldConnection->NetworkAddr, OldConnection->Endpoint, NULL, NULL);
-  if (err == RPC_S_OK) {
-    /* because of the way named pipes work, we'll transfer the connected pipe
-     * to the child, then reopen the server binding to continue listening */
-    NewConnection->conn = OldConnection->conn;
-    NewConnection->ovl = OldConnection->ovl;
-    OldConnection->conn = 0;
-    memset(&OldConnection->ovl, 0, sizeof(OldConnection->ovl));
-    *Connection = NewConnection;
-    RPCRT4_OpenConnection(OldConnection);
-  }
-  return err;
-}
-
 static RPC_STATUS RPCRT4_AllocBinding(RpcBinding** Binding, BOOL server)
 {
   RpcBinding* NewBinding;
@@ -402,7 +198,7 @@ RPC_STATUS RPCRT4_MakeBinding(RpcBinding** Binding, RpcConnection* Connection)
   TRACE("(RpcBinding == ^%p, Connection == ^%p)\n", Binding, Connection);
 
   RPCRT4_AllocBinding(&NewBinding, Connection->server);
-  NewBinding->Protseq = RPCRT4_strdupA(Connection->Protseq);
+  NewBinding->Protseq = RPCRT4_strdupA(rpcrt4_conn_get_name(Connection));
   NewBinding->NetworkAddr = RPCRT4_strdupA(Connection->NetworkAddr);
   NewBinding->Endpoint = RPCRT4_strdupA(Connection->Endpoint);
   NewBinding->FromConn = Connection;
@@ -471,7 +267,7 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
   /* we need to send a binding packet if we are client. */
   if (!(*Connection)->server) {
     RpcPktHdr *hdr;
-    DWORD count;
+    LONG count;
     BYTE *response;
     RpcPktHdr *response_hdr;
 
@@ -482,6 +278,7 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
                                  InterfaceId, TransferSyntax);
 
     status = RPCRT4_Send(*Connection, hdr, NULL, 0);
+    RPCRT4_FreeHeader(hdr);
     if (status != RPC_S_OK) {
       RPCRT4_DestroyConnection(*Connection);
       return status;
@@ -494,15 +291,10 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
       return E_OUTOFMEMORY;
     }
 
-    /* get a reply */
-    if (!ReadFile(NewConnection->conn, response, RPC_MAX_PACKET_SIZE, &count, NULL)) {
-      WARN("ReadFile failed with error %ld\n", GetLastError());
-      RPCRT4_DestroyConnection(*Connection);
-      return RPC_S_PROTOCOL_ERROR;
-    }
-
+    count = rpcrt4_conn_read(NewConnection, response, RPC_MAX_PACKET_SIZE);
     if (count < sizeof(response_hdr->common)) {
       WARN("received invalid header\n");
+      HeapFree(GetProcessHeap(), 0, response);
       RPCRT4_DestroyConnection(*Connection);
       return RPC_S_PROTOCOL_ERROR;
     }
@@ -513,12 +305,14 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
         response_hdr->common.rpc_ver_minor != RPC_VER_MINOR ||
         response_hdr->common.ptype != PKT_BIND_ACK) {
       WARN("invalid protocol version or rejection packet\n");
+      HeapFree(GetProcessHeap(), 0, response);
       RPCRT4_DestroyConnection(*Connection);
       return RPC_S_PROTOCOL_ERROR;
     }
 
     if (response_hdr->bind_ack.max_tsize < RPC_MIN_PACKET_SIZE) {
       WARN("server doesn't allow large enough packets\n");
+      HeapFree(GetProcessHeap(), 0, response);
       RPCRT4_DestroyConnection(*Connection);
       return RPC_S_PROTOCOL_ERROR;
     }
@@ -527,7 +321,9 @@ RPC_STATUS RPCRT4_OpenBinding(RpcBinding* Binding, RpcConnection** Connection,
 
     (*Connection)->MaxTransmissionSize = response_hdr->bind_ack.max_tsize;
     (*Connection)->ActiveInterface = *InterfaceId;
+    HeapFree(GetProcessHeap(), 0, response);
   }
+  Binding->FromConn = *Connection;
 
   return RPC_S_OK;
 }
@@ -1144,3 +940,113 @@ RPC_STATUS WINAPI RpcRevertToSelfEx(RPC_BINDING_HANDLE BindingHandle)
     FIXME("(%p): stub\n", BindingHandle);
     return RPC_S_OK;
 }
+
+/***********************************************************************
+ *             RpcBindingInqAuthInfoExA (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingInqAuthInfoExA( RPC_BINDING_HANDLE Binding, unsigned char ** ServerPrincName, unsigned long *AuthnLevel,
+                          unsigned long *AuthnSvc, RPC_AUTH_IDENTITY_HANDLE *AuthIdentity, unsigned long *AuthzSvc,
+                          unsigned long RpcQosVersion, RPC_SECURITY_QOS *SecurityQOS )
+{
+    FIXME("%p %p %p %p %p %p %lu %p\n", Binding, ServerPrincName, AuthnLevel,
+          AuthnSvc, AuthIdentity, AuthzSvc, RpcQosVersion, SecurityQOS);
+    return RPC_S_INVALID_BINDING;
+}
+
+/***********************************************************************
+ *             RpcBindingInqAuthInfoExW (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingInqAuthInfoExW( RPC_BINDING_HANDLE Binding, unsigned short ** ServerPrincName, unsigned long *AuthnLevel,
+                          unsigned long *AuthnSvc, RPC_AUTH_IDENTITY_HANDLE *AuthIdentity, unsigned long *AuthzSvc,
+                          unsigned long RpcQosVersion, RPC_SECURITY_QOS *SecurityQOS )
+{
+    FIXME("%p %p %p %p %p %p %lu %p\n", Binding, ServerPrincName, AuthnLevel,
+          AuthnSvc, AuthIdentity, AuthzSvc, RpcQosVersion, SecurityQOS);
+    return RPC_S_INVALID_BINDING;
+}
+
+/***********************************************************************
+ *             RpcBindingInqAuthInfoA (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingInqAuthInfoA( RPC_BINDING_HANDLE Binding, unsigned char ** ServerPrincName, unsigned long *AuthnLevel,
+                        unsigned long *AuthnSvc, RPC_AUTH_IDENTITY_HANDLE *AuthIdentity, unsigned long *AuthzSvc )
+{
+    FIXME("%p %p %p %p %p %p\n", Binding, ServerPrincName, AuthnLevel,
+          AuthnSvc, AuthIdentity, AuthzSvc);
+    return RPC_S_INVALID_BINDING;
+}
+
+/***********************************************************************
+ *             RpcBindingInqAuthInfoW (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingInqAuthInfoW( RPC_BINDING_HANDLE Binding, unsigned short ** ServerPrincName, unsigned long *AuthnLevel,
+                        unsigned long *AuthnSvc, RPC_AUTH_IDENTITY_HANDLE *AuthIdentity, unsigned long *AuthzSvc )
+{
+    FIXME("%p %p %p %p %p %p\n", Binding, ServerPrincName, AuthnLevel,
+          AuthnSvc, AuthIdentity, AuthzSvc);
+    return RPC_S_INVALID_BINDING;
+}
+
+/***********************************************************************
+ *             RpcBindingSetAuthInfoExA (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingSetAuthInfoExA( RPC_BINDING_HANDLE Binding, unsigned char *ServerPrincName,
+                          unsigned long AuthnLevel, unsigned long AuthnSvc,
+                          RPC_AUTH_IDENTITY_HANDLE AuthIdentity, unsigned long AuthzSvr,
+                          RPC_SECURITY_QOS *SecurityQos )
+{
+    FIXME("%p %s %lu %lu %p %lu %p\n", Binding, debugstr_a((const char*)ServerPrincName),
+          AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr, SecurityQos);
+    return RPC_S_OK;
+}
+
+/***********************************************************************
+ *             RpcBindingSetAuthInfoExW (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingSetAuthInfoExW( RPC_BINDING_HANDLE Binding, unsigned short *ServerPrincName, unsigned long AuthnLevel,
+                          unsigned long AuthnSvc, RPC_AUTH_IDENTITY_HANDLE AuthIdentity, unsigned long AuthzSvr,
+                          RPC_SECURITY_QOS *SecurityQos )
+{
+    FIXME("%p %s %lu %lu %p %lu %p\n", Binding, debugstr_w((const WCHAR*)ServerPrincName),
+          AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr, SecurityQos);
+    return RPC_S_OK;
+}
+
+/***********************************************************************
+ *             RpcBindingSetAuthInfoA (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingSetAuthInfoA( RPC_BINDING_HANDLE Binding, unsigned char *ServerPrincName, unsigned long AuthnLevel,
+                          unsigned long AuthnSvc, RPC_AUTH_IDENTITY_HANDLE AuthIdentity, unsigned long AuthzSvr )
+{
+    FIXME("%p %s %lu %lu %p %lu\n", Binding, debugstr_a((const char*)ServerPrincName),
+          AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr);
+    return RPC_S_OK;
+}
+
+/***********************************************************************
+ *             RpcBindingSetAuthInfoW (RPCRT4.@)
+ */
+RPCRTAPI RPC_STATUS RPC_ENTRY
+RpcBindingSetAuthInfoW( RPC_BINDING_HANDLE Binding, unsigned short *ServerPrincName, unsigned long AuthnLevel,
+                        unsigned long AuthnSvc, RPC_AUTH_IDENTITY_HANDLE AuthIdentity, unsigned long AuthzSvr )
+{
+    FIXME("%p %s %lu %lu %p %lu\n", Binding, debugstr_w((const WCHAR*)ServerPrincName),
+          AuthnLevel, AuthnSvc, AuthIdentity, AuthzSvr);
+    return RPC_S_OK;
+}
+
+/***********************************************************************
+ *             RpcBindingSetOption (RPCRT4.@)
+ */
+RPC_STATUS WINAPI RpcBindingSetOption(RPC_BINDING_HANDLE BindingHandle, ULONG Option, ULONG OptionValue)
+{
+    FIXME("(%p, %ld, %ld): stub\n", BindingHandle, Option, OptionValue);
+    return RPC_S_OK;
+}
