@@ -23,21 +23,32 @@
 
 #include "wine/rpcss_shared.h"
 
+struct protseq_ops;
+
 typedef struct _RpcConnection
 {
   struct _RpcConnection* Next;
   struct _RpcBinding* Used;
   BOOL server;
-  LPSTR Protseq;
   LPSTR NetworkAddr;
   LPSTR Endpoint;
-  HANDLE conn, thread;
-  OVERLAPPED ovl;
+  struct protseq_ops *ops;
   USHORT MaxTransmissionSize;
   /* The active interface bound to server. */
   RPC_SYNTAX_IDENTIFIER ActiveInterface;
 } RpcConnection;
 
+struct protseq_ops {
+  char *name;
+  RpcConnection *(*alloc)(void);
+  RPC_STATUS (*open_connection)(RpcConnection *conn);
+  HANDLE (*get_connect_wait_handle)(RpcConnection *conn);
+  RPC_STATUS (*handoff)(RpcConnection *old_conn, RpcConnection *new_conn);
+  int (*read)(RpcConnection *conn, void *buffer, unsigned int len);
+  int (*write)(RpcConnection *conn, const void *buffer, unsigned int len);
+  int (*close)(RpcConnection *conn);
+};
+
 /* don't know what MS's structure looks like */
 typedef struct _RpcBinding
 {
@@ -62,7 +73,7 @@ void RPCRT4_strfree(LPSTR src);
 #define RPCRT4_strdupA(x) RPCRT4_strndupA((x),-1)
 #define RPCRT4_strdupW(x) RPCRT4_strndupW((x),-1)
 
-RPC_STATUS RPCRT4_CreateConnection(RpcConnection** Connection, BOOL server, LPSTR Protseq, LPSTR NetworkAddr, LPSTR Endpoint, LPSTR NetworkOptions, RpcBinding* Binding);
+RPC_STATUS RPCRT4_CreateConnection(RpcConnection** Connection, BOOL server, LPCSTR Protseq, LPCSTR NetworkAddr, LPCSTR Endpoint, LPCSTR NetworkOptions, RpcBinding* Binding);
 RPC_STATUS RPCRT4_DestroyConnection(RpcConnection* Connection);
 RPC_STATUS RPCRT4_OpenConnection(RpcConnection* Connection);
 RPC_STATUS RPCRT4_CloseConnection(RpcConnection* Connection);
@@ -83,4 +94,36 @@ BOOL RPCRT4_RPCSSOnDemandCall(PRPCSS_NP_MESSAGE msg, char *vardata_payload, PRPC
 HANDLE RPCRT4_GetMasterMutex(void);
 HANDLE RPCRT4_RpcssNPConnect(void);
 
+static inline const char *rpcrt4_conn_get_name(RpcConnection *Connection)
+{
+  return Connection->ops->name;
+}
+
+static inline int rpcrt4_conn_read(RpcConnection *Connection,
+                     void *buffer, unsigned int len)
+{
+  return Connection->ops->read(Connection, buffer, len);
+}
+
+static inline int rpcrt4_conn_write(RpcConnection *Connection,
+                     const void *buffer, unsigned int len)
+{
+  return Connection->ops->write(Connection, buffer, len);
+}
+
+static inline int rpcrt4_conn_close(RpcConnection *Connection)
+{
+  return Connection->ops->close(Connection);
+}
+
+static inline HANDLE rpcrt4_conn_get_wait_object(RpcConnection *Connection)
+{
+  return Connection->ops->get_connect_wait_handle(Connection);
+}
+
+static inline RPC_STATUS rpcrt4_conn_handoff(RpcConnection *old_conn, RpcConnection *new_conn)
+{
+  return old_conn->ops->handoff(old_conn, new_conn);
+}
+
 #endif
