@@ -247,7 +247,8 @@ RPC_STATUS RPCRT4_Send(RpcConnection *Connection, RpcPktHdr *Header,
                        void *Buffer, unsigned int BufferLength)
 {
   PUCHAR buffer_pos;
-  DWORD hdr_size, count;
+  DWORD hdr_size;
+  LONG count;
 
   buffer_pos = Buffer;
   /* The packet building functions save the packet header size, so we can use it. */
@@ -264,9 +265,10 @@ RPC_STATUS RPCRT4_Send(RpcConnection *Connection, RpcPktHdr *Header,
     }
 
     /* transmit packet header */
-    if (!WriteFile(Connection->conn, Header, hdr_size, &count, NULL)) {
-      WARN("WriteFile failed with error %ld\n", GetLastError());
-      return GetLastError();
+    count = rpcrt4_conn_write(Connection, Header, hdr_size);
+    if (count<0) {
+      WARN("rpcrt4_conn_write failed\n");
+      return RPC_S_PROTOCOL_ERROR;
     }
 
     /* fragment consisted of header only and is the last one */
@@ -276,9 +278,10 @@ RPC_STATUS RPCRT4_Send(RpcConnection *Connection, RpcPktHdr *Header,
     }
 
     /* send the fragment data */
-    if (!WriteFile(Connection->conn, buffer_pos, Header->common.frag_len - hdr_size, &count, NULL)) {
-      WARN("WriteFile failed with error %ld\n", GetLastError());
-      return GetLastError();
+    count = rpcrt4_conn_write(Connection, buffer_pos, Header->common.frag_len - hdr_size);
+    if (count<0) {
+      WARN("rpcrt4_conn_write failed\n");
+      return RPC_S_PROTOCOL_ERROR;
     }
 
     buffer_pos += Header->common.frag_len - hdr_size;
@@ -298,7 +301,8 @@ RPC_STATUS RPCRT4_Receive(RpcConnection *Connection, RpcPktHdr **Header,
                           PRPC_MESSAGE pMsg)
 {
   RPC_STATUS status;
-  DWORD dwRead, hdr_length;
+  DWORD hdr_length;
+  LONG dwRead;
   unsigned short first_flag;
   unsigned long data_length;
   unsigned long buffer_length;
@@ -310,13 +314,7 @@ RPC_STATUS RPCRT4_Receive(RpcConnection *Connection, RpcPktHdr **Header,
   TRACE("(%p, %p, %p)\n", Connection, Header, pMsg);
 
   /* read packet common header */
-  if (!ReadFile(Connection->conn, &common_hdr, sizeof(common_hdr), &dwRead, NULL)) {
-    if (GetLastError() != ERROR_MORE_DATA) {
-      WARN("ReadFile failed with error %ld\n", GetLastError());
-      status = RPC_S_PROTOCOL_ERROR;
-      goto fail;
-    }
-  }
+  dwRead = rpcrt4_conn_read(Connection, &common_hdr, sizeof(common_hdr));
   if (dwRead != sizeof(common_hdr)) {
     WARN("Short read of header, %ld/%d bytes\n", dwRead, sizeof(common_hdr));
     status = RPC_S_PROTOCOL_ERROR;
@@ -342,14 +340,7 @@ RPC_STATUS RPCRT4_Receive(RpcConnection *Connection, RpcPktHdr **Header,
   memcpy(*Header, &common_hdr, sizeof(common_hdr));
 
   /* read the rest of packet header */
-  if (!ReadFile(Connection->conn, &(*Header)->common + 1,
-                hdr_length - sizeof(common_hdr), &dwRead, NULL)) {
-    if (GetLastError() != ERROR_MORE_DATA) {
-      WARN("ReadFile failed with error %ld\n", GetLastError());
-      status = RPC_S_PROTOCOL_ERROR;
-      goto fail;
-    }
-  }
+  dwRead = rpcrt4_conn_read(Connection, &(*Header)->common + 1, hdr_length - sizeof(common_hdr));
   if (dwRead != hdr_length - sizeof(common_hdr)) {
     WARN("bad header length, %ld/%ld bytes\n", dwRead, hdr_length - sizeof(common_hdr));
     status = RPC_S_PROTOCOL_ERROR;
@@ -387,13 +378,7 @@ RPC_STATUS RPCRT4_Receive(RpcConnection *Connection, RpcPktHdr **Header,
     }
 
     if (data_length == 0) dwRead = 0; else
-    if (!ReadFile(Connection->conn, buffer_ptr, data_length, &dwRead, NULL)) {
-      if (GetLastError() != ERROR_MORE_DATA) {
-        WARN("ReadFile failed with error %ld\n", GetLastError());
-        status = RPC_S_PROTOCOL_ERROR;
-        goto fail;
-      }
-    }
+    dwRead = rpcrt4_conn_read(Connection, buffer_ptr, data_length);
     if (dwRead != data_length) {
       WARN("bad data length, %ld/%ld\n", dwRead, data_length);
       status = RPC_S_PROTOCOL_ERROR;
@@ -413,13 +398,7 @@ RPC_STATUS RPCRT4_Receive(RpcConnection *Connection, RpcPktHdr **Header,
       TRACE("next header\n");
 
       /* read the header of next packet */
-      if (!ReadFile(Connection->conn, *Header, hdr_length, &dwRead, NULL)) {
-        if (GetLastError() != ERROR_MORE_DATA) {
-          WARN("ReadFile failed with error %ld\n", GetLastError());
-          status = GetLastError();
-          goto fail;
-        }
-      }
+      dwRead = rpcrt4_conn_read(Connection, *Header, hdr_length);
       if (dwRead != hdr_length) {
         WARN("invalid packet header size (%ld)\n", dwRead);
         status = RPC_S_PROTOCOL_ERROR;
@@ -491,6 +470,14 @@ RPC_STATUS WINAPI I_RpcSend(PRPC_MESSAGE pMsg)
   } else {
     cif = pMsg->RpcInterfaceInformation;
     if (!cif) return RPC_S_INTERFACE_NOT_FOUND; /* ? */
+
+    if (!bind->Endpoint || !bind->Endpoint[0])
+    {
+      TRACE("automatically resolving partially bound binding\n");
+      status = RpcEpResolveBinding(bind, cif);
+      if (status != RPC_S_OK) return status;
+    }
+
     status = RPCRT4_OpenBinding(bind, &conn, &cif->TransferSyntax,
                                 &cif->InterfaceId);
   }
@@ -554,6 +541,14 @@ RPC_STATUS WINAPI I_RpcReceive(PRPC_MESSAGE pMsg)
     } else {
       cif = pMsg->RpcInterfaceInformation;
       if (!cif) return RPC_S_INTERFACE_NOT_FOUND; /* ? */
+
+      if (!bind->Endpoint || !bind->Endpoint[0])
+      {
+        TRACE("automatically resolving partially bound binding\n");
+        status = RpcEpResolveBinding(bind, cif);
+        if (status != RPC_S_OK) return status;
+      }
+
       status = RPCRT4_OpenBinding(bind, &conn, &cif->TransferSyntax,
                                   &cif->InterfaceId);
     }
