@@ -34,7 +34,6 @@
 #include "winbase.h"
 #include "winerror.h"
 #include "winreg.h"
-#include "ntstatus.h"
 
 #include "rpc.h"
 #include "rpcndr.h"
@@ -262,7 +261,9 @@ static void RPCRT4_process_packet(RpcConnection* conn, RpcPktHdr* hdr, RPC_MESSA
         conn->MaxTransmissionSize = hdr->bind.max_tsize;
       }
 
-      if (RPCRT4_Send(conn, response, NULL, 0) != RPC_S_OK)
+      status = RPCRT4_Send(conn, response, NULL, 0);
+      RPCRT4_FreeHeader(response);
+      if (status != RPC_S_OK)
         goto fail;
 
       break;
@@ -276,6 +277,7 @@ static void RPCRT4_process_packet(RpcConnection* conn, RpcPktHdr* hdr, RPC_MESSA
                                            status);
 
         RPCRT4_Send(conn, response, NULL, 0);
+        RPCRT4_FreeHeader(response);
         break;
       }
 
@@ -343,6 +345,7 @@ fail:
   msg->Buffer = NULL;
   RPCRT4_FreeHeader(hdr);
   TlsSetValue(worker_tls, NULL);
+  HeapFree(GetProcessHeap(), 0, msg);
 }
 
 static DWORD CALLBACK RPCRT4_worker_thread(LPVOID the_arg)
@@ -415,6 +418,7 @@ static DWORD CALLBACK RPCRT4_io_thread(LPVOID the_arg)
     status = RPCRT4_Receive(conn, &hdr, msg);
     if (status != RPC_S_OK) {
       WARN("receive failed with error %lx\n", status);
+      HeapFree(GetProcessHeap(), 0, msg);
       break;
     }
 
@@ -431,7 +435,6 @@ static DWORD CALLBACK RPCRT4_io_thread(LPVOID the_arg)
 #endif
     msg = NULL;
   }
-  HeapFree(GetProcessHeap(), 0, msg);
   RPCRT4_DestroyConnection(conn);
   return 0;
 }
@@ -473,16 +476,17 @@ static DWORD CALLBACK RPCRT4_server_thread(LPVOID the_arg)
       conn = cps->conn;
       while (conn) {
         RPCRT4_OpenConnection(conn);
-        if (conn->ovl.hEvent) count++;
+        if (rpcrt4_conn_get_wait_object(conn))
+          count++;
         conn = conn->Next;
       }
       cps = cps->Next;
     }
     /* make array of connections */
     if (objs)
-	objs = HeapReAlloc(GetProcessHeap(), 0, objs, count*sizeof(HANDLE));
+      objs = HeapReAlloc(GetProcessHeap(), 0, objs, count*sizeof(HANDLE));
     else
-	objs = HeapAlloc(GetProcessHeap(), 0, count*sizeof(HANDLE));
+      objs = HeapAlloc(GetProcessHeap(), 0, count*sizeof(HANDLE));
 
     objs[0] = m_event;
     count = 1;
@@ -490,7 +494,8 @@ static DWORD CALLBACK RPCRT4_server_thread(LPVOID the_arg)
     while (cps) {
       conn = cps->conn;
       while (conn) {
-        if (conn->ovl.hEvent) objs[count++] = conn->ovl.hEvent;
+        if ((objs[count] = rpcrt4_conn_get_wait_object(conn)))
+          count++;
         conn = conn->Next;
       }
       cps = cps->Next;
@@ -526,18 +531,18 @@ static DWORD CALLBACK RPCRT4_server_thread(LPVOID the_arg)
       while (cps) {
         conn = cps->conn;
         while (conn) {
-          if (conn->ovl.hEvent == b_handle) break;
+          if (b_handle == rpcrt4_conn_get_wait_object(conn)) break;
           conn = conn->Next;
         }
         if (conn) break;
         cps = cps->Next;
       }
       cconn = NULL;
-      if (conn) RPCRT4_SpawnConnection(&cconn, conn);
-      LeaveCriticalSection(&server_cs);
-      if (!conn) {
+      if (conn)
+        RPCRT4_SpawnConnection(&cconn, conn);
+      else
         ERR("failed to locate connection for handle %p\n", b_handle);
-      }
+      LeaveCriticalSection(&server_cs);
       if (cconn) RPCRT4_new_client(cconn);
     }
   }
@@ -719,22 +724,35 @@ RPC_STATUS WINAPI RpcServerUseProtseqEpW( LPWSTR Protseq, UINT MaxCalls, LPWSTR
   return RpcServerUseProtseqEpExW( Protseq, MaxCalls, Endpoint, SecurityDescriptor, &policy );
 }
 
+/***********************************************************************
+ *             alloc_serverprotoseq (internal)
+ */
+static RpcServerProtseq *alloc_serverprotoseq(UINT MaxCalls, char *Protseq, char *Endpoint)
+{
+  RpcServerProtseq* ps;
+
+  ps = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RpcServerProtseq));
+  ps->MaxCalls = MaxCalls;
+  ps->Protseq = Protseq;
+  ps->Endpoint = Endpoint;
+
+  return ps;
+}
+
 /***********************************************************************
  *             RpcServerUseProtseqEpExA (RPCRT4.@)
  */
 RPC_STATUS WINAPI RpcServerUseProtseqEpExA( unsigned char *Protseq, UINT MaxCalls, unsigned char *Endpoint, LPVOID SecurityDescriptor,
                                             PRPC_POLICY lpPolicy )
 {
+  char *szps = (char*)Protseq, *szep = (char*)Endpoint;
   RpcServerProtseq* ps;
 
-  TRACE("(%s,%u,%s,%p,{%u,%lu,%lu})\n", debugstr_a( (char*)Protseq ), MaxCalls,
-       debugstr_a( (char*)Endpoint ), SecurityDescriptor,
+  TRACE("(%s,%u,%s,%p,{%u,%lu,%lu})\n", debugstr_a(szps), MaxCalls,
+       debugstr_a(szep), SecurityDescriptor,
        lpPolicy->Length, lpPolicy->EndpointFlags, lpPolicy->NICFlags );
 
-  ps = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RpcServerProtseq));
-  ps->MaxCalls = MaxCalls;
-  ps->Protseq = RPCRT4_strdupA((char*)Protseq);
-  ps->Endpoint = RPCRT4_strdupA((char*)Endpoint);
+  ps = alloc_serverprotoseq(MaxCalls, RPCRT4_strdupA(szps), RPCRT4_strdupA(szep));
 
   return RPCRT4_use_protseq(ps);
 }
@@ -751,10 +769,8 @@ RPC_STATUS WINAPI RpcServerUseProtseqEpExW( LPWSTR Protseq, UINT MaxCalls, LPWST
        debugstr_w( Endpoint ), SecurityDescriptor,
        lpPolicy->Length, lpPolicy->EndpointFlags, lpPolicy->NICFlags );
 
-  ps = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RpcServerProtseq));
-  ps->MaxCalls = MaxCalls;
-  ps->Protseq = RPCRT4_strdupWtoA(Protseq);
-  ps->Endpoint = RPCRT4_strdupWtoA(Endpoint);
+  ps = alloc_serverprotoseq(MaxCalls, RPCRT4_strdupWtoA(Protseq),
+                            RPCRT4_strdupWtoA(Endpoint));
 
   return RPCRT4_use_protseq(ps);
 }
@@ -978,6 +994,9 @@ RPC_STATUS WINAPI RpcServerListen( UINT MinimumCallThreads, UINT MaxCalls, UINT
 
   status = RPCRT4_start_listen(FALSE);
 
+  if (status == RPC_S_OK)
+    RPCRT4_sync_with_server_thread();
+
   if (DontWait || (status != RPC_S_OK)) return status;
 
   return RpcMgmtWaitServerListen();
@@ -999,7 +1018,7 @@ RPC_STATUS WINAPI RpcMgmtWaitServerListen( void )
   
   LeaveCriticalSection(&listen_cs);
 
-  RPCRT4_sync_with_server_thread();
+  FIXME("not waiting for server calls to finish\n");
 
   return RPC_S_OK;
 }
@@ -1050,3 +1069,23 @@ UINT WINAPI I_RpcWindowProc( void *hWnd, UINT Message, UINT wParam, ULONG lParam
 
   return 0;
 }
+
+/***********************************************************************
+ *             RpcMgmtInqIfIds (RPCRT4.@)
+ */
+RPC_STATUS WINAPI RpcMgmtInqIfIds(RPC_BINDING_HANDLE Binding, RPC_IF_ID_VECTOR **IfIdVector)
+{
+  FIXME("(%p,%p): stub\n", Binding, IfIdVector);
+  return RPC_S_INVALID_BINDING;
+}
+
+/***********************************************************************
+ *             RpcMgmtEpEltInqBegin (RPCRT4.@)
+ */
+RPC_STATUS WINAPI RpcMgmtEpEltInqBegin(RPC_BINDING_HANDLE Binding, unsigned long InquiryType,
+    RPC_IF_ID *IfId, unsigned long VersOption, UUID *ObjectUuid, RPC_EP_INQ_HANDLE* InquiryContext)
+{
+  FIXME("(%p,%lu,%p,%lu,%p,%p): stub\n",
+        Binding, InquiryType, IfId, VersOption, ObjectUuid, InquiryContext);
+  return RPC_S_INVALID_BINDING;
+}
