@@ -436,10 +436,6 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                 TRACE("According to a MS whitepaper pszTargetName is ignored.\n");
             }
             /* Handle all the flags */
-            if(fContextReq & ISC_REQ_ALLOCATE_MEMORY)
-            {
-                FIXME("InitializeSecurityContext(): ISC_REQ_ALLOCATE_MEMORY stub\n");
-            }
             if(fContextReq & ISC_REQ_CONFIDENTIALITY)
             {
                 FIXME("InitializeSecurityContext(): ISC_REQ_CONFIDENTIALITY stub\n");
@@ -544,22 +540,35 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                     HeapFree(GetProcessHeap(), 0, bin);
                     return ret;
                 }
-                
+
                 /* put the decoded client blob into the out buffer */
-                if(pOutput == NULL){
-                    TRACE("pOutput is NULL\n");
-                    HeapFree(GetProcessHeap(), 0, buffer);
-                    HeapFree(GetProcessHeap(), 0, bin);
-                    return SEC_E_INSUFFICIENT_MEMORY;
+
+                if (fContextReq & ISC_REQ_ALLOCATE_MEMORY)
+                {
+                    if (pOutput)
+                    {
+                        pOutput->cBuffers = 1;
+                        pOutput->pBuffers[0].pvBuffer = SECUR32_ALLOC(bin_len);
+                        pOutput->pBuffers[0].cbBuffer = bin_len;
+                    }
                 }
 
-                if(pOutput->cBuffers < 1)
+                if (!pOutput || !pOutput->cBuffers || pOutput->pBuffers[0].cbBuffer < bin_len)
                 {
-                    TRACE("pOutput->cBuffers is %ld\n", pOutput->cBuffers);
+                    TRACE("out buffer is NULL or has not enough space\n");
                     HeapFree(GetProcessHeap(), 0, buffer);
                     HeapFree(GetProcessHeap(), 0, bin);
-                    return SEC_E_INSUFFICIENT_MEMORY;
+                    return SEC_E_BUFFER_TOO_SMALL;
                 }
+
+                if (!pOutput->pBuffers[0].pvBuffer)
+                {
+                    TRACE("out buffer is NULL\n");
+                    HeapFree(GetProcessHeap(), 0, buffer);
+                    HeapFree(GetProcessHeap(), 0, bin);
+                    return SEC_E_INTERNAL_ERROR;
+                }
+
                 pOutput->pBuffers[0].cbBuffer = bin_len;
                 pOutput->pBuffers[0].BufferType = SECBUFFER_DATA;
                 memcpy(pOutput->pBuffers[0].pvBuffer, bin, bin_len);
@@ -570,18 +579,18 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
             {
                 /* handle second call here */
                 /* encode server data to base64 */
-                if(pInput == NULL)
+                if (!pInput || !pInput->cBuffers)
                 {
                     HeapFree(GetProcessHeap(), 0, buffer);
                     HeapFree(GetProcessHeap(), 0, bin);
                     return SEC_E_INCOMPLETE_MESSAGE;
                 }
-                
-                if(pInput->cBuffers < 1)
+
+                if (!pInput->pBuffers[0].pvBuffer)
                 {
                     HeapFree(GetProcessHeap(), 0, buffer);
                     HeapFree(GetProcessHeap(), 0, bin);
-                    return SEC_E_INCOMPLETE_MESSAGE;
+                    return SEC_E_INTERNAL_ERROR;
                 }
 
                 if(pInput->pBuffers[0].cbBuffer > max_len)
@@ -638,18 +647,32 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                     return ret;
                 }
 
-                if(pOutput == NULL)
+                /* put the decoded client blob into the out buffer */
+
+                if (fContextReq & ISC_REQ_ALLOCATE_MEMORY)
                 {
-                    HeapFree(GetProcessHeap(), 0, buffer);
-                    HeapFree(GetProcessHeap(), 0, bin);
-                    return SEC_E_INSUFFICIENT_MEMORY;
+                    if (pOutput)
+                    {
+                        pOutput->cBuffers = 1;
+                        pOutput->pBuffers[0].pvBuffer = SECUR32_ALLOC(bin_len);
+                        pOutput->pBuffers[0].cbBuffer = bin_len;
+                    }
                 }
 
-                if(pOutput->cBuffers < 1)
+                if (!pOutput || !pOutput->cBuffers || pOutput->pBuffers[0].cbBuffer < bin_len)
                 {
+                    TRACE("out buffer is NULL or has not enough space\n");
                     HeapFree(GetProcessHeap(), 0, buffer);
                     HeapFree(GetProcessHeap(), 0, bin);
-                    return SEC_E_INSUFFICIENT_MEMORY;
+                    return SEC_E_BUFFER_TOO_SMALL;
+                }
+
+                if (!pOutput->pBuffers[0].pvBuffer)
+                {
+                    TRACE("out buffer is NULL\n");
+                    HeapFree(GetProcessHeap(), 0, buffer);
+                    HeapFree(GetProcessHeap(), 0, bin);
+                    return SEC_E_INTERNAL_ERROR;
                 }
 
                 pOutput->pBuffers[0].cbBuffer = bin_len;
@@ -659,7 +682,6 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
                 ret = SEC_E_OK;
                 phNewContext->dwUpper = ctxt_attr;
                 phNewContext->dwLower = ret;
-
             }
             HeapFree(GetProcessHeap(), 0, buffer);
             HeapFree(GetProcessHeap(), 0, bin);
@@ -1005,7 +1027,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_DeleteSecurityContext(PCtxtHandle phContex
     TRACE("%p\n", phContext);
     if (phContext)
     {
-        ret = SEC_E_UNSUPPORTED_FUNCTION;
+        phContext->dwUpper = 0;
+        phContext->dwLower = 0;
+        ret = SEC_E_OK;
     }
     else
     {
