@@ -396,14 +396,20 @@ SecureProvider *SECUR32_addProvider(const SecurityFunctionTableA *fnTableA,
     {
         providerTable = HeapAlloc(GetProcessHeap(), 0, sizeof(SecureProviderTable));
         if (!providerTable)
+        {
+            LeaveCriticalSection(&cs);
             return NULL;
+        }
 
         list_init(&providerTable->table);
     }
 
     ret = HeapAlloc(GetProcessHeap(), 0, sizeof(SecureProvider));
     if (!ret)
+    {
+        LeaveCriticalSection(&cs);
         return NULL;
+    }
 
     list_add_tail(&providerTable->table, &ret->entry);
     ret->lib = NULL;
@@ -439,7 +445,10 @@ void SECUR32_addPackages(SecureProvider *provider, ULONG toAdd,
     {
         packageTable = HeapAlloc(GetProcessHeap(), 0, sizeof(SecurePackageTable));
         if (!packageTable)
+        {
+            LeaveCriticalSection(&cs);
             return;
+        }
 
         packageTable->numPackages = 0;
         list_init(&packageTable->table);
@@ -657,8 +666,7 @@ static void SECUR32_freeProviders(void)
     {
         LIST_FOR_EACH_ENTRY(provider, &providerTable->table, SecureProvider, entry)
         {
-            if (provider->moduleName)
-                SECUR32_FREE(provider->moduleName);
+            SECUR32_FREE(provider->moduleName);
             if (provider->lib)
                 FreeLibrary(provider->lib);
         }
@@ -680,19 +688,9 @@ static void SECUR32_freeProviders(void)
  */
 SECURITY_STATUS WINAPI FreeContextBuffer(PVOID pv)
 {
-    SECURITY_STATUS ret;
+    SECUR32_FREE(pv);
 
-    /* as it turns out, SECURITY_STATUSes are actually HRESULTS */
-    if (pv)
-    {
-        if (SECUR32_FREE(pv) == NULL)
-            ret = SEC_E_OK;
-        else
-            ret = HRESULT_FROM_WIN32(GetLastError());
-    }
-    else
-        ret = HRESULT_FROM_WIN32(ERROR_INVALID_PARAMETER);
-    return ret;
+    return SEC_E_OK;
 }
 
 /***********************************************************************
@@ -703,6 +701,8 @@ SECURITY_STATUS WINAPI EnumerateSecurityPackagesW(PULONG pcPackages,
 {
     SECURITY_STATUS ret = SEC_E_OK;
 
+    TRACE("(%p, %p)\n", pcPackages, ppPackageInfo);
+
     /* windows just crashes if pcPackages or ppPackageInfo is NULL, so will I */
     *pcPackages = 0;
     EnterCriticalSection(&cs);
@@ -737,6 +737,7 @@ SECURITY_STATUS WINAPI EnumerateSecurityPackagesW(PULONG pcPackages,
                     memcpy(pkgInfo, &package->infoW, sizeof(SecPkgInfoW));
                     if (package->infoW.Name)
                     {
+                        TRACE("Name[%ld] = %s\n", i - 1, debugstr_w(package->infoW.Name));
                         pkgInfo->Name = nextString;
                         lstrcpyW(nextString, package->infoW.Name);
                         nextString += lstrlenW(nextString) + 1;
@@ -745,6 +746,7 @@ SECURITY_STATUS WINAPI EnumerateSecurityPackagesW(PULONG pcPackages,
                         pkgInfo->Name = NULL;
                     if (package->infoW.Comment)
                     {
+                        TRACE("Comment[%ld] = %s\n", i - 1, debugstr_w(package->infoW.Comment));
                         pkgInfo->Comment = nextString;
                         lstrcpyW(nextString, package->infoW.Comment);
                         nextString += lstrlenW(nextString) + 1;
@@ -758,6 +760,7 @@ SECURITY_STATUS WINAPI EnumerateSecurityPackagesW(PULONG pcPackages,
         }
     }
     LeaveCriticalSection(&cs);
+    TRACE("<-- 0x%08lx\n", ret);
     return ret;
 }
 
