@@ -76,6 +76,7 @@ static const WCHAR UpdateIniFields[] = {'U','p','d','a','t','e','I','n','i','F',
 static const WCHAR RegisterDlls[]    = {'R','e','g','i','s','t','e','r','D','l','l','s',0};
 static const WCHAR UnregisterDlls[]  = {'U','n','r','e','g','i','s','t','e','r','D','l','l','s',0};
 static const WCHAR ProfileItems[]    = {'P','r','o','f','i','l','e','I','t','e','m','s',0};
+static const WCHAR WineFakeDlls[]    = {'W','i','n','e','F','a','k','e','D','l','l','s',0};
 
 
 /***********************************************************************
@@ -444,6 +445,7 @@ static BOOL do_register_dll( const struct register_dll_info *info, const WCHAR *
     HMODULE module;
     HRESULT res;
     SP_REGISTER_CONTROL_STATUSW status;
+    IMAGE_NT_HEADERS *nt;
 
     status.cbSize = sizeof(status);
     status.FileName = path;
@@ -473,6 +475,45 @@ static BOOL do_register_dll( const struct register_dll_info *info, const WCHAR *
         goto done;
     }
 
+    if ((nt = RtlImageNtHeader( module )) && !(nt->FileHeader.Characteristics & IMAGE_FILE_DLL))
+    {
+        /* file is an executable, not a dll */
+        STARTUPINFOW startup;
+        PROCESS_INFORMATION info;
+        WCHAR *cmd_line;
+        BOOL res;
+        static const WCHAR format[] = {'"','%','s','"',' ','%','s',0};
+        static const WCHAR default_args[] = {'/','R','e','g','S','e','r','v','e','r',0};
+
+        FreeLibrary( module );
+        module = NULL;
+        if (!args) args = default_args;
+        cmd_line = HeapAlloc( GetProcessHeap(), 0, (strlenW(path) + strlenW(args) + 4) * sizeof(WCHAR) );
+        sprintfW( cmd_line, format, path, args );
+        memset( &startup, 0, sizeof(startup) );
+        startup.cb = sizeof(startup);
+        TRACE( "executing %s\n", debugstr_w(cmd_line) );
+        res = CreateProcessW( NULL, cmd_line, NULL, NULL, FALSE, 0, NULL, NULL, &startup, &info );
+        HeapFree( GetProcessHeap(), 0, cmd_line );
+        if (!res)
+        {
+            status.FailureCode = SPREG_LOADLIBRARY;
+            status.Win32Error = GetLastError();
+            goto done;
+        }
+        CloseHandle( info.hThread );
+
+        if (WaitForSingleObject( info.hProcess, timeout*1000 ) == WAIT_TIMEOUT)
+        {
+            /* timed out, kill the process */
+            TerminateProcess( info.hProcess, 1 );
+            status.FailureCode = SPREG_TIMEOUT;
+            status.Win32Error = ERROR_TIMEOUT;
+        }
+        CloseHandle( info.hProcess );
+        goto done;
+    }
+
     if (flags & FLG_REGSVR_DLLREGISTER)
     {
         const char *entry_point = info->unregister ? "DllUnregisterServer" : "DllRegisterServer";
@@ -580,6 +621,48 @@ static BOOL register_dlls_callback( HINF hinf, PCWSTR field, void *arg )
     return ret;
 }
 
+/***********************************************************************
+ *            fake_dlls_callback
+ *
+ * Called once for each WineFakeDlls entry in a given section.
+ */
+static BOOL fake_dlls_callback( HINF hinf, PCWSTR field, void *arg )
+{
+    INFCONTEXT context;
+    BOOL ret = TRUE;
+    BOOL ok = SetupFindFirstLineW( hinf, field, NULL, &context );
+
+    for (; ok; ok = SetupFindNextLine( &context, &context ))
+    {
+        WCHAR *path, *p;
+        WCHAR buffer[MAX_INF_STRING_LENGTH];
+
+        /* get directory */
+        if (!(path = PARSER_get_dest_dir( &context ))) continue;
+
+        /* get dll name */
+        if (!SetupGetStringFieldW( &context, 3, buffer, sizeof(buffer)/sizeof(WCHAR), NULL ))
+            goto done;
+        if (!(p = HeapReAlloc( GetProcessHeap(), 0, path,
+                               (strlenW(path) + strlenW(buffer) + 2) * sizeof(WCHAR) ))) goto done;
+        path = p;
+        p += strlenW(p);
+        if (p == path || p[-1] != '\\') *p++ = '\\';
+        strcpyW( p, buffer );
+
+        /* get source dll */
+        if (SetupGetStringFieldW( &context, 4, buffer, sizeof(buffer)/sizeof(WCHAR), NULL ))
+            p = buffer;  /* otherwise use target base name as default source */
+
+        create_fake_dll( path, p );  /* ignore errors */
+
+    done:
+        HeapFree( GetProcessHeap(), 0, path );
+        if (!ret) break;
+    }
+    return ret;
+}
+
 /***********************************************************************
  *            update_ini_callback
  *
@@ -850,6 +933,9 @@ BOOL WINAPI SetupInstallFromInfSectionW( HWND owner, HINF hinf, PCWSTR section,
 
         if (!iterate_section_fields( hinf, section, RegisterDlls, register_dlls_callback, &info ))
             return FALSE;
+
+        if (!iterate_section_fields( hinf, section, WineFakeDlls, fake_dlls_callback, NULL ))
+            return FALSE;
     }
     if (flags & SPINST_UNREGSVR)
     {
