@@ -150,54 +150,6 @@ BOOL WINAPI SetupCopyOEMInfW(PCWSTR sourceinffile, PCWSTR sourcemedialoc,
   return FALSE;
 }
 
-/***********************************************************************
- *		SetupGetInfInformationA    (SETUPAPI.@)
- */
-BOOL WINAPI SetupGetInfInformationA( LPCVOID InfSpec, DWORD SearchControl,
-                                     PSP_INF_INFORMATION ReturnBuffer,
-                                     DWORD ReturnBufferSize, PDWORD RequiredSize)
-{
-    FIXME("(%p, %ld, %p, %ld, %p) Stub!\n",
-          InfSpec, SearchControl, ReturnBuffer, ReturnBufferSize, RequiredSize );
-    return TRUE;
-}
-
-/***********************************************************************
- *      SetupGetInfInformationW    (SETUPAPI.@)
- */
-BOOL WINAPI SetupGetInfInformationW( LPCVOID InfSpec, DWORD SearchControl,
-                                     PSP_INF_INFORMATION ReturnBuffer,
-                                     DWORD ReturnBufferSize, PDWORD RequiredSize)
-{
-    FIXME("(%p, %ld, %p, %ld, %p) Stub!\n",
-          InfSpec, SearchControl, ReturnBuffer, ReturnBufferSize, RequiredSize );
-    return TRUE;
-}
-
-/***********************************************************************
- *      SetupQueryInfFileInformationA    (SETUPAPI.@)
- */
-BOOL WINAPI SetupQueryInfFileInformationA(PSP_INF_INFORMATION InfInformation,
-                                          UINT InfIndex, PSTR ReturnBuffer,
-                                          DWORD ReturnBufferSize, PDWORD RequiredSize)
-{
-    FIXME("(%p, %u, %p, %ld, %p) Stub!\n",
-          InfInformation, InfIndex, ReturnBuffer, ReturnBufferSize, RequiredSize );
-    return TRUE;
-}
-
-/***********************************************************************
- *      SetupQueryInfFileInformationW    (SETUPAPI.@)
- */
-BOOL WINAPI SetupQueryInfFileInformationW(PSP_INF_INFORMATION InfInformation,
-                                          UINT InfIndex, PWSTR ReturnBuffer,
-                                          DWORD ReturnBufferSize, PDWORD RequiredSize) 
-{
-    FIXME("(%p, %u, %p, %ld, %p) Stub!\n",
-          InfInformation, InfIndex, ReturnBuffer, ReturnBufferSize, RequiredSize );
-    return TRUE;
-}
-
 /***********************************************************************
  *		SetupInitializeFileLogW(SETUPAPI.@)
  */
@@ -224,3 +176,12 @@ BOOL WINAPI SetupTerminateFileLog(HANDLE FileLogHandle)
     FIXME ("Stub %p\n",FileLogHandle);
     return TRUE;
 }
+
+/***********************************************************************
+ *		RegistryDelnode(SETUPAPI.@)
+ */
+BOOL WINAPI RegistryDelnode(DWORD x, DWORD y)
+{
+    FIXME("%08lx %08lx: stub\n", x, y);
+    return FALSE;
+}
