@@ -21,11 +21,11 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(shdocvw);
 
-#define CLIENTSITE_THIS(iface) DEFINE_THIS(WebBrowser, OleClientSite, iface)
+#define CLIENTSITE_THIS(iface) DEFINE_THIS(DocHost, OleClientSite, iface)
 
 static HRESULT WINAPI ClientSite_QueryInterface(IOleClientSite *iface, REFIID riid, void **ppv)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
 
     *ppv = NULL;
 
@@ -50,19 +50,19 @@ static HRESULT WINAPI ClientSite_QueryInterface(IOleClientSite *iface, REFIID ri
     }else if(IsEqualGUID(&IID_IOleDocumentSite, riid)) {
         TRACE("(%p)->(IID_IOleDocumentSite %p)\n", This, ppv);
         *ppv = DOCSITE(This);
-    }else if(IsEqualGUID(&IID_IOleClientSite, riid)) {
-        TRACE("(%p)->(IID_IOleClientSite %p)\n", This, ppv);
+    }else if(IsEqualGUID(&IID_IOleCommandTarget, riid)) {
+        TRACE("(%p)->(IID_IOleCommandTarget %p)\n", This, ppv);
         *ppv = CLOLECMD(This);
     }else if(IsEqualGUID(&IID_IDispatch, riid)) {
         TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
         *ppv = CLDISP(This);
     }else if(IsEqualGUID(&IID_IServiceProvider, riid)) {
         TRACE("(%p)->(IID_IServiceProvider %p)\n", This, ppv);
-        *ppv = CLSERVPROV(This);
+        *ppv = SERVPROV(This);
     }
 
     if(*ppv) {
-        IWebBrowser2_AddRef(WEBBROWSER(This));
+        IOleClientSite_AddRef(CLIENTSITE(This));
         return S_OK;
     }
 
@@ -73,19 +73,19 @@ static HRESULT WINAPI ClientSite_QueryInterface(IOleClientSite *iface, REFIID ri
 
 static ULONG WINAPI ClientSite_AddRef(IOleClientSite *iface)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
-    return IWebBrowser2_AddRef(WEBBROWSER(This));
+    DocHost *This = CLIENTSITE_THIS(iface);
+    return IDispatch_AddRef(This->disp);
 }
 
 static ULONG WINAPI ClientSite_Release(IOleClientSite *iface)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
-    return IWebBrowser2_Release(WEBBROWSER(This));
+    DocHost *This = CLIENTSITE_THIS(iface);
+    return IDispatch_Release(This->disp);
 }
 
 static HRESULT WINAPI ClientSite_SaveObject(IOleClientSite *iface)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -93,35 +93,35 @@ static HRESULT WINAPI ClientSite_SaveObject(IOleClientSite *iface)
 static HRESULT WINAPI ClientSite_GetMoniker(IOleClientSite *iface, DWORD dwAssign,
                                             DWORD dwWhichMoniker, IMoniker **ppmk)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)->(%ld %ld %p)\n", This, dwAssign, dwWhichMoniker, ppmk);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ClientSite_GetContainer(IOleClientSite *iface, IOleContainer **ppContainer)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)->(%p)\n", This, ppContainer);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ClientSite_ShowObject(IOleClientSite *iface)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ClientSite_OnShowWindow(IOleClientSite *iface, BOOL fShow)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fShow);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI ClientSite_RequestNewObjectLayout(IOleClientSite *iface)
 {
-    WebBrowser *This = CLIENTSITE_THIS(iface);
+    DocHost *This = CLIENTSITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -140,46 +140,46 @@ static const IOleClientSiteVtbl OleClientSiteVtbl = {
     ClientSite_RequestNewObjectLayout
 };
 
-#define INPLACESITE_THIS(iface) DEFINE_THIS(WebBrowser, OleInPlaceSite, iface)
+#define INPLACESITE_THIS(iface) DEFINE_THIS(DocHost, OleInPlaceSite, iface)
 
 static HRESULT WINAPI InPlaceSite_QueryInterface(IOleInPlaceSite *iface, REFIID riid, void **ppv)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI InPlaceSite_AddRef(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI InPlaceSite_Release(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI InPlaceSite_GetWindow(IOleInPlaceSite *iface, HWND *phwnd)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
 
     TRACE("(%p)->(%p)\n", This, phwnd);
 
-    *phwnd = This->doc_view_hwnd;
+    *phwnd = This->hwnd;
     return S_OK;
 }
 
 static HRESULT WINAPI InPlaceSite_ContextSensitiveHelp(IOleInPlaceSite *iface, BOOL fEnterMode)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fEnterMode);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI InPlaceSite_CanInPlaceActivate(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
 
     TRACE("(%p)\n", This);
 
@@ -189,7 +189,7 @@ static HRESULT WINAPI InPlaceSite_CanInPlaceActivate(IOleInPlaceSite *iface)
 
 static HRESULT WINAPI InPlaceSite_OnInPlaceActivate(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
 
     TRACE("(%p)\n", This);
 
@@ -199,7 +199,7 @@ static HRESULT WINAPI InPlaceSite_OnInPlaceActivate(IOleInPlaceSite *iface)
 
 static HRESULT WINAPI InPlaceSite_OnUIActivate(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -208,7 +208,7 @@ static HRESULT WINAPI InPlaceSite_GetWindowContext(IOleInPlaceSite *iface,
         IOleInPlaceFrame **ppFrame, IOleInPlaceUIWindow **ppDoc, LPRECT lprcPosRect,
         LPRECT lprcClipRect, LPOLEINPLACEFRAMEINFO lpFrameInfo)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
 
     TRACE("(%p)->(%p %p %p %p %p)\n", This, ppFrame, ppDoc, lprcPosRect,
           lprcClipRect, lpFrameInfo);
@@ -217,12 +217,12 @@ static HRESULT WINAPI InPlaceSite_GetWindowContext(IOleInPlaceSite *iface,
     *ppFrame = INPLACEFRAME(This);
     *ppDoc = NULL;
 
-    GetClientRect(This->doc_view_hwnd, lprcPosRect);
+    GetClientRect(This->hwnd, lprcPosRect);
     memcpy(lprcClipRect, lprcPosRect, sizeof(RECT));
 
     lpFrameInfo->cb = sizeof(*lpFrameInfo);
     lpFrameInfo->fMDIApp = FALSE;
-    lpFrameInfo->hwndFrame = This->shell_embedding_hwnd;
+    lpFrameInfo->hwndFrame = This->frame_hwnd;
     lpFrameInfo->haccel = NULL;
     lpFrameInfo->cAccelEntries = 0; /* FIXME: should be 5 */
 
@@ -231,35 +231,35 @@ static HRESULT WINAPI InPlaceSite_GetWindowContext(IOleInPlaceSite *iface,
 
 static HRESULT WINAPI InPlaceSite_Scroll(IOleInPlaceSite *iface, SIZE scrollExtent)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)->({%ld %ld})\n", This, scrollExtent.cx, scrollExtent.cy);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI InPlaceSite_OnUIDeactivate(IOleInPlaceSite *iface, BOOL fUndoable)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fUndoable);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI InPlaceSite_OnInPlaceDeactivate(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI InPlaceSite_DiscardUndoState(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI InPlaceSite_DeactivateAndUndo(IOleInPlaceSite *iface)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -267,7 +267,7 @@ static HRESULT WINAPI InPlaceSite_DeactivateAndUndo(IOleInPlaceSite *iface)
 static HRESULT WINAPI InPlaceSite_OnPosRectChange(IOleInPlaceSite *iface,
                                                   LPCRECT lprcPosRect)
 {
-    WebBrowser *This = INPLACESITE_THIS(iface);
+    DocHost *This = INPLACESITE_THIS(iface);
     FIXME("(%p)->(%p)\n", This, lprcPosRect);
     return E_NOTIMPL;
 }
@@ -292,31 +292,31 @@ static const IOleInPlaceSiteVtbl OleInPlaceSiteVtbl = {
     InPlaceSite_OnPosRectChange
 };
 
-#define DOCSITE_THIS(iface) DEFINE_THIS(WebBrowser, OleDocumentSite, iface)
+#define DOCSITE_THIS(iface) DEFINE_THIS(DocHost, OleDocumentSite, iface)
 
 static HRESULT WINAPI OleDocumentSite_QueryInterface(IOleDocumentSite *iface,
                                                      REFIID riid, void **ppv)
 {
-    WebBrowser *This = DOCSITE_THIS(iface);
+    DocHost *This = DOCSITE_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI OleDocumentSite_AddRef(IOleDocumentSite *iface)
 {
-    WebBrowser *This = DOCSITE_THIS(iface);
+    DocHost *This = DOCSITE_THIS(iface);
     return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI OleDocumentSite_Release(IOleDocumentSite *iface)
 {
-    WebBrowser *This = DOCSITE_THIS(iface);
+    DocHost *This = DOCSITE_THIS(iface);
     return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI OleDocumentSite_ActivateMe(IOleDocumentSite *iface,
                                                  IOleDocumentView *pViewToActivate)
 {
-    WebBrowser *This = DOCSITE_THIS(iface);
+    DocHost *This = DOCSITE_THIS(iface);
     IOleDocument *oledoc;
     RECT rect;
     HRESULT hres;
@@ -330,7 +330,7 @@ static HRESULT WINAPI OleDocumentSite_ActivateMe(IOleDocumentSite *iface,
     IOleDocument_CreateView(oledoc, INPLACESITE(This), NULL, 0, &This->view);
     IOleDocument_Release(oledoc);
 
-    GetClientRect(This->doc_view_hwnd, &rect);
+    GetClientRect(This->hwnd, &rect);
     IOleDocumentView_SetRect(This->view, &rect);
 
     hres = IOleDocumentView_Show(This->view, TRUE);
@@ -347,29 +347,29 @@ static const IOleDocumentSiteVtbl OleDocumentSiteVtbl = {
     OleDocumentSite_ActivateMe
 };
 
-#define DISP_THIS(iface) DEFINE_THIS(WebBrowser, Dispatch, iface)
+#define DISP_THIS(iface) DEFINE_THIS(DocHost, Dispatch, iface)
 
 static HRESULT WINAPI ClDispatch_QueryInterface(IDispatch *iface, REFIID riid, void **ppv)
 {
-    WebBrowser *This = DISP_THIS(iface);
+    DocHost *This = DISP_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI ClDispatch_AddRef(IDispatch *iface)
 {
-    WebBrowser *This = DISP_THIS(iface);
-    return IWebBrowser2_AddRef(WEBBROWSER(This));
+    DocHost *This = DISP_THIS(iface);
+    return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI ClDispatch_Release(IDispatch *iface)
 {
-    WebBrowser *This = DISP_THIS(iface);
-    return IWebBrowser2_Release(WEBBROWSER(This));
+    DocHost *This = DISP_THIS(iface);
+    return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI ClDispatch_GetTypeInfoCount(IDispatch *iface, UINT *pctinfo)
 {
-    WebBrowser *This = DISP_THIS(iface);
+    DocHost *This = DISP_THIS(iface);
 
     TRACE("(%p)->(%p)\n", This, pctinfo);
 
@@ -379,7 +379,7 @@ static HRESULT WINAPI ClDispatch_GetTypeInfoCount(IDispatch *iface, UINT *pctinf
 static HRESULT WINAPI ClDispatch_GetTypeInfo(IDispatch *iface, UINT iTInfo, LCID lcid,
                                              ITypeInfo **ppTInfo)
 {
-    WebBrowser *This = DISP_THIS(iface);
+    DocHost *This = DISP_THIS(iface);
 
     TRACE("(%p)->(%u %ld %p)\n", This, iTInfo, lcid, ppTInfo);
 
@@ -389,7 +389,7 @@ static HRESULT WINAPI ClDispatch_GetTypeInfo(IDispatch *iface, UINT iTInfo, LCID
 static HRESULT WINAPI ClDispatch_GetIDsOfNames(IDispatch *iface, REFIID riid, LPOLESTR *rgszNames,
                                                UINT cNames, LCID lcid, DISPID *rgDispId)
 {
-    WebBrowser *This = DISP_THIS(iface);
+    DocHost *This = DISP_THIS(iface);
 
     TRACE("(%p)->(%s %p %u %ld %p)\n", This, debugstr_guid(riid), rgszNames, cNames,
           lcid, rgDispId);
@@ -401,7 +401,7 @@ static HRESULT WINAPI ClDispatch_Invoke(IDispatch *iface, DISPID dispIdMember, R
                                         LCID lcid, WORD wFlags, DISPPARAMS *pDispParams,
                                         VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
-    WebBrowser *This = DISP_THIS(iface);
+    DocHost *This = DISP_THIS(iface);
     FIXME("(%p)->(%ld %s %ld %04x %p %p %p %p)\n", This, dispIdMember, debugstr_guid(riid),
           lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
     return E_NOTIMPL;
@@ -419,35 +419,35 @@ static const IDispatchVtbl DispatchVtbl = {
     ClDispatch_Invoke
 };
 
-#define SERVPROV_THIS(iface) DEFINE_THIS(WebBrowser, ClServiceProvider, iface)
+#define SERVPROV_THIS(iface) DEFINE_THIS(DocHost, ServiceProvider, iface)
 
 static HRESULT WINAPI ClServiceProvider_QueryInterface(IServiceProvider *iface, REFIID riid,
                                                        void **ppv)
 {
-    WebBrowser *This = SERVPROV_THIS(iface);
+    DocHost *This = SERVPROV_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI ClServiceProvider_AddRef(IServiceProvider *iface)
 {
-    WebBrowser *This = SERVPROV_THIS(iface);
-    return IWebBrowser2_AddRef(WEBBROWSER2(This));
+    DocHost *This = SERVPROV_THIS(iface);
+    return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI ClServiceProvider_Release(IServiceProvider *iface)
 {
-    WebBrowser *This = SERVPROV_THIS(iface);
-    return IWebBrowser2_Release(WEBBROWSER2(This));
+    DocHost *This = SERVPROV_THIS(iface);
+    return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI ClServiceProvider_QueryService(IServiceProvider *iface, REFGUID guidService,
                                                      REFIID riid, void **ppv)
 {
-    WebBrowser *This = SERVPROV_THIS(iface);
+    DocHost *This = SERVPROV_THIS(iface);
 
     if(IsEqualGUID(&IID_IHlinkFrame, guidService)) {
         TRACE("(%p)->(IID_IHlinkFrame %s %p)\n", This, debugstr_guid(riid), ppv);
-        return IWebBrowser2_QueryInterface(WEBBROWSER(This), riid, ppv);
+        return IDispatch_QueryInterface(This->disp, riid, ppv);
     }
 
     FIXME("(%p)->(%s %s %p)\n", This, debugstr_guid(guidService), debugstr_guid(riid), ppv);
@@ -464,18 +464,18 @@ static const IServiceProviderVtbl ServiceProviderVtbl = {
     ClServiceProvider_QueryService
 };
 
-void WebBrowser_ClientSite_Init(WebBrowser *This)
+void DocHost_ClientSite_Init(DocHost *This)
 {
-    This->lpOleClientSiteVtbl     = &OleClientSiteVtbl;
-    This->lpOleInPlaceSiteVtbl    = &OleInPlaceSiteVtbl;
-    This->lpOleDocumentSiteVtbl   = &OleDocumentSiteVtbl;
-    This->lpDispatchVtbl          = &DispatchVtbl;
-    This->lpClServiceProviderVtbl = &ServiceProviderVtbl;
+    This->lpOleClientSiteVtbl   = &OleClientSiteVtbl;
+    This->lpOleInPlaceSiteVtbl  = &OleInPlaceSiteVtbl;
+    This->lpOleDocumentSiteVtbl = &OleDocumentSiteVtbl;
+    This->lpDispatchVtbl        = &DispatchVtbl;
+    This->lpServiceProviderVtbl = &ServiceProviderVtbl;
 
     This->view = NULL;
 }
 
-void WebBrowser_ClientSite_Destroy(WebBrowser *This)
+void DocHost_ClientSite_Release(DocHost *This)
 {
     if(This->view)
         IOleDocumentView_Release(This->view);
