@@ -25,7 +25,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(shdocvw);
 
 static ATOM doc_view_atom = 0;
 
-static void navigate_complete(WebBrowser *This)
+static void navigate_complete(DocHost *This)
 {
     IDispatch *disp = NULL;
     DISPPARAMS dispparams;
@@ -58,7 +58,7 @@ static void navigate_complete(WebBrowser *This)
         IDispatch_Release(disp);
 }
 
-static LRESULT navigate2(WebBrowser *This)
+static LRESULT navigate2(DocHost *This)
 {
     IHlinkTarget *hlink;
     HRESULT hres;
@@ -88,7 +88,7 @@ static LRESULT navigate2(WebBrowser *This)
     return 0;
 }
 
-static LRESULT resize_document(WebBrowser *This, LONG width, LONG height)
+static LRESULT resize_document(DocHost *This, LONG width, LONG height)
 {
     RECT rect = {0, 0, width, height};
 
@@ -102,12 +102,12 @@ static LRESULT resize_document(WebBrowser *This, LONG width, LONG height)
 
 static LRESULT WINAPI doc_view_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
-    WebBrowser *This;
+    DocHost *This;
 
     static const WCHAR wszTHIS[] = {'T','H','I','S',0};
 
     if(msg == WM_CREATE) {
-        This = *(WebBrowser**)lParam;
+        This = *(DocHost**)lParam;
         SetPropW(hwnd, wszTHIS, This);
     }else {
         This = GetPropW(hwnd, wszTHIS);
@@ -120,10 +120,10 @@ static LRESULT WINAPI doc_view_proc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM l
         return navigate2(This);
     }
 
-    return DefWindowProcA(hwnd, msg, wParam, lParam);
+    return DefWindowProcW(hwnd, msg, wParam, lParam);
 }
 
-void create_doc_view_hwnd(WebBrowser *This)
+void create_doc_view_hwnd(DocHost *This)
 {
     RECT rect;
 
@@ -146,15 +146,15 @@ void create_doc_view_hwnd(WebBrowser *This)
         doc_view_atom = RegisterClassExW(&wndclass);
     }
 
-    GetClientRect(This->shell_embedding_hwnd, &rect);
-    This->doc_view_hwnd = CreateWindowExW(0, wszShell_DocObject_View,
+    GetClientRect(This->frame_hwnd, &rect); /* FIXME */
+    This->hwnd = CreateWindowExW(0, wszShell_DocObject_View,
          wszShell_DocObject_View,
-         WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_TABSTOP | WS_MAXIMIZEBOX,
-         rect.left, rect.top, rect.right, rect.bottom, This->shell_embedding_hwnd,
+         WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_TABSTOP,
+         rect.left, rect.top, rect.right, rect.bottom, This->frame_hwnd,
          NULL, shdocvw_hinstance, This);
 }
 
-void deactivate_document(WebBrowser *This)
+void deactivate_document(DocHost *This)
 {
     IOleInPlaceObjectWindowless *winobj;
     IOleObject *oleobj = NULL;
@@ -206,31 +206,31 @@ void deactivate_document(WebBrowser *This)
     This->document = NULL;
 }
 
-#define OLECMD_THIS(iface) DEFINE_THIS(WebBrowser, ClOleCommandTarget, iface)
+#define OLECMD_THIS(iface) DEFINE_THIS(DocHost, OleCommandTarget, iface)
 
 static HRESULT WINAPI ClOleCommandTarget_QueryInterface(IOleCommandTarget *iface,
         REFIID riid, void **ppv)
 {
-    WebBrowser *This = OLECMD_THIS(iface);
+    DocHost *This = OLECMD_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI ClOleCommandTarget_AddRef(IOleCommandTarget *iface)
 {
-    WebBrowser *This = OLECMD_THIS(iface);
-    return IWebBrowser2_AddRef(WEBBROWSER(This));
+    DocHost *This = OLECMD_THIS(iface);
+    return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI ClOleCommandTarget_Release(IOleCommandTarget *iface)
 {
-    WebBrowser *This = OLECMD_THIS(iface);
-    return IWebBrowser2_Release(WEBBROWSER(This));
+    DocHost *This = OLECMD_THIS(iface);
+    return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI ClOleCommandTarget_QueryStatus(IOleCommandTarget *iface,
         const GUID *pguidCmdGroup, ULONG cCmds, OLECMD prgCmds[], OLECMDTEXT *pCmdText)
 {
-    WebBrowser *This = OLECMD_THIS(iface);
+    DocHost *This = OLECMD_THIS(iface);
     FIXME("(%p)->(%s %lu %p %p)\n", This, debugstr_guid(pguidCmdGroup), cCmds, prgCmds,
           pCmdText);
     return E_NOTIMPL;
@@ -240,7 +240,7 @@ static HRESULT WINAPI ClOleCommandTarget_Exec(IOleCommandTarget *iface,
         const GUID *pguidCmdGroup, DWORD nCmdID, DWORD nCmdexecopt, VARIANT *pvaIn,
         VARIANT *pvaOut)
 {
-    WebBrowser *This = OLECMD_THIS(iface);
+    DocHost *This = OLECMD_THIS(iface);
     FIXME("(%p)->(%s %ld %ld %p %p)\n", This, debugstr_guid(pguidCmdGroup), nCmdID,
           nCmdexecopt, pvaIn, pvaOut);
     return E_NOTIMPL;
@@ -256,31 +256,31 @@ static const IOleCommandTargetVtbl OleCommandTargetVtbl = {
     ClOleCommandTarget_Exec
 };
 
-#define DOCHOSTUI_THIS(iface) DEFINE_THIS(WebBrowser, DocHostUIHandler, iface)
+#define DOCHOSTUI_THIS(iface) DEFINE_THIS(DocHost, DocHostUIHandler, iface)
 
 static HRESULT WINAPI DocHostUIHandler_QueryInterface(IDocHostUIHandler2 *iface,
                                                       REFIID riid, void **ppv)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     return IOleClientSite_QueryInterface(CLIENTSITE(This), riid, ppv);
 }
 
 static ULONG WINAPI DocHostUIHandler_AddRef(IDocHostUIHandler2 *iface)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     return IOleClientSite_AddRef(CLIENTSITE(This));
 }
 
 static ULONG WINAPI DocHostUIHandler_Release(IDocHostUIHandler2 *iface)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     return IOleClientSite_Release(CLIENTSITE(This));
 }
 
 static HRESULT WINAPI DocHostUIHandler_ShowContextMenu(IDocHostUIHandler2 *iface,
          DWORD dwID, POINT *ppt, IUnknown *pcmdtReserved, IDispatch *pdispReserved)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     HRESULT hres;
 
     TRACE("(%p)->(%ld %p %p %p)\n", This, dwID, ppt, pcmdtReserved, pdispReserved);
@@ -299,7 +299,7 @@ static HRESULT WINAPI DocHostUIHandler_ShowContextMenu(IDocHostUIHandler2 *iface
 static HRESULT WINAPI DocHostUIHandler_GetHostInfo(IDocHostUIHandler2 *iface,
         DOCHOSTUIINFO *pInfo)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, pInfo);
@@ -320,7 +320,7 @@ static HRESULT WINAPI DocHostUIHandler_ShowUI(IDocHostUIHandler2 *iface, DWORD d
         IOleInPlaceActiveObject *pActiveObject, IOleCommandTarget *pCommandTarget,
         IOleInPlaceFrame *pFrame, IOleInPlaceUIWindow *pDoc)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%ld %p %p %p %p)\n", This, dwID, pActiveObject, pCommandTarget,
           pFrame, pDoc);
     return E_NOTIMPL;
@@ -328,14 +328,14 @@ static HRESULT WINAPI DocHostUIHandler_ShowUI(IDocHostUIHandler2 *iface, DWORD d
 
 static HRESULT WINAPI DocHostUIHandler_HideUI(IDocHostUIHandler2 *iface)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI DocHostUIHandler_UpdateUI(IDocHostUIHandler2 *iface)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -343,7 +343,7 @@ static HRESULT WINAPI DocHostUIHandler_UpdateUI(IDocHostUIHandler2 *iface)
 static HRESULT WINAPI DocHostUIHandler_EnableModeless(IDocHostUIHandler2 *iface,
                                                       BOOL fEnable)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fEnable);
     return E_NOTIMPL;
 }
@@ -351,7 +351,7 @@ static HRESULT WINAPI DocHostUIHandler_EnableModeless(IDocHostUIHandler2 *iface,
 static HRESULT WINAPI DocHostUIHandler_OnDocWindowActivate(IDocHostUIHandler2 *iface,
                                                            BOOL fActivate)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fActivate);
     return E_NOTIMPL;
 }
@@ -359,7 +359,7 @@ static HRESULT WINAPI DocHostUIHandler_OnDocWindowActivate(IDocHostUIHandler2 *i
 static HRESULT WINAPI DocHostUIHandler_OnFrameWindowActivate(IDocHostUIHandler2 *iface,
                                                              BOOL fActivate)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%x)\n", This, fActivate);
     return E_NOTIMPL;
 }
@@ -367,7 +367,7 @@ static HRESULT WINAPI DocHostUIHandler_OnFrameWindowActivate(IDocHostUIHandler2
 static HRESULT WINAPI DocHostUIHandler_ResizeBorder(IDocHostUIHandler2 *iface,
         LPCRECT prcBorder, IOleInPlaceUIWindow *pUIWindow, BOOL fRameWindow)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%p %p %X)\n", This, prcBorder, pUIWindow, fRameWindow);
     return E_NOTIMPL;
 }
@@ -375,7 +375,7 @@ static HRESULT WINAPI DocHostUIHandler_ResizeBorder(IDocHostUIHandler2 *iface,
 static HRESULT WINAPI DocHostUIHandler_TranslateAccelerator(IDocHostUIHandler2 *iface,
         LPMSG lpMsg, const GUID *pguidCmdGroup, DWORD nCmdID)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%p %p %ld)\n", This, lpMsg, pguidCmdGroup, nCmdID);
     return E_NOTIMPL;
 }
@@ -383,7 +383,7 @@ static HRESULT WINAPI DocHostUIHandler_TranslateAccelerator(IDocHostUIHandler2 *
 static HRESULT WINAPI DocHostUIHandler_GetOptionKeyPath(IDocHostUIHandler2 *iface,
         LPOLESTR *pchKey, DWORD dw)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
 
     TRACE("(%p)->(%p %ld)\n", This, pchKey, dw);
 
@@ -396,7 +396,7 @@ static HRESULT WINAPI DocHostUIHandler_GetOptionKeyPath(IDocHostUIHandler2 *ifac
 static HRESULT WINAPI DocHostUIHandler_GetDropTarget(IDocHostUIHandler2 *iface,
         IDropTarget *pDropTarget, IDropTarget **ppDropTarget)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)\n", This);
     return E_NOTIMPL;
 }
@@ -404,7 +404,7 @@ static HRESULT WINAPI DocHostUIHandler_GetDropTarget(IDocHostUIHandler2 *iface,
 static HRESULT WINAPI DocHostUIHandler_GetExternal(IDocHostUIHandler2 *iface,
         IDispatch **ppDispatch)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%p)\n", This, ppDispatch);
     return E_NOTIMPL;
 }
@@ -412,7 +412,7 @@ static HRESULT WINAPI DocHostUIHandler_GetExternal(IDocHostUIHandler2 *iface,
 static HRESULT WINAPI DocHostUIHandler_TranslateUrl(IDocHostUIHandler2 *iface,
         DWORD dwTranslate, OLECHAR *pchURLIn, OLECHAR **ppchURLOut)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
 
     TRACE("(%p)->(%ld %s %p)\n", This, dwTranslate, debugstr_w(pchURLIn), ppchURLOut);
 
@@ -426,7 +426,7 @@ static HRESULT WINAPI DocHostUIHandler_TranslateUrl(IDocHostUIHandler2 *iface,
 static HRESULT WINAPI DocHostUIHandler_FilterDataObject(IDocHostUIHandler2 *iface,
         IDataObject *pDO, IDataObject **ppDORet)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     FIXME("(%p)->(%p %p)\n", This, pDO, ppDORet);
     return E_NOTIMPL;
 }
@@ -434,17 +434,17 @@ static HRESULT WINAPI DocHostUIHandler_FilterDataObject(IDocHostUIHandler2 *ifac
 static HRESULT WINAPI DocHostUIHandler_GetOverrideKeyPath(IDocHostUIHandler2 *iface,
         LPOLESTR *pchKey, DWORD dw)
 {
-    WebBrowser *This = DOCHOSTUI_THIS(iface);
+    DocHost *This = DOCHOSTUI_THIS(iface);
     IDocHostUIHandler2 *handler;
     HRESULT hres;
 
     TRACE("(%p)->(%p %ld)\n", This, pchKey, dw);
 
-    if(!This->client)
+    if(!This->hostui)
         return S_OK;
 
-    hres = IOleClientSite_QueryInterface(This->client, &IID_IDocHostUIHandler2,
-                                         (void**)&handler);
+    hres = IDocHostUIHandler_QueryInterface(This->hostui, &IID_IDocHostUIHandler2,
+                                            (void**)&handler);
     if(SUCCEEDED(hres)) {
         hres = IDocHostUIHandler2_GetOverrideKeyPath(handler, pchKey, dw);
         IDocHostUIHandler2_Release(handler);
@@ -478,12 +478,29 @@ static const IDocHostUIHandler2Vtbl DocHostUIHandler2Vtbl = {
     DocHostUIHandler_GetOverrideKeyPath
 };
 
-void WebBrowser_DocHost_Init(WebBrowser *This)
+void DocHost_Init(DocHost *This, IDispatch *disp)
 {
-    This->lpDocHostUIHandlerVtbl   = &DocHostUIHandler2Vtbl;
-    This->lpClOleCommandTargetVtbl = &OleCommandTargetVtbl;
+    This->lpDocHostUIHandlerVtbl = &DocHostUIHandler2Vtbl;
+    This->lpOleCommandTargetVtbl = &OleCommandTargetVtbl;
 
+    This->disp = disp;
+
+    This->document = NULL;
     This->hostui = NULL;
 
-    This->doc_view_hwnd = NULL;
+    This->hwnd = NULL;
+    This->frame_hwnd = NULL;
+    This->url = NULL;
+
+    DocHost_ClientSite_Init(This);
+    DocHost_Frame_Init(This);
+    DocHost_Events_Init(This);
+}
+
+void DocHost_Release(DocHost *This)
+{
+    DocHost_ClientSite_Release(This);
+    DocHost_Events_Release(This);
+
+    SysFreeString(This->url);
 }
