@@ -288,7 +288,7 @@ static const IHttpNegotiateVtbl HttpNegotiateVtbl = {
     HttpNegotiate_OnResponse
 };
 
-static IBindStatusCallback *create_callback(WebBrowser *This, PBYTE post_data,
+static IBindStatusCallback *create_callback(DocHost *This, PBYTE post_data,
         ULONG post_data_len, LPWSTR headers, VARIANT_BOOL *cancel)
 {
     BindStatusCallback *ret = HeapAlloc(GetProcessHeap(), 0, sizeof(BindStatusCallback));
@@ -315,7 +315,7 @@ static IBindStatusCallback *create_callback(WebBrowser *This, PBYTE post_data,
     return BINDSC(ret);
 }
 
-static void on_before_navigate2(WebBrowser *This, LPWSTR url, PBYTE post_data, ULONG post_data_len,
+static void on_before_navigate2(DocHost *This, LPWSTR url, PBYTE post_data, ULONG post_data_len,
                                 LPWSTR headers, VARIANT_BOOL *cancel)
 {
     VARIANT var_url, var_flags, var_frame_name, var_post_data, var_post_data2, var_headers;
@@ -369,7 +369,7 @@ static void on_before_navigate2(WebBrowser *This, LPWSTR url, PBYTE post_data, U
     V_BSTR(&var_url) = SysAllocString(url);
 
     V_VT(params+6) = (VT_DISPATCH);
-    V_DISPATCH(params+6) = (IDispatch*)WEBBROWSER2(This);
+    V_DISPATCH(params+6) = This->disp;
 
     call_sink(This->cp_wbe2, DISPID_BEFORENAVIGATE2, &dispparams);
 
@@ -378,7 +378,7 @@ static void on_before_navigate2(WebBrowser *This, LPWSTR url, PBYTE post_data, U
         SafeArrayDestroy(V_ARRAY(&var_post_data));
 }
 
-static HRESULT navigate(WebBrowser *This, IMoniker *mon, IBindCtx *bindctx,
+static HRESULT navigate(DocHost *This, IMoniker *mon, IBindCtx *bindctx,
                         IBindStatusCallback *callback)
 {
     IOleObject *oleobj;
@@ -435,13 +435,13 @@ static HRESULT navigate(WebBrowser *This, IMoniker *mon, IBindCtx *bindctx,
     hres = IOleObject_SetClientSite(oleobj, CLIENTSITE(This));
     IOleObject_Release(oleobj);
 
-    PostMessageW(This->doc_view_hwnd, WB_WM_NAVIGATE2, 0, 0);
+    PostMessageW(This->hwnd, WB_WM_NAVIGATE2, 0, 0);
 
     return hres;
 
 }
 
-HRESULT navigate_url(WebBrowser *This, LPCWSTR url, PBYTE post_data, ULONG post_data_len,
+HRESULT navigate_url(DocHost *This, LPCWSTR url, PBYTE post_data, ULONG post_data_len,
                      LPWSTR headers)
 {
     IBindStatusCallback *callback;
@@ -450,6 +450,9 @@ HRESULT navigate_url(WebBrowser *This, LPCWSTR url, PBYTE post_data, ULONG post_
     VARIANT_BOOL cancel = VARIANT_FALSE;
     HRESULT hres;
 
+    if(!This->hwnd)
+        create_doc_view_hwnd(This);
+
     hres = CreateURLMoniker(NULL, url, &mon);
     if(FAILED(hres)) {
         WARN("CreateURLMoniker failed: %08lx\n", hres);
@@ -469,7 +472,7 @@ HRESULT navigate_url(WebBrowser *This, LPCWSTR url, PBYTE post_data, ULONG post_
     return hres;
 }
 
-HRESULT navigate_hlink(WebBrowser *This, IMoniker *mon, IBindCtx *bindctx,
+HRESULT navigate_hlink(DocHost *This, IMoniker *mon, IBindCtx *bindctx,
                        IBindStatusCallback *callback)
 {
     IHttpNegotiate *http_negotiate;
@@ -586,7 +589,7 @@ static HRESULT WINAPI HlinkFrame_Navigate(IHlinkFrame *iface, DWORD grfHLNF, LPB
         return E_NOTIMPL;
     }
 
-    return navigate_hlink(This, mon, pbc, pibsc);
+    return navigate_hlink(&This->doc_host, mon, pbc, pibsc);
 }
 
 static HRESULT WINAPI HlinkFrame_OnNavigate(IHlinkFrame *iface, DWORD grfHLNF,
