@@ -33,8 +33,8 @@ static ATOM shell_embedding_atom = 0;
 
 static LRESULT resize_window(WebBrowser *This, LONG width, LONG height)
 {
-    if(This->doc_view_hwnd)
-        SetWindowPos(This->doc_view_hwnd, NULL, 0, 0, width, height,
+    if(This->doc_host.hwnd)
+        SetWindowPos(This->doc_host.hwnd, NULL, 0, 0, width, height,
                      SWP_NOZORDER | SWP_NOACTIVATE);
 
     return 0;
@@ -58,7 +58,7 @@ static LRESULT WINAPI shell_embedding_proc(HWND hwnd, UINT msg, WPARAM wParam, L
         return resize_window(This, LOWORD(lParam), HIWORD(lParam));
     }
 
-    return DefWindowProcA(hwnd, msg, wParam, lParam);
+    return DefWindowProcW(hwnd, msg, wParam, lParam);
 }
 
 static void create_shell_embedding_hwnd(WebBrowser *This)
@@ -91,10 +91,12 @@ static void create_shell_embedding_hwnd(WebBrowser *This)
         IOleInPlaceSite_Release(inplace);
     }
 
-    This->shell_embedding_hwnd = CreateWindowExW(0, wszShellEmbedding, wszShellEmbedding,
-         WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_TABSTOP | WS_MAXIMIZEBOX,
-         0, 0, 0, 0, parent,
-         NULL, shdocvw_hinstance, This);
+    This->doc_host.frame_hwnd = This->shell_embedding_hwnd = CreateWindowExW(
+            WS_EX_WINDOWEDGE,
+            wszShellEmbedding, wszShellEmbedding,
+            WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_TABSTOP,
+            0, 0, 0, 0, parent,
+            NULL, shdocvw_hinstance, This);
 }
 
 /**********************************************************************
@@ -131,23 +133,28 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, LPOLECLIENTSITE
     if(This->client == pClientSite)
         return S_OK;
 
-    if(This->doc_view_hwnd) {
-        DestroyWindow(This->doc_view_hwnd);
-        This->doc_view_hwnd = NULL;
+    if(This->doc_host.hwnd) {
+        DestroyWindow(This->doc_host.hwnd);
+        This->doc_host.hwnd = NULL;
     }
     if(This->shell_embedding_hwnd) {
         DestroyWindow(This->shell_embedding_hwnd);
         This->shell_embedding_hwnd = NULL;
     }
 
-    if(This->hostui)
-        IDocHostUIHandler_Release(This->hostui);
+    if(This->inplace) {
+        IOleInPlaceSite_Release(This->inplace);
+        This->inplace = NULL;
+    }
+
+    if(This->doc_host.hostui)
+        IDocHostUIHandler_Release(This->doc_host.hostui);
     if(This->client)
         IOleClientSite_Release(This->client);
 
     if(!pClientSite) {
-        if(This->document)
-            deactivate_document(This);
+        if(This->doc_host.document)
+            deactivate_document(&This->doc_host);
         This->client = NULL;
         return S_OK;
     }
@@ -156,9 +163,9 @@ static HRESULT WINAPI OleObject_SetClientSite(IOleObject *iface, LPOLECLIENTSITE
     IOleClientSite_AddRef(pClientSite);
 
     hres = IOleClientSite_QueryInterface(This->client, &IID_IDocHostUIHandler,
-                                         (void**)&This->hostui);
+                                         (void**)&This->doc_host.hostui);
     if(FAILED(hres))
-        This->hostui = NULL;
+        This->doc_host.hostui = NULL;
 
     create_shell_embedding_hwnd(This);
 
@@ -241,40 +248,43 @@ static HRESULT WINAPI OleObject_DoVerb(IOleObject *iface, LONG iVerb, struct tag
     switch (iVerb)
     {
     case OLEIVERB_SHOW:
-    case OLEIVERB_INPLACEACTIVATE: {
-        IOleInPlaceSite *inplace;
-
+    case OLEIVERB_INPLACEACTIVATE:
         TRACE("OLEIVERB_INPLACEACTIVATE\n");
 
         if(!pActiveSite)
             return E_INVALIDARG;
 
-        hres = IOleClientSite_QueryInterface(pActiveSite, &IID_IOleInPlaceSite, (void**)&inplace);
+        if(This->inplace) {
+            IOleInPlaceSite_Release(This->inplace);
+            This->inplace = NULL;
+        }
+
+        hres = IOleClientSite_QueryInterface(pActiveSite, &IID_IOleInPlaceSite, (void**)&This->inplace);
         if(FAILED(hres)) {
             WARN("Could not get IOleInPlaceSite\n");
             return hres;
         }
 
-        hres = IOleInPlaceSite_CanInPlaceActivate(inplace);
+        hres = IOleInPlaceSite_CanInPlaceActivate(This->inplace);
         if(hres != S_OK) {
             WARN("CanInPlaceActivate returned: %08lx\n", hres);
-            IOleInPlaceSite_Release(inplace);
+            IOleInPlaceSite_Release(This->inplace);
             return E_FAIL;
         }
 
-        hres = IOleInPlaceSite_GetWindow(inplace, &This->iphwnd);
+        hres = IOleInPlaceSite_GetWindow(This->inplace, &This->iphwnd);
         if(FAILED(hres))
             This->iphwnd = hwndParent;
 
-        IOleInPlaceSite_OnInPlaceActivate(inplace);
+        IOleInPlaceSite_OnInPlaceActivate(This->inplace);
 
-        IOleInPlaceSite_GetWindowContext(inplace, &This->frame, &This->uiwindow,
+        IOleInPlaceSite_GetWindowContext(This->inplace, &This->frame, &This->uiwindow,
                                          &This->pos_rect, &This->clip_rect,
                                          &This->frameinfo);
 
 
         if(iVerb == OLEIVERB_INPLACEACTIVATE)
-            IOleInPlaceSite_Release(inplace);
+            IOleInPlaceSite_Release(This->inplace);
 
         SetWindowPos(This->shell_embedding_hwnd, NULL,
                      This->pos_rect.left, This->pos_rect.top,
@@ -295,8 +305,8 @@ static HRESULT WINAPI OleObject_DoVerb(IOleObject *iface, LONG iVerb, struct tag
 
         TRACE("OLEIVERB_SHOW\n");
 
-        IOleInPlaceSite_OnUIActivate(inplace);
-        IOleInPlaceSite_Release(inplace);
+        IOleInPlaceSite_OnUIActivate(This->inplace);
+        IOleInPlaceSite_Release(This->inplace);
 
         if(This->frame)
             IOleInPlaceFrame_SetActiveObject(This->frame, ACTIVEOBJ(This), wszitem);
@@ -308,7 +318,6 @@ static HRESULT WINAPI OleObject_DoVerb(IOleObject *iface, LONG iVerb, struct tag
          */
 
         return S_OK;
-    }
     default:
         FIXME("stub for %ld\n", iVerb);
         break;
@@ -392,14 +401,11 @@ static HRESULT WINAPI OleObject_EnumAdvise(IOleObject *iface, IEnumSTATDATA **pp
 static HRESULT WINAPI OleObject_GetMiscStatus(IOleObject *iface, DWORD dwAspect, DWORD *pdwStatus)
 {
     WebBrowser *This = OLEOBJ_THIS(iface);
-    HRESULT hres;
 
     TRACE("(%p)->(%lx, %p)\n", This, dwAspect, pdwStatus);
 
-    hres = OleRegGetMiscStatus(&CLSID_WebBrowser, dwAspect, pdwStatus);
-
-    if (FAILED(hres))
-        *pdwStatus = 0;
+    *pdwStatus = OLEMISC_SETCLIENTSITEFIRST|OLEMISC_ACTIVATEWHENVISIBLE|OLEMISC_INSIDEOUT
+        |OLEMISC_CANTLINKINSIDE|OLEMISC_RECOMPOSEONRESIZE;
 
     return S_OK;
 }
@@ -505,6 +511,11 @@ static HRESULT WINAPI OleInPlaceObject_SetObjectRects(IOleInPlaceObject *iface,
 
     TRACE("(%p)->(%p %p)\n", This, lprcPosRect, lprcClipRect);
 
+    memcpy(&This->pos_rect, lprcPosRect, sizeof(RECT));
+
+    if(lprcClipRect)
+        memcpy(&This->clip_rect, lprcClipRect, sizeof(RECT));
+
     if(This->shell_embedding_hwnd) {
         SetWindowPos(This->shell_embedding_hwnd, NULL,
                      lprcPosRect->left, lprcPosRect->top,
@@ -753,6 +764,7 @@ void WebBrowser_OleObject_Init(WebBrowser *This)
     This->lpWBOleCommandTargetVtbl     = &OleCommandTargetVtbl;
 
     This->client = NULL;
+    This->inplace = NULL;
     This->container = NULL;
     This->iphwnd = NULL;
     This->frame_hwnd = NULL;
