@@ -51,6 +51,36 @@ extern HRESULT SHDOCVW_GetShellInstanceObjectClassObject(REFCLSID rclsid,
 
 typedef struct ConnectionPoint ConnectionPoint;
 
+typedef struct {
+    const IOleClientSiteVtbl      *lpOleClientSiteVtbl;
+    const IOleInPlaceSiteVtbl     *lpOleInPlaceSiteVtbl;
+    const IDocHostUIHandler2Vtbl  *lpDocHostUIHandlerVtbl;
+    const IOleDocumentSiteVtbl    *lpOleDocumentSiteVtbl;
+    const IOleCommandTargetVtbl   *lpOleCommandTargetVtbl;
+    const IDispatchVtbl           *lpDispatchVtbl;
+    const IServiceProviderVtbl    *lpServiceProviderVtbl;
+
+    /* Interfaces of InPlaceFrame object */
+    const IOleInPlaceFrameVtbl          *lpOleInPlaceFrameVtbl;
+
+    IDispatch *disp;
+
+    IUnknown *document;
+    IOleDocumentView *view;
+    IDocHostUIHandler *hostui;
+
+    HWND hwnd;
+    HWND frame_hwnd;
+
+    LPOLESTR url;
+
+    /* Connection points */
+
+    ConnectionPoint *cp_wbe2;
+    ConnectionPoint *cp_wbe;
+    ConnectionPoint *cp_pns;
+} DocHost;
+
 typedef struct {
     /* Interfaces available via WebBrowser object */
 
@@ -67,30 +97,11 @@ typedef struct {
     const IOleCommandTargetVtbl         *lpWBOleCommandTargetVtbl;
     const IHlinkFrameVtbl               *lpHlinkFrameVtbl;
 
-    /* Interfaces available for embeded document */
-
-    const IOleClientSiteVtbl            *lpOleClientSiteVtbl;
-    const IOleInPlaceSiteVtbl           *lpOleInPlaceSiteVtbl;
-    const IDocHostUIHandler2Vtbl        *lpDocHostUIHandlerVtbl;
-    const IOleDocumentSiteVtbl          *lpOleDocumentSiteVtbl;
-    const IOleCommandTargetVtbl         *lpClOleCommandTargetVtbl;
-    const IDispatchVtbl                 *lpDispatchVtbl;
-    const IServiceProviderVtbl          *lpClServiceProviderVtbl;
-
-    /* Interfaces of InPlaceFrame object */
-
-    const IOleInPlaceFrameVtbl          *lpOleInPlaceFrameVtbl;
-
     LONG ref;
 
-    IUnknown *document;
-
     IOleClientSite *client;
     IOleContainer *container;
-    IOleDocumentView *view;
-    IDocHostUIHandler *hostui;
-
-    LPOLESTR url;
+    IOleInPlaceSite *inplace;
 
     /* window context */
 
@@ -102,16 +113,27 @@ typedef struct {
     RECT clip_rect;
     OLEINPLACEFRAMEINFO frameinfo;
 
-    HWND doc_view_hwnd;
     HWND shell_embedding_hwnd;
 
-    /* Connection points */
+    VARIANT_BOOL visible;
+    VARIANT_BOOL menu_bar;
+    VARIANT_BOOL address_bar;
+    VARIANT_BOOL status_bar;
+    VARIANT_BOOL tool_bar;
 
-    ConnectionPoint *cp_wbe2;
-    ConnectionPoint *cp_wbe;
-    ConnectionPoint *cp_pns;
+    DocHost doc_host;
 } WebBrowser;
 
+typedef struct {
+    const IWebBrowser2Vtbl *lpWebBrowser2Vtbl;
+
+    LONG ref;
+
+    HWND frame_hwnd;
+
+    DocHost doc_host;
+} InternetExplorer;
+
 #define WEBBROWSER(x)   ((IWebBrowser*)                 &(x)->lpWebBrowser2Vtbl)
 #define WEBBROWSER2(x)  ((IWebBrowser2*)                &(x)->lpWebBrowser2Vtbl)
 #define OLEOBJ(x)       ((IOleObject*)                  &(x)->lpOleObjectVtbl)
@@ -132,9 +154,9 @@ typedef struct {
 #define DOCHOSTUI(x)    ((IDocHostUIHandler*)           &(x)->lpDocHostUIHandlerVtbl)
 #define DOCHOSTUI2(x)   ((IDocHostUIHandler2*)          &(x)->lpDocHostUIHandlerVtbl)
 #define DOCSITE(x)      ((IOleDocumentSite*)            &(x)->lpOleDocumentSiteVtbl)
-#define CLOLECMD(x)     ((IOleCommandTarget*)           &(x)->lpClOleCommandTargetVtbl)
+#define CLOLECMD(x)     ((IOleCommandTarget*)           &(x)->lpOleCommandTargetVtbl)
 #define CLDISP(x)       ((IDispatch*)                   &(x)->lpDispatchVtbl)
-#define CLSERVPROV(x)   ((IServiceProvider*)            &(x)->lpClServiceProviderVtbl)
+#define SERVPROV(x)     ((IServiceProvider*)            &(x)->lpServiceProviderVtbl)
 
 #define INPLACEFRAME(x) ((IOleInPlaceFrame*)            &(x)->lpOleInPlaceFrameVtbl)
 
@@ -145,21 +167,28 @@ void WebBrowser_ClassInfo_Init(WebBrowser*);
 void WebBrowser_Events_Init(WebBrowser*);
 void WebBrowser_HlinkFrame_Init(WebBrowser*);
 
-void WebBrowser_ClientSite_Init(WebBrowser*);
-void WebBrowser_DocHost_Init(WebBrowser*);
-
-void WebBrowser_Frame_Init(WebBrowser*);
-
 void WebBrowser_OleObject_Destroy(WebBrowser*);
-void WebBrowser_Events_Destroy(WebBrowser*);
-void WebBrowser_ClientSite_Destroy(WebBrowser*);
+
+void DocHost_Init(DocHost*,IDispatch*);
+void DocHost_ClientSite_Init(DocHost*);
+void DocHost_Events_Init(DocHost*);
+void DocHost_Frame_Init(DocHost*);
+
+void DocHost_Release(DocHost*);
+void DocHost_ClientSite_Release(DocHost*);
+void DocHost_Events_Release(DocHost*);
 
 HRESULT WebBrowser_Create(IUnknown*,REFIID,void**);
 
-void create_doc_view_hwnd(WebBrowser *This);
-void deactivate_document(WebBrowser*);
+void create_doc_view_hwnd(DocHost*);
+void deactivate_document(DocHost*);
 void call_sink(ConnectionPoint*,DISPID,DISPPARAMS*);
-HRESULT navigate_url(WebBrowser*,LPCWSTR,PBYTE,ULONG,LPWSTR);
+HRESULT navigate_url(DocHost*,LPCWSTR,PBYTE,ULONG,LPWSTR);
+
+HRESULT create_mozctl(REFIID,void**);
+
+HRESULT InternetExplorer_Create(IUnknown*,REFIID,void**);
+void InternetExplorer_WebBrowser_Init(InternetExplorer*);
 
 #define WB_WM_NAVIGATE2 (WM_USER+100)
 
@@ -175,7 +204,7 @@ static inline void SHDOCVW_UnlockModule(void) { InterlockedDecrement( &SHDOCVW_r
 extern HINSTANCE shdocvw_hinstance;
 extern void register_iewindow_class(void);
 extern void unregister_iewindow_class(void);
-extern BOOL create_ie_window(LPCWSTR url);
-extern IClassFactory *get_class_factory(void);
+
+HRESULT register_class_object(BOOL);
 
 #endif /* __WINE_SHDOCVW_H */
