@@ -1259,7 +1259,7 @@ BOOL WINAPI SHGetPathFromIDListW(LPCITEMIDLIST pidl, LPWSTR pszPath)
     DWORD dwAttributes;
     STRRET strret;
 
-    TRACE_(shell)("(pidl=%p,%p)\n", pidl, debugstr_w(pszPath));
+    TRACE_(shell)("(pidl=%p,%p)\n", pidl, pszPath);
     pdump(pidl);
 
     if (!pidl)
@@ -1380,6 +1380,12 @@ LPITEMIDLIST _ILCreateMyComputer()
     return _ILCreateGuid(PT_GUID, &CLSID_MyComputer);
 }
 
+LPITEMIDLIST _ILCreateMyDocuments()
+{
+    TRACE("()\n");
+    return _ILCreateGuid(PT_GUID, &CLSID_MyDocuments);
+}
+
 LPITEMIDLIST _ILCreateIExplore()
 {
     TRACE("()\n");
