@@ -748,7 +748,7 @@ int WINAPI SHFileOperationA(LPSHFILEOPSTRUCTA lpFileOp)
 
 typedef struct
 {
-    WIN32_FIND_DATAW wfd;
+    DWORD attributes;
     LPWSTR szDirectory;
     LPWSTR szFilename;
     LPWSTR szFullPath;
@@ -760,73 +760,28 @@ typedef struct
 typedef struct
 {
     FILE_ENTRY *feFiles;
+    DWORD num_alloc;
     DWORD dwNumFiles;
     BOOL bAnyFromWildcard;
     BOOL bAnyDirectories;
     BOOL bAnyDontExist;
 } FILE_LIST;
 
-/* count the number of expanded files from a given wildcard */
-static DWORD count_wildcard_files(LPCWSTR szWildFile)
+
+static inline void grow_list(FILE_LIST *list)
 {
-    WIN32_FIND_DATAW wfd;
-    HANDLE hFile = FindFirstFileW(szWildFile, &wfd);
-    BOOL res = TRUE;
-    DWORD dwCount = 0;
-    
-    if (hFile == INVALID_HANDLE_VALUE)
-        return 0;
-    
-    while (res)
-    {
-        dwCount++;
-        res = FindNextFileW(hFile, &wfd);
-    }
-
-    FindClose(hFile);
-    return dwCount;
-}
-
-/* counts the number of files in a file list including expanded wildcard files */
-static DWORD count_files(LPCWSTR szFileList)
-{
-    DWORD dwCount = 0;
-    LPCWSTR p = szFileList;
-    LPCWSTR q = p + 1;
-    LPCWSTR str = p;
-
-    /* test empty list */
-    if (!szFileList[0] && !szFileList[1])
-        return -1;
-    
-    /* p,q search: stop when we reach double null terminator */
-    while (*p || *q)
-    {
-        if (!*q)
-        {
-            if (StrPBrkW(str, wWildcardChars))
-                dwCount += count_wildcard_files(str);
-            else
-                dwCount++;
-
-            str = q + 1;
-        }
-
-        p++;
-        q++;
-    }
-
-    return dwCount;
+    FILE_ENTRY *new = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, list->feFiles,
+                                  list->num_alloc * 2 * sizeof(*new) );
+    list->feFiles = new;
+    list->num_alloc *= 2;
 }
 
 /* adds a file to the FILE_ENTRY struct
- * returns TRUE if szFile is a directory, FALSE otherwise
  */
-static BOOL add_file_to_entry(FILE_ENTRY *feFile, LPWSTR szFile, BOOL bFromWildcard)
+static void add_file_to_entry(FILE_ENTRY *feFile, LPWSTR szFile)
 {
     DWORD dwLen = strlenW(szFile) + 1;
     LPWSTR ptr;
-    HANDLE h;
 
     feFile->szFullPath = HeapAlloc(GetProcessHeap(), 0, dwLen * sizeof(WCHAR));
     strcpyW(feFile->szFullPath, szFile);
@@ -842,19 +797,7 @@ static BOOL add_file_to_entry(FILE_ENTRY *feFile, LPWSTR szFile, BOOL bFromWildc
         feFile->szFilename = HeapAlloc(GetProcessHeap(), 0, dwLen * sizeof(WCHAR));
         strcpyW(feFile->szFilename, ptr + 1); /* skip over backslash */
     }
-
-    feFile->bFromWildcard = bFromWildcard;
-    h = FindFirstFileW(feFile->szFullPath, &feFile->wfd);
-    if (h != INVALID_HANDLE_VALUE)
-    {
-        FindClose(h);
-        if (IsAttribDir(feFile->wfd.dwFileAttributes))
-            return TRUE;
-    }
-    else
-        feFile->wfd.dwFileAttributes &= ~FILE_ATTRIBUTE_DIRECTORY;
-
-    return FALSE;
+    feFile->bFromWildcard = FALSE;
 }
 
 static LPWSTR wildcard_to_file(LPWSTR szWildCard, LPWSTR szFileName)
@@ -878,19 +821,21 @@ static void parse_wildcard_files(FILE_LIST *flList, LPWSTR szFile, LPDWORD pdwLi
 {
     WIN32_FIND_DATAW wfd;
     HANDLE hFile = FindFirstFileW(szFile, &wfd);
+    FILE_ENTRY *file;
     LPWSTR szFullPath;
-    BOOL res = TRUE, bDir;
-    
-    while (res)
-    {
-        szFullPath = wildcard_to_file(szFile, wfd.cFileName);
-        bDir = add_file_to_entry(&flList->feFiles[(*pdwListIndex)++],
-                                 szFullPath, TRUE);
-        HeapFree(GetProcessHeap(), 0, szFullPath);
-        res = FindNextFileW(hFile, &wfd);
+    BOOL res;
 
-        if (bDir)
-            flList->bAnyDirectories = TRUE;
+    for (res = TRUE; res; res = FindNextFileW(hFile, &wfd))
+    {
+        if (IsDotDir(wfd.cFileName)) continue;
+        if (*pdwListIndex >= flList->num_alloc) grow_list( flList );
+        szFullPath = wildcard_to_file(szFile, wfd.cFileName);
+        file = &flList->feFiles[(*pdwListIndex)++];
+        add_file_to_entry(file, szFullPath);
+        file->bFromWildcard = TRUE;
+        file->attributes = wfd.dwFileAttributes;
+        if (IsAttribDir(file->attributes)) flList->bAnyDirectories = TRUE;
+        HeapFree(GetProcessHeap(), 0, szFullPath);
     }
 
     FindClose(hFile);
@@ -901,7 +846,7 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
 {
     LPCWSTR ptr = szFiles;
     WCHAR szCurFile[MAX_PATH];
-    DWORD i, dwDirLen;
+    DWORD i = 0, dwDirLen;
 
     if (!szFiles)
         return ERROR_INVALID_PARAMETER;
@@ -909,19 +854,19 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
     flList->bAnyFromWildcard = FALSE;
     flList->bAnyDirectories = FALSE;
     flList->bAnyDontExist = FALSE;
-    flList->dwNumFiles = count_files(szFiles);
+    flList->num_alloc = 32;
+    flList->dwNumFiles = 0;
 
     /* empty list */
-    if (flList->dwNumFiles == -1)
+    if (!szFiles[0])
         return ERROR_ACCESS_DENIED;
         
     flList->feFiles = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                flList->dwNumFiles * sizeof(FILE_ENTRY));
+                                flList->num_alloc * sizeof(FILE_ENTRY));
 
-    for (i = 0; i < flList->dwNumFiles; i++)
+    while (*ptr)
     {
-        if (!*ptr)
-            return ERROR_ACCESS_DENIED;
+        if (i >= flList->num_alloc) grow_list( flList );
 
         /* change relative to absolute path */
         if (PathIsRelativeW(ptr))
@@ -943,21 +888,21 @@ static HRESULT parse_file_list(FILE_LIST *flList, LPCWSTR szFiles)
             flList->bAnyFromWildcard = TRUE;
             i--;
         }
-        else if (add_file_to_entry(&flList->feFiles[i], szCurFile, FALSE))
-            flList->bAnyDirectories = TRUE;
-
-        /* record whether the file exists */
-        if (!PathFileExistsW(flList->feFiles[i].szFullPath))
-        {
-            flList->feFiles[i].bExists = FALSE;
-            flList->bAnyDontExist = TRUE;
-        }
         else
-            flList->feFiles[i].bExists = TRUE;
+        {
+            FILE_ENTRY *file = &flList->feFiles[i];
+            add_file_to_entry(file, szCurFile);
+            file->attributes = GetFileAttributesW( file->szFullPath );
+            file->bExists = (file->attributes != INVALID_FILE_ATTRIBUTES);
+            if (!file->bExists) flList->bAnyDontExist = TRUE;
+            if (IsAttribDir(file->attributes)) flList->bAnyDirectories = TRUE;
+        }
 
         /* advance to the next string */
         ptr += strlenW(ptr) + 1;
+        i++;
     }
+    flList->dwNumFiles = i;
 
     return S_OK;
 }
@@ -1019,7 +964,7 @@ static void copy_to_dir(LPSHFILEOPSTRUCTW lpFileOp, FILE_ENTRY *feFrom, FILE_ENT
 
     PathCombineW(szDestPath, feTo->szFullPath, feFrom->szFilename);
 
-    if (IsAttribFile(feFrom->wfd.dwFileAttributes))
+    if (IsAttribFile(feFrom->attributes))
         SHNotifyCopyFileW(feFrom->szFullPath, szDestPath, FALSE);
     else if (!(lpFileOp->fFlags & FOF_FILESONLY && feFrom->bFromWildcard))
         copy_dir_to_dir(lpFileOp, feFrom, szDestPath);
@@ -1069,7 +1014,7 @@ static HRESULT copy_files(LPSHFILEOPSTRUCTW lpFileOp, FILE_LIST *flFrom, FILE_LI
 
     if (flFrom->dwNumFiles > 1 && flTo->dwNumFiles == 1 &&
         !PathFileExistsW(flTo->feFiles[0].szFullPath) &&
-        IsAttribFile(fileDest->wfd.dwFileAttributes))
+        IsAttribFile(fileDest->attributes))
     {
         bCancelIfAnyDirectories = TRUE;
     }
@@ -1083,7 +1028,7 @@ static HRESULT copy_files(LPSHFILEOPSTRUCTW lpFileOp, FILE_LIST *flFrom, FILE_LI
 
     if (!(lpFileOp->fFlags & FOF_MULTIDESTFILES) && flFrom->dwNumFiles != 1 &&
         PathFileExistsW(fileDest->szFullPath) &&
-        IsAttribFile(fileDest->wfd.dwFileAttributes))
+        IsAttribFile(fileDest->attributes))
     {
         return ERROR_CANCELLED;
     }
@@ -1095,31 +1040,31 @@ static HRESULT copy_files(LPSHFILEOPSTRUCTW lpFileOp, FILE_LIST *flFrom, FILE_LI
         if (lpFileOp->fFlags & FOF_MULTIDESTFILES)
             fileDest = &flTo->feFiles[i];
 
-        if (IsAttribDir(entryToCopy->wfd.dwFileAttributes) &&
+        if (IsAttribDir(entryToCopy->attributes) &&
             !lstrcmpW(entryToCopy->szFullPath, fileDest->szDirectory))
         {
             return ERROR_SUCCESS;
         }
 
-        if (IsAttribDir(entryToCopy->wfd.dwFileAttributes) && bCancelIfAnyDirectories)
+        if (IsAttribDir(entryToCopy->attributes) && bCancelIfAnyDirectories)
             return ERROR_CANCELLED;
 
         create_dest_dirs(fileDest->szDirectory);
 
         if (!strcmpW(entryToCopy->szFullPath, fileDest->szFullPath))
         {
-            if (IsAttribFile(entryToCopy->wfd.dwFileAttributes))
+            if (IsAttribFile(entryToCopy->attributes))
                 return ERROR_NO_MORE_SEARCH_HANDLES;
             else
                 return ERROR_SUCCESS;
         }
 
         if ((flFrom->dwNumFiles > 1 && flTo->dwNumFiles == 1) ||
-            (flFrom->dwNumFiles == 1 && IsAttribDir(fileDest->wfd.dwFileAttributes)))
+            (flFrom->dwNumFiles == 1 && IsAttribDir(fileDest->attributes)))
         {
             copy_to_dir(lpFileOp, entryToCopy, fileDest);
         }
-        else if (IsAttribDir(entryToCopy->wfd.dwFileAttributes))
+        else if (IsAttribDir(entryToCopy->attributes))
         {
             copy_dir_to_dir(lpFileOp, entryToCopy, fileDest->szFullPath);
         }
@@ -1153,7 +1098,7 @@ static HRESULT delete_files(LPSHFILEOPSTRUCTW lpFileOp, FILE_LIST *flFrom)
         fileEntry = &flFrom->feFiles[i];
 
         /* delete the file or directory */
-        if (IsAttribFile(fileEntry->wfd.dwFileAttributes))
+        if (IsAttribFile(fileEntry->attributes))
             bPathExists = DeleteFileW(fileEntry->szFullPath);
         else if (!(lpFileOp->fFlags & FOF_FILESONLY && fileEntry->bFromWildcard))
             bPathExists = SHELL_DeleteDirectoryW(fileEntry->szFullPath, bConfirm);
@@ -1197,7 +1142,7 @@ static void move_to_dir(LPSHFILEOPSTRUCTW lpFileOp, FILE_ENTRY *feFrom, FILE_ENT
 
     PathCombineW(szDestPath, feTo->szFullPath, feFrom->szFilename);
 
-    if (IsAttribFile(feFrom->wfd.dwFileAttributes))
+    if (IsAttribFile(feFrom->attributes))
         SHNotifyMoveFileW(feFrom->szFullPath, szDestPath);
     else if (!(lpFileOp->fFlags & FOF_FILESONLY && feFrom->bFromWildcard))
         move_dir_to_dir(lpFileOp, feFrom, szDestPath);
@@ -1246,7 +1191,7 @@ static HRESULT move_files(LPSHFILEOPSTRUCTW lpFileOp, FILE_LIST *flFrom, FILE_LI
         if (!PathFileExistsW(fileDest->szDirectory))
             return ERROR_CANCELLED;
 
-        if (fileDest->bExists && IsAttribDir(fileDest->wfd.dwFileAttributes))
+        if (fileDest->bExists && IsAttribDir(fileDest->attributes))
             move_to_dir(lpFileOp, entryToMove, fileDest);
         else
             SHNotifyMoveFileW(entryToMove->szFullPath, fileDest->szFullPath);
