@@ -81,6 +81,7 @@ static void test_ParseDisplayName(void)
     DWORD res;
     WCHAR cTestDirW [MAX_PATH] = {0};
     ITEMIDLIST *newPIDL;
+    BOOL bRes;
 
     hr = SHGetDesktopFolder(&IDesktopFolder);
     if(hr != S_OK) return;
@@ -102,6 +103,26 @@ static void test_ParseDisplayName(void)
         NULL, NULL, cTestDirW, NULL, &newPIDL, 0);
     ok((hr == HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND)) || (hr == E_FAIL) || (hr == E_INVALIDARG), 
         "ParseDisplayName returned %08lx, expected 80070002, E_FAIL or E_INVALIDARG\n", hr);
+
+    /* I thought that perhaps the DesktopFolder's ParseDisplayName would recognize the
+     * path corresponding to CSIDL_PERSONAL and return a CLSID_MyDocuments PIDL. Turns
+     * out it doesn't. The magic seems to happen in the file dialogs, then. */
+    if (!pSHGetSpecialFolderPathW || !pILFindLastID) goto finished;
+    
+    bRes = pSHGetSpecialFolderPathW(NULL, cTestDirW, CSIDL_PERSONAL, FALSE);
+    ok(bRes, "SHGetSpecialFolderPath(CSIDL_PERSONAL) failed! %ld\n", GetLastError());
+    if (!bRes) goto finished;
+
+    hr = IShellFolder_ParseDisplayName(IDesktopFolder, NULL, NULL, cTestDirW, NULL, &newPIDL, 0);
+    ok(SUCCEEDED(hr), "DesktopFolder->ParseDisplayName failed. hr = %08lx.\n", hr);
+    if (FAILED(hr)) goto finished;
+
+    ok(pILFindLastID(newPIDL)->mkid.abID[0] == 0x31, "Last pidl should be of type "
+       "PT_FOLDER, but is: %02x\n", pILFindLastID(newPIDL)->mkid.abID[0]);
+    IMalloc_Free(ppM, newPIDL);
+    
+finished:
+    IShellFolder_Release(IDesktopFolder);
 }
 
 /* creates a file with the specified name for tests */
@@ -602,6 +623,13 @@ static void test_GetAttributesOf(void)
     WCHAR wszMyComputer[] = { 
         ':',':','{','2','0','D','0','4','F','E','0','-','3','A','E','A','-','1','0','6','9','-',
         'A','2','D','8','-','0','8','0','0','2','B','3','0','3','0','9','D','}',0 };
+    char  cCurrDirA [MAX_PATH] = {0};
+    WCHAR cCurrDirW [MAX_PATH];
+    static const WCHAR cTestDirW[] = {'t','e','s','t','d','i','r',0};
+    static const WCHAR cBackSlash[] = {'\\',0};
+    IShellFolder *IDesktopFolder, *testIShellFolder;
+    ITEMIDLIST *newPIDL;
+    int len;
 
     hr = SHGetDesktopFolder(&psfDesktop);
     ok (SUCCEEDED(hr), "SHGetDesktopFolder failed! hr = %08lx\n", hr);
@@ -658,6 +686,67 @@ static void test_GetAttributesOf(void)
                     "Wrong MyComputer attributes: %08lx, expected: %08lx\n", dwFlags, dwMyComputerFlags); }
 
     IShellFolder_Release(psfMyComputer);
+
+    /* create test directory */
+    CreateFilesFolders();
+
+    GetCurrentDirectoryA(MAX_PATH, cCurrDirA);
+    len = lstrlenA(cCurrDirA);
+
+    if (len == 0) {
+	trace("GetCurrentDirectoryA returned empty string. Skipping test_EnumObjects_and_CompareIDs\n");
+	return;
+    }
+    if(cCurrDirA[len-1] == '\\')
+	cCurrDirA[len-1] = 0;
+
+    MultiByteToWideChar(CP_ACP, 0, cCurrDirA, -1, cCurrDirW, MAX_PATH);
+ 
+    hr = SHGetDesktopFolder(&IDesktopFolder);
+    ok(hr == S_OK, "SHGetDesktopfolder failed %08lx\n", hr);
+
+    hr = IShellFolder_ParseDisplayName(IDesktopFolder, NULL, NULL, cCurrDirW, NULL, &newPIDL, 0);
+    ok(hr == S_OK, "ParseDisplayName failed %08lx\n", hr);
+
+    hr = IShellFolder_BindToObject(IDesktopFolder, newPIDL, NULL, (REFIID)&IID_IShellFolder, (LPVOID *)&testIShellFolder);
+    ok(hr == S_OK, "BindToObject failed %08lx\n", hr);
+
+    IMalloc_Free(ppM, newPIDL);
+
+    /* get relative PIDL */
+    hr = IShellFolder_ParseDisplayName(testIShellFolder, NULL, NULL, (LPWSTR)cTestDirW, NULL, &newPIDL, 0);
+    ok(hr == S_OK, "ParseDisplayName failed %08lx\n", hr);
+
+    /* test the shell attributes of the test directory using the relative PIDL */
+    dwFlags = SFGAO_FOLDER;
+    hr = IShellFolder_GetAttributesOf(testIShellFolder, 1, (LPCITEMIDLIST*)&newPIDL, &dwFlags);
+    ok (SUCCEEDED(hr), "Desktop->GetAttributesOf() failed! hr = %08lx\n", hr);
+    ok ((dwFlags&SFGAO_FOLDER), "Wrong directory attribute for relative PIDL: %08lx\n", dwFlags);
+
+    /* free memory */
+    IMalloc_Free(ppM, newPIDL);
+
+    /* append testdirectory name to path */
+    strcatW(cCurrDirW, cBackSlash);
+    strcatW(cCurrDirW, cTestDirW);
+
+    hr = IShellFolder_ParseDisplayName(IDesktopFolder, NULL, NULL, cCurrDirW, NULL, &newPIDL, 0);
+    ok(hr == S_OK, "ParseDisplayName failed %08lx\n", hr);
+
+    /* test the shell attributes of the test directory using the absolute PIDL */
+    dwFlags = SFGAO_FOLDER;
+    hr = IShellFolder_GetAttributesOf(IDesktopFolder, 1, (LPCITEMIDLIST*)&newPIDL, &dwFlags);
+    ok (SUCCEEDED(hr), "Desktop->GetAttributesOf() failed! hr = %08lx\n", hr);
+    ok ((dwFlags&SFGAO_FOLDER), "Wrong directory attribute for absolute PIDL: %08lx\n", dwFlags);
+        
+    /* free memory */
+    IMalloc_Free(ppM, newPIDL);
+
+    IShellFolder_Release(testIShellFolder);
+
+    Cleanup();
+
+    IShellFolder_Release(IDesktopFolder);
 }    
 
 static void test_SHGetPathFromIDList(void)
@@ -769,7 +858,7 @@ static void test_SHGetPathFromIDList(void)
 
 	/* Test if we can get the path from the start menu "program files" PIDL. */
     hShell32 = GetModuleHandleA("shell32");
-    pSHGetSpecialFolderLocation = (HRESULT(WINAPI*)(HWND,int,LPITEMIDLIST*))GetProcAddress(hShell32, "SHGetSpecialFolderLocation");
+    pSHGetSpecialFolderLocation = (void *)GetProcAddress(hShell32, "SHGetSpecialFolderLocation");
 
     hr = pSHGetSpecialFolderLocation(NULL, CSIDL_PROGRAM_FILES, &pidlPrograms);
     ok(SUCCEEDED(hr), "SHGetFolderLocation failed: 0x%08lx\n", hr);
@@ -816,12 +905,13 @@ static void test_EnumObjects_and_CompareIDs(void)
 
     test_EnumObjects(testIShellFolder);
 
-    hr = IShellFolder_Release(testIShellFolder);
-    ok(hr == S_OK, "IShellFolder_Release failed %08lx\n", hr);
+    IShellFolder_Release(testIShellFolder);
 
     Cleanup();
 
     IMalloc_Free(ppM, newPIDL);
+
+    IShellFolder_Release(IDesktopFolder);
 }
 
 /* A simple implementation of an IPropertyBag, which returns fixed values for
