@@ -53,7 +53,6 @@ typedef struct tagCLASS
     INT              cbClsExtra;    /* Class extra bytes */
     INT              cbWndExtra;    /* Window extra bytes */
     LPWSTR           menuName;      /* Default menu name (Unicode followed by ASCII) */
-    SEGPTR           segMenuName;   /* Default menu name as SEGPTR */
     HINSTANCE        hInstance;     /* Module that created the task */
     HICON            hIcon;         /* Default icon */
     HICON            hIconSm;       /* Default small icon */
@@ -76,14 +75,14 @@ static CLASS *get_class_ptr( HWND hwnd, BOOL write_access )
     if (ptr)
     {
         if (ptr != WND_OTHER_PROCESS && ptr != WND_DESKTOP) return ptr->class;
+        if (!write_access) return CLASS_OTHER_PROCESS;
 
-        if (write_access && (ptr == WND_DESKTOP || IsWindow( hwnd ))) /* check other processes */
+        /* modifying classes in other processes is not allowed */
+        if (ptr == WND_DESKTOP || IsWindow( hwnd ))
         {
-            /* modifying classes in other processes is not allowed */
             SetLastError( ERROR_ACCESS_DENIED );
             return NULL;
         }
-        return CLASS_OTHER_PROCESS;
     }
     SetLastError( ERROR_INVALID_WINDOW_HANDLE );
     return NULL;
@@ -149,7 +148,7 @@ static BOOL set_server_info( HWND hwnd, INT offset, LONG newval )
  *
  * Get the class winproc for a given proc type
  */
-static WNDPROC16 CLASS_GetProc( CLASS *classPtr, WINDOWPROCTYPE type )
+static WNDPROC CLASS_GetProc( CLASS *classPtr, BOOL unicode )
 {
     WNDPROC proc = classPtr->winprocA;
 
@@ -158,9 +157,9 @@ static WNDPROC16 CLASS_GetProc( CLASS *classPtr, WINDOWPROCTYPE type )
         /* if we have a Unicode proc, use it if we have no ASCII proc
          * or if we have both and Unicode was requested
          */
-        if (!proc || type == WIN_PROC_32W) proc = classPtr->winprocW;
+        if (!proc || unicode) proc = classPtr->winprocW;
     }
-    return WINPROC_GetProc( proc, type );
+    return WINPROC_GetProc( proc, unicode );
 }
 
 
@@ -170,29 +169,20 @@ static WNDPROC16 CLASS_GetProc( CLASS *classPtr, WINDOWPROCTYPE type )
  * Set the class winproc for a given proc type.
  * Returns the previous window proc.
  */
-static WNDPROC16 CLASS_SetProc( CLASS *classPtr, WNDPROC newproc, WINDOWPROCTYPE type )
+static void CLASS_SetProc( CLASS *classPtr, WNDPROC newproc, BOOL unicode )
 {
-    WNDPROC *proc = &classPtr->winprocA;
-    WNDPROC16 ret;
+    WNDPROC proc = WINPROC_AllocProc( newproc, unicode );
 
-    if (classPtr->winprocW)
+    if (WINPROC_IsUnicode( proc, unicode ))
     {
-        /* if we have a Unicode proc, use it if we have no ASCII proc
-         * or if we have both and Unicode was requested
-         */
-        if (!*proc || type == WIN_PROC_32W) proc = &classPtr->winprocW;
+        classPtr->winprocA = 0;
+        classPtr->winprocW = proc;
     }
-    ret = WINPROC_GetProc( *proc, type );
-    *proc = WINPROC_AllocProc( newproc, type );
-    /* now clear the one that we didn't set */
-    if (classPtr->winprocA && classPtr->winprocW)
+    else
     {
-        if (proc == &classPtr->winprocA)
-            classPtr->winprocW = 0;
-        else
-            classPtr->winprocA = 0;
+        classPtr->winprocA = proc;
+        classPtr->winprocW = 0;
     }
-    return ret;
 }
 
 
@@ -208,20 +198,6 @@ inline static LPSTR CLASS_GetMenuNameA( CLASS *classPtr )
 }
 
 
-/***********************************************************************
- *           CLASS_GetMenuName16
- *
- * Get the menu name as a SEGPTR.
- */
-inline static SEGPTR CLASS_GetMenuName16( CLASS *classPtr )
-{
-    if (!HIWORD(classPtr->menuName)) return (SEGPTR)classPtr->menuName;
-    if (!classPtr->segMenuName)
-        classPtr->segMenuName = MapLS( CLASS_GetMenuNameA(classPtr) );
-    return classPtr->segMenuName;
-}
-
-
 /***********************************************************************
  *           CLASS_GetMenuNameW
  *
@@ -240,8 +216,6 @@ inline static LPWSTR CLASS_GetMenuNameW( CLASS *classPtr )
  */
 static void CLASS_SetMenuNameA( CLASS *classPtr, LPCSTR name )
 {
-    UnMapLS( classPtr->segMenuName );
-    classPtr->segMenuName = 0;
     if (HIWORD(classPtr->menuName)) HeapFree( GetProcessHeap(), 0, classPtr->menuName );
     if (HIWORD(name))
     {
@@ -262,8 +236,6 @@ static void CLASS_SetMenuNameA( CLASS *classPtr, LPCSTR name )
  */
 static void CLASS_SetMenuNameW( CLASS *classPtr, LPCWSTR name )
 {
-    UnMapLS( classPtr->segMenuName );
-    classPtr->segMenuName = 0;
     if (HIWORD(classPtr->menuName)) HeapFree( GetProcessHeap(), 0, classPtr->menuName );
     if (HIWORD(name))
     {
@@ -292,7 +264,6 @@ static void CLASS_FreeClass( CLASS *classPtr )
     list_remove( &classPtr->entry );
     if (classPtr->hbrBackground > (HBRUSH)(COLOR_GRADIENTINACTIVECAPTION + 1))
         DeleteObject( classPtr->hbrBackground );
-    UnMapLS( classPtr->segMenuName );
     HeapFree( GetProcessHeap(), 0, classPtr->menuName );
     HeapFree( GetProcessHeap(), 0, classPtr );
     USER_Unlock();
@@ -447,8 +418,8 @@ static CLASS *register_builtin( const struct builtin_class_descr *descr )
     classPtr->hCursor       = LoadCursorA( 0, (LPSTR)descr->cursor );
     classPtr->hbrBackground = descr->brush;
 
-    if (descr->procA) classPtr->winprocA = WINPROC_AllocProc( descr->procA, WIN_PROC_32A );
-    if (descr->procW) classPtr->winprocW = WINPROC_AllocProc( descr->procW, WIN_PROC_32W );
+    if (descr->procA) classPtr->winprocA = WINPROC_AllocProc( descr->procA, FALSE );
+    if (descr->procW) classPtr->winprocW = WINPROC_AllocProc( descr->procW, TRUE );
     release_class_ptr( classPtr );
     return classPtr;
 }
@@ -493,9 +464,9 @@ void CLASS_RegisterBuiltinClasses(void)
  * Add a new window using this class, and set the necessary
  * information inside the window structure.
  */
-void CLASS_AddWindow( CLASS *class, WND *win, WINDOWPROCTYPE type )
+void CLASS_AddWindow( CLASS *class, WND *win, BOOL unicode )
 {
-    if (type == WIN_PROC_32W)
+    if (unicode)
     {
         if (!(win->winproc = class->winprocW)) win->winproc = class->winprocA;
     }
@@ -505,6 +476,7 @@ void CLASS_AddWindow( CLASS *class, WND *win, WINDOWPROCTYPE type )
     }
     win->class    = class;
     win->clsStyle = class->style;
+    if (WINPROC_IsUnicode( win->winproc, unicode )) win->flags |= WIN_ISUNICODE;
 }
 
 
@@ -593,7 +565,7 @@ ATOM WINAPI RegisterClassExA( const WNDCLASSEXA* wc )
     classPtr->hIconSm       = wc->hIconSm;
     classPtr->hCursor       = wc->hCursor;
     classPtr->hbrBackground = wc->hbrBackground;
-    classPtr->winprocA      = WINPROC_AllocProc( wc->lpfnWndProc, WIN_PROC_32A );
+    classPtr->winprocA      = WINPROC_AllocProc( wc->lpfnWndProc, FALSE );
     CLASS_SetMenuNameA( classPtr, wc->lpszMenuName );
     release_class_ptr( classPtr );
     return atom;
@@ -631,7 +603,7 @@ ATOM WINAPI RegisterClassExW( const WNDCLASSEXW* wc )
     classPtr->hIconSm       = wc->hIconSm;
     classPtr->hCursor       = wc->hCursor;
     classPtr->hbrBackground = wc->hbrBackground;
-    classPtr->winprocW      = WINPROC_AllocProc( wc->lpfnWndProc, WIN_PROC_32W );
+    classPtr->winprocW      = WINPROC_AllocProc( wc->lpfnWndProc, TRUE );
     CLASS_SetMenuNameW( classPtr, wc->lpszMenuName );
     release_class_ptr( classPtr );
     return atom;
@@ -714,40 +686,6 @@ WORD WINAPI GetClassWord( HWND hwnd, INT offset )
 }
 
 
-/***********************************************************************
- *		GetClassLong (USER.131)
- */
-LONG WINAPI GetClassLong16( HWND16 hwnd16, INT16 offset )
-{
-    CLASS *class;
-    LONG ret;
-    HWND hwnd = (HWND)(ULONG_PTR)hwnd16;  /* no need for full handle */
-
-    TRACE("%p %d\n",hwnd, offset);
-
-    switch( offset )
-    {
-    case GCLP_WNDPROC:
-        if (!(class = get_class_ptr( hwnd, FALSE ))) return 0;
-        if (class == CLASS_OTHER_PROCESS) break;
-        ret = (LONG)CLASS_GetProc( class, WIN_PROC_16 );
-        release_class_ptr( class );
-        return ret;
-    case GCLP_MENUNAME:
-        if (!(class = get_class_ptr( hwnd, FALSE ))) return 0;
-        if (class == CLASS_OTHER_PROCESS) break;
-        ret = (LONG)CLASS_GetMenuName16( class );
-        release_class_ptr( class );
-        return ret;
-    default:
-        return GetClassLongA( hwnd, offset );
-    }
-    FIXME( "offset %d not supported on other process window %p\n", offset, hwnd );
-    SetLastError( ERROR_INVALID_HANDLE );
-    return 0;
-}
-
-
 /***********************************************************************
  *		GetClassLongW (USER32.@)
  */
@@ -844,7 +782,7 @@ DWORD WINAPI GetClassLongW( HWND hwnd, INT offset )
         retvalue = (DWORD)class->hInstance;
         break;
     case GCLP_WNDPROC:
-        retvalue = (DWORD)CLASS_GetProc( class, WIN_PROC_32W );
+        retvalue = (DWORD)CLASS_GetProc( class, TRUE );
         break;
     case GCLP_MENUNAME:
         retvalue = (DWORD)CLASS_GetMenuNameW( class );
@@ -884,7 +822,7 @@ DWORD WINAPI GetClassLongA( HWND hwnd, INT offset )
     }
 
     if (offset == GCLP_WNDPROC)
-        retvalue = (DWORD)CLASS_GetProc( class, WIN_PROC_32A );
+        retvalue = (DWORD)CLASS_GetProc( class, FALSE );
     else  /* GCL_MENUNAME */
         retvalue = (DWORD)CLASS_GetMenuNameA( class );
 
@@ -927,33 +865,6 @@ WORD WINAPI SetClassWord( HWND hwnd, INT offset, WORD newval )
 }
 
 
-/***********************************************************************
- *		SetClassLong (USER.132)
- */
-LONG WINAPI SetClassLong16( HWND16 hwnd16, INT16 offset, LONG newval )
-{
-    CLASS *class;
-    LONG retval;
-    HWND hwnd = (HWND)(ULONG_PTR)hwnd16;  /* no need for full handle */
-
-    TRACE("%p %d %lx\n", hwnd, offset, newval);
-
-    switch(offset)
-    {
-    case GCLP_WNDPROC:
-        if (!(class = get_class_ptr( hwnd, TRUE ))) return 0;
-        retval = (LONG)CLASS_SetProc( class, (WNDPROC)newval, WIN_PROC_16 );
-        release_class_ptr( class );
-        return retval;
-    case GCLP_MENUNAME:
-        newval = (LONG)MapSL( newval );
-        /* fall through */
-    default:
-        return SetClassLongA( hwnd, offset, newval );
-    }
-}
-
-
 /***********************************************************************
  *		SetClassLongW (USER32.@)
  */
@@ -982,7 +893,8 @@ DWORD WINAPI SetClassLongW( HWND hwnd, INT offset, LONG newval )
         retval = 0;  /* Old value is now meaningless anyway */
         break;
     case GCLP_WNDPROC:
-        retval = (DWORD)CLASS_SetProc( class, (WNDPROC)newval, WIN_PROC_32W );
+        retval = (DWORD)CLASS_GetProc( class, TRUE );
+        CLASS_SetProc( class, (WNDPROC)newval, TRUE );
         break;
     case GCLP_HBRBACKGROUND:
         retval = (DWORD)class->hbrBackground;
@@ -1048,7 +960,10 @@ DWORD WINAPI SetClassLongA( HWND hwnd, INT offset, LONG newval )
     if (!(class = get_class_ptr( hwnd, TRUE ))) return 0;
 
     if (offset == GCLP_WNDPROC)
-        retval = (DWORD)CLASS_SetProc( class, (WNDPROC)newval, WIN_PROC_32A );
+    {
+        retval = (DWORD)CLASS_GetProc( class, FALSE );
+        CLASS_SetProc( class, (WNDPROC)newval, FALSE );
+    }
     else  /* GCL_MENUNAME */
     {
         CLASS_SetMenuNameA( class, (LPCSTR)newval );
@@ -1169,7 +1084,7 @@ BOOL WINAPI GetClassInfoExA( HINSTANCE hInstance, LPCSTR name, WNDCLASSEXA *wc )
         return FALSE;
     }
     wc->style         = classPtr->style;
-    wc->lpfnWndProc   = (WNDPROC)CLASS_GetProc( classPtr, WIN_PROC_32A );
+    wc->lpfnWndProc   = (WNDPROC)CLASS_GetProc( classPtr, FALSE );
     wc->cbClsExtra    = classPtr->cbClsExtra;
     wc->cbWndExtra    = classPtr->cbWndExtra;
     wc->hInstance     = (hInstance == user32_module) ? 0 : hInstance;
@@ -1204,7 +1119,7 @@ BOOL WINAPI GetClassInfoExW( HINSTANCE hInstance, LPCWSTR name, WNDCLASSEXW *wc
         return FALSE;
     }
     wc->style         = classPtr->style;
-    wc->lpfnWndProc   = (WNDPROC)CLASS_GetProc( classPtr, WIN_PROC_32W );
+    wc->lpfnWndProc   = (WNDPROC)CLASS_GetProc( classPtr, TRUE );
     wc->cbClsExtra    = classPtr->cbClsExtra;
     wc->cbWndExtra    = classPtr->cbWndExtra;
     wc->hInstance     = (hInstance == user32_module) ? 0 : hInstance;
