@@ -55,7 +55,6 @@
 #include "wingdi.h"
 #include "wownt32.h"
 #include "winerror.h"
-#include "ntstatus.h"
 #include "excpt.h"
 #include "wine/winbase16.h"
 #include "wine/winuser16.h"
@@ -285,7 +284,7 @@ static BOOL is_dib_monochrome( const BITMAPINFO* info )
     }
     else  /* assume BITMAPINFOHEADER */
     {
-        RGBQUAD *rgb = info->bmiColors;
+        const RGBQUAD *rgb = info->bmiColors;
 
         /* Check if the first color is black */
         if ((rgb->rgbRed == 0) && (rgb->rgbGreen == 0) &&
@@ -541,7 +540,7 @@ static BOOL CURSORICON_GetResIconEntry( LPVOID dir, int n,
         return FALSE;
     icon = &resdir->idEntries[n].ResInfo.icon;
     *width = icon->bWidth;
-    *height = icon->bWidth;
+    *height = icon->bHeight;
     *bits = resdir->idEntries[n].wBitCount;
     return TRUE;
 }
@@ -2115,7 +2114,6 @@ static HBITMAP BITMAP_Load( HINSTANCE instance, LPCWSTR name,
         if (!instance)
         {
             /* OEM bitmap: try to load the resource from user32.dll */
-            if (HIWORD(name)) return 0;
             instance = user32_module;
         }
 
@@ -2203,17 +2201,6 @@ end:
  *
  * See LoadImageW.
  */
-
-/* filter for page-fault exceptions */
-static WINE_EXCEPTION_FILTER(page_fault)
-{
-    if (GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION)
-        return EXCEPTION_EXECUTE_HANDLER;
-    return EXCEPTION_CONTINUE_SEARCH;
-}
-
-/*********************************************************************/
-
 HANDLE WINAPI LoadImageA( HINSTANCE hinst, LPCSTR name, UINT type,
                               INT desiredx, INT desiredy, UINT loadflags)
 {
@@ -2228,7 +2215,7 @@ HANDLE WINAPI LoadImageA( HINSTANCE hinst, LPCSTR name, UINT type,
         u_name = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
         MultiByteToWideChar( CP_ACP, 0, name, -1, u_name, len );
     }
-    __EXCEPT(page_fault) {
+    __EXCEPT_PAGE_FAULT {
         SetLastError( ERROR_INVALID_PARAMETER );
         return 0;
     }
