@@ -195,6 +195,10 @@ BOOL WINAPI ReleaseCapture(void)
     SERVER_END_REQ;
 
     if (previous) SendMessageW( previous, WM_CAPTURECHANGED, 0, 0 );
+
+    /* Somebody may have missed some mouse movements */
+    mouse_event( MOUSEEVENTF_MOVE, 0, 0, 0, 0 );
+
     return ret;
 }
 
@@ -236,11 +240,14 @@ DWORD WINAPI GetQueueStatus( UINT flags )
 {
     DWORD ret = 0;
 
-    if (flags & ~0xff)
-        FIXME("QS_xxxx flags (%04x) are not handled\n", flags & ~0xff);
+    if (flags & ~(QS_ALLINPUT | QS_ALLPOSTMESSAGE | QS_SMRESULT))
+    {
+        SetLastError( ERROR_INVALID_FLAGS );
+        return 0;
+    }
 
     /* check for pending X events */
-    USER_Driver->pMsgWaitForMultipleObjectsEx( 0, NULL, 0, QS_ALLINPUT, 0 );
+    USER_Driver->pMsgWaitForMultipleObjectsEx( 0, NULL, 0, flags, 0 );
 
     SERVER_START_REQ( get_queue_status )
     {
