@@ -55,7 +55,7 @@ static void *user_handles[NB_USER_HANDLES];
  * Create a window handle with the server.
  */
 static WND *create_window_handle( HWND parent, HWND owner, ATOM atom,
-                                  HINSTANCE instance, WINDOWPROCTYPE type )
+                                  HINSTANCE instance, BOOL unicode )
 {
     WORD index;
     WND *win;
@@ -122,9 +122,10 @@ static WND *create_window_handle( HWND parent, HWND owner, ATOM atom,
     win->parent     = full_parent;
     win->owner      = full_owner;
     win->dwMagic    = WND_MAGIC;
+    win->flags      = 0;
     win->cbWndExtra = extra_bytes;
     memset( win->wExtra, 0, extra_bytes );
-    CLASS_AddWindow( class, win, type );
+    CLASS_AddWindow( class, win, unicode );
     return win;
 }
 
@@ -845,26 +846,21 @@ static void dump_window_styles( DWORD style, DWORD exstyle )
  *
  * Implementation of CreateWindowEx().
  */
-static HWND WIN_CreateWindowEx( CREATESTRUCTA *cs, ATOM classAtom,
-				WINDOWPROCTYPE type )
+static HWND WIN_CreateWindowEx( CREATESTRUCTA *cs, ATOM classAtom, UINT flags )
 {
     INT sw = SW_SHOW;
     WND *wndPtr;
     HWND hwnd, parent, owner, top_child = 0;
-    BOOL unicode = (type == WIN_PROC_32W);
+    BOOL unicode = (flags & WIN_ISUNICODE) != 0;
     MDICREATESTRUCTA mdi_cs;
 
     TRACE("%s %s ex=%08lx style=%08lx %d,%d %dx%d parent=%p menu=%p inst=%p params=%p\n",
-          (type == WIN_PROC_32W) ? debugstr_w((LPCWSTR)cs->lpszName) : debugstr_a(cs->lpszName),
-          (type == WIN_PROC_32W) ? debugstr_w((LPCWSTR)cs->lpszClass) : debugstr_a(cs->lpszClass),
+          unicode ? debugstr_w((LPCWSTR)cs->lpszName) : debugstr_a(cs->lpszName),
+          unicode ? debugstr_w((LPCWSTR)cs->lpszClass) : debugstr_a(cs->lpszClass),
           cs->dwExStyle, cs->style, cs->x, cs->y, cs->cx, cs->cy,
           cs->hwndParent, cs->hMenu, cs->hInstance, cs->lpCreateParams );
-
     if(TRACE_ON(win)) dump_window_styles( cs->style, cs->dwExStyle );
 
-    TRACE("winproc type is %d (%s)\n", type, (type == WIN_PROC_16) ? "WIN_PROC_16" :
-	    ((type == WIN_PROC_32A) ? "WIN_PROC_32A" : "WIN_PROC_32W") );
-
     /* Fix the styles for MDI children */
     if (cs->dwExStyle & WS_EX_MDICHILD)
     {
@@ -975,7 +971,7 @@ static HWND WIN_CreateWindowEx( CREATESTRUCTA *cs, ATOM classAtom,
 
     /* Create the window structure */
 
-    if (!(wndPtr = create_window_handle( parent, owner, classAtom, cs->hInstance, type )))
+    if (!(wndPtr = create_window_handle( parent, owner, classAtom, cs->hInstance, unicode )))
         return 0;
     hwnd = wndPtr->hwndSelf;
 
@@ -988,13 +984,13 @@ static HWND WIN_CreateWindowEx( CREATESTRUCTA *cs, ATOM classAtom,
     wndPtr->dwExStyle      = cs->dwExStyle;
     wndPtr->wIDmenu        = 0;
     wndPtr->helpContext    = 0;
-    wndPtr->flags          = (type == WIN_PROC_16) ? 0 : WIN_ISWIN32;
     wndPtr->pVScroll       = NULL;
     wndPtr->pHScroll       = NULL;
     wndPtr->userdata       = 0;
     wndPtr->hIcon          = 0;
     wndPtr->hIconSmall     = 0;
     wndPtr->hSysMenu       = 0;
+    wndPtr->flags         |= (flags & WIN_ISWIN32);
 
     if (wndPtr->dwStyle & WS_SYSMENU) SetSystemMenu( hwnd, 0 );
 
@@ -1032,7 +1028,7 @@ static HWND WIN_CreateWindowEx( CREATESTRUCTA *cs, ATOM classAtom,
         req->style     = wndPtr->dwStyle;
         req->ex_style  = wndPtr->dwExStyle;
         req->instance  = (void *)wndPtr->hInstance;
-        req->is_unicode = (type == WIN_PROC_32W);
+        req->is_unicode = (wndPtr->flags & WIN_ISUNICODE) != 0;
         req->extra_offset = -1;
         wine_server_call( req );
     }
@@ -1171,7 +1167,7 @@ HWND16 WINAPI CreateWindowEx16( DWORD exStyle, LPCSTR className,
     cs.lpszClass      = className;
     cs.dwExStyle      = exStyle;
 
-    return HWND_16( WIN_CreateWindowEx( &cs, classAtom, WIN_PROC_16 ));
+    return HWND_16( WIN_CreateWindowEx( &cs, classAtom, 0 ));
 }
 
 
@@ -1224,7 +1220,7 @@ HWND WINAPI CreateWindowExA( DWORD exStyle, LPCSTR className,
     cs.lpszClass      = className;
     cs.dwExStyle      = exStyle;
 
-    return WIN_CreateWindowEx( &cs, classAtom, WIN_PROC_32A );
+    return WIN_CreateWindowEx( &cs, classAtom, WIN_ISWIN32 );
 }
 
 
@@ -1279,7 +1275,7 @@ HWND WINAPI CreateWindowExW( DWORD exStyle, LPCWSTR className,
 
     /* Note: we rely on the fact that CREATESTRUCTA and */
     /* CREATESTRUCTW have the same layout. */
-    return WIN_CreateWindowEx( (CREATESTRUCTA *)&cs, classAtom, WIN_PROC_32W );
+    return WIN_CreateWindowEx( (CREATESTRUCTA *)&cs, classAtom, WIN_ISWIN32 | WIN_ISUNICODE );
 }
 
 
@@ -1683,7 +1679,7 @@ BOOL WINAPI IsWindowUnicode( HWND hwnd )
 
     if (wndPtr != WND_OTHER_PROCESS)
     {
-        retvalue = (WINPROC_GetProcType( wndPtr->winproc ) == WIN_PROC_32W);
+        retvalue = (wndPtr->flags & WIN_ISUNICODE) != 0;
         WIN_ReleasePtr( wndPtr );
     }
     else
@@ -1831,7 +1827,7 @@ WORD WINAPI SetWindowWord( HWND hwnd, INT offset, WORD newval )
  *
  * Helper function for GetWindowLong().
  */
-static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, WINDOWPROCTYPE type )
+static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, BOOL unicode )
 {
     LONG_PTR retvalue = 0;
     WND *wndPtr;
@@ -1888,24 +1884,6 @@ static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, WINDOWPROCTYPE type )
     {
         if (offset > (int)(wndPtr->cbWndExtra - sizeof(LONG)))
         {
-          /*
-            * Some programs try to access last element from 16 bit
-            * code using illegal offset value. Hopefully this is
-            * what those programs really expect.
-            */
-           if (type == WIN_PROC_16 &&
-               wndPtr->cbWndExtra >= 4 &&
-               offset == wndPtr->cbWndExtra - sizeof(WORD))
-           {
-               INT offset2 = wndPtr->cbWndExtra - sizeof(LONG);
-
-               ERR( "- replaced invalid offset %d with %d\n",
-                    offset, offset2 );
-
-                retvalue = *(LONG_PTR *)(((char *)wndPtr->wExtra) + offset2);
-                WIN_ReleasePtr( wndPtr );
-                return retvalue;
-            }
             WARN("Invalid offset %d\n", offset );
             WIN_ReleasePtr( wndPtr );
             SetLastError( ERROR_INVALID_INDEX );
@@ -1914,7 +1892,7 @@ static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, WINDOWPROCTYPE type )
         retvalue = *(LONG_PTR *)(((char *)wndPtr->wExtra) + offset);
         /* Special case for dialog window procedure */
         if ((offset == DWLP_DLGPROC) && (wndPtr->flags & WIN_ISDIALOG))
-            retvalue = (LONG_PTR)WINPROC_GetProc( (WNDPROC)retvalue, type );
+            retvalue = (LONG_PTR)WINPROC_GetProc( (WNDPROC)retvalue, unicode );
         WIN_ReleasePtr( wndPtr );
         return retvalue;
     }
@@ -1925,7 +1903,7 @@ static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, WINDOWPROCTYPE type )
     case GWL_STYLE:      retvalue = wndPtr->dwStyle; break;
     case GWL_EXSTYLE:    retvalue = wndPtr->dwExStyle; break;
     case GWLP_ID:        retvalue = (ULONG_PTR)wndPtr->wIDmenu; break;
-    case GWLP_WNDPROC:   retvalue = (ULONG_PTR)WINPROC_GetProc( wndPtr->winproc, type ); break;
+    case GWLP_WNDPROC:   retvalue = (ULONG_PTR)WINPROC_GetProc( wndPtr->winproc, unicode ); break;
     case GWLP_HINSTANCE: retvalue = (ULONG_PTR)wndPtr->hInstance; break;
     default:
         WARN("Unknown offset %d\n", offset );
@@ -1945,15 +1923,14 @@ static LONG_PTR WIN_GetWindowLong( HWND hwnd, INT offset, WINDOWPROCTYPE type )
  * 0 is the failure code. However, in the case of failure SetLastError
  * must be set to distinguish between a 0 return value and a failure.
  */
-static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval,
-                                   WINDOWPROCTYPE type )
+static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval, BOOL unicode )
 {
     STYLESTRUCT style;
     BOOL ok;
     LONG_PTR retval = 0;
     WND *wndPtr;
 
-    TRACE( "%p %d %lx %x\n", hwnd, offset, newval, type );
+    TRACE( "%p %d %lx %c\n", hwnd, offset, newval, unicode ? 'W' : 'A' );
 
     if (is_broadcast(hwnd))
     {
@@ -2008,10 +1985,12 @@ static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval,
         }
     case GWLP_WNDPROC:
     {
-        WINDOWPROCTYPE old_type = WINPROC_GetProcType( wndPtr->winproc );
-        retval = (ULONG_PTR)WINPROC_GetProc( wndPtr->winproc, type );
-        wndPtr->winproc = WINPROC_AllocProc( (WNDPROC)newval, type );
-        if (old_type == type)
+        UINT old_flags = wndPtr->flags;
+        retval = (ULONG_PTR)WINPROC_GetProc( wndPtr->winproc, unicode );
+        wndPtr->winproc = WINPROC_AllocProc( (WNDPROC)newval, unicode );
+        if (WINPROC_IsUnicode( wndPtr->winproc, unicode )) wndPtr->flags |= WIN_ISUNICODE;
+        else wndPtr->flags &= ~WIN_ISUNICODE;
+        if (!((old_flags ^ wndPtr->flags) & WIN_ISUNICODE))
         {
             WIN_ReleasePtr( wndPtr );
             return retval;
@@ -2024,11 +2003,11 @@ static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval,
     case GWLP_USERDATA:
         break;
     case DWLP_DLGPROC:
-        if ((wndPtr->cbWndExtra + sizeof(LONG_PTR) >= DWLP_DLGPROC) && (wndPtr->flags & WIN_ISDIALOG))
+        if ((wndPtr->cbWndExtra - sizeof(LONG_PTR) >= DWLP_DLGPROC) && (wndPtr->flags & WIN_ISDIALOG))
         {
             WNDPROC *ptr = (WNDPROC *)((char *)wndPtr->wExtra + DWLP_DLGPROC);
-            retval = (ULONG_PTR)WINPROC_GetProc( *ptr, type );
-            *ptr = WINPROC_AllocProc( (WNDPROC)newval, type );
+            retval = (ULONG_PTR)WINPROC_GetProc( *ptr, unicode );
+            *ptr = WINPROC_AllocProc( (WNDPROC)newval, unicode );
             WIN_ReleasePtr( wndPtr );
             return retval;
         }
@@ -2077,7 +2056,7 @@ static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval,
             break;
         case GWLP_WNDPROC:
             req->flags = SET_WIN_UNICODE;
-            req->is_unicode = (type == WIN_PROC_32W);
+            req->is_unicode = (wndPtr->flags & WIN_ISUNICODE) != 0;
             break;
         case GWLP_USERDATA:
             req->flags = SET_WIN_USERDATA;
@@ -2144,7 +2123,47 @@ static LONG_PTR WIN_SetWindowLong( HWND hwnd, INT offset, LONG_PTR newval,
  */
 LONG WINAPI GetWindowLong16( HWND16 hwnd, INT16 offset )
 {
-    return WIN_GetWindowLong( WIN_Handle32(hwnd), offset, WIN_PROC_16 );
+    WND *wndPtr;
+    LONG_PTR retvalue;
+    BOOL is_winproc = (offset == GWLP_WNDPROC);
+
+    if (offset >= 0)
+    {
+        if (!(wndPtr = WIN_GetPtr( WIN_Handle32(hwnd) )))
+        {
+            SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+            return 0;
+        }
+        if (wndPtr != WND_OTHER_PROCESS && wndPtr != WND_DESKTOP)
+        {
+            if (offset > (int)(wndPtr->cbWndExtra - sizeof(LONG)))
+            {
+                /*
+                 * Some programs try to access last element from 16 bit
+                 * code using illegal offset value. Hopefully this is
+                 * what those programs really expect.
+                 */
+                if (wndPtr->cbWndExtra >= 4 && offset == wndPtr->cbWndExtra - sizeof(WORD))
+                {
+                    INT offset2 = wndPtr->cbWndExtra - sizeof(LONG);
+                    ERR( "- replaced invalid offset %d with %d\n", offset, offset2 );
+                    offset = offset2;
+                }
+                else
+                {
+                    WARN("Invalid offset %d\n", offset );
+                    WIN_ReleasePtr( wndPtr );
+                    SetLastError( ERROR_INVALID_INDEX );
+                    return 0;
+                }
+            }
+            is_winproc = ((offset == DWLP_DLGPROC) && (wndPtr->flags & WIN_ISDIALOG));
+            WIN_ReleasePtr( wndPtr );
+        }
+    }
+    retvalue = GetWindowLongA( WIN_Handle32(hwnd), offset );
+    if (is_winproc) retvalue = (LONG_PTR)WINPROC_GetProc16( (WNDPROC)retvalue, FALSE );
+    return retvalue;
 }
 
 
@@ -2153,7 +2172,7 @@ LONG WINAPI GetWindowLong16( HWND16 hwnd, INT16 offset )
  */
 LONG WINAPI GetWindowLongA( HWND hwnd, INT offset )
 {
-    return WIN_GetWindowLong( hwnd, offset, WIN_PROC_32A );
+    return WIN_GetWindowLong( hwnd, offset, FALSE );
 }
 
 
@@ -2162,7 +2181,7 @@ LONG WINAPI GetWindowLongA( HWND hwnd, INT offset )
  */
 LONG WINAPI GetWindowLongW( HWND hwnd, INT offset )
 {
-    return WIN_GetWindowLong( hwnd, offset, WIN_PROC_32W );
+    return WIN_GetWindowLong( hwnd, offset, TRUE );
 }
 
 
@@ -2171,7 +2190,31 @@ LONG WINAPI GetWindowLongW( HWND hwnd, INT offset )
  */
 LONG WINAPI SetWindowLong16( HWND16 hwnd, INT16 offset, LONG newval )
 {
-    return WIN_SetWindowLong( WIN_Handle32(hwnd), offset, newval, WIN_PROC_16 );
+    WND *wndPtr;
+    BOOL is_winproc = (offset == GWLP_WNDPROC);
+
+    if (offset == DWLP_DLGPROC)
+    {
+        if (!(wndPtr = WIN_GetPtr( WIN_Handle32(hwnd) )))
+        {
+            SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+            return 0;
+        }
+        if (wndPtr != WND_OTHER_PROCESS && wndPtr != WND_DESKTOP)
+        {
+            is_winproc = ((wndPtr->cbWndExtra - sizeof(LONG_PTR) >= DWLP_DLGPROC) &&
+                          (wndPtr->flags & WIN_ISDIALOG));
+            WIN_ReleasePtr( wndPtr );
+        }
+    }
+
+    if (is_winproc)
+    {
+        WNDPROC new_proc = WINPROC_AllocProc16( (WNDPROC16)newval );
+        WNDPROC old_proc = (WNDPROC)SetWindowLongA( WIN_Handle32(hwnd), offset, (LONG_PTR)new_proc );
+        return (LONG)WINPROC_GetProc16( (WNDPROC)old_proc, FALSE );
+    }
+    else return SetWindowLongA( WIN_Handle32(hwnd), offset, newval );
 }
 
 
@@ -2182,7 +2225,7 @@ LONG WINAPI SetWindowLong16( HWND16 hwnd, INT16 offset, LONG newval )
  */
 LONG WINAPI SetWindowLongA( HWND hwnd, INT offset, LONG newval )
 {
-    return WIN_SetWindowLong( hwnd, offset, newval, WIN_PROC_32A );
+    return WIN_SetWindowLong( hwnd, offset, newval, FALSE );
 }
 
 
@@ -2257,7 +2300,7 @@ LONG WINAPI SetWindowLongW(
     INT offset, /* [in] offset, in bytes, of location to alter */
     LONG newval /* [in] new value of location */
 ) {
-    return WIN_SetWindowLong( hwnd, offset, newval, WIN_PROC_32W );
+    return WIN_SetWindowLong( hwnd, offset, newval, TRUE );
 }
 
 
