@@ -70,9 +70,6 @@ static const SCRIPT_PROPERTIES *Global_Script[MAX_SCRIPTS] =
 
 typedef struct scriptcache {
        HDC hdc;
-       DWORD GlyphToChar[256];
-       int HaveWidths;
-       ABC CharWidths[256];
 } Scriptcache;
 
 BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpv)
@@ -126,8 +123,12 @@ HRESULT WINAPI ScriptGetFontProperties(HDC hdc, SCRIPT_CACHE *psc, SCRIPT_FONTPR
 {
     HDC phdc;
     Scriptcache *pScriptcache;
+    TEXTMETRICW ptm;
 
-    FIXME("%p,%p,%p\n", hdc, psc, sfp);
+    TRACE("%p,%p,%p\n", hdc, psc, sfp);
+
+    if (!psc || !sfp)
+        return E_INVALIDARG;
     if  (!hdc && !*psc) {
         TRACE("No Script_Cache (psc) and no hdc. Ask for one. Hdc=%p, psc=%p\n", hdc, *psc);
 	return E_PENDING;
@@ -136,9 +137,8 @@ HRESULT WINAPI ScriptGetFontProperties(HDC hdc, SCRIPT_CACHE *psc, SCRIPT_FONTPR
             pScriptcache = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(Scriptcache) );
             pScriptcache->hdc = (HDC) hdc;
             phdc = hdc;
-            pScriptcache->HaveWidths = 0;
             *psc = (Scriptcache *) pScriptcache;
-       }   else
+        }   else
             if  (*psc) {
                 pScriptcache = (Scriptcache *) *psc;
                 phdc = pScriptcache->hdc;
@@ -148,13 +148,14 @@ HRESULT WINAPI ScriptGetFontProperties(HDC hdc, SCRIPT_CACHE *psc, SCRIPT_FONTPR
         return E_INVALIDARG;
 
     /* return something sensible? */
-    if (NULL != sfp) {
-        sfp->wgBlank       = 0;
-        sfp->wgDefault     = 0;
-        sfp->wgInvalid     = 0;
-        sfp->wgKashida     = 1;
-        sfp->iKashidaWidth = 0;
-    }
+    sfp->wgBlank       = 0;
+    if  (GetTextMetricsW(phdc, &ptm)) 
+        sfp->wgDefault = ptm.tmDefaultChar;
+    else
+        sfp->wgDefault = 0;
+    sfp->wgInvalid     = 0;
+    sfp->wgKashida     = 0xffff;
+    sfp->iKashidaWidth = 0;
     return 0;
 }
 
@@ -303,6 +304,7 @@ HRESULT WINAPI ScriptIsComplex(const WCHAR* pwcInChars, int cInChars, DWORD dwFl
   FIXME("(%s,%d,0x%lx): stub\n",  debugstr_wn(pwcInChars, cInChars), cInChars, dwFlags);
    return E_NOTIMPL;
 }
+
 /***********************************************************************
  *      ScriptShape (USP10.@)
  *
@@ -333,10 +335,9 @@ HRESULT WINAPI ScriptShape(HDC hdc, SCRIPT_CACHE *psc, const WCHAR *pwcChars,
 	return E_PENDING;
     }   else 
         if  (hdc && !*psc) {
-            pScriptcache = HeapAlloc( GetProcessHeap(), 0, sizeof(Scriptcache) );
+            pScriptcache = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(Scriptcache) );
             pScriptcache->hdc = (HDC) hdc;
             phdc = hdc;
-            pScriptcache->HaveWidths = 0;
             *psc = (Scriptcache *) pScriptcache;
        }   else
             if  (*psc) {
@@ -354,16 +355,14 @@ HRESULT WINAPI ScriptShape(HDC hdc, SCRIPT_CACHE *psc, const WCHAR *pwcChars,
         TRACE("After:  ");
         for (cnt = 0; cnt < cChars; cnt++) {
              TRACE("%04x",pwOutGlyphs[cnt]);
-             pScriptcache->GlyphToChar[pwOutGlyphs[cnt]] = pwcChars[cnt]; /* save for ScriptPlace */
         }
-       TRACE("\n");
+        TRACE("\n");
     }
     else {
         TRACE("After:  ");
         for (cnt = 0; cnt < cChars; cnt++) {                           /* no translate so set up */
              pwOutGlyphs[cnt] = pwcChars[cnt];                         /* copy in to out and     */
              TRACE("%04x",pwOutGlyphs[cnt]);
-             pScriptcache->GlyphToChar[pwcChars[cnt]] = pwcChars[cnt]; /* set up a dummy table   */
         }
        TRACE("\n");
     }
@@ -417,34 +416,41 @@ HRESULT WINAPI ScriptPlace(HDC hdc, SCRIPT_CACHE *psc, const WORD *pwGlyphs,
 
     /*   Here we need to calculate the width of the run unit.  At this point the input string
      *   has been converted to glyphs and we till need to translate back to the original chars
-     *   to get the correct ABC widths.  To make life easier the ABC widths are saved in the 
-     *   SCRIPT_CACHE.  This is ok as the cache must be invalidated if the font or font size 
-     *   changes.   */
+     *   to get the correct ABC widths.   */
 
-     lpABC = pScriptcache->CharWidths;
+     lpABC = HeapAlloc(GetProcessHeap(), 0 , sizeof(ABC)*cGlyphs);
      pABC->abcA = 0; 
      pABC->abcB = 0; 
      pABC->abcC = 0; 
-     if  (!pScriptcache->HaveWidths) {
-         if  (GetCharABCWidthsW(phdc, 0, 255, lpABC )) 
-             pScriptcache->HaveWidths = 1;
-         else {
-             WARN("Could not get ABC values\n");
+     if  (!GetCharABCWidthsI(phdc, 0, cGlyphs, (WORD *) pwGlyphs, lpABC )) 
+     {
+         WARN("Could not get ABC values\n");
+         for (wcnt = 0; wcnt < cGlyphs; wcnt++) {
+             piAdvance[wcnt] = 0;
+             pGoffset[wcnt].du = 0;
+             pGoffset[wcnt].dv = 0;
          }
      }
-
-     for (wcnt = 0; wcnt < cGlyphs ; wcnt++) {          /* add up the char lengths  */
-         TRACE("     Glyph=%04x, abcA=%d,  abcB=%d,  abcC=%d  wcnt=%d\n",
-                              pwGlyphs[wcnt], 
-                              lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcA,
-                              lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcB,
-                              lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcC, wcnt);
-         pABC->abcA += lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcA;
-         pABC->abcB += lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcB;
-         pABC->abcC += lpABC[pScriptcache->GlyphToChar[pwGlyphs[wcnt]]].abcC;
+     else
+     {
+         for (wcnt = 0; wcnt < cGlyphs ; wcnt++) {          /* add up the char lengths  */
+             TRACE("     Glyph=%04x,  abcA=%d,  abcB=%d,  abcC=%d  wcnt=%d\n",
+                                  pwGlyphs[wcnt],  
+                                  lpABC[wcnt].abcA,
+                                  lpABC[wcnt].abcB,
+                                  lpABC[wcnt].abcC, wcnt);
+             pABC->abcA += lpABC[wcnt].abcA;
+             pABC->abcB += lpABC[wcnt].abcB;
+             pABC->abcC += lpABC[wcnt].abcC;
+             piAdvance[wcnt] = lpABC[wcnt].abcA + lpABC[wcnt].abcB + lpABC[wcnt].abcC;
+             pGoffset[wcnt].du = 0;
+             pGoffset[wcnt].dv = 0;
+         }
      }
      TRACE("Total for run:   abcA=%d,  abcB=%d,  abcC=%d\n", pABC->abcA, pABC->abcB, pABC->abcC);
 
+     HeapFree(GetProcessHeap(), 0, lpABC );
+
      return 0;
 }
 
@@ -459,7 +465,11 @@ HRESULT WINAPI ScriptGetCMap(HDC hdc, SCRIPT_CACHE *psc, const WCHAR *pwcInChars
     int cnt;
     DWORD hr;
     Scriptcache *pScriptcache;
-    FIXME("(%p,%p,%s,%d,0x%lx,%p): semi-stub\n", hdc, psc, debugstr_wn(pwcInChars,cChars), cChars, dwFlags, pwOutGlyphs);
+    FIXME("(%p,%p,%s,%d,0x%lx,%p): semi-stub\n", hdc, psc, debugstr_wn(pwcInChars,cChars), 
+                                                 cChars, dwFlags, pwOutGlyphs);
+
+    if  (!psc || !pwcInChars || !pwOutGlyphs)
+        return E_INVALIDARG;
 
     if  (!hdc && !*psc) {
         TRACE("No Script_Cache (psc) and no hdc. Ask for one. Hdc=%p, psc=%p\n", hdc, *psc);
@@ -469,7 +479,6 @@ HRESULT WINAPI ScriptGetCMap(HDC hdc, SCRIPT_CACHE *psc, const WCHAR *pwcInChars
             pScriptcache = HeapAlloc( GetProcessHeap(), 0, sizeof(Scriptcache) );
             pScriptcache->hdc = hdc;
             phdc = hdc;
-            pScriptcache->HaveWidths = 0;
             *psc = pScriptcache;
         }   else
             if  (*psc) {
@@ -486,7 +495,6 @@ HRESULT WINAPI ScriptGetCMap(HDC hdc, SCRIPT_CACHE *psc, const WCHAR *pwcInChars
     TRACE("After:  ");
     for (cnt = 0; cnt < cChars; cnt++) {
          TRACE("%04x",pwOutGlyphs[cnt]);
-         pScriptcache->GlyphToChar[pwOutGlyphs[cnt]] = pwcInChars[cnt]; /* save for ScriptPlace */
     }
     TRACE("\n");
 
@@ -502,8 +510,40 @@ HRESULT WINAPI ScriptTextOut(const HDC hdc, SCRIPT_CACHE *psc, int x, int y, UIN
                              int iReserved, const WORD *pwGlyphs, int cGlyphs, const int *piAdvance, 
                              const int *piJustify, const GOFFSET *pGoffset)
 {
-     FIXME("(%p, %p, %d, %d, %04x, %p, %p, %p, %d, %p, %d, %p, %p, %p): stub\n",
-           hdc, psc, x, y, fuOptions, lprc, psa, pwcReserved, iReserved, pwGlyphs, cGlyphs,
-           piAdvance, piJustify, pGoffset);
-     return E_NOTIMPL;
+    HDC phdc;
+    DWORD hr;
+    Scriptcache *pScriptcache;
+    TRACE     ("(%p, %p, %d, %d, %04x, %p, %p, %p, %d, %p, %d, %p, %p, %p): stub\n",
+         hdc, psc, x, y, fuOptions, lprc, psa, pwcReserved, iReserved, pwGlyphs, cGlyphs,
+         piAdvance, piJustify, pGoffset);
+
+    if  (!psc || !piAdvance || !psa || !pwGlyphs)
+        return E_INVALIDARG;
+        
+    if  (!hdc && !*psc) {
+        TRACE("No Script_Cache (psc) and no hdc. Ask for one. Hdc=%p, psc=%p\n", hdc, *psc);
+	return E_PENDING;
+    }   else 
+        if  (hdc && !*psc) {
+            pScriptcache = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(Scriptcache) );
+            pScriptcache->hdc = hdc;
+            phdc = hdc;
+            *psc = pScriptcache;
+        }   else
+            if  (*psc) {
+                pScriptcache = *psc;
+                phdc = pScriptcache->hdc;
+            }
+
+    fuOptions &= ETO_CLIPPED + ETO_OPAQUE;
+    if  (!psa->fNoGlyphIndex)                                     /* Have Glyphs?                      */
+        fuOptions |= ETO_GLYPH_INDEX;                             /* Say don't do tranlastion to glyph */
+
+    hr = ExtTextOutW(phdc, x, y, fuOptions, lprc, pwGlyphs, cGlyphs, NULL);
+
+    if  (hr) return S_OK;
+    else {
+        FIXME("ExtTextOut returned:=%ld\n", hr);
+        return hr;
+    }
 }
