@@ -173,9 +173,9 @@ HRESULT WINAPI IWineD3DImpl_QueryInterface(IWineD3D *iface,REFIID riid,LPVOID *p
         || IsEqualGUID(riid, &IID_IWineD3DDevice)) {
         IUnknown_AddRef(iface);
         *ppobj = This;
-        return WINED3D_OK;
+        return S_OK;
     }
-
+    *ppobj = NULL;
     return E_NOINTERFACE;
 }
 
@@ -203,7 +203,7 @@ ULONG WINAPI IWineD3DImpl_Release(IWineD3D *iface) {
  * IWineD3D parts follows
  **********************************************************/
 
-static BOOL IWineD3DImpl_FillGLCaps(WineD3D_GL_Info *gl_info, Display* display) {
+BOOL IWineD3DImpl_FillGLCaps(WineD3D_GL_Info *gl_info, Display* display) {
     const char *GL_Extensions    = NULL;
     const char *GLX_Extensions   = NULL;
     const char *gl_string        = NULL;
@@ -489,6 +489,9 @@ static BOOL IWineD3DImpl_FillGLCaps(WineD3D_GL_Info *gl_info, Display* display)
                 glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &gl_max);
                 TRACE_(d3d_caps)(" FOUND: ARB Pixel Shader support - GL_MAX_TEXTURE_IMAGE_UNITS_ARB=%u\n", gl_max);
                 gl_info->max_samplers = min(MAX_SAMPLERS, gl_max);
+            } else if (strcmp(ThisExtn, "GL_ARB_shading_language_100") == 0) {
+                TRACE_(d3d_caps)(" FOUND: GL Shading Language v100 support\n");
+                gl_info->supported[ARB_SHADING_LANGUAGE_100] = TRUE;
             } else if (strcmp(ThisExtn, "GL_ARB_multisample") == 0) {
                 TRACE_(d3d_caps)(" FOUND: ARB Multisample support\n");
                 gl_info->supported[ARB_MULTISAMPLE] = TRUE;
@@ -1674,28 +1677,48 @@ HRESULT WINAPI IWineD3DImpl_GetDeviceCaps(IWineD3D *iface, UINT Adapter, WINED3D
     *pCaps->MaxStreams          = MAX_STREAMS;
     *pCaps->MaxStreamStride     = 1024;
 
-    if (((wined3d_settings.vs_mode == VS_HW) && GL_SUPPORT(ARB_VERTEX_PROGRAM)) || (wined3d_settings.vs_mode == VS_SW) || (DeviceType == WINED3DDEVTYPE_REF)) {
-      *pCaps->VertexShaderVersion = D3DVS_VERSION(1,1);
-
-      if (This->gl_info.gl_vendor == VENDOR_MESA ||
-          This->gl_info.gl_vendor == VENDOR_WINE) {
-        *pCaps->MaxVertexShaderConst = 95;
-      } else {
-        *pCaps->MaxVertexShaderConst = WINED3D_VSHADER_MAX_CONSTANTS;
-      }
+    if (wined3d_settings.vs_mode == VS_HW && GL_SUPPORT(ARB_SHADING_LANGUAGE_100) && DeviceType != WINED3DDEVTYPE_REF) {
+        /* FIXME: Uncomment the line below and remove the line beneath that to set Vertex Shader 2.0+ capability */
+        /* *pCaps->VertexShaderVersion = D3DVS_VERSION(3,0); */
+        *pCaps->VertexShaderVersion = D3DVS_VERSION(1,1);
+        TRACE_(d3d_caps)("Hardware Vertex Shaders versions 2.0+ enabled\n");
+    } else if (wined3d_settings.vs_mode == VS_HW && GL_SUPPORT(ARB_VERTEX_PROGRAM) && DeviceType != WINED3DDEVTYPE_REF) {
+        *pCaps->VertexShaderVersion = D3DVS_VERSION(1,1);
+        TRACE_(d3d_caps)("Hardware Vertex Shader version 1.1 enabled\n");
+    } else if (wined3d_settings.vs_mode == VS_SW || DeviceType == WINED3DDEVTYPE_REF) {
+        *pCaps->VertexShaderVersion = D3DVS_VERSION(1,1);
+        TRACE_(d3d_caps)("Software Vertex Shader version 1.1 enabled\n");
     } else {
         *pCaps->VertexShaderVersion  = 0;
-        *pCaps->MaxVertexShaderConst = 0;
+        TRACE_(d3d_caps)("Vertex Shader functionality not available\n");
     }
 
-    if ((wined3d_settings.ps_mode == PS_HW) && GL_SUPPORT(ARB_FRAGMENT_PROGRAM) && (DeviceType != WINED3DDEVTYPE_REF)) {
+    if (This->gl_info.gl_vendor == VENDOR_MESA || This->gl_info.gl_vendor == VENDOR_WINE) {
+        *pCaps->MaxVertexShaderConst = 95;
+    } else {
+        *pCaps->MaxVertexShaderConst = WINED3D_VSHADER_MAX_CONSTANTS;
+    }
+
+    if (wined3d_settings.ps_mode == PS_HW && GL_SUPPORT(ARB_SHADING_LANGUAGE_100) && DeviceType != WINED3DDEVTYPE_REF) {
+        /* FIXME: Uncomment the following line and remove the line beneath that to set Pixel Shader 2.0+ capability */
+        /* *pCaps->PixelShaderVersion = D3DPS_VERSION(3,0); */
         *pCaps->PixelShaderVersion    = D3DPS_VERSION(1,4);
         *pCaps->PixelShader1xMaxValue = 1.0;
+        TRACE_(d3d_caps)("Hardware Pixel Shaders versions 2.0+ enabled\n");
+    } else if (wined3d_settings.ps_mode == PS_HW && GL_SUPPORT(ARB_FRAGMENT_PROGRAM) && DeviceType != WINED3DDEVTYPE_REF) {
+        *pCaps->PixelShaderVersion    = D3DPS_VERSION(1,4);
+        *pCaps->PixelShader1xMaxValue = 1.0;
+        TRACE_(d3d_caps)("Hardware Pixel Shaders version 1.4 enabled\n");
+    /* FIXME: Uncomment this when there is support for software Pixel Shader 1.4 and PS_SW is defined
+    } else if (wined3d_settings.ps_mode == PS_SW || DeviceType == WINED3DDEVTYPE_REF) {
+        *pCaps->PixelShaderVersion    = D3DPS_VERSION(1,4);
+        *pCaps->PixelShader1xMaxValue = 1.0;
+        TRACE_(d3d_caps)("Software Pixel Shader version 1.4 enabled\n"); */
     } else {
         *pCaps->PixelShaderVersion    = 0;
         *pCaps->PixelShader1xMaxValue = 0.0;
+        TRACE_(d3d_caps)("Pixel Shader functionality not available\n");
     }
-    /* TODO: ARB_FRAGMENT_PROGRAM_100 */
 
     /* ------------------------------------------------
        The following fields apply to d3d9 only
@@ -1744,13 +1767,11 @@ HRESULT WINAPI IWineD3DImpl_GetDeviceCaps(IWineD3D *iface, UINT Adapter, WINED3D
 /* Note due to structure differences between dx8 and dx9 D3DPRESENT_PARAMETERS,
    and fields being inserted in the middle, a new structure is used in place    */
 HRESULT  WINAPI  IWineD3DImpl_CreateDevice(IWineD3D *iface, UINT Adapter, WINED3DDEVTYPE DeviceType, HWND hFocusWindow,
-                                           DWORD BehaviourFlags, WINED3DPRESENT_PARAMETERS* pPresentationParameters,
-                                           IWineD3DDevice** ppReturnedDeviceInterface, IUnknown *parent,
-                                           D3DCB_CREATEADDITIONALSWAPCHAIN D3DCB_CreateAdditionalSwapChain) {
+                                           DWORD BehaviourFlags, IWineD3DDevice** ppReturnedDeviceInterface,
+                                           IUnknown *parent) {
 
     IWineD3DDeviceImpl *object  = NULL;
     IWineD3DImpl       *This    = (IWineD3DImpl *)iface;
-    IWineD3DSwapChainImpl *swapchain;
 
     /* Validate the adapter number */
     if (Adapter >= IWineD3D_GetAdapterCount(iface)) {
@@ -1775,11 +1796,8 @@ HRESULT  WINAPI  IWineD3DImpl_CreateDevice(IWineD3D *iface, UINT Adapter, WINED3
     /* Set the state up as invalid until the device is fully created */
     object->state   = WINED3DERR_DRIVERINTERNALERROR;
 
-    TRACE("(%p)->(Adptr:%d, DevType: %x, FocusHwnd: %p, BehFlags: %lx, PresParms: %p, RetDevInt: %p)\n", This, Adapter, DeviceType,
-          hFocusWindow, BehaviourFlags, pPresentationParameters, ppReturnedDeviceInterface);
-    TRACE("(%p)->(DepthStencil:(%u,%s), BackBufferFormat:(%u,%s))\n", This,
-          *(pPresentationParameters->AutoDepthStencilFormat), debug_d3dformat(*(pPresentationParameters->AutoDepthStencilFormat)),
-          *(pPresentationParameters->BackBufferFormat), debug_d3dformat(*(pPresentationParameters->BackBufferFormat)));
+    TRACE("(%p)->(Adptr:%d, DevType: %x, FocusHwnd: %p, BehFlags: %lx, RetDevInt: %p)\n", This, Adapter, DeviceType,
+          hFocusWindow, BehaviourFlags, ppReturnedDeviceInterface);
 
     /* Save the creation parameters */
     object->createParms.AdapterOrdinal = Adapter;
@@ -1791,86 +1809,24 @@ HRESULT  WINAPI  IWineD3DImpl_CreateDevice(IWineD3D *iface, UINT Adapter, WINED3
     object->adapterNo                    = Adapter;
     object->devType                      = DeviceType;
 
-    /* FIXME: Use for dx7 code eventually too! */
-    /* Deliberately no indentation here, as this if will be removed when dx8 support merged in */
-    if (This->dxVersion >= 8) {
-        TRACE("(%p) : Creating stateblock\n", This);
-        /* Creating the startup stateBlock - Note Special Case: 0 => Don't fill in yet! */
-        if (WINED3D_OK != IWineD3DDevice_CreateStateBlock((IWineD3DDevice *)object,
-                                         WINED3DSBT_INIT,
-                                        (IWineD3DStateBlock **)&object->stateBlock,
-                                        NULL)  || NULL == object->stateBlock) {   /* Note: No parent needed for initial internal stateblock */
-            WARN("Failed to create stateblock\n");
-            goto create_device_error;
-        }
-        TRACE("(%p) : Created stateblock (%p)\n", This, object->stateBlock);
-        object->updateStateBlock = object->stateBlock;
-        IWineD3DStateBlock_AddRef((IWineD3DStateBlock*)object->updateStateBlock);
-        /* Setup surfaces for the backbuffer, frontbuffer and depthstencil buffer */
-
-        /* Setup some defaults for creating the implicit swapchain */
-        ENTER_GL();
-        IWineD3DImpl_FillGLCaps(&This->gl_info, IWineD3DImpl_GetAdapterDisplay(iface, Adapter));
-        LEAVE_GL();
-
-        /* Setup the implicit swapchain */
-        TRACE("Creating implicit swapchain\n");
-        if (WINED3D_OK != D3DCB_CreateAdditionalSwapChain((IUnknown *) object->parent, pPresentationParameters, (IWineD3DSwapChain **)&swapchain) || swapchain == NULL) {
-            WARN("Failed to create implicit swapchain\n");
-            goto create_device_error;
-        }
-
-        object->renderTarget = swapchain->backBuffer;
-        IWineD3DSurface_AddRef(object->renderTarget);
-        /* Depth Stencil support */
-        object->stencilBufferTarget = object->depthStencilBuffer;
-        if (NULL != object->stencilBufferTarget) {
-            IWineD3DSurface_AddRef(object->stencilBufferTarget);
-        }
-
-        /* Set up some starting GL setup */
-        ENTER_GL();
-        /*
-        * Initialize openGL extension related variables
-        *  with Default values
-        */
-
-        This->isGLInfoValid = IWineD3DImpl_FillGLCaps(&This->gl_info, swapchain->display);
-        /* Setup all the devices defaults */
-        IWineD3DStateBlock_InitStartupStateBlock((IWineD3DStateBlock *)object->stateBlock);
-#if 0
-        IWineD3DImpl_CheckGraphicsMemory();
-#endif
-        LEAVE_GL();
-
-        { /* Set a default viewport */
-            D3DVIEWPORT9 vp;
-            vp.X      = 0;
-            vp.Y      = 0;
-            vp.Width  = *(pPresentationParameters->BackBufferWidth);
-            vp.Height = *(pPresentationParameters->BackBufferHeight);
-            vp.MinZ   = 0.0f;
-            vp.MaxZ   = 1.0f;
-            IWineD3DDevice_SetViewport((IWineD3DDevice *)object, &vp);
-        }
-
-
-        /* Initialize the current view state */
-        object->modelview_valid = 1;
-        object->proj_valid = 0;
-        object->view_ident = 1;
-        object->last_was_rhw = 0;
-        glGetIntegerv(GL_MAX_LIGHTS, &object->maxConcurrentLights);
-        TRACE("(%p,%d) All defaults now set up, leaving CreateDevice with %p\n", This, Adapter, object);
-
-        /* Clear the screen */
-        IWineD3DDevice_Clear((IWineD3DDevice *) object, 0, NULL, D3DCLEAR_STENCIL|D3DCLEAR_ZBUFFER|D3DCLEAR_TARGET, 0x00, 1.0, 0);
-
-    } else { /* End of FIXME: remove when dx8 merged in */
-
-        FIXME("(%p) Incomplete stub for d3d8\n", This);
-
+    TRACE("(%p) : Creating stateblock\n", This);
+    /* Creating the startup stateBlock - Note Special Case: 0 => Don't fill in yet! */
+    if (WINED3D_OK != IWineD3DDevice_CreateStateBlock((IWineD3DDevice *)object,
+                                      WINED3DSBT_INIT,
+                                    (IWineD3DStateBlock **)&object->stateBlock,
+                                    NULL)  || NULL == object->stateBlock) {   /* Note: No parent needed for initial internal stateblock */
+        WARN("Failed to create stateblock\n");
+        goto create_device_error;
     }
+    TRACE("(%p) : Created stateblock (%p)\n", This, object->stateBlock);
+    object->updateStateBlock = object->stateBlock;
+    IWineD3DStateBlock_AddRef((IWineD3DStateBlock*)object->updateStateBlock);
+    /* Setup surfaces for the backbuffer, frontbuffer and depthstencil buffer */
+
+    /* Setup some defaults for creating the implicit swapchain */
+    ENTER_GL();
+    IWineD3DImpl_FillGLCaps(&This->gl_info, IWineD3DImpl_GetAdapterDisplay(iface, Adapter));
+    LEAVE_GL();
 
     /* set the state of the device to valid */
     object->state = WINED3D_OK;
@@ -1901,10 +1857,6 @@ create_device_error:
         IWineD3DSurface_Release(object->stencilBufferTarget);
         object->stencilBufferTarget = NULL;
     }
-    if (swapchain != NULL) {
-        IWineD3DSwapChain_Release((IWineD3DSwapChain *)swapchain);
-        swapchain = NULL;
-    }
     HeapFree(GetProcessHeap(), 0, object);
     *ppReturnedDeviceInterface = NULL;
     return WINED3DERR_INVALIDCALL;
