@@ -38,8 +38,9 @@ HRESULT WINAPI IWineD3DStateBlockImpl_QueryInterface(IWineD3DStateBlock *iface,R
         || IsEqualGUID(riid, &IID_IWineD3DStateBlock)){
         IUnknown_AddRef(iface);
         *ppobj = This;
-        return D3D_OK;
+        return S_OK;
     }
+    *ppobj = NULL;
     return E_NOINTERFACE;
 }
 
@@ -111,7 +112,7 @@ HRESULT WINAPI IWineD3DStateBlockImpl_GetParent(IWineD3DStateBlock *iface, IUnkn
     IWineD3DStateBlockImpl *This = (IWineD3DStateBlockImpl *)iface;
     IUnknown_AddRef(This->parent);
     *pParent = This->parent;
-    return D3D_OK;
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DStateBlockImpl_GetDevice(IWineD3DStateBlock *iface, IWineD3DDevice** ppDevice){
@@ -120,7 +121,7 @@ HRESULT WINAPI IWineD3DStateBlockImpl_GetDevice(IWineD3DStateBlock *iface, IWine
 
     *ppDevice = (IWineD3DDevice*)This->wineD3DDevice;
     IWineD3DDevice_AddRef(*ppDevice);
-    return D3D_OK;
+    return WINED3D_OK;
 
 }
 
@@ -370,7 +371,7 @@ HRESULT WINAPI IWineD3DStateBlockImpl_Capture(IWineD3DStateBlock *iface){
 
     TRACE("(%p) : Updated state block %p ------------------^\n", targetStateBlock, This);
 
-    return D3D_OK;
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DStateBlockImpl_Apply(IWineD3DStateBlock *iface){
@@ -581,7 +582,7 @@ should really perform a delta so that only the changes get updated*/
     memcpy(&((IWineD3DDeviceImpl*)pDevice)->stateBlock->changed, &This->changed, sizeof(((IWineD3DDeviceImpl*)pDevice)->stateBlock->changed));
     TRACE("(%p) : Applied state block %p ------------------^\n", This, pDevice);
 
-    return D3D_OK;
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStateBlock* iface) {
@@ -785,9 +786,9 @@ HRESULT WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStateBlock*
         This->samplerState[i][WINED3DSAMP_ADDRESSV         ] = D3DTADDRESS_WRAP;
         This->samplerState[i][WINED3DSAMP_ADDRESSW         ] = D3DTADDRESS_WRAP;
         This->samplerState[i][WINED3DSAMP_BORDERCOLOR      ] = 0x00;
-        This->samplerState[i][WINED3DSAMP_MAGFILTER        ] = D3DTEXF_POINT;
-        This->samplerState[i][WINED3DSAMP_MINFILTER        ] = D3DTEXF_POINT;
-        This->samplerState[i][WINED3DSAMP_MIPFILTER        ] = D3DTEXF_NONE;
+        This->samplerState[i][WINED3DSAMP_MAGFILTER        ] = WINED3DTEXF_POINT;
+        This->samplerState[i][WINED3DSAMP_MINFILTER        ] = WINED3DTEXF_POINT;
+        This->samplerState[i][WINED3DSAMP_MIPFILTER        ] = WINED3DTEXF_NONE;
         This->samplerState[i][WINED3DSAMP_MIPMAPLODBIAS    ] = 0;
         This->samplerState[i][WINED3DSAMP_MAXMIPLEVEL      ] = 0;
         This->samplerState[i][WINED3DSAMP_MAXANISOTROPY    ] = 1;
@@ -852,7 +853,7 @@ HRESULT WINAPI IWineD3DStateBlockImpl_InitStartupStateBlock(IWineD3DStateBlock*
     This->wineD3DDevice->currentPalette = 0;
 
     TRACE("-----------------------> Device defaults now set up...\n");
-    return D3D_OK;
+    return WINED3D_OK;
 }
 
 /**********************************************************
