@@ -1,10 +1,12 @@
 /*
  * IWineD3DSurface Implementation
  *
+ * Copyright 2000-2001 TransGaming Technologies Inc.
  * Copyright 2002-2005 Jason Edmeades
  * Copyright 2002-2003 Raphael Junqueira
  * Copyright 2004 Christian Costa
  * Copyright 2005 Oliver Stieber
+ * Copyright 2006 Stefan Dsinger for CodeWeavers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,8 +47,9 @@ HRESULT WINAPI IWineD3DSurfaceImpl_QueryInterface(IWineD3DSurface *iface, REFIID
         || IsEqualGUID(riid, &IID_IWineD3DSurface)) {
         IUnknown_AddRef((IUnknown*)iface);
         *ppobj = This;
-        return WINED3D_OK;
+        return S_OK;
     }
+    *ppobj = NULL;
     return E_NOINTERFACE;
 }
 
@@ -69,6 +72,17 @@ ULONG WINAPI IWineD3DSurfaceImpl_Release(IWineD3DSurface *iface) {
             glDeleteTextures(1, &This->glDescription.textureName);
             LEAVE_GL();
         }
+
+        if(This->Flags & SFLAG_DIBSECTION) {
+            /* Release the DC */
+            SelectObject(This->hDC, This->dib.holdbitmap);
+            DeleteDC(This->hDC);
+            /* Release the DIB section */
+            DeleteObject(This->dib.DIBsection);
+            This->dib.bitmap_data = NULL;
+            This->resource.allocatedMemory = NULL;
+        }
+
         IWineD3DResourceImpl_CleanUp((IWineD3DResource *)iface);
 
         TRACE("(%p) Released\n", This);
@@ -218,7 +232,10 @@ HRESULT WINAPI IWineD3DSurfaceImpl_GetContainer(IWineD3DSurface* iface, REFIID r
     }
 
     TRACE("Relaying to QueryInterface\n");
-    return IUnknown_QueryInterface(container, riid, ppContainer);
+    if (IUnknown_QueryInterface(container, riid, ppContainer) != S_OK)
+        return WINED3DERR_INVALIDCALL;
+
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DSurfaceImpl_GetDesc(IWineD3DSurface *iface, WINED3DSURFACE_DESC *pDesc) {
@@ -241,7 +258,7 @@ void WINAPI IWineD3DSurfaceImpl_SetGlTextureDesc(IWineD3DSurface *iface, UINT te
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
     TRACE("(%p) : setting textureName %u, target %i\n", This, textureName, target);
     if (This->glDescription.textureName == 0 && textureName != 0) {
-        This->Dirty = TRUE;
+        This->Flags |= SFLAG_DIRTY;
         IWineD3DSurface_AddDirtyRect(iface, NULL);
     }
     This->glDescription.textureName = textureName;
@@ -271,13 +288,13 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
     static UINT messages = 0; /* holds flags to disable fixme messages */
 
     /* fixme: should we really lock as such? */
-    if (This->inTexture && This->inPBuffer) {
+    if((This->Flags & (SFLAG_INTEXTURE | SFLAG_INPBUFFER)) ==
+                      (SFLAG_INTEXTURE | SFLAG_INPBUFFER) ) {
         FIXME("Warning: Surface is in texture memory or pbuffer\n");
-        This->inTexture = 0;
-        This->inPBuffer = 0;
+        This->Flags &= ~(SFLAG_INTEXTURE | SFLAG_INPBUFFER);
     }
 
-    if (FALSE == This->lockable) {
+    if (!(This->Flags & SFLAG_LOCKABLE)) {
         /* Note: UpdateTextures calls CopyRects which calls this routine to populate the
               texture regions, and since the destination is an unlockable region we need
               to tolerate this                                                           */
@@ -308,22 +325,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         TRACE("(%p) : rect@%p flags(%08lx), output lockedRect@%p, memory@%p\n", This, pRect, Flags, pLockedRect, This->resource.allocatedMemory);
     }
 
-    /* DXTn formats don't have exact pitches as they are to the new row of blocks,
-         where each block is 4x4 pixels, 8 bytes (dxt1) and 16 bytes (dxt2/3/4/5)
-          ie pitch = (width/4) * bytes per block                                  */
-    if (This->resource.format == WINED3DFMT_DXT1) /* DXT1 is 8 bytes per block */
-        pLockedRect->Pitch = (This->currentDesc.Width >> 2) << 3;
-    else if (This->resource.format == WINED3DFMT_DXT2 || This->resource.format == WINED3DFMT_DXT3 ||
-             This->resource.format == WINED3DFMT_DXT4 || This->resource.format == WINED3DFMT_DXT5) /* DXT2/3/4/5 is 16 bytes per block */
-        pLockedRect->Pitch = (This->currentDesc.Width >> 2) << 4;
-    else {
-        if (NP2_REPACK == wined3d_settings.nonpower2_mode || This->resource.usage & WINED3DUSAGE_RENDERTARGET) {
-            /* Front and back buffers are always lockes/unlocked on currentDesc.Width */
-            pLockedRect->Pitch = This->bytesPerPixel * This->currentDesc.Width;  /* Bytes / row */
-        } else {
-            pLockedRect->Pitch = This->bytesPerPixel * This->pow2Width;
-        }
-    }
+    pLockedRect->Pitch = IWineD3DSurface_GetPitch(iface);
 
     if (NULL == pRect) {
         pLockedRect->pBits = This->resource.allocatedMemory;
@@ -346,7 +348,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         This->lockedRect.bottom = pRect->bottom;
     }
 
-    if (This->nonpow2) {
+    if (This->Flags & SFLAG_NONPOW2) {
         TRACE("Locking non-power 2 texture\n");
     }
 
@@ -355,15 +357,18 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         These resources may be POOL_SYSTEMMEM, so they must not access the device */
         TRACE("locking an ordinarary surface\n");
         /* Check to see if memory has already been allocated from the surface*/
-        if (NULL == This->resource.allocatedMemory) { /* TODO: check to see if an update has been performed on the surface (an update could just clobber allocatedMemory */
+        if ((NULL == This->resource.allocatedMemory) ||
+            (This->Flags & SFLAG_NEWDC) ){ /* TODO: check to see if an update has been performed on the surface (an update could just clobber allocatedMemory */
             /* Non-system memory surfaces */
 
             /*Surface has no memory currently allocated to it!*/
             TRACE("(%p) Locking rect\n" , This);
-            This->resource.allocatedMemory = HeapAlloc(GetProcessHeap() ,0 , This->pow2Size);
+            if(!This->resource.allocatedMemory) {
+                This->resource.allocatedMemory = HeapAlloc(GetProcessHeap() ,0 , This->pow2Size);
+            }
             if (0 != This->glDescription.textureName) {
                 /* Now I have to copy thing bits back */
-                This->activeLock = TRUE; /* When this flag is set to true, loading the surface again won't free THis->resource.allocatedMemory */
+                This->Flags |= SFLAG_ACTIVELOCK; /* When this flag is set to true, loading the surface again won't free THis->resource.allocatedMemory */
                 /* TODO: make activeLock a bit more intelligent, maybe implement a method to purge the texture memory. */
                 ENTER_GL();
     
@@ -437,7 +442,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         
                         internally the texture is still stored in a boxed format so any references to textureName will get a boxed texture with width pow2width and not a texture of width currentDesc.Width.
                         */
-                        if (This->nonpow2) {
+                        if (This->Flags & SFLAG_NONPOW2) {
                             BYTE* dataa, *datab;
                             int pitcha = 0, pitchb = 0;
                             int y;
@@ -489,7 +494,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         if (This->resource.allocatedMemory == NULL)
                 This->resource.allocatedMemory = HeapAlloc(GetProcessHeap() ,0 ,This->resource.size);
 
-        This->activeLock = TRUE; /*When this flag is set to true, loading the surface again won't free THis->resource.allocatedMemory*/
+        This->Flags |= SFLAG_ACTIVELOCK; /*When this flag is set to true, loading the surface again won't free THis->resource.allocatedMemory*/
         pLockedRect->pBits = This->resource.allocatedMemory;
 
         glFlush();
@@ -651,9 +656,9 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKE
         }
     }
 
-    TRACE("returning memory@%p, pitch(%d) dirtyfied(%d)\n", pLockedRect->pBits, pLockedRect->Pitch, This->Dirty);
+    TRACE("returning memory@%p, pitch(%d) dirtyfied(%d)\n", pLockedRect->pBits, pLockedRect->Pitch, This->Flags & SFLAG_DIRTY ? 0 : 1);
 
-    This->locked = TRUE;
+    This->Flags |= SFLAG_LOCKED;
     return WINED3D_OK;
 }
 
@@ -664,7 +669,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UnlockRect(IWineD3DSurface *iface) {
     const char *buffername = "";
     IWineD3DSwapChainImpl *swapchain = NULL;
 
-    if (FALSE == This->locked) {
+    if (!(This->Flags & SFLAG_LOCKED)) {
         WARN("trying to Unlock an unlocked surf@%p\n", This);
         return WINED3DERR_INVALIDCALL;
     }
@@ -687,9 +692,9 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UnlockRect(IWineD3DSurface *iface) {
         IWineD3DSwapChain_Release((IWineD3DSwapChain *)swapchain);
     }
 
-    TRACE("(%p %s) : dirtyfied(%d)\n", This, buffername, This->Dirty);
+    TRACE("(%p %s) : dirtyfied(%d)\n", This, buffername, This->Flags & SFLAG_DIRTY ? 1 : 0);
 
-    if (FALSE == This->Dirty) {
+    if (!(This->Flags & SFLAG_DIRTY)) {
         TRACE("(%p) : Not Dirtified so nothing to do, return now\n", This);
         goto unlock_end;
     }
@@ -715,6 +720,22 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UnlockRect(IWineD3DSurface *iface) {
             GLint  prev_draw;
             GLint  prev_rasterpos[4];
 
+            /* Some drivers(radeon dri, others?) don't like exceptions during
+             * glDrawPixels. If the surface is a DIB section, it might be in GDIMode
+             * after ReleaseDC. Reading it will cause an exception, which x11drv will
+             * catch to put the dib section in InSync mode, which leads to a crash
+             * and a blocked x server on my radeon card.
+             *
+             * The following lines read the dib section so it is put in inSync mode
+             * before glDrawPixels is called and the crash is prevented. There won't
+             * be any interfering gdi accesses, because UnlockRect is called from
+             * ReleaseDC, and the app won't use the dc any more afterwards.
+             */
+            if(This->Flags & SFLAG_DIBSECTION) {
+                volatile BYTE read;
+                read = This->resource.allocatedMemory[0];
+            }
+
             ENTER_GL();
 
             glFlush();
@@ -910,21 +931,208 @@ HRESULT WINAPI IWineD3DSurfaceImpl_UnlockRect(IWineD3DSurface *iface) {
     }
 
     unlock_end:
-    This->locked = FALSE;
+    This->Flags &= ~SFLAG_LOCKED;
     memset(&This->lockedRect, 0, sizeof(RECT));
     return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DSurfaceImpl_GetDC(IWineD3DSurface *iface, HDC *pHDC) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
-    FIXME("No support for GetDC yet for surface %p\n", This);
-    return WINED3DERR_INVALIDCALL;
+    WINED3DLOCKED_RECT lock;
+    UINT usage;
+    BITMAPINFO* b_info;
+    HDC ddc;
+    DWORD *masks;
+    HRESULT hr;
+    RGBQUAD col[256];
+
+    TRACE("(%p)->(%p)\n",This,pHDC);
+
+    /* Give more detailed info for ddraw */
+    if (This->Flags & SFLAG_DCINUSE)
+        return DDERR_DCALREADYCREATED;
+
+    /* Can't GetDC if the surface is locked */
+    if (This->Flags & SFLAG_LOCKED)
+        return WINED3DERR_INVALIDCALL;
+
+    memset(&lock, 0, sizeof(lock)); /* To be sure */
+
+    /* Create a DIB section if there isn't a hdc yet */
+    if(!This->hDC) {
+        if(This->Flags & SFLAG_ACTIVELOCK) {
+            ERR("Creating a DIB section while a lock is active. Uncertain consequences\n");
+        }
+
+        switch (This->bytesPerPixel) {
+            case 2:
+            case 4:
+                /* Allocate extra space to store the RGB bit masks. */
+                b_info = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(BITMAPINFOHEADER) + 3 * sizeof(DWORD));
+                break;
+
+            case 3:
+                b_info = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(BITMAPINFOHEADER));
+                break;
+
+            default:
+                /* Allocate extra space for a palette. */
+                b_info = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
+                                  sizeof(BITMAPINFOHEADER)
+                                  + sizeof(RGBQUAD)
+                                  * (1 << (This->bytesPerPixel * 8)));
+                break;
+        }
+
+        b_info->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+        b_info->bmiHeader.biWidth = This->pow2Width;
+        b_info->bmiHeader.biHeight = -This->pow2Height;
+        b_info->bmiHeader.biPlanes = 1;
+        b_info->bmiHeader.biBitCount = This->bytesPerPixel * 8;
+
+        b_info->bmiHeader.biSizeImage = This->resource.size;
+
+        b_info->bmiHeader.biXPelsPerMeter = 0;
+        b_info->bmiHeader.biYPelsPerMeter = 0;
+        b_info->bmiHeader.biClrUsed = 0;
+        b_info->bmiHeader.biClrImportant = 0;
+
+        /* Get the bit masks */
+        masks = (DWORD *) &(b_info->bmiColors);
+        switch (This->resource.format) {
+            case WINED3DFMT_R8G8B8:
+                usage = DIB_RGB_COLORS;
+                b_info->bmiHeader.biCompression = BI_RGB;
+                break;
+
+            case WINED3DFMT_X1R5G5B5:
+            case WINED3DFMT_A1R5G5B5:
+            case WINED3DFMT_A4R4G4B4:
+            case WINED3DFMT_X4R4G4B4:
+            case WINED3DFMT_R3G3B2:
+            case WINED3DFMT_A8R3G3B2:
+            case WINED3DFMT_A2B10G10R10:
+            case WINED3DFMT_A8B8G8R8:
+            case WINED3DFMT_X8B8G8R8:
+            case WINED3DFMT_A2R10G10B10:
+            case WINED3DFMT_R5G6B5:
+            case WINED3DFMT_A16B16G16R16:
+                usage = 0;
+                b_info->bmiHeader.biCompression = BI_BITFIELDS;
+                masks[0] = get_bitmask_red(This->resource.format);
+                masks[1] = get_bitmask_green(This->resource.format);
+                masks[2] = get_bitmask_blue(This->resource.format);
+                break;
+
+            default:
+                /* Don't know palette */
+                b_info->bmiHeader.biCompression = BI_RGB;
+                usage = 0;
+                break;
+        }
+
+        ddc = CreateDCA("DISPLAY", NULL, NULL, NULL);
+        if (ddc == 0) {
+            HeapFree(GetProcessHeap(), 0, b_info);
+            return HRESULT_FROM_WIN32(GetLastError());
+        }
+
+        TRACE("Creating a DIB section with size %ldx%ldx%d, size=%ld\n", b_info->bmiHeader.biWidth, b_info->bmiHeader.biHeight, b_info->bmiHeader.biBitCount, b_info->bmiHeader.biSizeImage);
+        This->dib.DIBsection = CreateDIBSection(ddc, b_info, usage, &This->dib.bitmap_data, 0 /* Handle */, 0 /* Offset */);
+        DeleteDC(ddc);
+
+        if (!This->dib.DIBsection) {
+            ERR("CreateDIBSection failed!\n");
+            return HRESULT_FROM_WIN32(GetLastError());
+        }
+        HeapFree(GetProcessHeap(), 0, b_info);
+
+        TRACE("DIBSection at : %p\n", This->dib.bitmap_data);
+
+        /* copy the existing surface to the dib section */
+        if(This->resource.allocatedMemory) {
+            memcpy(This->dib.bitmap_data, This->resource.allocatedMemory, This->resource.size);
+            /* We won't need that any more */
+            HeapFree(GetProcessHeap(), 0, This->resource.allocatedMemory);
+        }
+
+        /* Use the dib section from now on */
+        This->resource.allocatedMemory = This->dib.bitmap_data;
+
+        /* Now allocate a HDC */
+        This->hDC = CreateCompatibleDC(0);
+        This->dib.holdbitmap = SelectObject(This->hDC, This->dib.DIBsection);
+        TRACE("using wined3d palette %p\n", This->palette);
+        SelectPalette(This->hDC,
+                      This->palette ? This->palette->hpal : 0,
+                      FALSE);
+
+        /* This is to make LockRect read the gl Texture although memory is allocated */
+        This->Flags |= SFLAG_NEWDC;
+
+        This->Flags |= SFLAG_DIBSECTION;
+    }
+
+    /* Lock the surface */
+    hr = IWineD3DSurface_LockRect(iface,
+                                  &lock,
+                                  NULL,
+                                  0);
+    This->Flags &= ~SFLAG_NEWDC;
+    if(FAILED(hr)) {
+        ERR("IWineD3DSurface_LockRect failed with hr = %08lx\n", hr);
+        /* keep the dib section */
+        return hr;
+    }
+
+    if(This->resource.format == WINED3DFMT_P8 ||
+        This->resource.format == WINED3DFMT_A8P8) {
+        unsigned int n;
+        if(This->palette) {
+            PALETTEENTRY ent[256];
+
+            GetPaletteEntries(This->palette->hpal, 0, 256, ent);
+            for (n=0; n<256; n++) {
+                col[n].rgbRed   = ent[n].peRed;
+                col[n].rgbGreen = ent[n].peGreen;
+                col[n].rgbBlue  = ent[n].peBlue;
+                col[n].rgbReserved = 0;
+            }
+        } else {
+            IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
+
+            for (n=0; n<256; n++) {
+                col[n].rgbRed   = device->palettes[device->currentPalette][n].peRed;
+                col[n].rgbGreen = device->palettes[device->currentPalette][n].peGreen;
+                col[n].rgbBlue  = device->palettes[device->currentPalette][n].peBlue;
+                col[n].rgbReserved = 0;
+            }
+
+        }
+        SetDIBColorTable(This->hDC, 0, 256, col);
+    }
+
+    *pHDC = This->hDC;
+    TRACE("returning %p\n",*pHDC);
+    This->Flags |= SFLAG_DCINUSE;
+
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DSurfaceImpl_ReleaseDC(IWineD3DSurface *iface, HDC hDC) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
-    FIXME("No support for ReleaseDC yet for surface %p\n", This);
-    return WINED3DERR_INVALIDCALL;
+
+    TRACE("(%p)->(%p)\n",This,hDC);
+
+    if (!(This->Flags & SFLAG_DCINUSE))
+        return D3DERR_INVALIDCALL;
+
+    /* we locked first, so unlock now */
+    IWineD3DSurface_UnlockRect(iface);
+
+    This->Flags &= ~SFLAG_DCINUSE;
+
+    return WINED3D_OK;
 }
 
 /* ******************************************************
@@ -933,16 +1141,16 @@ HRESULT WINAPI IWineD3DSurfaceImpl_ReleaseDC(IWineD3DSurface *iface, HDC hDC) {
 HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
 
-    if (This->inTexture) {
+    if (This->Flags & SFLAG_INTEXTURE) {
         TRACE("Surface already in texture\n");
         return WINED3D_OK;
     }
-    if (This->Dirty == FALSE) {
+    if (!(This->Flags & SFLAG_DIRTY)) {
         TRACE("surface isn't dirty\n");
         return WINED3D_OK;
     }
 
-    This->Dirty = FALSE;
+    This->Flags &= ~SFLAG_DIRTY;
 
     /* Resources are placed in system RAM and do not need to be recreated when a device is lost.
     *  These resources are not bound by device size or format restrictions. Because of this,
@@ -957,7 +1165,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
         return WINED3DERR_INVALIDCALL;
     }
 
-    if (This->inPBuffer) {
+    if (This->Flags & SFLAG_INPBUFFER) {
         ENTER_GL();
 
         if (This->glDescription.level != 0)
@@ -987,7 +1195,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
             glReadBuffer(prevRead);
             vcheckGLcall("glReadBuffer");
             TRACE("Updating target %d\n", This->glDescription.target);
-            This->inTexture = TRUE;
+            This->Flags |= SFLAG_INTEXTURE;
         }
         LEAVE_GL();
         return WINED3D_OK;
@@ -1079,7 +1287,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
 
             LEAVE_GL();
 
-            if(This->activeLock == FALSE){
+            if(!(This->Flags & SFLAG_DONOTFREE)){
                 HeapFree(GetProcessHeap(), 0, This->resource.allocatedMemory);
                 This->resource.allocatedMemory = NULL;
             }
@@ -1087,7 +1295,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
     } else {
 
        /* TODO: possibly use texture rectangle (though we are probably more compatible without it) */
-        if (NP2_REPACK == wined3d_settings.nonpower2_mode && This->nonpow2 == TRUE) {
+        if (NP2_REPACK == wined3d_settings.nonpower2_mode && (This->Flags & SFLAG_NONPOW2)) {
 
 
             TRACE("non power of two support\n");
@@ -1179,7 +1387,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_LoadTexture(IWineD3DSurface *iface) {
             */
         }
 #endif
-        if(This->activeLock == FALSE){
+        if(!(This->Flags & SFLAG_DONOTFREE)){
             HeapFree(GetProcessHeap(),0,This->resource.allocatedMemory);
             This->resource.allocatedMemory = NULL;
         }
@@ -1219,7 +1427,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_SaveSnapshot(IWineD3DSurface *iface, const ch
 /* check to see if were a 'virtual' texture e.g. were not a pbuffer of texture were a back buffer*/
     IWineD3DSurface_GetContainer(iface, &IID_IWineD3DSwapChain, (void **)&swapChain);
 
-    if (swapChain || This->inPBuffer) { /* if were not a real texture then read the back buffer into a real texture*/
+    if (swapChain || (This->Flags & SFLAG_INPBUFFER)) { /* if were not a real texture then read the back buffer into a real texture*/
 /* we don't want to interfere with the back buffer so read the data into a temporary texture and then save the data out of the temporary texture */
         GLint prevRead;
         ENTER_GL();
@@ -1333,12 +1541,12 @@ HRESULT WINAPI IWineD3DSurfaceImpl_SaveSnapshot(IWineD3DSurface *iface, const ch
 
 HRESULT WINAPI IWineD3DSurfaceImpl_CleanDirtyRect(IWineD3DSurface *iface) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
-    This->Dirty = FALSE;
+    This->Flags &= ~SFLAG_DIRTY;
     This->dirtyRect.left   = This->currentDesc.Width;
     This->dirtyRect.top    = This->currentDesc.Height;
     This->dirtyRect.right  = 0;
     This->dirtyRect.bottom = 0;
-    TRACE("(%p) : Dirty?%d, Rect:(%ld,%ld,%ld,%ld)\n", This, This->Dirty, This->dirtyRect.left,
+    TRACE("(%p) : Dirty?%d, Rect:(%ld,%ld,%ld,%ld)\n", This, This->Flags & SFLAG_DIRTY ? 1 : 0, This->dirtyRect.left,
           This->dirtyRect.top, This->dirtyRect.right, This->dirtyRect.bottom);
     return WINED3D_OK;
 }
@@ -1349,7 +1557,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_CleanDirtyRect(IWineD3DSurface *iface) {
 extern HRESULT WINAPI IWineD3DSurfaceImpl_AddDirtyRect(IWineD3DSurface *iface, CONST RECT* pDirtyRect) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
     IWineD3DBaseTexture *baseTexture = NULL;
-    This->Dirty = TRUE;
+    This->Flags |= SFLAG_DIRTY;
     if (NULL != pDirtyRect) {
         This->dirtyRect.left   = min(This->dirtyRect.left,   pDirtyRect->left);
         This->dirtyRect.top    = min(This->dirtyRect.top,    pDirtyRect->top);
@@ -1361,7 +1569,7 @@ extern HRESULT WINAPI IWineD3DSurfaceImpl_AddDirtyRect(IWineD3DSurface *iface, C
         This->dirtyRect.right  = This->currentDesc.Width;
         This->dirtyRect.bottom = This->currentDesc.Height;
     }
-    TRACE("(%p) : Dirty?%d, Rect:(%ld,%ld,%ld,%ld)\n", This, This->Dirty, This->dirtyRect.left,
+    TRACE("(%p) : Dirty?%d, Rect:(%ld,%ld,%ld,%ld)\n", This, This->Flags & SFLAG_DIRTY, This->dirtyRect.left,
           This->dirtyRect.top, This->dirtyRect.right, This->dirtyRect.bottom);
     /* if the container is a basetexture then mark it dirty. */
     if (IWineD3DSurface_GetContainer(iface, &IID_IWineD3DBaseTexture, (void **)&baseTexture) == WINED3D_OK) {
@@ -1427,7 +1635,7 @@ HRESULT WINAPI IWineD3DSurfaceImpl_SetFormat(IWineD3DSurface *iface, WINED3DFORM
         This->pow2Size      = 0;
     }
 
-    This->lockable = (WINED3DFMT_D16_LOCKABLE == format) ? TRUE : This->lockable;
+    This->Flags |= (WINED3DFMT_D16_LOCKABLE == format) ? SFLAG_LOCKABLE : 0;
 
     This->resource.format = format;
 
@@ -1440,16 +1648,282 @@ HRESULT WINAPI IWineD3DSurfaceImpl_SetFormat(IWineD3DSurface *iface, WINED3DFORM
 HRESULT WINAPI IWineD3DSurfaceImpl_SetPBufferState(IWineD3DSurface *iface, BOOL inPBuffer, BOOL  inTexture) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
 
-    This->inPBuffer = inPBuffer;
-    This->inTexture = inTexture;
+    if(inPBuffer) {
+        This->Flags |= SFLAG_INPBUFFER;
+    } else {
+        This->Flags &= ~SFLAG_INPBUFFER;
+    }
+
+    if(inTexture) {
+        This->Flags |= SFLAG_INTEXTURE;
+    } else {
+        This->Flags &= ~SFLAG_INTEXTURE;
+    }
+
     return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DSurfaceImpl_Flip(IWineD3DSurface *iface, IWineD3DSurface *override, DWORD Flags) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
+    IWineD3DDevice *D3D = (IWineD3DDevice *) This->resource.wineD3DDevice;
+    TRACE("(%p)->(%p,%lx)\n", This, override, Flags);
+
+    /* Flipping is only supported on RenderTargets */
+    if( !(This->resource.usage & WINED3DUSAGE_RENDERTARGET) ) return DDERR_NOTFLIPPABLE;
+
+    if(override) {
+        /* DDraw sets this for the X11 surfaces, so don't confuse the user 
+         * FIXME("(%p) Target override is not supported by now\n", This);
+         * Additionally, it isn't really possible to support triple-buffering
+         * properly on opengl at all
+         */
+    }
+
+    /* Flipping a OpenGL surface -> Use WineD3DDevice::Present */
+    return IWineD3DDevice_Present(D3D, NULL, NULL, 0, NULL);
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_Blt(IWineD3DSurface *iface, RECT *DestRect, IWineD3DSurface *SrcSurface, RECT *SrcRect, DWORD Flags, DDBLTFX *DDBltFx) {
     FIXME("This is unimplemented for now(d3d7 merge)\n");
     return WINED3DERR_INVALIDCALL;
 }
 
+HRESULT WINAPI IWineD3DSurfaceImpl_GetBltStatus(IWineD3DSurface *iface, DWORD Flags) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
+    TRACE("(%p)->(%lx)\n", This, Flags);
+
+    switch (Flags)
+    {
+    case DDGBS_CANBLT:
+    case DDGBS_ISBLTDONE:
+        return DD_OK;
+
+    default:
+        return DDERR_INVALIDPARAMS;
+    }
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_GetFlipStatus(IWineD3DSurface *iface, DWORD Flags) {
+    /* XXX: DDERR_INVALIDSURFACETYPE */
+
+    TRACE("(%p)->(%08lx)\n",iface,Flags);
+    switch (Flags) {
+    case DDGFS_CANFLIP:
+    case DDGFS_ISFLIPDONE:
+        return DD_OK;
+
+    default:
+        return DDERR_INVALIDPARAMS;
+    }
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_IsLost(IWineD3DSurface *iface) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    TRACE("(%p)\n", This);
+
+    return This->Flags & SFLAG_LOST ? DDERR_SURFACELOST : WINED3D_OK;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_Restore(IWineD3DSurface *iface) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    TRACE("(%p)\n", This);
+
+    /* So far we don't lose anything :) */
+    This->Flags &= ~SFLAG_LOST;
+    return WINED3D_OK;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_BltFast(IWineD3DSurface *iface, DWORD dstx, DWORD dsty, IWineD3DSurface *Source, RECT *rsrc, DWORD trans) {
+    FIXME("This is unimplemented for now(d3d7 merge)\n");
+    return WINED3DERR_INVALIDCALL;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_SetPixelFormat(IWineD3DSurface *iface, WINED3DFORMAT Format, BYTE *Surface, DWORD Size) {
+    FIXME("This is unimplemented for now(d3d7 merge)\n");
+    return WINED3DERR_INVALIDCALL;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_GetPalette(IWineD3DSurface *iface, IWineD3DPalette **Pal) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    TRACE("(%p)->(%p)\n", This, Pal);
+
+    *Pal = (IWineD3DPalette *) This->palette;
+    return DD_OK;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_RealizePalette(IWineD3DSurface *iface) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    RGBQUAD col[256];
+    IWineD3DPaletteImpl *pal = This->palette;
+    unsigned int n;
+    TRACE("(%p)\n", This);
+
+    if(This->resource.format == WINED3DFMT_P8 ||
+       This->resource.format == WINED3DFMT_A8P8)
+    {
+        TRACE("Dirtifying surface\n");
+        This->Flags |= SFLAG_DIRTY;
+    }
+
+    TRACE("(%p): Updating the palette\n", This);
+    for (n=0; n<256; n++) {
+        col[n].rgbRed   = pal->palents[n].peRed;
+        col[n].rgbGreen = pal->palents[n].peGreen;
+        col[n].rgbBlue  = pal->palents[n].peBlue;
+        col[n].rgbReserved = 0;
+    }
+    SetDIBColorTable(This->hDC, 0, 256, col);
+
+    return WINED3D_OK;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_SetPalette(IWineD3DSurface *iface, IWineD3DPalette *Pal) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    IWineD3DPaletteImpl *PalImpl = (IWineD3DPaletteImpl *) Pal;
+    TRACE("(%p)->(%p)\n", This, Pal);
+
+    if(This->palette != NULL) 
+        if(This->resource.usage & WINED3DUSAGE_RENDERTARGET)
+            This->palette->Flags &= ~DDPCAPS_PRIMARYSURFACE;
+
+    if(PalImpl != NULL) {
+        if(This->resource.usage & WINED3DUSAGE_RENDERTARGET) {
+            /* Set the device's main palette if the palette
+             * wasn't a primary palette before
+             */
+            if(!(PalImpl->Flags & DDPCAPS_PRIMARYSURFACE)) {
+                IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
+                unsigned int i;
+
+                for(i=0; i < 256; i++) {
+                    device->palettes[device->currentPalette][i] = PalImpl->palents[i];
+                }
+            }
+
+            (PalImpl)->Flags |= DDPCAPS_PRIMARYSURFACE;
+        }
+    }
+    This->palette = PalImpl;
+
+    return IWineD3DSurface_RealizePalette(iface);
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_SetColorKey(IWineD3DSurface *iface, DWORD Flags, DDCOLORKEY *CKey) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    BOOL dirtify = FALSE;
+    TRACE("(%p)->(%08lx,%p)\n", This, Flags, CKey);
+
+    if ((Flags & DDCKEY_COLORSPACE) != 0) {
+        FIXME(" colorkey value not supported (%08lx) !\n", Flags);
+        return DDERR_INVALIDPARAMS;
+    }
+
+    /* Dirtify the surface, but only if a key was changed */
+    if(CKey) {
+        switch (Flags & ~DDCKEY_COLORSPACE) {
+            case DDCKEY_DESTBLT:
+                if(!(This->CKeyFlags & DDSD_CKDESTBLT)) {
+                    dirtify = TRUE;
+                } else {
+                    dirtify = memcmp(&This->DestBltCKey, CKey, sizeof(*CKey) ) != 0;
+                }
+                This->DestBltCKey = *CKey;
+                This->CKeyFlags |= DDSD_CKDESTBLT;
+                break;
+
+            case DDCKEY_DESTOVERLAY:
+                if(!(This->CKeyFlags & DDSD_CKDESTOVERLAY)) {
+                    dirtify = TRUE;
+                } else {
+                    dirtify = memcmp(&This->DestOverlayCKey, CKey, sizeof(*CKey)) != 0;
+                }
+                This->DestOverlayCKey = *CKey;
+                This->CKeyFlags |= DDSD_CKDESTOVERLAY;
+                break;
+
+            case DDCKEY_SRCOVERLAY:
+                if(!(This->CKeyFlags & DDSD_CKSRCOVERLAY)) {
+                    dirtify = TRUE;
+                } else {
+                    dirtify = memcmp(&This->SrcOverlayCKey, CKey, sizeof(*CKey)) != 0;
+                }
+                This->SrcOverlayCKey = *CKey;
+                This->CKeyFlags |= DDSD_CKSRCOVERLAY;
+                break;
+
+            case DDCKEY_SRCBLT:
+                if(!(This->CKeyFlags & DDSD_CKSRCBLT)) {
+                    dirtify = TRUE;
+                } else {
+                    dirtify = memcmp(&This->SrcBltCKey, CKey, sizeof(*CKey)) != 0;
+                }
+                This->SrcBltCKey = *CKey;
+                This->CKeyFlags |= DDSD_CKSRCBLT;
+                break;
+        }
+    }
+    else {
+        switch (Flags & ~DDCKEY_COLORSPACE) {
+            case DDCKEY_DESTBLT:
+                dirtify = This->CKeyFlags & DDSD_CKDESTBLT;
+                This->CKeyFlags &= ~DDSD_CKDESTBLT;
+                break;
+
+            case DDCKEY_DESTOVERLAY:
+                dirtify = This->CKeyFlags & DDSD_CKDESTOVERLAY;
+                This->CKeyFlags &= ~DDSD_CKDESTOVERLAY;
+                break;
+
+            case DDCKEY_SRCOVERLAY:
+                dirtify = This->CKeyFlags & DDSD_CKSRCOVERLAY;
+                This->CKeyFlags &= ~DDSD_CKSRCOVERLAY;
+                break;
+
+            case DDCKEY_SRCBLT:
+                dirtify = This->CKeyFlags & DDSD_CKSRCBLT;
+                This->CKeyFlags &= ~DDSD_CKSRCBLT;
+                break;
+        }
+    }
+
+    if(dirtify) {
+        TRACE("Color key changed, dirtifing surface\n");
+        This->Flags |= SFLAG_DIRTY;
+    }
+
+    return WINED3D_OK;
+}
+
+HRESULT WINAPI IWineD3DSurfaceImpl_PrivateSetup(IWineD3DSurface *iface) {
+    /* Nothing to do for now */
+    return WINED3D_OK;
+}
+
+DWORD WINAPI IWineD3DSurfaceImpl_GetPitch(IWineD3DSurface *iface) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    DWORD ret;
+    TRACE("(%p)\n", This);
+
+    /* DXTn formats don't have exact pitches as they are to the new row of blocks,
+         where each block is 4x4 pixels, 8 bytes (dxt1) and 16 bytes (dxt2/3/4/5)
+          ie pitch = (width/4) * bytes per block                                  */
+    if (This->resource.format == WINED3DFMT_DXT1) /* DXT1 is 8 bytes per block */
+        ret = (This->currentDesc.Width >> 2) << 3;
+    else if (This->resource.format == WINED3DFMT_DXT2 || This->resource.format == WINED3DFMT_DXT3 ||
+             This->resource.format == WINED3DFMT_DXT4 || This->resource.format == WINED3DFMT_DXT5) /* DXT2/3/4/5 is 16 bytes per block */
+        ret = (This->currentDesc.Width >> 2) << 4;
+    else {
+        if (NP2_REPACK == wined3d_settings.nonpower2_mode || This->resource.usage & WINED3DUSAGE_RENDERTARGET) {
+            /* Front and back buffers are always lockes/unlocked on currentDesc.Width */
+            ret = This->bytesPerPixel * This->currentDesc.Width;  /* Bytes / row */
+        } else {
+            ret = This->bytesPerPixel * This->pow2Width;
+        }
+    }
+    TRACE("(%p) Returning %ld\n", This, ret);
+    return ret;
+}
+
 const IWineD3DSurfaceVtbl IWineD3DSurface_Vtbl =
 {
     /* IUnknown */
@@ -1475,6 +1949,18 @@ const IWineD3DSurfaceVtbl IWineD3DSurface_Vtbl =
     IWineD3DSurfaceImpl_GetDC,
     IWineD3DSurfaceImpl_ReleaseDC,
     IWineD3DSurfaceImpl_Flip,
+    IWineD3DSurfaceImpl_Blt,
+    IWineD3DSurfaceImpl_GetBltStatus,
+    IWineD3DSurfaceImpl_GetFlipStatus,
+    IWineD3DSurfaceImpl_IsLost,
+    IWineD3DSurfaceImpl_Restore,
+    IWineD3DSurfaceImpl_BltFast,
+    IWineD3DSurfaceImpl_SetPixelFormat,
+    IWineD3DSurfaceImpl_GetPalette,
+    IWineD3DSurfaceImpl_SetPalette,
+    IWineD3DSurfaceImpl_RealizePalette,
+    IWineD3DSurfaceImpl_SetColorKey,
+    IWineD3DSurfaceImpl_GetPitch,
     /* Internal use: */
     IWineD3DSurfaceImpl_CleanDirtyRect,
     IWineD3DSurfaceImpl_AddDirtyRect,
@@ -1485,5 +1971,6 @@ const IWineD3DSurfaceVtbl IWineD3DSurface_Vtbl =
     IWineD3DSurfaceImpl_SetGlTextureDesc,
     IWineD3DSurfaceImpl_GetGlDesc,
     IWineD3DSurfaceImpl_GetData,
-    IWineD3DSurfaceImpl_SetFormat
+    IWineD3DSurfaceImpl_SetFormat,
+    IWineD3DSurfaceImpl_PrivateSetup
 };
