@@ -38,8 +38,9 @@ HRESULT WINAPI IWineD3DVolumeImpl_QueryInterface(IWineD3DVolume *iface, REFIID r
         || IsEqualGUID(riid, &IID_IWineD3DVolume)){
         IUnknown_AddRef(iface);
         *ppobj = This;
-        return WINED3D_OK;
+        return S_OK;
     }
+    *ppobj = NULL;
     return E_NOINTERFACE;
 }
 
@@ -141,7 +142,10 @@ HRESULT WINAPI IWineD3DVolumeImpl_GetContainer(IWineD3DVolume *iface, REFIID rii
     }
 
     TRACE("Relaying to QueryInterface\n");
-    return IUnknown_QueryInterface(This->container, riid, ppContainer);
+    if (IUnknown_QueryInterface(This->container, riid, ppContainer) != S_OK)
+        return WINED3DERR_INVALIDCALL;
+
+    return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DVolumeImpl_GetDesc(IWineD3DVolume *iface, WINED3DVOLUME_DESC* pDesc) {
@@ -287,23 +291,23 @@ HRESULT WINAPI IWineD3DVolumeImpl_LoadTexture(IWineD3DVolume *iface, GLenum gl_l
     TRACE("Calling glTexImage3D %x level=%d, intfmt=%x, w=%d, h=%d,d=%d, 0=%d, glFmt=%x, glType=%x, Mem=%p\n",
             GL_TEXTURE_3D,
             gl_level,
-            D3DFmt2GLIntFmt(myDevice, This->currentDesc.Format),
+            D3DFmt2GLIntFmt(myDevice, This->resource.format),
             This->currentDesc.Width,
             This->currentDesc.Height,
             This->currentDesc.Depth,
             0,
-            D3DFmt2GLFmt(myDevice, This->currentDesc.Format),
-            D3DFmt2GLType(myDevice, This->currentDesc.Format),
+            D3DFmt2GLFmt(myDevice, This->resource.format),
+            D3DFmt2GLType(myDevice, This->resource.format),
             This->resource.allocatedMemory);
     glTexImage3D(GL_TEXTURE_3D,
                     gl_level,
-                    D3DFmt2GLIntFmt(myDevice, This->currentDesc.Format),
+                    D3DFmt2GLIntFmt(myDevice, This->resource.format),
                     This->currentDesc.Width,
                     This->currentDesc.Height,
                     This->currentDesc.Depth,
                     0,
-                    D3DFmt2GLFmt(myDevice, This->currentDesc.Format),
-                    D3DFmt2GLType(myDevice, This->currentDesc.Format),
+                    D3DFmt2GLFmt(myDevice, This->resource.format),
+                    D3DFmt2GLType(myDevice, This->resource.format),
                     This->resource.allocatedMemory);
     checkGLcall("glTexImage3D");
     return WINED3D_OK;
