@@ -39,6 +39,7 @@
 
 #include "d3d9.h"
 #include "d3d9types.h"
+#include "ddraw.h"
 #include "wine/wined3d_interface.h"
 #include "wine/wined3d_gl.h"
 
@@ -282,6 +283,7 @@ do {
 /* Advance declaration of structures to satisfy compiler */
 typedef struct IWineD3DStateBlockImpl IWineD3DStateBlockImpl;
 typedef struct IWineD3DSurfaceImpl    IWineD3DSurfaceImpl;
+typedef struct IWineD3DPaletteImpl    IWineD3DPaletteImpl;
 
 /* Tracking */
 
@@ -373,41 +375,14 @@ void drawPrimitive(IWineD3DDevice *iface,
                     long  StartIdx,
                     short idxBytes,
                     const void *idxData,
-                    int   minIndex);
+                    int   minIndex,
+                    WineDirect3DVertexStridedData *DrawPrimStrideData);
 
-/*****************************************************************************
- * Structures required to draw primitives 
- */
+#define eps 1e-8
 
-typedef struct Direct3DStridedData {
-    BYTE     *lpData;        /* Pointer to start of data               */
-    DWORD     dwStride;      /* Stride between occurances of this data */
-    DWORD     dwType;        /* Type (as in D3DVSDT_TYPE)              */
-} Direct3DStridedData;
-
-typedef struct Direct3DVertexStridedData {
-    union {
-        struct {
-             Direct3DStridedData  position;
-             Direct3DStridedData  blendWeights;
-             Direct3DStridedData  blendMatrixIndices;
-             Direct3DStridedData  normal;
-             Direct3DStridedData  pSize;
-             Direct3DStridedData  diffuse;
-             Direct3DStridedData  specular;
-             Direct3DStridedData  texCoords[MAX_TEXTURES];
-             Direct3DStridedData  position2; /* tween data */
-             Direct3DStridedData  normal2;   /* tween data */
-             Direct3DStridedData  tangent;
-             Direct3DStridedData  binormal;
-             Direct3DStridedData  tessFactor;
-             Direct3DStridedData  fog;
-             Direct3DStridedData  depth;
-             Direct3DStridedData  sample;
-        } s;
-        Direct3DStridedData input[16];  /* Indexed by constants in D3DVSDE_REGISTER */
-    } u;
-} Direct3DVertexStridedData;
+/* Routine to fill gl caps for swapchains and IWineD3D */
+BOOL IWineD3DImpl_FillGLCaps(WineD3D_GL_Info *gl_info,
+                             Display* display);
 
 /*****************************************************************************
  * Internal representation of a light
@@ -567,6 +542,7 @@ typedef struct IWineD3DDeviceImpl
 
     /* Device state management */
     HRESULT                 state;
+    BOOL                    d3d_initialized;
 
     /* Screen buffer resources */
     glContext contextCache[CONTEXT_CACHE];
@@ -576,7 +552,10 @@ typedef struct IWineD3DDeviceImpl
 
     /* process vertex shaders using software or hardware */
     BOOL softwareVertexProcessing;
-    
+
+    /* DirectDraw stuff */
+    HWND ddraw_window;
+
 } IWineD3DDeviceImpl;
 
 extern const IWineD3DDeviceVtbl IWineD3DDevice_Vtbl;
@@ -755,6 +734,13 @@ typedef struct IWineD3DCubeTextureImpl
 
 extern const IWineD3DCubeTextureVtbl IWineD3DCubeTexture_Vtbl;
 
+typedef struct _WINED3DVOLUMET_DESC
+{
+    UINT                    Width;
+    UINT                    Height;
+    UINT                    Depth;
+} WINED3DVOLUMET_DESC;
+
 /*****************************************************************************
  * IWineD3DVolume implementation structure (extends IUnknown)
  */
@@ -765,7 +751,7 @@ typedef struct IWineD3DVolumeImpl
     IWineD3DResourceClass      resource;
 
     /* WineD3DVolume Information */
-    D3DVOLUME_DESC          currentDesc;
+    WINED3DVOLUMET_DESC      currentDesc;
     IWineD3DBase            *container;
     UINT                    bytesPerPixel;
 
@@ -808,6 +794,16 @@ typedef struct _WINED3DSURFACET_DESC
     UINT                    Height;
 } WINED3DSURFACET_DESC;
 
+/*****************************************************************************
+ * Structure for DIB Surfaces (GetDC and GDI surfaces)
+ */
+typedef struct wineD3DSurface_DIB {
+    HBITMAP DIBsection;
+    void* bitmap_data;
+    HGDIOBJ holdbitmap;
+    BOOL client_memory;
+} wineD3DSurface_DIB;
+
 /*****************************************************************************
  * IWineD3DSurface implementation structure
  */
@@ -820,12 +816,13 @@ struct IWineD3DSurfaceImpl
     /* IWineD3DSurface fields */
     IWineD3DBase              *container;
     WINED3DSURFACET_DESC      currentDesc;
+    IWineD3DPaletteImpl      *palette;
 
     UINT                      textureName;
     UINT                      bytesPerPixel;
 
     /* TODO: move this off into a management class(maybe!) */
-    BOOL                      nonpow2;
+    WORD                      Flags;
 
     UINT                      pow2Width;
     UINT                      pow2Height;
@@ -837,23 +834,63 @@ struct IWineD3DSurfaceImpl
     float                     pow2scalingFactorY; /* =  (Height / pow2Height) */
 #endif
 
-    BOOL                      lockable;
-    BOOL                      discard;
-    BOOL                      locked;
-    BOOL                      activeLock;
-    
     RECT                      lockedRect;
     RECT                      dirtyRect;
-    BOOL                      Dirty;
-    
-    BOOL                      inTexture;
-    BOOL                      inPBuffer;
 
     glDescriptor              glDescription;
+
+    /* For GetDC */
+    wineD3DSurface_DIB        dib;
+    HDC                       hDC;
+
+    /* Color keys for DDraw */
+    DDCOLORKEY                DestBltCKey;
+    DDCOLORKEY                DestOverlayCKey;
+    DDCOLORKEY                SrcOverlayCKey;
+    DDCOLORKEY                SrcBltCKey;
+    DWORD                     CKeyFlags;
+
 };
 
 extern const IWineD3DSurfaceVtbl IWineD3DSurface_Vtbl;
 
+/* Surface flags: */
+#define SFLAG_OVERSIZE    0x0001 /* Surface is bigger than gl size, blts only */
+#define SFLAG_CONVERTED   0x0002 /* Converted for color keying or Palettized */
+#define SFLAG_DIBSECTION  0x0004 /* Has a DIB section attached for getdc */
+#define SFLAG_DIRTY       0x0008 /* Surface was locked by the app */
+#define SFLAG_LOCKABLE    0x0010 /* Surface can be locked */
+#define SFLAG_DISCARD     0x0020 /* ??? */
+#define SFLAG_LOCKED      0x0040 /* Surface is locked atm */
+#define SFLAG_ACTIVELOCK  0x0080 /* Not locked, but surface memory is needed */
+#define SFLAG_INTEXTURE   0x0100 /* ??? */
+#define SFLAG_INPBUFFER   0x0200 /* ??? */
+#define SFLAG_NONPOW2     0x0400 /* Surface sizes are not a power of 2 */
+#define SFLAG_DYNLOCK     0x0800 /* Surface is often locked by the app */
+#define SFLAG_DYNCHANGE   0x1800 /* Surface contents are changed very often, implies DYNLOCK */
+#define SFLAG_DCINUSE     0x2000 /* Set between GetDC and ReleaseDC calls */
+#define SFLAG_NEWDC       0x4000 /* To inform LockRect about a new dc */
+#define SFLAG_LOST        0x8000 /* Surface lost flag for DDraw */
+
+/* In some conditions the surface memory must not be freed:
+ * SFLAG_OVERSIZE: Not all data can be kept in GL
+ * SFLAG_CONVERTED: Converting the data back would take too long
+ * SFLAG_DIBSECTION: The dib code manages the memory
+ * SFLAG_DIRTY: GL surface isn't up to date
+ * SFLAG_LOCKED: The app requires access to the surface data
+ * SFLAG_ACTIVELOCK: Some wined3d code needs the memory
+ * SFLAG_DYNLOCK: Avoid freeing the data for performance
+ * SFLAG_DYNCHANGE: Same reason as DYNLOCK
+ */
+#define SFLAG_DONOTFREE  (SFLAG_OVERSIZE   | \
+                          SFLAG_CONVERTED  | \
+                          SFLAG_DIBSECTION | \
+                          SFLAG_DIRTY      | \
+                          SFLAG_LOCKED     | \
+                          SFLAG_ACTIVELOCK | \
+                          SFLAG_DYNLOCK    | \
+                          SFLAG_DYNCHANGE    )
+
 /*****************************************************************************
  * IWineD3DVertexDeclaration implementation structure
  */
@@ -1167,7 +1204,23 @@ enum vsConstantsEnum {
     VS_CONSTANT_FLOAT
 };
 
+struct SHADER_OPCODE_ARG;
 typedef void (*shader_fct_t)();
+typedef void (*SHADER_HANDLER) (struct SHADER_OPCODE_ARG*);
+
+
+/* This must be 0 in the main branch until GLSL is at least mostly implemented.
+   Also, think about making it a winecfg option to use GLSL (if the card supports it)
+   or ARB_vertex_program. Ideally, we want to use GLSL if it's available, but until
+   everything is implemented, we'll probably have better luck with the ARB generation */
+#define USING_GLSL 0
+
+#define SHADER_PGMSIZE 65535
+typedef struct SHADER_BUFFER {
+    char* buffer;
+    size_t bsize;
+    unsigned int lineNo;
+} SHADER_BUFFER;
 
 typedef struct SHADER_OPCODE {
     unsigned int  opcode;
@@ -1175,16 +1228,57 @@ typedef struct SHADER_OPCODE {
     const char*   glname;
     CONST UINT    num_params;
     shader_fct_t  soft_fct;
+    SHADER_HANDLER hw_fct;
+    SHADER_HANDLER hw_glsl_fct;
     DWORD         min_version;
     DWORD         max_version;
 } SHADER_OPCODE;
 
+typedef struct SHADER_OPCODE_ARG {
+    IWineD3DBaseShader* shader;
+    CONST SHADER_OPCODE* opcode;
+    DWORD dst;
+    DWORD src[4];
+    SHADER_BUFFER* buffer;
+} SHADER_OPCODE_ARG;
+
+typedef struct SHADER_LIMITS {
+    unsigned int temporary;
+    unsigned int texture;
+    unsigned int constant_int;
+    unsigned int constant_float;
+    unsigned int constant_bool;
+    unsigned int address;
+} SHADER_LIMITS;
+
+/** Keeps track of details for TEX_M#x# shader opcodes which need to 
+    maintain state information between multiple codes */
+typedef struct SHADER_PARSE_STATE {
+    unsigned int current_row;
+    DWORD texcoord_w[2];
+} SHADER_PARSE_STATE;
+
+/* Base Shader utility functions. 
+ * (may move callers into the same file in the future) */
+extern int shader_addline(
+    SHADER_BUFFER* buffer,
+    const char* fmt, ...);
+
+extern const SHADER_OPCODE* shader_get_opcode(
+    IWineD3DBaseShader *iface, 
+    const DWORD code);
+
 /*****************************************************************************
  * IDirect3DBaseShader implementation structure
  */
 typedef struct IWineD3DBaseShaderClass
 {
     DWORD                           version;
+    DWORD                           hex_version;
+    DWORD                           temps_used;
+    DWORD                           textures_used;
+    SHADER_LIMITS                   limits;
+    SHADER_PARSE_STATE              parse_state;
     CONST SHADER_OPCODE             *shader_ins;
     CONST DWORD                     *function;
     UINT                            functionLength;
@@ -1200,6 +1294,27 @@ typedef struct IWineD3DBaseShaderImpl {
     IWineD3DBaseShaderClass         baseShader;
 } IWineD3DBaseShaderImpl;
 
+extern void shader_get_registers_used(
+    IWineD3DBaseShader *iface,
+    CONST DWORD* pToken);
+
+extern void shader_program_dump_decl_usage(
+    DWORD dcl,
+    DWORD param);
+
+extern void generate_base_shader(
+    IWineD3DBaseShader *iface,
+    SHADER_BUFFER* buffer,
+    CONST DWORD* pFunction);
+
+extern void shader_dump_ins_modifiers(
+    const DWORD output);
+
+inline static int shader_get_regtype(const DWORD param) {
+    return (((param & D3DSP_REGTYPE_MASK) >> D3DSP_REGTYPE_SHIFT) |
+            ((param & D3DSP_REGTYPE_MASK2) >> D3DSP_REGTYPE_SHIFT2));
+}
+
 /*****************************************************************************
  * IDirect3DVertexShader implementation structure
  */
@@ -1264,4 +1379,36 @@ typedef struct IWineD3DPixelShaderImpl {
 
 extern const SHADER_OPCODE IWineD3DPixelShaderImpl_shader_ins[];
 extern const IWineD3DPixelShaderVtbl IWineD3DPixelShader_Vtbl;
+
+/*****************************************************************************
+ * IWineD3DPalette implementation structure
+ */
+struct IWineD3DPaletteImpl {
+    /* IUnknown parts */
+    const IWineD3DPaletteVtbl  *lpVtbl;
+    LONG                       ref;
+
+    IUnknown                   *parent;
+    IWineD3DDeviceImpl         *wineD3DDevice;
+
+    /* IWineD3DPalette */
+    HPALETTE                   hpal;
+    WORD                       palVersion;     /*|               */
+    WORD                       palNumEntries;  /*|  LOGPALETTE   */
+    PALETTEENTRY               palents[256];   /*|               */
+    /* This is to store the palette in 'screen format' */
+    int                        screen_palents[256];
+    DWORD                      Flags;
+};
+
+extern const IWineD3DPaletteVtbl IWineD3DPalette_Vtbl;
+DWORD IWineD3DPaletteImpl_Size(DWORD dwFlags);
+
+/* DirectDraw utility functions */
+extern WINED3DFORMAT pixelformat_for_depth(DWORD depth);
+LONG get_bitmask_red(WINED3DFORMAT fmt);
+LONG get_bitmask_green(WINED3DFORMAT fmt);
+LONG get_bitmask_blue(WINED3DFORMAT fmt);
+LONG get_bitmask_alpha(WINED3DFORMAT fmt);
+
 #endif
