@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2003, 2004 Stefan Leichter
  * Copyright (C) 2005, 2006 Detlef Riekenberg
+ * Copyright (C) 2006 Dmitry Timoshkov
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +25,7 @@
 #include "winbase.h"
 #include "winerror.h"
 #include "wingdi.h"
+#include "winuser.h"
 #include "winreg.h"
 #include "winspool.h"
 
@@ -328,6 +330,89 @@ static void test_AddMonitor(void)
 
 }
 
+/* ########################### */
+
+static void test_DeleteMonitor(void)
+{
+    MONITOR_INFO_2A mi2a; 
+    struct  monitor_entry * entry = NULL;
+    DWORD   res;
+
+    entry = find_installed_monitor();
+
+    if (!entry) {
+        trace("No usable Monitor found: Skip tests\n");
+        return;
+    }
+
+    mi2a.pName = winetest_monitor;
+    mi2a.pEnvironment = entry->env;
+    mi2a.pDLLName = entry->dllname;
+
+    /* Testing DeleteMonitor with real options */
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, entry->env, winetest_monitor);
+    ok(res, "returned %ld with %ld (expected '!= 0')\n", res, GetLastError());
+
+    /* Delete the Monitor twice */
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, entry->env, winetest_monitor);
+    /* NT: ERROR_UNKNOWN_PRINT_MONITOR (3000), 9x: ERROR_INVALID_PARAMETER (87) */
+    ok( !res &&
+        ((GetLastError() == ERROR_UNKNOWN_PRINT_MONITOR) ||
+        (GetLastError() == ERROR_INVALID_PARAMETER)), 
+        "returned %ld with %ld (expected '0' with: ERROR_UNKNOWN_PRINT_MONITOR" \
+        " or ERROR_INVALID_PARAMETER)\n", res, GetLastError());
+
+    /* the environment */
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, NULL, winetest_monitor);
+    ok(res, "returned %ld with %ld (expected '!=0')\n", res, GetLastError());
+
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, "", winetest_monitor);
+    ok(res, "returned %ld with %ld (expected '!=0')\n", res, GetLastError());
+
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, "bad_env", winetest_monitor);
+    ok(res, "returned %ld with %ld (expected '!=0')\n", res, GetLastError());
+
+    /* the monitor-name */
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, entry->env, NULL);
+    /* NT: ERROR_INVALID_PARAMETER (87),  9x: ERROR_INVALID_NAME (123)*/
+    ok( !res &&
+        ((GetLastError() == ERROR_INVALID_PARAMETER) ||
+        (GetLastError() == ERROR_INVALID_NAME)),
+        "returned %ld with %ld (expected '0' with: ERROR_INVALID_PARAMETER or " \
+        "ERROR_INVALID_NAME)\n", res, GetLastError());
+
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA(NULL, entry->env, "");
+    /* NT: ERROR_INVALID_PARAMETER (87),  9x: ERROR_INVALID_NAME (123)*/
+    ok( !res && 
+        ((GetLastError() == ERROR_INVALID_PARAMETER) ||
+        (GetLastError() == ERROR_INVALID_NAME)),
+        "returned %ld with %ld (expected '0' with: ERROR_INVALID_PARAMETER or " \
+        "ERROR_INVALID_NAME)\n", res, GetLastError());
+
+    AddMonitorA(NULL, 2, (LPBYTE) &mi2a);
+    SetLastError(MAGIC_DEAD);
+    res = DeleteMonitorA("", entry->env, winetest_monitor);
+    ok(res, "returned %ld with %ld (expected '!=0')\n", res, GetLastError());
+
+    /* cleanup */
+    DeleteMonitorA(NULL, entry->env, winetest_monitor);
+}
+
+
 /* ########################### */
 
 static void test_EnumMonitors(void)
@@ -911,6 +996,159 @@ static void test_SetDefaultPrinter(void)
 
 }
 
+static void test_GetPrinterDriver(void)
+{
+    LPSTR default_printer;
+    HANDLE hprn;
+    BOOL ret;
+    BYTE *buf;
+    INT level;
+    DWORD needed, filled;
+
+    default_printer = find_default_printer();
+    if (!default_printer)
+    {
+        trace("There is no default printer installed, skiping the test\n");
+        return;
+    }
+
+    hprn = 0;
+    ret = OpenPrinter(default_printer, &hprn, NULL);
+    if (!ret)
+    {
+        trace("There is no printers installed, skiping the test\n");
+        return;
+    }
+    ok(hprn != 0, "wrong hprn %p\n", hprn);
+
+    for (level = -1; level <= 7; level++)
+    {
+        SetLastError(0xdeadbeef);
+        needed = (DWORD)-1;
+        ret = GetPrinterDriver(hprn, NULL, level, NULL, 0, &needed);
+        ok(!ret, "level %d: GetPrinterDriver should fail\n", level);
+        if (level >= 1 && level <= 6)
+        {
+            /* Not all levels are supported on all Windows-Versions */
+            if(GetLastError() == ERROR_INVALID_LEVEL) continue;
+            ok(GetLastError() == ERROR_INSUFFICIENT_BUFFER, "wrong error %ld\n", GetLastError());
+            ok(needed > 0,"not expected needed buffer size %ld\n", needed);
+        }
+        else
+        {
+            /* ERROR_OUTOFMEMORY found on win9x */
+            ok( ((GetLastError() == ERROR_INVALID_LEVEL) ||
+                 (GetLastError() == ERROR_OUTOFMEMORY)),
+                "%d: returned %d with %ld (expected '0' with: " \
+                "ERROR_INVALID_LEVEL or ERROR_OUTOFMEMORY)\n",
+                level, ret, GetLastError());
+            /* needed is modified in win9x. The modified Value depends on the
+               default Printer. testing for "needed == (DWORD)-1" will fail */
+            continue;
+        }
+
+        buf = HeapAlloc(GetProcessHeap(), 0, needed);
+
+        SetLastError(0xdeadbeef);
+        filled = -1;
+        ret = GetPrinterDriver(hprn, NULL, level, buf, needed, &filled);
+        ok(ret, "level %d: GetPrinterDriver error %ld\n", level, GetLastError());
+        ok(needed == filled, "needed %ld != filled %ld\n", needed, filled);
+
+        if (level == 2)
+        {
+            DRIVER_INFO_2 *di_2 = (DRIVER_INFO_2 *)buf;
+            DWORD calculated = sizeof(*di_2);
+
+            /* MSDN is wrong: The Drivers on the win9x-CD's have cVersion=0x0400
+               NT351: 1, NT4.0+w2k(Kernelmode): 2, w2k and above(Usermode): 3  */
+            ok((di_2->cVersion >= 0 && di_2->cVersion <= 3) ||
+                (di_2->cVersion == 0x0400), "di_2->cVersion = %ld\n", di_2->cVersion);
+            ok(di_2->pName != NULL, "not expected NULL ptr\n");
+            ok(di_2->pEnvironment != NULL, "not expected NULL ptr\n");
+            ok(di_2->pDriverPath != NULL, "not expected NULL ptr\n");
+            ok(di_2->pDataFile != NULL, "not expected NULL ptr\n");
+            ok(di_2->pConfigFile != NULL, "not expected NULL ptr\n");
+
+            trace("cVersion %ld\n", di_2->cVersion);
+            trace("pName %s\n", di_2->pName);
+            calculated += strlen(di_2->pName) + 1;
+            trace("pEnvironment %s\n", di_2->pEnvironment);
+            calculated += strlen(di_2->pEnvironment) + 1;
+            trace("pDriverPath %s\n", di_2->pDriverPath);
+            calculated += strlen(di_2->pDriverPath) + 1;
+            trace("pDataFile %s\n", di_2->pDataFile);
+            calculated += strlen(di_2->pDataFile) + 1;
+            trace("pConfigFile %s\n", di_2->pConfigFile);
+            calculated += strlen(di_2->pConfigFile) + 1;
+
+            /* XP allocates memory for both ANSI and unicode names */
+            ok(filled >= calculated,"calculated %ld != filled %ld\n", calculated, filled);
+        }
+
+        HeapFree(GetProcessHeap(), 0, buf);
+    }
+
+    SetLastError(0xdeadbeef);
+    ret = ClosePrinter(hprn);
+    ok(ret, "ClosePrinter error %ld\n", GetLastError());
+}
+
+static void test_DEVMODE(const DEVMODE *dm, LONG dmSize, LPCSTR exp_prn_name)
+{
+    /* On NT3.51, some fields in DEVMODE are empty/zero
+      (dmDeviceName, dmSpecVersion, dmDriverVersion and dmDriverExtra)
+       We skip the Tests on this Platform */
+    if (dm->dmSpecVersion || dm->dmDriverVersion || dm->dmDriverExtra) {
+        /* The Printername can be larger (MAX_PATH) than CCHDEVICENAME  */
+        ok(!strncmp(exp_prn_name, (LPCSTR)dm->dmDeviceName, CCHDEVICENAME), 
+            "expected '%s', got '%s'\n", exp_prn_name, dm->dmDeviceName);
+        ok(dm->dmSize + dm->dmDriverExtra == dmSize,
+            "%u != %ld\n", dm->dmSize + dm->dmDriverExtra, dmSize);
+    }
+    trace("dmFields %08lx\n", dm->dmFields);
+}
+
+static void test_DocumentProperties(void)
+{
+    LPSTR default_printer;
+    HANDLE hprn;
+    LONG dm_size, ret;
+    DEVMODE *dm;
+
+    default_printer = find_default_printer();
+    if (!default_printer)
+    {
+        trace("There is no default printer installed, skiping the test\n");
+        return;
+    }
+
+    hprn = 0;
+    ret = OpenPrinter(default_printer, &hprn, NULL);
+    if (!ret)
+    {
+        trace("There is no printers installed, skiping the test\n");
+        return;
+    }
+    ok(hprn != 0, "wrong hprn %p\n", hprn);
+
+    dm_size = DocumentProperties(0, hprn, NULL, NULL, NULL, 0);
+    trace("DEVMODE required size %ld\n", dm_size);
+    ok(dm_size >= sizeof(DEVMODE), "unexpected DocumentProperties ret value %ld\n", dm_size);
+
+    dm = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dm_size);
+
+    ret = DocumentProperties(0, hprn, NULL, dm, dm, DM_OUT_BUFFER);
+    ok(ret == IDOK, "DocumentProperties ret value %ld != expected IDOK\n", ret);
+
+    test_DEVMODE(dm, dm_size, default_printer);
+
+    HeapFree(GetProcessHeap(), 0, dm);
+
+    SetLastError(0xdeadbeef);
+    ret = ClosePrinter(hprn);
+    ok(ret, "ClosePrinter error %ld\n", GetLastError());
+}
 
 START_TEST(info)
 {
@@ -921,9 +1159,12 @@ START_TEST(info)
     find_default_printer();
 
     test_AddMonitor();
+    test_DeleteMonitor();
+    test_DocumentProperties();
     test_EnumMonitors(); 
     test_GetDefaultPrinter();
     test_GetPrinterDriverDirectory();
     test_OpenPrinter();
+    test_GetPrinterDriver();
     test_SetDefaultPrinter();
 }
