@@ -122,7 +122,7 @@ static LPOPENCONTEXT TABLET_FindOpenContext(HCTX hCtx)
 
 static void LoadTablet(void)
 {
-    TRACE("Initilizing the tablet to hwnd %p\n",hwndDefault);
+    TRACE("Initializing the tablet to hwnd %p\n",hwndDefault);
     gLoaded= TRUE;
     pLoadTabletInfo(hwndDefault);
 }
@@ -852,7 +852,7 @@ int WINAPI WTDataPeek(HCTX hCtx, UINT wBegin, UINT wEnd,
 
     if (bgn == context->PacketsQueued ||  end == context->PacketsQueued)
     {
-        TRACE("%i %i %i \n", bgn, end, context->PacketsQueued);
+        TRACE("%i %i %i\n", bgn, end, context->PacketsQueued);
         LeaveCriticalSection(&csTablet);
         return 0;
     }
