@@ -34,7 +34,6 @@
 #define PI M_PI
 #endif
 #include <string.h>
-#include <X11/Intrinsic.h>
 
 #include "x11drv.h"
 #include "x11font.h"
@@ -908,7 +907,7 @@ X11DRV_RoundRect( X11DRV_PDEVICE *physDev, INT left, INT top, INT right,
 COLORREF
 X11DRV_SetPixel( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color )
 {
-    Pixel pixel;
+    unsigned long pixel;
     POINT pt;
 
     pt.x = x;
@@ -1213,7 +1212,7 @@ X11DRV_PolyPolyline( X11DRV_PDEVICE *physDev, const POINT* pt, const DWORD* coun
 static void X11DRV_InternalFloodFill(XImage *image, X11DRV_PDEVICE *physDev,
                                      int x, int y,
                                      int xOrg, int yOrg,
-                                     Pixel pixel, WORD fillType )
+                                     unsigned long pixel, WORD fillType )
 {
     int left, right;
 
@@ -1272,6 +1271,11 @@ static void X11DRV_InternalFloodFill(XImage *image, X11DRV_PDEVICE *physDev,
 }
 
 
+static int ExtFloodFillXGetImageErrorHandler( Display *dpy, XErrorEvent *event, void *arg )
+{
+    return (event->request_code == X_GetImage && event->error_code == BadMatch);
+}
+
 /**********************************************************************
  *          X11DRV_ExtFloodFill
  */
@@ -1292,10 +1296,12 @@ X11DRV_ExtFloodFill( X11DRV_PDEVICE *physDev, INT x, INT y, COLORREF color,
     GetRgnBox( physDev->region, &rect );
 
     wine_tsx11_lock();
+    X11DRV_expect_error( gdi_display, ExtFloodFillXGetImageErrorHandler, NULL );
     image = XGetImage( gdi_display, physDev->drawable,
                        physDev->org.x + rect.left, physDev->org.y + rect.top,
                        rect.right - rect.left, rect.bottom - rect.top,
                        AllPlanes, ZPixmap );
+    if(X11DRV_check_error()) image = NULL;
     wine_tsx11_unlock();
     if (!image) return FALSE;
 
