@@ -167,7 +167,7 @@ static CRITICAL_SECTION xrender_cs = { &critsect_debug, -1, 0, 0, 0, 0 };
  */
 void X11DRV_XRender_Init(void)
 {
-    int error_base, event_base, i;
+    int event_base, i;
     XRenderPictFormat pf;
 
     if (client_side_with_render &&
@@ -200,9 +200,9 @@ LOAD_OPTIONAL_FUNCPTR(XRenderSetPictureTransform)
 
 
         wine_tsx11_lock();
-        if(pXRenderQueryExtension(gdi_display, &event_base, &error_base)) {
+        if(pXRenderQueryExtension(gdi_display, &event_base, &xrender_error_base)) {
             X11DRV_XRender_Installed = TRUE;
-            TRACE("Xrender is up and running error_base = %d\n", error_base);
+            TRACE("Xrender is up and running error_base = %d\n", xrender_error_base);
             screen_format = pXRenderFindVisualFormat(gdi_display, visual);
             if(!screen_format)
             {
@@ -344,8 +344,7 @@ static void FreeEntry(int entry)
             formatEntry->realized = NULL;
             if(formatEntry->bitmaps) {
                 for(i = 0; i < formatEntry->nrealized; i++)
-                    if(formatEntry->bitmaps[i])
-                        HeapFree(GetProcessHeap(), 0, formatEntry->bitmaps[i]);
+                    HeapFree(GetProcessHeap(), 0, formatEntry->bitmaps[i]);
                 HeapFree(GetProcessHeap(), 0, formatEntry->bitmaps);
                 formatEntry->bitmaps = NULL;
                 HeapFree(GetProcessHeap(), 0, formatEntry->gis);
@@ -1554,14 +1553,14 @@ BOOL X11DRV_AlphaBlend(X11DRV_PDEVICE *devDst, INT xDst, INT yDst, INT widthDst,
         top_down = TRUE;
         dstbits += widthSrc * (heightSrc - 1) * 4;
         y2 = ySrc;
-        y = y2 + heightSrc;
+        y = y2 + heightSrc - 1;
     }
     else
     {
         y = dib.dsBmih.biHeight - ySrc - 1;
-        y2 = y - heightSrc;
+        y2 = y - heightSrc + 1;
     }
-    for(; y > y2; y--) {
+    for(; y >= y2; y--) {
         memcpy(dstbits, (char *)dib.dsBm.bmBits + y * dib.dsBm.bmWidthBytes + xSrc * 4,
                widthSrc * 4);
         dstbits += (top_down ? -1 : 1) * widthSrc * 4;
