@@ -25,7 +25,9 @@
 #ifndef __WINE_D3DTYPES_H
 #define __WINE_D3DTYPES_H
 
-/* #include <windows.h> FIXME: Need to include for compatibility. Inclusion caused compile fail */
+#ifndef __WINESRC__
+# include <windows.h>
+#endif
 
 #include <float.h>
 #include <ddraw.h>
