@@ -1220,6 +1220,17 @@ BOOLAPI InternetEnumPerSiteCookieDecisionA(LPSTR,unsigned long *,unsigned long *
 BOOLAPI InternetEnumPerSiteCookieDecisionW(LPWSTR,unsigned long *,unsigned long *,unsigned long);
 #define InternetEnumPerSiteCookieDecision WINELIB_NAME_AW(InternetEnumPerSiteCookieDecision)
 
+#define INTERNET_COOKIE_IS_SECURE       0x00000001
+#define INTERNET_COOKIE_IS_SESSION      0x00000002
+#define INTERNET_COOKIE_THIRD_PARTY     0x00000010
+#define INTERNET_COOKIE_PROMPT_REQUIRED 0x00000020
+#define INTERNET_COOKIE_EVALUATE_P3P    0x00000040
+#define INTERNET_COOKIE_APPLY_P3P       0x00000080
+#define INTERNET_COOKIE_P3P_ENABLED     0x00000100
+#define INTERNET_COOKIE_IS_RESTRICTED   0x00000200
+#define INTERNET_COOKIE_IE6             0x00000400
+#define INTERNET_COOKIE_IS_LEGACY       0x00000800
+
 BOOLAPI InternetGetCookieExA(LPCSTR,LPCSTR,LPSTR,LPDWORD,DWORD,LPVOID);
 BOOLAPI InternetGetCookieExW(LPCWSTR,LPCWSTR,LPWSTR,LPDWORD,DWORD,LPVOID);
 #define InternetGetCookieEx WINELIB_NAME_AW(InternetGetCookieEx)
@@ -1369,11 +1380,12 @@ INTERNETAPI DWORD WINAPI InternetConfirmZoneCrossingW(HWND ,LPWSTR ,LPWSTR ,BOOL
 
 
 #define NORMAL_CACHE_ENTRY              0x00000001
+#define STICKY_CACHE_ENTRY              0x00000004
+#define EDITED_CACHE_ENTRY              0x00000008
 #define COOKIE_CACHE_ENTRY              0x00100000
 #define URLHISTORY_CACHE_ENTRY          0x00200000
 #define TRACK_OFFLINE_CACHE_ENTRY       0x00000010
 #define TRACK_ONLINE_CACHE_ENTRY        0x00000020
-#define STICKY_CACHE_ENTRY              0x00000004
 #define SPARSE_CACHE_ENTRY              0x00010000
 
 #define URLCACHE_FIND_DEFAULT_FILTER    NORMAL_CACHE_ENTRY             \
