@@ -21,7 +21,8 @@
 #ifndef __WINE_EXPLORER_PRIVATE_H
 #define __WINE_EXPLORER_PRIVATE_H
 
-extern void manage_desktop(void);
+extern void manage_desktop( char *arg );
+extern void initialize_hal(void);
 extern void initialize_systray(void);
 
 #endif  /* __WINE_EXPLORER_PRIVATE_H */
